@@ -7,7 +7,7 @@
 
 #include "mew.h"
 
-private char version_message[] = "version 3.3 20030522 Yasunari Momoi";
+private char version_message[] = "version 4.2 20040809 Yasunari Momoi";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -74,6 +74,8 @@ private int	MboxType;
 private int	Backup;
 private int	GetCur;
 private int	UseCL;
+private int	PreserveUnixFrom;
+private int	CreateMTime = TRUE;
 private int	Exit = 0;
 
 /****************************************************************
@@ -96,11 +98,11 @@ private int	maildir_names(const char *, char **, char **, char **);
 private int	new_inbox_file(int, char[]);
 private FILE	*open_new_inbox_file(int *, char[]);
 private int	get_from_dir(int, char *, char *, int);
-private void	process_maildir(int);
+private int	process_maildir(int);
 private int	lock_mbox(char *);
 private void	unlock_mbox(char *);
-private void	process_mbox(int);
-private void	process_stdin(int);
+private int	process_mbox(int);
+private int	process_stdin(int);
 private void	process(void);
 private int	check_mbox_type(const char *);
 private void	sanity_check(void);
@@ -221,6 +223,8 @@ help_message[] = {
 	"    -a            Retrieve all mail from maildir/{cur,new} directory.",
 	"                  (no backup) (for maildir)",
 	"    -c            Use Content-Length: field. (for mbox)",
+	"    -u            Don't create inboxdir/.mew-mtime file.",
+	"    -f            Preserve Unix From (Envelope Sender). (for mbox)",
 	NULL
 };
 
@@ -522,7 +526,7 @@ open_new_inbox_file(int *seq, char inboxfile[])
 	if (fd < 0)
 		warning("open(%s) for write", inboxfile);
 	else {
-		if ((fp = fdopen(fd, "w")) == NULL)
+		if ((fp = fdopen(fd, FDWRITE)) == NULL)
 			warning("open(%s) for write", inboxfile);
 	}
 	return fp;
@@ -588,7 +592,7 @@ get_from_dir(int seq, char *fromdir, char *backupdir, int backup)
 	return seq;
 }
 
-private void
+private int
 process_maildir(int seq)
 {
 	char *newdir, *curdir;
@@ -596,7 +600,7 @@ process_maildir(int seq)
 		error("maildir name is not set (%s)", Mbox);
 	if (GetCur)
 		seq = get_from_dir(seq, curdir, NULL, Backup);
-	get_from_dir(seq, newdir, curdir, Backup);
+	return get_from_dir(seq, newdir, curdir, Backup);
 }
 
 private int
@@ -607,7 +611,7 @@ lock_mbox(char *lockfile)
 
 	while (TRUE) {
 		if ((fd = open(lockfile, O_WRONLY | O_CREAT | O_EXCL)) < 0) {
-			if (errno == EACCES)
+			if (errno == EACCES || errno == EROFS)
 				return 1; /* doesn't need a lockfile, maybe. */
 			else if (errno != EEXIST)
 				error("open(%s)", lockfile);
@@ -631,7 +635,7 @@ unlock_mbox(char *lockfile)
 		unlink(lockfile);
 }
 
-private void
+private int
 process_mbox(int seq)
 {
 	char inboxfile[PATH_MAX];
@@ -661,7 +665,7 @@ process_mbox(int seq)
 		warning("flock(%s)", Mbox);  goto rerr;
 	}
 #endif
-	if ((srcfp = fdopen(srcfd, "r")) == NULL) {
+	if ((srcfp = fdopen(srcfd, FDREAD)) == NULL) {
 		warning("fdopen(%s) for read", Mbox);  goto rerr;
 	}
 
@@ -674,6 +678,11 @@ process_mbox(int seq)
 				dstfp = open_new_inbox_file(&seq, inboxfile);
 				if (dstfp == NULL)
 					goto rerr;
+				if (PreserveUnixFrom &&
+				    fputs(ln, dstfp) == EOF) {
+					warning("fputs(%s)", inboxfile);
+					goto werr;
+				}
 				state = ST_HEADER;
 			}
 			break;
@@ -704,6 +713,11 @@ process_mbox(int seq)
 				dstfp = open_new_inbox_file(&seq, inboxfile);
 				if (dstfp == NULL)
 					goto rerr;
+				if (PreserveUnixFrom &&
+				    fputs(ln, dstfp) == EOF) {
+					warning("fputs(%s)", inboxfile);
+					goto werr;
+				}
 				state = ST_HEADER;
 				break;
 			}
@@ -745,7 +759,7 @@ process_mbox(int seq)
 	}
 	fclose(srcfp);
 	unlock_mbox(MboxLock);
-	return;
+	return seq;
 
  werr:
 	if (dstfp)
@@ -756,9 +770,10 @@ process_mbox(int seq)
 		fclose(srcfp);
 	unlock_mbox(MboxLock);
 	error("process_mbox(%s)", Mbox);
+	return -1;		/* error. not reached */
 }
 
-private void
+private int
 process_stdin(int seq)
 {
 	char inboxfile[PATH_MAX];
@@ -782,7 +797,7 @@ process_stdin(int seq)
 		fclose(dstfp);
 		printf("%d\n", seq);
 	}
-	return;
+	return seq;
 
  werr:
 	if (dstfp)
@@ -790,26 +805,47 @@ process_stdin(int seq)
 	unlink(inboxfile);
  rerr:
 	error("process_stdin");
+	return -1;		/* error. not reached */
 }
 
 private void
 process(void)
 {
+	char mtimefile[PATH_MAX];
+	FILE *fp;
+	size_t wb;
+	int len = strlen(MEW_MTIME_PHRASE);
 	int seq = get_last_seq();
+	int newseq = 0;
 
 	switch (MboxType) {
 	case T_MAILDIR:
-		process_maildir(seq);
+		newseq = process_maildir(seq);
 		break;
 	case T_MBOX:
-		process_mbox(seq);
+		newseq = process_mbox(seq);
 		break;
 	case T_STDIN:
-		process_stdin(seq);
+		newseq = process_stdin(seq);
 		break;
 	default:
 		error("unknown mbox type (%s)", Mbox);
 	}
+
+	/* update .mew-mtime file if new mail arrived */
+	if (!CreateMTime || newseq <= seq)
+		return;		/* no new mail */
+	if (strlen(InboxDir) + strlen(MEW_MTIME_FILE) + 1 > PATH_MAX)
+		error("pathname too long (%s%s)", InboxDir, MEW_MTIME_FILE);
+	sprintf(mtimefile, "%s/%s", InboxDir, MEW_MTIME_FILE);
+
+	if ((fp = fopen(mtimefile, FDWRITE)) == NULL)
+		error("can't create file (%s)", mtimefile);
+	if ((wb = fwrite(MEW_MTIME_PHRASE, sizeof(char), len, fp)) != len) {
+		fclose(fp);
+		error("fwrite failed (%d, %s)", wb, mtimefile);
+	}
+	fclose(fp);
 }
 
 private void
@@ -840,7 +876,7 @@ main(int argc, char **argv)
 	warn_prog = progname;
 	init_env(argc, argv);
 
-	while ((ch = Getopt(argc, argv, "abcd:hi:m:svp")) != EOF) {
+	while ((ch = Getopt(argc, argv, "abcd:fhi:m:suvp")) != EOF) {
 		switch (ch) {
 		case 'a':
 			GetCur = TRUE;
@@ -857,6 +893,9 @@ main(int argc, char **argv)
 				error("pathname too long (%s)", Optarg);
 			sprintf(Mbox, "%s", Optarg);
 			break;
+		case 'f':
+			PreserveUnixFrom = TRUE;
+			break;
 		case 'i':
 			if (strlen(Optarg) + 1 > PATH_MAX)
 				error("pathname too long (%s)", Optarg);
@@ -869,6 +908,9 @@ main(int argc, char **argv)
 		case 's':
 			MboxType = T_STDIN;
 			break;
+		case 'u':
+			CreateMTime = FALSE;
+			break;
 		case 'v':
 			version(progname);
 			exit(EXIT_SUCCESS);
