@@ -142,7 +142,7 @@ highlighted specially."
   :group 'mew-highlight
   :type 'boolean)
 
-(defcustom mew-fancy-summary-ml-regex "[\[(][^])\n\r]*[\])]"
+(defcustom mew-fancy-summary-ml-regex "[\[(][^])|\n\r]*[\])|]"
   "*Regex string as a prefix of the subject of mails posted to ML."
   :group 'mew-highlight
   :type 'regexp)
@@ -271,7 +271,7 @@ return t, otherwise nil."
     (set (make-local-variable 'font-lock-fontify-region-function)
 	 'mew-fancy-summary-region)
     (setq mew-fancy-summary-scan-form
-	  (mew-summary-scan-form (mew-summary-folder-name 'ext)))
+		  (mew-get-summary-form (mew-summary-folder-name 'ext)))
     (setq mew-fancy-summary-thread-column -1)
     (font-lock-mode 1)))
 
@@ -293,7 +293,7 @@ return t, otherwise nil."
   (when (mew-thread-p)
     (mew-fancy-summary-remove-invisible)
     (setq mew-fancy-summary-scan-form
-	  (mew-summary-scan-form (substring (mew-summary-folder-name 'ext) 1)))
+		  (mew-get-summary-form (substring (mew-summary-folder-name 'ext) 1)))
     (setq mew-fancy-summary-thread-column (mew-vinfo-get-column))
     ;; XEmacs + lazy-lock ???
     (when mew-xemacs-p
@@ -374,25 +374,37 @@ return t, otherwise nil."
 	 (setq linebeg (point))
 	 (setq col 0)
 	 (cond
-	  ;; Normal line
+	  ;; unread line
 	  ((and (not (mew-in-decode-syntax-p))
-		(looking-at mew-regex-msg)
-		(setq numend (match-end 0))
-		(not (looking-at mew-regex-msg-mark)))
+                (setq numend (point-min))
+		(eq (char-after) mew-mark-unread))
 	   (let ((form mew-fancy-summary-scan-form) entry)
 	     (while form
 	       (setq entry (car form))
 	       (when (= col mew-fancy-summary-thread-column)
-		 (setq col (+ col
-			      (mew-fancy-summary-do-highlight 'thread t))))
+                 (setq col (+ col (mew-fancy-summary-do-highlight 'thread t))))
+	       (setq col (+ col (mew-fancy-summary-do-highlight entry nil numend)))
+	       (setq form (cdr form))))
+	   (looking-at "[^\r\n]*")
+	   (setq lineend (match-end 0))
+	   (put-text-property linebeg (1+ linebeg) 'face 'mew-face-mark-unread))
+	  ;; Normal line
+	  ((and (not (mew-in-decode-syntax-p))
+                (setq numend (point-min))
+                (not (looking-at mew-regex-mark)))
+	   (let ((form mew-fancy-summary-scan-form) entry)
+	     (while form
+	       (setq entry (car form))
+	       (when (= col mew-fancy-summary-thread-column)
+                 (setq col (+ col (mew-fancy-summary-do-highlight 'thread t))))
 	       (setq col (+ col (mew-fancy-summary-do-highlight entry nil numend)))
 	       (setq form (cdr form))))
 	   (looking-at "[^\r\n]*")
 	   (setq lineend (match-end 0)))
 	  ;; Marked line
 	  ((and (not (mew-in-decode-syntax-p))
-		(looking-at mew-regex-msg-mark))
-	   (let ((mark (string-to-char (match-string 2))))
+                (looking-at mew-regex-mark))
+           (let ((mark (string-to-char (match-string 0))))
 	     (looking-at "[^\r\n]*")
 	     (setq lineend (match-end 0))
 	     (put-text-property linebeg lineend
@@ -494,16 +506,15 @@ return t, otherwise nil."
 			 (move-to-column (+ (current-column) range))
 			 (point)
 		       (goto-char beg))))
-	     (prelen (and (stringp mew-scan-form-from-me-prefix)
-			  (string-width mew-scan-form-from-me-prefix)))
+			 (prelen (and (stringp mew-summary-form-from-me-prefix)
+						  (string-width mew-summary-form-from-me-prefix)))
 	     elen)
 	(if (string-match "[^\r\n]*[^ \r\n]" entry)
 	    (setq entry (substring entry (match-beginning 0) (match-end 0)))
 	  (setq entry ""))
 	(setq elen (string-width entry))
 	(if (not (and prelen (> prelen 0)
-		      (eq (string-match mew-scan-form-from-me-prefix entry)
-			  0)))
+					  (eq (string-match mew-summary-form-from-me-prefix entry) 0)))
 	    ;; Mail to me
 	    (if (mew-fancy-summary-special-p entry range)
 		(list (cons elen (mew-fancy-summary-get-face 'special t))
@@ -521,7 +532,15 @@ return t, otherwise nil."
      ((eq elem 'subj)
       (list (cons mew-fancy-summary-ml-regex
 		  (mew-fancy-summary-get-face 'ml t))
-	    (cons range face)))
+ 	    (cons
+	     (if (> mew-fancy-summary-thread-column 0)
+		 (save-excursion
+ 		   (move-to-column mew-fancy-summary-thread-column)
+		   (- range
+		      (* (mew-thread-get-property (point))
+			 mew-thread-indent-width)))
+	       range)
+	     face)))
      (t
       (list (cons range face))))))
 
