@@ -8,7 +8,7 @@
 ;; (eval-after-load "mew" '(require 'mew-nmz))
 ;;
 
-(defconst mew-nmz-version "mew-nmz.el 0.70")
+(defconst mew-nmz-version "mew-nmz.el 0.80")
 
 (eval-when-compile
   (require 'mew)
@@ -37,9 +37,9 @@
 
 (defvar mew-nmz-prog-mknmz "mknmz" "*Namazu make index program.")
 (defvar mew-nmz-prog-mknmz-args '("-q")
-  "*Mknmz's argment, in addition to \"-U\", \"-h\".")
+  "*Mknmz's argument, in addition to \"-U\", \"-h\".")
 (defvar mew-nmz-prog-mknmz-include "~/Namazu/mknmz-inc.pl" "*Include file for mknmz.")
-(defvar mew-nmz-mknmz-use-timestamp nil
+(defvar mew-nmz-mknmz-use-timestamp (featurep 'meadow)
   "*If non-nil, use NMZ.stamp file (see. immknmz --timestamp=on).")
 
 (defvar mew-nmz-prog-gcnmz "gcnmz" "*Namazu refresh index program.")
@@ -131,15 +131,14 @@ A good example is \"+from:\".")
   "Show original message location.
 If executed with 'C-u', jump to original message folder and number."
   (interactive "P")
-  (if (not (mew-virtual-p))
-      (message "This command can be used in Virtual mode only.")
     (save-excursion
+      (mew-summary-goto-message)
       (let ((fld (mew-summary-folder-name))
 	    (msg (mew-summary-message-number)))
 	(message "Original message at %s/%s" fld msg)
-	(when arg
+	(when (and arg (mew-virtual-p))
 	  (mew-nmz-goto-folder-msg fld msg)
-	  (message "Original message at %s/%s...jump done" fld msg))))))
+	  (message "Original message at %s/%s...jump done" fld msg)))))
 
 ;; macros
 (defmacro mew-nmz-expand-folder (fld)
@@ -402,19 +401,17 @@ If executed with 'C-u', remove index files at the specified folder."
 	     (with-temp-buffer
 	       (message "checking %s..." (file-name-nondirectory ufname))
 	       (insert-file-contents ufname)
-	       (while (re-search-forward "/\\([1-9][0-9]*\\)$" nil t)
+	       (while (re-search-forward "/\\([0-9]+\\)$" nil t)
 		 (setq msgnums (cons (string-to-number (match-string 1)) msgnums))))
 	     (message "checking %s..." (buffer-name))
 	     (goto-char (point-min))
 	     (while (not (eobp))
-	       (if (and
-		    (looking-at "^ *\\([1-9][0-9]*\\)")
-		    (not (memq (string-to-number (match-string 1)) msgnums))
-		    (not (mew-in-decode-syntax-p)))
+	       (if (and (mew-sumsyn-match mew-regex-sumsyn-short)
+			(not (memq (string-to-number (mew-sumsyn-message-number)) msgnums))
+			(not (mew-in-decode-syntax-p)))
 		   (progn
 		     (setq umsgs (1+ umsgs))
-		     (if (mew-summary-marked-p)
-			 ()
+		     (when (mew-summary-markable)
 		       (mew-summary-mark-as mew-nmz-mark-unindexed)
 		       (setq mmsgs (1+ mmsgs)))))
 	       (forward-line))
@@ -442,7 +439,7 @@ If executed with 'C-u', search *child* message."
 	(proto (or (mew-proto-to-refile (mew-summary-folder-name 'ext))
 		   "+"))
 	(case (mew-sinfo-get-case))
-	refiledir mess ref pid rh)
+	refiledir mess ref pid rh pos)
     (if mid
 	(setq pid (list mid) idh nil)
       (if (not (or (mew-summary-message-number) (mew-syntax-number)))
@@ -453,9 +450,9 @@ If executed with 'C-u', search *child* message."
 			  (mew-summary-folder-name) (mew-summary-message-number))
 			 (mew-buffer-message)))
 	  (set-buffer mess)
-	  (mew-sinfo-set-case case)
-	  (mew-sinfo-set-proto proto)
-	  (setq refilefld (append (car (mew-refile-guess nil t)) refilefld))
+	  (let ((mew-inherit-refile-proto proto)
+		(mew-inherit-refile-case case))
+	    (setq refilefld (append (car (mew-refile-guess nil t)) refilefld)))
 	  (if child
 	      (setq idh (car (cdr idh)))
 	    (setq idh (car idh)))
@@ -532,23 +529,20 @@ If executed with 'C-u', search *child* message."
 		))))
 	(if (null range)
 	    (message "No message found.")
-	  (if (mew-virtual-p)
-	      (save-excursion
-		(goto-char (point-min))
-		(if (re-search-forward
-		     (concat " \006 " (regexp-quote fld) " " (car range) "$")
-		     nil t)
-		    (progn
-		      (setq fld (buffer-name))
-		      (goto-char (match-beginning 0))
-		      (beginning-of-line)
-		      (looking-at "^ *\\([1-9][0-9]*\\)[^0-9]")
-		      (setq range (list (match-string 1)))))))
+	  (when (mew-virtual-p)
+	    (save-excursion
+	      (goto-char (point-min))
+	      (when (re-search-forward
+		     (concat "\r \\(" (regexp-quote fld) "\\)? +" (car range) " ") nil t)
+		(setq fld (buffer-name))
+		(goto-char (match-beginning 0))
+		(beginning-of-line)
+		(setq pos (point)))))
 	  (if (listp (car range))
 	      (setq fld (car (car range))
 		    mess (car (cdr (car range))))
 	    (setq mess (car range)))
-	  (mew-nmz-goto-folder-msg fld mess)
+	  (mew-nmz-goto-folder-msg fld mess pos)
 	  (message "Searching %s...%s/%s" message fld mess))))))
 
 (defun mew-nmz-search-msgid-at-point ()
@@ -579,7 +573,7 @@ If executed with 'C-u', search *child* message."
   "Namazu pick messages according to a pick pattern which you input,
 then put the '*' mark onto them. "
   (interactive)
-  (mew-summary-or-thread
+  (mew-pickable
    (if (eq (point-min) (point-max))
        (message "No messages in this buffer.")
      (let ((fld (mew-summary-folder-name))
@@ -624,7 +618,7 @@ then put the '*' mark onto them. "
 	       (setq i 0)
 	       (setq msgs msgsback)
 	       (while (and msgs (not (eobp)))
-		 (when (re-search-forward (mew-regex-jmp-msg (car msgs)) nil t)
+		 (when (mew-nmz-re-search-message (car msgs))
 		   (mew-summary-mark-as mew-mark-review)
 		   (forward-line))
 		 (setq i (1+ i))
@@ -636,99 +630,126 @@ then put the '*' mark onto them. "
 
 ;;
 ;; "Namazu virtual" function.
+(defvar mew-nmz-input-folders-single t)
+(defvar mew-nmz-input-folders-asterisk t)
+
+(defun mew-nmz-input-folders ()
+  (let ((map (copy-keymap mew-input-folder-map)))
+    (define-key map "*" 'mew-input-folder-self-insert)
+    (let ((mew-input-folder-map map))
+      (or (mew-input-refile-folders
+	   (list (concat (or (mew-summary-folder-name) "+")
+			 (if mew-nmz-input-folders-asterisk "*" "")))
+	   mew-nmz-input-folders-single nil "+")
+	  '("+*")))))
+
 (defun mew-nmz-virtual ()
   "Another virtual mode with namazu."
   (interactive)
   (if (not (mew-summary-or-virtual-p))
       (message "This command cannot be used in this mode.")
-    (let ((vfld (concat
-		 "*"
-		 (mew-input-string "Namazu virtual folder name %s(%s): "
-				   "" ;; dummy
-				   "vnamazu")))
-	  (flds (mew-input-refile-folders (list (concat (mew-summary-folder-name) "*"))
-					  t nil "+"))
-	  (pattern (mew-nmz-input-pick-pattern))
-	  (buf (generate-new-buffer mew-buffer-prefix))
-	  (file (mew-make-temp-name))
-	  nmzdirs fldmsgs fld msgs fasts scans count func lra)
+    (let* ((flds (mew-nmz-input-folders))
+	   (pattern (mew-nmz-input-pick-pattern (mapconcat 'identity flds ",")))
+	   (vfld (mew-summary-get-vfolder flds))
+	   (buf (generate-new-buffer mew-buffer-prefix))
+	   (file (mew-make-temp-name))
+	   nmzdirs fldmsgs fld msgs fasts scans count func lra pickflds
+	   line lines beg med end)
       (if (null mew-nmz-indexed-folders)
 	  (mew-nmz-gather-indexed-folder))
-      (setq nmzdirs (mew-nmz-flds-to-indexs (or flds '("+*"))))
+      (setq nmzdirs (mew-nmz-flds-to-indexs flds))
       (if (null nmzdirs)
 	  (message "Please make namazu index.")
-	(setq fldmsgs (mew-nmz-multi-pick nmzdirs pattern))
-	(if (null fldmsgs)
-	    (message "No message pick.")
-	  (while fldmsgs
-	    (if (and mew-nmz-use-fast-pick
-		     (mew-nmz-folder-newp (car (car fldmsgs))))
-		(setq fasts (cons (car fldmsgs) fasts))
-	      (setq scans (cons (car fldmsgs) scans)))
-	    (setq fldmsgs (cdr fldmsgs)))
-	  (setq fasts (nreverse fasts))
-	  (setq scans (nreverse scans))
-	  ;;
-	  (mew-summary-switch-to-folder vfld)
-	  (mew-erase-buffer)
-	  (setq count 1)
-	  (when fasts
-	    (while (setq fld (car (car fasts)))
-	      (message "Namazu picking...(%s)" fld)
-	      (setq pattern "")
-	      (setq msgs (cdr (car fasts)))
+	(mew-summary-switch-to-folder vfld)
+	(when (mew-summary-exclusive-p)
+	  (setq fldmsgs (mew-nmz-multi-pick nmzdirs pattern))
+	  (if (null fldmsgs)
+	      (message "No message pick.")
+	    (while fldmsgs
+	      (setq pickflds (cons (car (car fldmsgs)) pickflds))
+	      (if (and mew-nmz-use-fast-pick
+		       (mew-nmz-folder-newp (car (car fldmsgs))))
+		  (setq fasts (cons (car fldmsgs) fasts))
+		(setq scans (cons (car fldmsgs) scans)))
+	      (setq fldmsgs (cdr fldmsgs)))
+	    (setq fasts (nreverse fasts))
+	    (setq scans (nreverse scans))
+	    (setq pickflds (nreverse pickflds))
+	    ;;
+	    (mew-erase-buffer)
+	    (mew-vinfo-set-flds pickflds)
+	    (setq count 1)
+	    (when fasts
+	      (while (setq fld (car (car fasts)))
+		(message "Namazu picking...(%s)" fld)
+		(setq lines nil)
+		(setq msgs (cdr (car fasts)))
+		(set-buffer buf)
+		(buffer-disable-undo (current-buffer))
+		(mew-erase-buffer)
+		(mew-frwlet
+		 mew-cs-m17n mew-cs-dummy
+		 (insert-file-contents
+		  (expand-file-name mew-summary-cache-file (mew-expand-folder fld))))
+		(goto-char (point-min))
+		(while (and (not (eobp)) msgs)
+		  (when (mew-nmz-re-search-message (car msgs))
+		    (beginning-of-line)
+		    (when (looking-at
+			   "^\\([^\r]*\\)\r +\\([^\n]*\n\\)")
+		      (setq lines (cons (list
+					 (match-string 1)
+					 (format "\r %s %s"
+						 fld (match-string 2)))
+					lines))
+		      (setq count (1+ count))))
+		  (forward-line 1)
+		  (setq msgs (cdr msgs)))
+		(setq lines (nreverse lines))
+		(when lines
+		  (mew-summary-switch-to-folder vfld)
+		  (goto-char (point-max))
+		  (while (setq line (car lines))
+		    (mew-elet 
+		     (setq beg (point))
+		     (insert (car line))
+		     (setq med (point))
+		     (insert (car (cdr line)))
+		     (setq end (point))
+		     (goto-char beg)
+		     (mew-front-nonsticky beg med) ;; for XEmacs
+		     (if mew-use-highlight-mouse-line
+			 (put-text-property
+			  beg med 'mouse-face mew-highlight-mouse-line-face))
+		     (forward-line)
+		     (put-text-property med (1- (point)) 'invisible t))
+		    (goto-char (point-max))
+		    (setq lines (cdr lines)))
+		  (mew-summary-set-count-line))
+		(setq fasts (cdr fasts))))
+	    (if (null scans)
+		(message "Namazu picking...done")
 	      (set-buffer buf)
 	      (buffer-disable-undo (current-buffer))
 	      (mew-erase-buffer)
+	      (while (setq fld (car (car scans)))
+		(setq lra (cons (cons fld fld) lra))
+		(insert "CD: " fld "\n")
+		(setq msgs (cdr (car scans)))
+		(while msgs
+		  (insert (car msgs) "\n")
+		  (setq msgs (cdr msgs)))
+		(setq scans (cdr scans)))
 	      (mew-frwlet
-	       mew-cs-m17n mew-cs-dummy
-	       (insert-file-contents
-		(expand-file-name mew-summary-cache-file (mew-expand-folder fld))))
-	      (goto-char (point-min))
-	      (while (and (not (eobp)) msgs)
-		(when (re-search-forward (concat "^ *" (car msgs) "[^0-9]") nil t)
-		  (beginning-of-line)
-		  (when (looking-at	;; new format "\r <msgid> <par-id> \006 folder msg"
-			 "^ *\\([1-9][0-9]*\\)[^0-9]\\([^\r]*\r\\)\\([^\n]*\\)")
-		    (setq pattern
-			  ;; 123 .... \r  <msgid> <par-id> folder num
-			  (concat pattern
-				  (format "%5d %s%s \006 %s %s\n"
-					  count
-					  (match-string 2)	;; don't use match-string
-					  (match-string 3)	;; for fancy-hightlight
-					  fld (car msgs))))
-		    (setq count (1+ count))))
-		(setq msgs (cdr msgs))
-		(forward-line 1))
-	      (mew-summary-switch-to-folder vfld)
-	      (goto-char (point-max))
-	      (mew-elet
-	       (insert pattern))
-	      (setq fasts (cdr fasts))))
-	  (if (null scans)
-	      (message "Namazu picking...done")
-	    (set-buffer buf)
-	    (buffer-disable-undo (current-buffer))
-	    (mew-erase-buffer)
-	    (while (setq fld (car (car scans)))
-	      (setq lra (cons (cons fld fld) lra))
-	      (insert "CD: " fld "\n")
-	      (setq msgs (cdr (car scans)))
-	      (while msgs
-		(insert (car msgs) "\n")
-		(setq msgs (cdr msgs)))
-	      (setq scans (cdr scans)))
-	    (mew-frwlet
-	     mew-cs-text-for-read mew-cs-text-for-write
-	     (write-region (point-min) (point-max) file nil 'nomsg))
-	    (setq scans t))
-	  (mew-remove-buffer buf)
-	  (mew-summary-switch-to-folder vfld)
-	  (when scans
-	    (setq func `(lambda () (when (file-exists-p ,file)
-				     (delete-file ,file))))
-	    (mew-local-retrieve 'vir (list "-i" file) count func lra 'noerase)))))))
+	       mew-cs-text-for-read mew-cs-text-for-write
+	       (write-region (point-min) (point-max) file nil 'nomsg))
+	      (setq scans t))
+	    (mew-remove-buffer buf)
+	    (mew-summary-switch-to-folder vfld)
+	    (when scans
+	      (setq func `(lambda () (when (file-exists-p ,file)
+				       (delete-file ,file))))
+	      (mew-local-retrieve 'vir (list "-i" file) func lra t))))))))
 
 ;;
 ;; Use namazu-mode.
@@ -750,10 +771,10 @@ then put the '*' mark onto them. "
 	 (mew-nmz-gather-indexed-folder))
      (setq mew-nmz-namazu-last-folder (mew-summary-folder-name 'ext))
      (if (or arg (not (mew-nmz-namazu-goto-namazu)))
-	 (let ((flds (mew-input-folders (concat (mew-summary-folder-name) "*")))
-	       (pattern (mew-nmz-input-pick-pattern))
-	       fld nmzdirs dirlen)
-	   (setq nmzdirs (mew-nmz-flds-to-indexs (or flds '("+*"))))
+	 (let* ((flds (mew-nmz-input-folders))
+		(pattern (mew-nmz-input-pick-pattern (mapconcat 'identity flds ",")))
+		fld nmzdirs dirlen)
+	   (setq nmzdirs (mew-nmz-flds-to-indexs flds))
 	   (setq dirlen (length nmzdirs))
 	   (if (not (> dirlen mew-nmz-db-max))
 	       ()
@@ -843,18 +864,21 @@ then put the '*' mark onto them. "
 
 ;;
 ;; Input "Namazu pattern" funcions.
-(defun mew-nmz-input-pick-pattern ()
+(defun mew-nmz-input-pick-pattern (&optional fld-str)
   "Input mew-nmz pick pattern."
   (mew-input-clear)
   (let ((mew-nmz-gather-header-list (mew-nmz-pick-pattern-gather-header)))
     (setq mew-input-complete-function (function mew-nmz-pick-pattern))
     (let ((keymap (copy-keymap mew-input-map)) pattern)
       (define-key keymap " " nil)
-      (setq pattern (read-from-minibuffer "Namazu pick pattern: "
-					  mew-nmz-pick-default-field
-					  keymap
-					  nil
-					  'mew-nmz-pick-pattern-hist))
+      (setq pattern (read-from-minibuffer
+		     (if fld-str
+			 (format "Namazu pattern for '%s': " fld-str)
+		       "Namazu pattern: ")
+		     mew-nmz-pick-default-field
+		     keymap
+		     nil
+		     'mew-nmz-pick-pattern-hist))
       (mew-decode-syntax-delete)
       ;; for M-n, M-p
       (setq mew-input-pick-pattern-hist
@@ -998,13 +1022,16 @@ then put the '*' mark onto them. "
       (setq flds (cdr flds)))
     (mew-uniq-list nmzdirs)))
 
-(defun mew-nmz-goto-folder-msg (fld msg)
+(defun mew-nmz-goto-folder-msg (fld msg &optional pos)
   (mew-summary-visit-folder fld)
   (while (processp mew-summary-buffer-process)
     (sit-for 1)
     (discard-input))
-  (mew-summary-jump-message msg)
-  (mew-summary-thread-move-cursor)
+  (if pos
+      (goto-char pos)
+    (goto-char (point-min))
+    (mew-nmz-re-search-message msg))
+  (mew-thread-move-cursor)
   (mew-summary-display 'force))
 
 (defun mew-nmz-make-temp-name (prefix)
@@ -1049,12 +1076,8 @@ then put the '*' mark onto them. "
     (message "Ignore url(%s) in %s." url "NMZ.field.uri")))
 
 (defun mew-nmz-index-newp (fld)
-  (let ((touchtime (if mew-touch-folder-p
-		       (mew-file-get-time
-			(expand-file-name mew-summary-touch-file
-					  (mew-expand-folder fld)))
-		     (mew-file-get-time
-		      (file-chase-links (mew-expand-folder fld)))))
+  (let ((touchtime (mew-file-get-time
+		    (file-chase-links (mew-expand-folder fld))))
 	(stamptime (if mew-nmz-mknmz-use-timestamp
 		       (mew-file-get-time
 			(expand-file-name
@@ -1070,33 +1093,30 @@ then put the '*' mark onto them. "
      (t nil))))
 
 (defun mew-nmz-folder-newp (fld)
-  (let ((buf (get-buffer fld)))
-    (if (not buf)
-	(let ((dir (file-chase-links (mew-expand-folder fld)))
-	      (tdir (if mew-touch-folder-p
-			(mew-file-get-time
-			 (expand-file-name mew-summary-touch-file
-					   (mew-expand-folder fld)))
-		      (mew-file-get-time
-		       (file-chase-links (mew-expand-folder fld)))))
-	      (tfile (mew-file-get-time
-		      (expand-file-name mew-summary-cache-file
-					(mew-expand-folder fld)))))
-	  (if (and mew-touch-folder-p (null tdir)
-		   (file-directory-p dir)
-		   (mew-dir-messages (mew-expand-folder dir)))
-	      nil
-	    (cond
-	     ((null tdir) t)
-	     ((null tfile) nil)
-	     ((> (car tdir) (car tfile)) nil)
-	     ((= (car tdir) (car tfile))
-	      (if (> (nth 1 tdir) (nth 1 tfile)) nil t)) ;; nil if same
-	     (t t))))
-      (set-buffer buf)
-      (if (and (not (mew-summary-folder-dir-newp))
-	       (mew-summary-exclusive-p))
-	  t nil))))
+  (save-excursion
+    (let ((buf (get-buffer fld)))
+      (if (not buf)
+	  (let ((dir (file-chase-links (mew-expand-folder fld)))
+		(tdir (mew-file-get-time
+		       (file-chase-links (mew-expand-folder fld))))
+		(tfile (mew-file-get-time
+			(expand-file-name mew-summary-cache-file
+					  (mew-expand-folder fld)))))
+	    (if (and (null tdir)
+		     (file-directory-p dir)
+		     (mew-dir-messages (mew-expand-folder dir)))
+		nil
+	      (cond
+	       ((null tdir) t)
+	       ((null tfile) nil)
+	       ((> (car tdir) (car tfile)) nil)
+	       ((= (car tdir) (car tfile))
+		(if (> (nth 1 tdir) (nth 1 tfile)) nil t)) ;; nil if same
+	       (t t))))
+	(set-buffer buf)
+	(if (and (not (mew-summary-folder-dir-newp))
+		 (mew-summary-exclusive-p))
+	    t nil)))))
 
 (defun mew-nmz-expand-folder-regexp (fld)
   "Convert folder to namazu-index-dir with '*' expand."
@@ -1175,6 +1195,19 @@ then put the '*' mark onto them. "
 	  (if (file-writable-p nfile)
 	      (delete-file nfile))))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Moving to message
+;;;
+
+(defun mew-nmz-re-search-message (msg)
+  (let ((here (point)))
+    (if (not (re-search-forward (concat "^.*\r +" msg "[^0-9]") nil t))
+	(progn (goto-char here)
+	       nil)
+      (beginning-of-line)
+      t)))
+
 (provide 'mew-nmz)
 
 ;;; Copyright Notice:
