@@ -26,22 +26,26 @@
 ;;
 
 (defvar mew-addrbook-orig-alist nil
-  "(key addr) or (key (addr1, addr2) nickname name)")
+  "(key (addr1, addr2) nickname name) or (key addr)")
 
 (defsubst mew-adbkorigent-by-shortname (user)
   (mew-addrbook-alias-hit user mew-addrbook-orig-alist))
 
 ;;
 
-(defvar mew-addrbook-alist nil
+(defvar mew-addrbook-alist nil ;; mew-alias-auto-alist is appended
   "(key addr) or (key (addr1, addr2) nickname name)")
 
 (defsubst mew-adbkent-by-addr-with-alist (addr alist)
-  (mew-assoc-member-case-equal addr mew-addrbook-alist 1))
+  (mew-assoc-member-case-equal addr alist 1))
 
 (defsubst mew-adbkent-by-addr (addr)
   (mew-adbkent-by-addr-with-alist addr mew-addrbook-alist))
 
+(defsubst mew-adbkent-by-addr2 (addr)
+  ;; for nickname and name, mew-addrbook-orig-alist is enough and fast
+  (mew-adbkent-by-addr-with-alist addr mew-addrbook-orig-alist))
+
 (defsubst mew-adbkent-shortname (adbkent)
   (nth 0 adbkent))
 
@@ -61,10 +65,10 @@
   (mew-adbkent-addrs (mew-adbkent-by-addr addr)))
 
 (defun mew-addrbook-nickname-get (addr)
-  (mew-adbkent-nickname (mew-adbkent-by-addr addr)))
+  (mew-adbkent-nickname (mew-adbkent-by-addr2 addr)))
 
 (defun mew-addrbook-name-get (addr)
-  (mew-adbkent-name (mew-adbkent-by-addr addr)))
+  (mew-adbkent-name (mew-adbkent-by-addr2 addr)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -88,18 +92,42 @@
 ;;; Setup
 ;;;
 
+(defun mew-alias-short-to-full (alist)
+  (let (ret)
+    (while alist
+      (setq ret (cons (cdr (car alist)) ret))
+      (setq alist (cdr alist)))
+    (nreverse ret)))
+
+;; duplicated?
+(defun mew-alias-full-to-short (alist)
+  (let (ret ent user)
+    (while alist
+      (setq ent (car alist))
+      (setq alist (cdr alist))
+      (setq user (downcase (mew-addrstr-extract-user (car ent))))
+      (setq ret (cons (cons user ent) ret)))
+    (nreverse ret)))
+
 (defun mew-addrbook-setup ()
   (if (and mew-alias-auto-file (null mew-alias-auto-alist))
       ;; make auto-alist only at the initialization time
       ;; not at update time (auto-alist have not been saved yet)
       (setq mew-alias-auto-alist (mew-lisp-load mew-alias-auto-file)))
+  (if mew-use-full-alias
+      (if (mew-autoent-addr (car mew-alias-auto-alist))
+	  (setq mew-alias-auto-alist
+		(mew-alias-short-to-full mew-alias-auto-alist)))
+    (if (null (mew-autoent-addr (car mew-alias-auto-alist)))
+	(setq mew-alias-auto-alist
+	      (mew-alias-full-to-short mew-alias-auto-alist))))
   (setq mew-addrbook-orig-alist (mew-addrbook-make-alist))
   (mew-addrbook-concat-uniq)
   (add-hook 'kill-emacs-hook 'mew-addrbook-clean-up))
 
 (defun mew-addrbook-concat-uniq ()
   (setq mew-addrbook-alist
-	(append mew-addrbook-orig-alist (copy-alist mew-alias-auto-alist)))
+	(append mew-addrbook-orig-alist (copy-sequence mew-alias-auto-alist)))
   (setq mew-addrbook-alist (mew-uniq-alist mew-addrbook-alist)))
 
 (defun mew-addrbook-clean-up ()
@@ -174,33 +202,38 @@ returned."
 (defun mew-addrbook-alias-add (addr)
   ;; Let's downcase user-name for .mew-alias since it is automatically
   ;; maintained.
-  (when (and (stringp addr) (string-match "@" addr))
-    (let* ((user (downcase (mew-addrstr-extract-user addr)))
-	   (match-auto (mew-autoent-by-shortname user))
-	   (match-adbk-orig (mew-adbkorigent-by-shortname user)))
-      (cond
-       ((string= user "")
-	;; do nothing
-	)
-       (match-adbk-orig
-	;; do nothing
-	)
-       (match-auto
+  (when (and (stringp addr) (string-match "@" addr)
+	     (or (null mew-addrbook-alias-not-learn-list)
+		 (not (mew-member-match addr mew-addrbook-alias-not-learn-list))))
+    (if mew-use-full-alias
+	(unless (member (list addr) mew-alias-auto-alist)
+	  (mew-addrbook-alias-cons (list addr)))
+      (let* ((user (downcase (mew-addrstr-extract-user addr)))
+	     (match-auto (mew-autoent-by-shortname user))
+	     (match-adbk-orig (mew-adbkorigent-by-shortname user)))
 	(cond
-	 ((string= addr (mew-autoent-addr match-auto))
-	  ;; Move the entry to the top for the recent-used-first.
-	  (setq mew-alias-auto-alist (delq match-auto mew-alias-auto-alist))
-	  (setq mew-alias-auto-alist (cons match-auto mew-alias-auto-alist)))
-	 (mew-addrbook-override-by-newone
-	  ;; override match-auto by (user addr)
-	  (setq mew-alias-auto-alist (delq match-auto mew-alias-auto-alist))
-	  (setq mew-addrbook-alist (delete match-auto mew-addrbook-alist))
-	  (mew-addrbook-alias-cons (list user addr)))
+	 ((string= user "")
+	  ;; do nothing
+	  )
+	 (match-adbk-orig
+	  ;; do nothing
+	  )
+	 (match-auto
+	  (cond
+	   ((string= addr (mew-autoent-addr match-auto))
+	    ;; Move the entry to the top for the recent-used-first.
+	    (setq mew-alias-auto-alist (delq match-auto mew-alias-auto-alist))
+	    (setq mew-alias-auto-alist (cons match-auto mew-alias-auto-alist)))
+	   (mew-addrbook-override-by-newone
+	    ;; override match-auto by (user addr)
+	    (setq mew-alias-auto-alist (delq match-auto mew-alias-auto-alist))
+	    (setq mew-addrbook-alist (delete match-auto mew-addrbook-alist))
+	    (mew-addrbook-alias-cons (list user addr)))
+	   (t
+	    ;; the old entry remains
+	    )))
 	 (t
-	  ;; the old entry remains
-	  )))
-       (t
-	(mew-addrbook-alias-cons (list user addr)))))))
+	  (mew-addrbook-alias-cons (list user addr))))))))
 
 (defun mew-addrbook-alias-cons (user-addr)
   (setq mew-alias-auto-alist (cons user-addr mew-alias-auto-alist))
@@ -236,14 +269,14 @@ returned."
       (setq pars (cdr pars))
       (when (file-readable-p par)
 	(setq ret t)
-	(insert-file-contents par)
+	(mew-insert-file-contents par)
 	(setq path (file-name-directory par))
 	;; include children files
 	(while (re-search-forward "^<[ \t]*\\([^ \t\n]+\\).*$" nil t)
 	  (setq chr (expand-file-name (mew-match-string 1) path))
 	  (replace-match "" nil t)
 	  (when (and (file-readable-p chr) (not (member chr files)))
-	    (insert-file-contents chr)
+	    (mew-insert-file-contents chr)
 	    (setq files (cons chr files))))
 	(goto-char (point-max))))
     ;; remove commets
@@ -299,14 +332,11 @@ returned."
 (defun mew-addrbook-strsafe (var)
   (if (or (string= "" var) (string= "*" var))
       nil
-    (mew-replace-character var 0 32)))
+    (mew-replace-character var 0 mew-sp)))
 
 (defun mew-addrbook-make-alist ()
-  (let ((buf (generate-new-buffer mew-buffer-prefix))
-	alias colon addrs nick name alist)
-    (save-excursion
-      (set-buffer buf)
-      (mew-erase-buffer)
+  (let (alias colon addrs nick name alist)
+    (with-temp-buffer
       (when (mew-addrbook-insert-file
 	     mew-addrbook-file mew-addrbook-comment-regex 'unquote)
 	(goto-char (point-min))
@@ -324,7 +354,6 @@ returned."
 	      (setq nick nil)
 	      (setq name nil))
 	    (setq alist (cons (list alias addrs nick name) alist))))))
-    (mew-remove-buffer buf)
     (nreverse alist)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -344,7 +373,7 @@ it will add an alias."
 	from shortname addrs name)
     (save-excursion
       (set-buffer buf)
-      (unless (mew-header-p)
+      (unless (mew-msghder-p)
 	(setq buf (mew-cache-hit fld msg 'must-hit))
 	(set-buffer buf))
       (setq from (mew-header-get-value mew-from:))
@@ -451,9 +480,9 @@ The keys that are defined for this mode are:
        ((and name (string-match "^\"[^\"]*[^\000-\177]" name))
 	(setq errmsg "Remove quote around non-ASCII Name")))))
     (if errmsg
-	(message errmsg)
+	(message "%s" errmsg)
       (save-excursion
-	(setq buf (find-file-noselect
+	(setq buf (mew-find-file-noselect2
 		   (expand-file-name mew-addrbook-file mew-conf-path)))
 	(set-buffer buf)
 	(goto-char (point-min))
@@ -507,7 +536,7 @@ The keys that are defined for this mode are:
 ;;; Editing auto-alist
 ;;;
 
-(defun mew-summary-addrbook-read-buffer ()
+(defun mew-summary-alias-read-buffer ()
   (save-excursion
     (goto-char (point-min))
     ;; An error occurs if the expression is broken.
@@ -516,7 +545,11 @@ The keys that are defined for this mode are:
     (mew-addrbook-concat-uniq)
     nil)) ;; to write
 
-(defun mew-summary-addrbook-edit ()
+(defun mew-summary-alias-edit ()
+  "Editing the auto alias file which contains a list of alias-address
+pairs. Remove unnecessary entries and save the file by
+'\\[save-buffer]'. After saving, the modification is automatically
+reflected."
   (interactive)
   (let ((file mew-alias-auto-file))
     (setq mew-alias-auto-alist
@@ -524,15 +557,17 @@ The keys that are defined for this mode are:
     (mew-lisp-save mew-alias-auto-file mew-alias-auto-alist)
     (unless (file-name-absolute-p file)
       (setq file (expand-file-name file mew-conf-path)))
-    (find-file file)
+    (switch-to-buffer (mew-find-file-noselect file))
     (emacs-lisp-mode)
-    (add-hook 'local-write-file-hooks 'mew-summary-addrbook-read-buffer)))
+    (if (boundp 'write-file-functions)
+	(add-hook 'write-file-functions 'mew-summary-alias-read-buffer nil 'local)
+      (add-hook 'local-write-file-hooks 'mew-summary-alias-read-buffer))))
 
 (provide 'mew-addrbook)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2002 Mew developing team.
+;; Copyright (C) 1999-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
