@@ -8,7 +8,7 @@
 (require 'mew)
 
 (defun mew-auth-select (grt lst)
-  (mew-auth-select2 (mew-split grt 32) lst))
+  (mew-auth-select2 (mew-split grt mew-sp) lst))
 
 (defun mew-auth-select2 (auths lst)
   (let (auth n preference strongest)
