@@ -16,8 +16,8 @@
     ("Q" . mew-q-decode-string)))
 
 (defvar mew-header-decode-error-switch
-  `(("B" . ,mew-error-illegal-b-encoding)
-    ("Q" . ,mew-error-illegal-q-encoding)))
+  `(("B" . ,mew-error-invalid-b-encoding)
+    ("Q" . ,mew-error-invalid-q-encoding)))
 
 (defsubst mew-header-encode-get-func (b-or-q)
   (cdr (mew-assoc-case-equal b-or-q mew-header-encode-switch 0)))
@@ -119,9 +119,9 @@
     (while (< i len)
       (setq char (aref str i))
       (cond
-       ((char-equal char 32)
+       ((char-equal char mew-sp)
 	(aset ret j ?_))
-       ((and (> char 32)
+       ((and (> char mew-sp)
 	     (< char 126)
 	     (not (char-equal char ?=))
 	     (not (char-equal char ??))
@@ -180,7 +180,7 @@
 	  (setq char (aref qpstr i))
 	  (cond
 	   ((char-equal char ?_)
-	    (aset ret j 32))
+	    (aset ret j mew-sp))
 	   ((char-equal char key)
 	    (aset ret j (+ (* (mew-hexchar-to-int (aref qpstr (1+ i))) 16)
 			   (mew-hexchar-to-int (aref qpstr (+ i 2)))))
@@ -334,10 +334,10 @@
 	      (throw 'quote nil))
 	     ((> c 127)
 	      (mew-encode-error
-	       "Only ASCII is allowed in quoted-string in the header. ")))
+	       "Only ASCII is allowed in quoted-string in the header")))
 	    (setq I (1+ I))))
 	(if open
-	    (mew-encode-error "Quote string must be closed in the header. "))
+	    (mew-encode-error "Quote string must be closed in the header"))
 	(mew-header-encode-cond ?a)
 	(setq i I))
        ;; end of quote
@@ -364,9 +364,9 @@
 	      (setq i (1+ i))))
 	  (if (and qp (eq status 'non-ascii))
 	      (mew-encode-error
-	       "Only ASCII is allowed in quoted-string in the header. ")))
+	       "Only ASCII is allowed in quoted-string in the header")))
 	(if open
-	    (mew-encode-error "Comment must be closed in the header. "))
+	    (mew-encode-error "Comment must be closed in the header"))
 	(mew-header-encode-cond2 'comment)
 	(if (= i len)
 	    ()
@@ -377,7 +377,7 @@
        ;; route
        ((char-equal c ?<)
 	(mew-header-encode-cond2 nil)
-	(if (or (char-equal (char-before (point)) 32)
+	(if (or (char-equal (char-before (point)) mew-sp)
 		(char-equal (char-before (point)) ?\t))
 	    (insert "<")
 	  (insert " <"))
@@ -393,19 +393,19 @@
 	      (setq open nil)
 	      (throw 'route nil))
 	     ((> c 127)
-	      (mew-encode-error "<> must contain ASCII only. "))
+	      (mew-encode-error "<> must contain ASCII only"))
 	     (t
 	      (insert c)))
 	    (setq i (1+ i))))
 	(if open
-	    (mew-encode-error "<> must be closed in the header. "))
+	    (mew-encode-error "<> must be closed in the header"))
 	(if (= i len)
 	    ()
 	  (insert ">")
 	  (setq bound (1+ i)))
 	(setq status 'space))
        ((char-equal c ?>)
-	(mew-encode-error "Unbalanced <> in the header. "))
+	(mew-encode-error "Unbalanced <> in the header"))
        ;; end of <>
 
        ;; group
@@ -424,14 +424,14 @@
 	(insert ":")
 	(setq i (1+ i))
 	(unless (char-equal (aref str i) ?\;)
-	  (mew-encode-error ": must be followed by ; in the header. "))
+	  (mew-encode-error ": must be followed by ; in the header"))
 	(insert ";")
 	(setq bound (1+ i))
 	(setq status 'space))
        ;; end of group
 
        ;; space
-       ((or (char-equal c 32) (char-equal c ?\t))
+       ((or (char-equal c mew-sp) (char-equal c ?\t))
 	(cond
 	 ((or (eq status 'ascii) (eq status 'space))
 	  (insert (substring str bound i)) ;; 'ascii
@@ -449,13 +449,13 @@
 	(catch 'comma
 	  (while (< i len)
 	    (setq c (aref str i))
-	    (if (or (char-equal c 32) (char-equal c ?\t) (char-equal c ?\n))
+	    (if (or (char-equal c mew-sp) (char-equal c ?\t) (char-equal c ?\n))
 		() ;; loop
 	      (throw 'comma nil))
 	    (setq i (1+ i))))
 	;; get back to the end of white spaces
 	(setq bound i)
-	(setq c 32)
+	(setq c mew-sp)
 	(setq i (1- i))
 	(setq status 'space))
        ;; end of comma
@@ -620,6 +620,9 @@
     (beginning-of-line)
     (char-equal (following-char) ?\")))
 
+(defsubst mew-header-decode-in-addr-spec ()
+  (eq (get-text-property (match-beginning 0) 'mew-addr-spec) t))
+
 (defun mew-decode-warning-body (err msg)
   (mew-xinfo-set-warning
    (cons 
@@ -631,7 +634,7 @@
 	  (format "Charset (%s) for body is mis-matched.\n" msg)
 	"Charset for body is not specified.\n"))
      ((eq err 'multi)
-      (format "Illegal encoding (%s) for multipart.\n" msg)))
+      (format "Invalid encoding (%s) for multipart.\n" msg)))
     (mew-xinfo-get-warning))))
 
 (defun mew-decode-warning-fields (key err)
@@ -661,19 +664,19 @@
        ((eq err 'raw)
 	(concat "The '" param "' parameter has raw text.\n"))
        ((eq err 'ctl)
-	(concat "The '" param "' has illegal control codes.\n"))
+	(concat "The '" param "' has invalid control codes.\n"))
        ((eq err 'enc)
 	(concat "The '" param "' parameter has encoded-word.\n")))
       (mew-xinfo-get-warning)))))
 
 (defun mew-header-decode-region (key rbeg rend &optional draftp)
   "RFC 2047 decoding. This is liberal on the one point from RFC 2047.
-That is, each line may be more than 75. "
+That is, each line may be more than 75."
   (setq key (capitalize key))
   (let* ((type (mew-field-type-for-decoding key)) (nl "\0"))
     (save-restriction
       (narrow-to-region rbeg rend)
-      ;; Handling illegal raw text.
+      ;; Handling invalid raw text.
       (goto-char (point-min))
       (if draftp
 	  (mew-cs-decode-region (point-min) (point-max) mew-cs-m17n)
@@ -707,8 +710,8 @@ That is, each line may be more than 75. "
 	  ;; encoded-word in quoted-string should not be decoded
 	  ;; according to RFC 2047. However, if users wish
 	  ;; (ie mew-decode-broken is *non-nil*), decode it.
-	  ;; Regular expression cannot express quoted string. So,
-	  ;; canonicalize as follows:
+	  ;; Regular expression cannot express a quoted string in 
+	  ;; general case. So, canonicalize as follows:
 	  ;;     Not quoted line (may be null line)
           ;;     "quoted line"
 	  ;;     Not quoted line
@@ -718,6 +721,12 @@ That is, each line may be more than 75. "
 	  (goto-char (point-min))
 	  (while (re-search-forward "\n" nil t)
 	    (replace-match nl nil t))
+	  ;;
+	  (goto-char (point-min))
+	  (while (re-search-forward mew-regex-id nil t)
+	    (put-text-property (match-beginning 0) (match-end 0)
+			       'mew-addr-spec t))
+	  ;;
 	  (goto-char (point-min))
 	  ;; Perfect regular expression for quoted strings.
 	  ;; See "Marstering Regular Expressions", O'REILLY
@@ -730,22 +739,27 @@ That is, each line may be more than 75. "
 	  (goto-char (point-min)) ;; anyway
 	  ;;
 	  (while (re-search-forward mew-header-decode-regex nil t)
-	    (if (mew-header-decode-in-quotedp)
-		(when mew-decode-broken
-		  (mew-decode-warning-fields key 'quoted)
-		  (mew-header-decode-string key)
-		  (while (looking-at mew-header-decode-regex2)
-		    (mew-header-decode-string key)))
+	    (cond
+	     ((mew-header-decode-in-addr-spec)
+	      ) ;; do nothing
+	     ((mew-header-decode-in-quotedp)
+	      (when mew-decode-broken
+		(mew-decode-warning-fields key 'quoted)
+		(mew-header-decode-string key)
+		(while (looking-at mew-header-decode-regex2)
+		  (mew-header-decode-string key))))
+	     (t
 	      (mew-header-decode-string key)
 	      (while (looking-at mew-header-decode-regex2)
-		(mew-header-decode-string key))))
+		(mew-header-decode-string key)))))
 	  ;; Getting back to the original
 	  (goto-char (point-min))
 	  (while (re-search-forward "\n" nil t)
 	    (replace-match "" nil t))
 	  (goto-char (point-min))
 	  (while (re-search-forward nl nil t)
-	    (replace-match "\n" nil t)))))
+	    (replace-match "\n" nil t))
+	  (remove-text-properties (point-min) (point-max) '(mew-addr-spec t)))))
       ;;
       (if (and mew-decode-broken (member key mew-decode-ws-fields))
 	  (let (ws)
@@ -758,7 +772,7 @@ That is, each line may be more than 75. "
 	      (or ws (setq ws t))
 	      (replace-match " " nil t))
 	    (goto-char (point-min))
-	    ;; Continuous SPCs are not illegal, just for appearance.
+	    ;; Continuous SPCs are not invalid, just for appearance.
 	    (while (re-search-forward "  +" nil t)
 	      (or ws (setq ws t))
 	      (replace-match " " nil t))
@@ -816,9 +830,7 @@ That is, each line may be more than 75. "
       (setq max (1+ max))
       (while ext
 	(setq ea (car ext))
-	(setq paramname (nth 0 ea))
-	(setq paramvalue (nth 1 ea))
-	(setq num (nth 2 ea))
+	(mew-set '(paramname paramvalue num) ea)
 	(if (setq entry (assoc paramname ext-sort))
 	    (progn
 	      (aset (nth 1 entry) num paramvalue)
@@ -834,9 +846,7 @@ That is, each line may be more than 75. "
       (while ext-sort
 	(setq ea (car ext-sort))
 	(setq paramvalue nil)
-	(setq paramname (nth 0 ea))
-	(setq entry (nth 1 ea))
-	(setq charset (nth 2 ea))
+	(mew-set '(paramname entry charset) ea)
 	(setq num 0)
 	(catch 'concat-loop
 	  (while (< num max)
@@ -922,9 +932,9 @@ That is, each line may be more than 75. "
   value)
 
 (defun mew-param-sanity-check (ent)
-  (let* ((param (nth 0 ent))
-	 (value (nth 1 ent))
-	 (new (mew-header-sanity-check-string value)))
+  (let (param value new)
+    (mew-set '(param value) ent)
+    (setq new (mew-header-sanity-check-string value))
     (unless (string= value new) (mew-decode-warning-params param 'ctl))
     (list param new)))
 
@@ -932,7 +942,7 @@ That is, each line may be more than 75. "
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
