@@ -18,7 +18,7 @@
 ;;
 ;;
 
-(defun mew-cfent-value (case key def)
+(defun mew-cfent-value (case key def &optional symbol)
   (let ((cases (and case (mapcar 'mew-chop (mew-split case ?,))))
 	ent ret)
     (or (member mew-case-default cases)
@@ -27,8 +27,10 @@
     (catch 'loop
       (while cases
 	(setq ent (assoc key (mew-cfent-by-case (car cases))))
-	(setq ret (cdr ent))
-	(if ent (throw 'loop nil))
+	(when ent
+	  (setq ret (cdr ent))
+	  (if symbol (setq ret (symbol-value ret)))
+	  (if ent (throw 'loop nil)))
 	(setq cases (cdr cases)))
       (setq ret def))
     ret))
@@ -206,6 +208,12 @@
 (defun mew-pop-ssl-port (&optional case)
   (mew-cfent-value case "pop-ssl-port" mew-pop-ssl-port))
 
+(defun mew-pop-proxy-server (&optional case)
+  (mew-cfent-value case "pop-proxy-server" mew-pop-proxy-server))
+
+(defun mew-pop-proxy-port (&optional case)
+  (mew-cfent-value case "pop-proxy-port" mew-pop-proxy-port))
+
 (defun mew-pop-user (&optional case)
   (mew-cfent-value case "pop-user" mew-pop-user))
 
@@ -259,6 +267,9 @@
 (defun mew-imap-header-only (&optional case)
   (mew-cfent-value case "imap-header-only" mew-imap-header-only))
 
+(defun mew-imap-prefix-list (&optional case)
+  (mew-cfent-value case "imap-prefix-list" mew-imap-prefix-list))
+
 ;;
 
 (defun mew-nntp-server (&optional case)
@@ -322,6 +333,9 @@
 (defun mew-imap-trash-folder (&optional case)
   (mew-cfent-value case "imap-trash-folder" mew-imap-trash-folder))
 
+(defun mew-imap-trash-folder-list (&optional case)
+  (mew-cfent-value case "imap-trash-folder-list" mew-imap-trash-folder-list))
+
 (defun mew-header-alist (&optional case)
   (mew-cfent-value case "header-alist" mew-header-alist))
 
@@ -331,11 +345,56 @@
 (defun mew-content-type (&optional case)
   (mew-capitalize (mew-cfent-value case "content-type" mew-content-type)))
 
+(defun mew-use-retrieve (&optional case)
+  (mew-cfent-value case "use-retrieve" mew-use-retrieve))
+
+(defun mew-imap-spam-field (&optional case)
+  (mew-cfent-value case "imap-spam-field" mew-imap-spam-field))
+
+(defun mew-imap-spam-word (&optional case)
+  (mew-cfent-value case "imap-spam-word" mew-imap-spam-word))
+
+(defun mew-imap-spam-folder (&optional case)
+  (mew-cfent-value case "imap-spam-folder" mew-imap-spam-folder))
+
 ;;
 
 (defun mew-proto (&optional case)
   (mew-cfent-value case "proto" mew-proto))
 
+;;
+
+(defun mew-ssh-prog (&optional case)
+  (mew-cfent-value case "ssh-prog" mew-ssh-prog))
+
+(defun mew-ssh-prog-args (&optional case)
+  (mew-cfent-value case "ssh-prog-args" mew-ssh-prog-args))
+
+(defun mew-ssh-prog-ver (&optional case)
+  (mew-cfent-value case "ssh-prog-ver" mew-ssh-prog-ver))
+
+(defun mew-spam-prog (&optional case)
+  (mew-cfent-value case "spam-prog" mew-spam-prog))
+
+(defun mew-spam-prog-args (&optional case)
+  (mew-cfent-value case "spam-prog-args" mew-spam-prog-args))
+
+(defun mew-ham-prog (&optional case)
+  (mew-cfent-value case "ham-prog" mew-ham-prog))
+
+(defun mew-ham-prog-args (&optional case)
+  (mew-cfent-value case "ham-prog-args" mew-ham-prog-args))
+
+;;
+
+(defun mew-refile-guess-alist (&optional case)
+  (mew-cfent-value case "refile-guess-alist" mew-refile-guess-alist 'symbol))
+
+;;
+
+(defun mew-use-old-pgp (&optional case)
+  (mew-cfent-value case "use-old-pgp" mew-use-old-pgp))
+
 ;;;
 ;;; Setting case
 ;;;
@@ -386,22 +445,27 @@ non-nil, set the case for input, too."
  (2) If called with '\\[universal-argument]', set the case for output.
  (3) Otherwise, set the case for input."
   (interactive "P")
-  (mew-summary-or-thread
-   (if output
-       (mew-case-set-output)
-     (mew-case-set-input))
-   (save-excursion
-     (let ((bufs mew-buffers))
-       (while bufs
-	 (if (get-buffer (car bufs))
-	     (progn
-	       (set-buffer (car bufs))
-	       (cond
-		((mew-summary-p)
-		 (mew-summary-mode-name mew-mode-name-summary))
-		((mew-virtual-p)
-		 (mew-summary-mode-name mew-mode-name-virtual)))))
-	 (setq bufs (cdr bufs)))))))
+  (let (case inbox)
+    (if output
+	(setq case (mew-case-set-output))
+      (setq case (mew-case-set-input)))
+    (save-excursion
+      (let ((bufs mew-buffers))
+	(while bufs
+	  (if (get-buffer (car bufs))
+	      (progn
+		(set-buffer (car bufs))
+		(cond
+		 ((mew-summary-p)
+		  (mew-summary-mode-name mew-mode-name-summary))
+		 ((mew-virtual-p)
+		  (mew-summary-mode-name mew-mode-name-virtual)))))
+	  (setq bufs (cdr bufs)))))
+    (when mew-visit-inbox-after-setting-case
+      (setq inbox (mew-case-folder
+		   case
+		   (mew-proto-inbox-folder (mew-proto case) case)))
+      (mew-summary-visit-folder inbox))))
 
 (defun mew-draft-set-case (&optional arg)
   "Guess case and set the case for output to it. The value is
