@@ -90,69 +90,111 @@
        (goto-char (point-max))
        (insert "\n"))
      (when (and (integerp mew-header-max-length)
-		(> (mew-count-lines (point-min) (point))
-		   mew-header-max-length))
+		(> (count-lines (point-min) (point)) mew-header-max-length))
        (mew-xinfo-set-action
-	(substitute-command-keys
-	 "Too long header. To see the message, type '\\<mew-summary-mode-map>\\[mew-summary-analyze-again]'"))
+	(mew-substitute-for-summary
+	 "Too long header. To see the message, type '\\[mew-summary-analyze-again]'"))
        (error ""))
      (save-restriction
        (narrow-to-region (point-min) (point))
        (goto-char (point-min))
        ,@body)))
 
+(defun mew-summary-toggle-header-veil ()
+  "If 'mew-use-header-veil' is non-nil, field lines of To: and Cc:
+over 'mew-header-veil-count' are covered with invisible veils.
+This commands toggles visibility of these lines."
+  (interactive)
+  (when mew-use-header-veil
+    (mew-summary-msg-or-part
+     (save-excursion
+       (set-buffer (mew-buffer-message))
+       (when (mew-msghder-p)
+	 (let* ((win (get-buffer-window (current-buffer)))
+		(pos (window-start win)))
+	   (mew-toggle-header-veil (mew-minfo-get-veil-to))
+	   (mew-toggle-header-veil (mew-minfo-get-veil-cc))
+	   (set-window-start win pos)))))))
+
+(defun mew-header-veil-put (key ov)
+  (let ((case-fold-search t)
+	(regex (concat "^" key)))
+    (goto-char (point-min))
+    (when (re-search-forward regex nil t)
+      (beginning-of-line)
+      (let ((start (point)) end)
+	(forward-line)
+	(mew-header-goto-next)
+	(setq end (point))
+	(when (> (count-lines start end) mew-header-veil-count)
+	  (goto-char start)
+	  (forward-line mew-header-veil-count)
+	  (move-overlay ov (1- (point)) (1- end)))))))
+
+(defun mew-header-veil ()
+  (when mew-use-header-veil
+    (unless (mew-minfo-get-veil-to)
+      (mew-minfo-set-veil-to (mew-header-veil-make)))
+    (unless (mew-minfo-get-veil-cc)
+      (mew-minfo-set-veil-cc (mew-header-veil-make)))
+    (mew-header-veil-put "To:" (mew-minfo-get-veil-to))
+    (mew-header-veil-put "Cc:" (mew-minfo-get-veil-cc))))
+
 (defun mew-header-arrange (beg end)
-  (save-restriction
-    (narrow-to-region beg end)
-    (mew-elet
-     (let (ch-beg ch-end vs-beg vs-end contents cbeg)
-       (setq ch-beg (next-single-property-change (point-min) 'mew-noncontents))
-       (when ch-beg
-	 (setq vs-beg (next-single-property-change (point-min) 'mew-visible))
-	 (cond
-	  (vs-beg
-	   (setq vs-end (next-single-property-change vs-beg 'mew-visible)))
-	  (mew-field-other-visible
-	   (setq vs-beg (next-single-property-change (point-min) 'mew-others))
-	   (if vs-beg 
-	       (setq vs-end (next-single-property-change vs-beg 'mew-others))
+  (let (vispos)
+    (save-restriction
+      (narrow-to-region beg end)
+      (mew-elet
+       (let (ch-beg ch-end vs-beg vs-end contents cbeg)
+	 (setq ch-beg (next-single-property-change (point-min) 'mew-noncontents))
+	 (when ch-beg
+	   (setq vs-beg (next-single-property-change (point-min) 'mew-visible))
+	   (cond
+	    (vs-beg
+	     (setq vs-end (next-single-property-change vs-beg 'mew-visible)))
+	    (mew-field-other-visible
+	     (setq vs-beg (next-single-property-change (point-min) 'mew-others))
+	     (if vs-beg 
+		 (setq vs-end (next-single-property-change vs-beg 'mew-others))
+	       (setq vs-beg ch-beg)
+	       (setq vs-end ch-beg)))
+	    (t
 	     (setq vs-beg ch-beg)
 	     (setq vs-end ch-beg)))
-	  (t
-	   (setq vs-beg ch-beg)
-	   (setq vs-end ch-beg)))
-	 (setq ch-end (point-max))
-	 (unless vs-end (setq vs-end (point-max)))
-	 (mew-decode-header-property-region ch-beg ch-end)
-	 ;; This must be "buffer-substring".
-	 (setq contents (buffer-substring ch-beg ch-end))
-	 (delete-region ch-beg ch-end)
-	 (cond
-	  ((mew-nspec-visiblep (mew-nspec-by-key "Content-"))
-	   ;; visible
-	   (goto-char vs-end)
-	   (setq cbeg (point))
-	   (insert contents)
-	   (put-text-property cbeg (point) 'mew-visible t)) ;; used later
-	  (t
-	   ;; invisible
-	   (goto-char vs-beg)
-	   (setq cbeg (point))
-	   (insert contents)
-	   (put-text-property cbeg (point) 'mew-visible nil))) ;; for XEmacs
-	 (goto-char (point-max)))
-       (mew-decode-syntax-insert-privacy)
-       (mew-decode-syntax-insert-warning)
-       (goto-char (if (get-text-property (point-min) 'mew-visible)
-		      (point-min)
-                    (or (next-single-property-change (point-min) 'mew-visible)
-			(point-max))))
-       (recenter 0)
-       (save-excursion (mew-highlight-x-face (point-min) (point-max))))))
-  (mew-header-goto-end)
-  (if (eobp)
-      (mew-header-set "\n") ;; analyzed
-    (mew-header-set nil))) ;; asis
+	   (setq ch-end (point-max))
+	   (unless vs-end (setq vs-end (point-max)))
+	   (mew-decode-header-property-region ch-beg ch-end)
+	   ;; This must be "buffer-substring".
+	   (setq contents (buffer-substring ch-beg ch-end))
+	   (delete-region ch-beg ch-end)
+	   (cond
+	    ((mew-nspec-visiblep (mew-nspec-by-key "Content-"))
+	     ;; visible
+	     (goto-char vs-end)
+	     (setq cbeg (point))
+	     (insert contents)
+	     (put-text-property cbeg (point) 'mew-visible t)) ;; used later
+	    (t
+	     ;; invisible
+	     (goto-char vs-beg)
+	     (setq cbeg (point))
+	     (insert contents)
+	     (put-text-property cbeg (point) 'mew-visible nil))) ;; for XEmacs
+	   (goto-char (point-max)))
+	 (mew-decode-syntax-insert-privacy)
+	 (mew-decode-syntax-insert-warning)
+	 (save-excursion (mew-highlight-x-face (point-min) (point-max)))
+	 (setq vispos (if (get-text-property (point-min) 'mew-visible)
+			(point-min)
+		      (or (next-single-property-change (point-min) 'mew-visible)
+			  (point-max)))))))
+    (mew-header-veil)
+    (mew-header-goto-end)
+    (if (eobp)
+	(mew-header-set "\n") ;; analyzed
+      (mew-header-set nil))   ;; asis
+    ;; Emacs 21.3.50 or later need this.
+    (set-window-start (selected-window) vispos)))
 
 (defun mew-decode-header-property-region (BEG END)
   ;; see also mew-highlight-header
@@ -213,11 +255,11 @@ Return the existence MIME-Version: and the value of Subject:."
 	  ((string= key mew-x-mew-uidl:)
 	   (when (and (eq cnt 1)
 		      (setq fld (mew-cinfo-get-fld))
-		      (not (string= fld mew-trash-folder))
-		      (not (mew-folder-remotep (mew-folder-folder fld))))
+		      (not (and mew-trash-folder (string= fld mew-trash-folder)))
+		      (not (mew-folder-remotep (mew-case:folder-folder fld))))
 	     (mew-xinfo-set-action
-	      (substitute-command-keys 
-	       "Too large, truncated (the 'T' mark). To get the entire message, type '\\<mew-summary-mode-map>\\[mew-summary-retrieve-message]'"))))
+	      (mew-substitute-for-summary 
+	       "Too large, truncated (the 'T' mark). To get the entire message, type '\\[mew-summary-retrieve-message]'"))))
 	  ((string-match "^Content-" key)
 	   ;; Due to PGP/MIME, properties are not put here.
 	   :; This must be "buffer-substring".
@@ -309,8 +351,7 @@ Return a part syntax after moving the beginning of the content body."
 	 (mew-header-goto-next)
 	 (setq attr (assoc key mew-mime-fields))
 	 (when attr
-	   (setq n (nth 1 attr))
-	   (setq act (nth 2 attr))
+	   (mew-set '(nil n act) attr)
 	   (cond
 	    ((eq act 'analyze)
 	     (setq value (mew-param-decode
@@ -327,9 +368,7 @@ Return a part syntax after moving the beginning of the content body."
 	     (setq value (mew-buffer-substring med (1- (point)))))
 	    ((eq act 'id)
 	     (setq value (mew-buffer-substring med (1- (point))))
-	     (if (string-match mew-regex-id value)
-		 (setq value (match-string 1 value))
-	       (setq value nil))))
+	     (setq value (mew-idstr-get-first-id value))))
 	   (aset vec n value)))))
     (if (eobp)
 	(insert "\n")
@@ -444,7 +483,8 @@ Return a part syntax after moving the beginning of the content body."
 	(or (mew-syntax-get-cd syntax) (mew-syntax-set-cd syntax subj))
 	;; (point-min), (point) - 1, (point), (point-max)
 	(mew-highlight-body-region (point) (point-max))
-	(mew-decode-syntax-rfc822 syntax))))))) ;; return value
+	(mew-decode-syntax-rfc822
+	 syntax (not (mew-xinfo-get-text-body))))))))) ;; return value
 
 ;;;
 ;;; Decoding singlepart
@@ -787,7 +827,7 @@ Return a part syntax after moving the beginning of the content body."
 			   " does not exist")))
      (t
       (setq file3result (funcall func file1 file2))
-      (setq file3 (nth 0 file3result) result (nth 1 file3result))))
+      (mew-set '(file3 result) file3result)))
     ;;
     (if (and func existp (file-exists-p file3))
 	(mew-flet 
