@@ -22,7 +22,7 @@ Mew -- Messaging in the Emacs World
 
 %s
 
-Copyright (C) 1994-2003 Kazu Yamamoto
+Copyright (C) 1994-2004 Kazu Yamamoto
 
 Please send comments to Kazu@Mew.org.
 
@@ -73,7 +73,7 @@ Please send comments to Kazu@Mew.org.
 				   (expand-file-name mew-icon-mew
 						     mew-icon-directory)))))
 	(indent-to (startup-center-spaces mew-logo))
-	(set-extent-begin-glyph (mew-overlay-make (point) (point)) mew-logo)
+	(set-extent-end-glyph (mew-overlay-make (point) (point)) mew-logo)
 	(goto-char (point-min))
 	(mew-redraw time))
        ((and mew-gemacs-p window-system
@@ -82,9 +82,10 @@ Please send comments to Kazu@Mew.org.
 	(goto-char (point-max))
 	(setq mew-logo (create-image 
 			(expand-file-name mew-icon-mew mew-icon-directory)))
-	(setq indent (/ (- (frame-width)
+	(setq indent (/ (- (window-width)
 			   (truncate (car (image-size mew-logo)))) 2))
-	(insert (make-string indent 32))
+	(setq indent (max 0 indent))
+	(insert (make-string indent mew-sp))
 	(insert-image mew-logo)
 	(goto-char (point-min))
 	(mew-redraw time))
@@ -108,7 +109,7 @@ Please send comments to Kazu@Mew.org.
   (insert "/\\\\ - \\\\/")
   (center-line)
   (end-of-line)
-  (insert (make-string (1- (- (window-width) (current-column))) ?\ ))
+  (insert (make-string (1- (- (window-width) (current-column))) mew-sp))
   (end-of-line)
   (mew-redraw time)
   (if (and mew-demo window-system) (mew-demo)))
@@ -118,8 +119,8 @@ Please send comments to Kazu@Mew.org.
 	 (wl (window-width))
 	 (ul (length (regexp-quote "/\\ - \\/")))
 	 (pl (/ (- wl ul) 2))
-	 (pre (make-string pl 32))
-	 (suf (make-string (1- (- (- wl pl) ul)) 32)))
+	 (pre (make-string pl mew-sp))
+	 (suf (make-string (1- (- (- wl pl) ul)) mew-sp)))
     (save-window-excursion
       (while list
 	(mew-demo-print (car list) pre suf)
