@@ -41,9 +41,18 @@
   (make-local-variable 'mail-header-separator)
   (setq mail-header-separator mew-header-separator)
   (make-local-variable 'comment-start)
-  (setq comment-start ">")
+  (setq comment-start mew-comment-start)
+  (make-local-variable 'comment-start-skip)
+  (setq comment-start-skip mew-comment-start-skip)
   (make-local-hook 'after-change-functions)
-  (add-hook 'after-change-functions 'mew-draft-dynamic-highlight nil 'local))
+  (add-hook 'after-change-functions 'mew-draft-dynamic-highlight nil 'local)
+  (if (boundp 'write-file-functions)
+      (add-hook 'write-file-functions 'mew-encode-make-backup nil 'local)
+    (add-hook 'local-write-file-hooks 'mew-encode-make-backup))
+  (make-local-variable 'after-save-hook)
+  (when mew-require-final-newline
+    (make-local-variable 'require-final-newline)
+    (setq require-final-newline t)))
 
 (defun mew-draft-mode (&optional encrypted)
   "\\<mew-draft-mode-map>
@@ -62,6 +71,7 @@ To send a draft, type \\[mew-draft-make-message] and \\[mew-draft-send-message].
 	To compose a multipart message, you should execute this 
 	command first.
 
+\\[mew-draft-set-case]	Guess case and set the case for output to it.
 \\[mew-draft-rehighlight]  Highlight header and body again.
 
 \\[mew-draft-kill]	Kill this draft.
@@ -171,24 +181,20 @@ Moreover, '~/.mew-fib' supports aliases like;
 
 "
   (interactive)
-  (mew-draft-set-local-variables)
-  (add-hook 'local-write-file-hooks 'mew-encode-make-backup)
   (setq major-mode 'mew-draft-mode)
+  (setq mode-line-buffer-identification mew-mode-line-id)
+  (mew-draft-mode-name)
+  (mew-draft-set-local-variables)
   (use-local-map mew-draft-mode-map)
   (set-syntax-table mew-draft-mode-syntax-table)
-  (mew-ainfo-set-icon (file-name-nondirectory (buffer-file-name)))
   (cd (expand-file-name mew-home))
-  (when mew-require-final-newline
-    (make-local-variable 'require-final-newline)
-    (setq require-final-newline t))
   (mew-draft-setup-decoration)
+  (mew-ainfo-set-icon (file-name-nondirectory (buffer-file-name)))
   (mew-tinfo-set-encrypted-p encrypted)
   (mew-tinfo-set-privacy-err nil)
   (mew-tinfo-set-privacy-type nil)
-  (setq mode-line-buffer-identification mew-mode-line-id)
-  (mew-draft-mode-name)
   (run-hooks 'text-mode-hook 'mew-draft-mode-hook)
-  ;; must be here for auto-fill
+  ;; auto-fill-function is set by mew-draft-mode-hook
   (when auto-fill-function
     (make-local-variable 'auto-fill-function)
     (setq auto-fill-function 'mew-draft-auto-fill))
@@ -261,7 +267,7 @@ Moreover, '~/.mew-fib' supports aliases like;
       (setq switch-func 'switch-to-buffer-other-window)))
     (mew-frwlet
      mew-cs-m17n mew-cs-dummy
-     (funcall switch-func (find-file-noselect draft-path)))
+     (funcall switch-func (mew-find-file-noselect draft-path)))
     ;; draft buffer
     (mew-set-buffer-cs mew-cs-m17n)
     ;; copy config, first
@@ -381,7 +387,7 @@ the Body: field."
 	     (file-exists-p (expand-file-name mew-x-face-file)))
 	(let (xface)
 	  (with-temp-buffer
-	    (insert-file-contents (expand-file-name mew-x-face-file))
+	    (mew-insert-file-contents (expand-file-name mew-x-face-file))
 	    (setq xface (mew-buffer-substring (point-min)
 					      (max (buffer-size) 1))))
 	  (mew-draft-header-insert mew-x-face: xface)))
@@ -393,6 +399,7 @@ the Body: field."
     (mew-draft-header-insert-alist (mew-header-alist case))
     ;; X-Mailer: must be the last
     (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
+    ;; (mew-header-set "\n") is enough. But highlighting delayed.
     (mew-header-prepared)
     ;; on the body
     (if nl (insert "\n"))
@@ -440,20 +447,24 @@ citation prefix and label.
 	(set-buffer (mew-buffer-message))
 	(save-restriction
 	  (widen)
-	  (let ((mark-active t) (transient-mark-mode nil))
+	  (let ((mark-active t))
 	    (cond
 	     (arg 
 	      (setq beg (point-min) end (point-max)))
-	     ((mew-mark) 
+	     ((and (not mew-cite-ignore-region)
+		   (mew-mark)
+		   (/= (point) (mew-mark))
+		   (not (and mew-cite-ignore-mouse-region
+			     (mew-mouse-region-p))))
 	      (setq beg (region-beginning) end (region-end)))
-	     ((mew-header-p)
+	     ((mew-msghder-p)
 	      ;; header exists in Message mode
 	      (mew-header-goto-body)
 	      (setq beg (point) end (point-max)))
 	     (t
 	      (setq beg (point-min) end (point-max)))))
 	  (setq cite (mew-buffer-substring beg end))))
-      (push-mark (point) t t) ;; for C-x C-x
+      (mew-push-mark)
       (insert cite)
       (mew-draft-auto-set-input-method))))
 
@@ -496,13 +507,16 @@ citation prefix and label.
 	(save-restriction
 	  ;; first prepare "cite"
 	  (widen)
-	  (let ((mark-active t) (transient-mark-mode nil))
+	  (let ((mark-active t))
 	    (cond
 	     ;; arg will be effect in mew-cite-original
-	     ((and (mew-mark) (or (not mew-cite-ignore-mouse-region)
-				  (not (mew-mouse-region-p))))
+	     ((and (not mew-cite-ignore-region)
+		   (mew-mark)
+		   (/= (point) (mew-mark))
+		   (not (and mew-cite-ignore-mouse-region
+			     (mew-mouse-region-p))))
 	      (setq beg (region-beginning) end (region-end)))
-	     ((mew-header-p)
+	     ((mew-msghder-p)
 	      ;; header exists in Message mode. Skip the header
 	      ;; because we will concatenate it to cite later.
 	      (mew-header-goto-body)
@@ -515,7 +529,7 @@ citation prefix and label.
 	(setq tbuf (or nonmewbuf
 		       (save-excursion
 			 (when (get-buffer msg-buf)
-			   (if (mew-header-p) (current-buffer))))
+			   (if (mew-msghder-p) (current-buffer))))
 		       ;; header exists only in cache if multipart
 		       (mew-cache-hit fld msg)))
 	(if (get-buffer tbuf)
@@ -526,32 +540,30 @@ citation prefix and label.
 	  (mew-header-goto-end)
 	  (setq cite (concat (mew-buffer-substring (point-min) (point)) 
 			     "\n" cite))
-          (setq irt-msgid (mew-header-get-value mew-message-id:))))
+          (setq irt-msgid (mew-idstr-get-first-id
+			   (mew-header-get-value mew-message-id:)))))
       ;; 
       ;; Draft mode, insert the header and the body.
       ;;
 
       ;; Append message-id to In-Reply-To:
-      (if (and irt-msgid (mew-header-p))
+      (if (and irt-msgid (mew-msghder-p))
           (save-excursion
-            (let ((irt (mew-header-get-value mew-in-reply-to:))
-                  (irtl nil) rb)
-              (when irt
-                (setq rb irt)
-                (while (string-match "<[^>]+>" rb)
-                  (setq irtl (cons (match-string 0 rb) irtl)) 
-                  (setq rb (substring rb (match-end 0))))
-                (if (member irt-msgid irtl)
-                    (setq irt-msgid nil)))
-              (when irt-msgid
-                (setq irt (concat irt (if irt "\n\t") irt-msgid))
-                (mew-header-delete-lines (list mew-in-reply-to:))
-                (unless irtl (goto-char (mew-header-end)))
-                (mew-draft-header-insert mew-in-reply-to: irt)))))
+            (let* ((mew-references-max-count nil)
+		   (irt (mew-header-get-value mew-in-reply-to:))
+		   (irtl (mew-idstr-to-id-list irt 'rev))
+		   irtstr)
+	      (mew-addq irtl irt-msgid)
+	      (setq irtl (nreverse irtl))
+	      (setq irtstr (mew-id-list-to-idstr irtl))
+	      (mew-header-delete-lines (list mew-in-reply-to:))
+	      (unless irt (goto-char (mew-header-end)))
+	      (mew-draft-header-insert mew-in-reply-to: irtstr))))
       (save-restriction
 	;; this gets complicated due to supercite, please do not care
 	(narrow-to-region (point)(point)) ;; for (goto-char (point-min))
 	(insert cite)
+	;; not for C-x C-x. Do not use mew-push-mark.
 	(push-mark (point) t t)
 	(goto-char (point-min)))
       (cond
@@ -586,7 +598,7 @@ citation prefix and label.
 	  ;; header has been already cited. So, delete it.
 	  (delete-region beg (progn (mew-header-goto-body) (point))))
       (insert label)
-      (push-mark (point) t t) ;; for C-x C-x
+      (mew-push-mark)
       (and (bolp) (insert prefix))
       (while (= 0 (forward-line))
 	(or (= (point) (point-max))
@@ -659,25 +671,38 @@ The citation style is 'from_address> ', e.g. 'kazu> '"
 ;;; Misc
 ;;;
 
+(defun mew-draft-save-buffer ()
+  "Save this draft."
+  (interactive)
+  (let ((after-change-functions nil))
+    (save-excursion
+      (mew-header-clear 'keep-read-only)
+      (insert-before-markers "\n") ;; for mew-summary-reply
+      (save-buffer)
+      (delete-region (1- (point)) (point))
+      (mew-header-prepared)
+      (set-buffer-modified-p nil))))
+
 (defun mew-draft-kill ()
   "Kill this draft."
   (interactive)
-  (if (y-or-n-p "Kill draft message? ")
-      (let* ((attachdir (mew-attachdir)) ;; attachdir must be here
-	     (draft (buffer-file-name))
-	     (buf (current-buffer))
-	     (mdi (concat draft mew-draft-info-suffix)))
-	(mew-overlay-delete-buffer)
-	(save-buffer)
-	(mew-delete-file draft)
-	(mew-delete-file mdi)
-	(if (and (mew-tinfo-get-other-frame) (> (length (frame-list)) 1))
-	    (delete-frame)
-	  (mew-current-get-window-config))
-	(mew-remove-buffer buf)
-	(mew-delete-directory-recursively attachdir)
-	(message "Draft was killed"))
-    (message "Draft was not killed")))
+  (if (not (y-or-n-p "Kill draft message? "))
+      (message "Draft was not killed")
+    (let* ((attachdir (mew-attachdir)) ;; attachdir must be here
+	   (draft (buffer-file-name))
+	   (buf (current-buffer))
+	   (mdi (concat draft mew-draft-info-suffix)))
+      (mew-elet
+       (mew-overlay-delete-buffer))
+      (save-buffer)
+      (mew-delete-file draft)
+      (mew-delete-file mdi)
+      (if (and (mew-tinfo-get-other-frame) (> (length (frame-list)) 1))
+	  (delete-frame)
+	(mew-current-get-window-config))
+      (mew-delete-directory-recursively attachdir)
+      (mew-remove-buffer buf)
+      (message "Draft was killed"))))
 
 (defun mew-draft-insert-signature (&optional arg)
   "Insert the signature file specified by mew-signature-file.
@@ -716,7 +741,7 @@ you can set the case."
 	    (goto-char (1- (mew-attach-begin))))
 	  (end-of-line)
 	  (unless (bolp) (insert "\n")))
-	(insert-file-contents sigfile)))))
+	(mew-insert-file-contents sigfile)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -781,14 +806,23 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 (defun mew-draft-make-message (&optional privacy signer)
   "Compose a MIME message then put it into a queue folder."
   (interactive)
-  (mew-draft-process-message 'queue privacy signer))
+  (if (and (interactive-p) ;; prevent the loop
+	   mew-use-old-pgp
+	   mew-protect-privacy-with-old-pgp-signature)
+      (mew-pgp-sign-message)
+    (mew-draft-process-message 'queue privacy signer)))
 
 (defun mew-draft-send-message ()
   "Compose a MIME message then send it."
   (interactive)
-  (mew-draft-process-message 'send))
+  (if (and (interactive-p) ;; just in case
+	   mew-use-old-pgp
+	   mew-protect-privacy-with-old-pgp-signature)
+      (mew-pgp-sign-message)
+    (mew-draft-process-message 'send)))
 
 (defun mew-draft-process-message (action &optional privacy signer)
+  (run-hooks 'mew-make-message-hook)
   (let* ((case (or (mew-tinfo-get-case) mew-case-default))
 	 (old-case case)
 	 guessed-case)
@@ -819,10 +853,11 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
   (run-hooks 'mew-send-hook)
   (let* ((buf (current-buffer))
 	 (pnm (mew-smtp-info-name case))
-	 resentp fcc sendit err)
+	 (queue (mew-queue-folder case))
+	 resentp fcc sendit msg err)
     (if (get-process pnm)
 	(message "Another message is being sent. Try later")
-      (mew-draft-remove-illegal-fields)
+      (mew-draft-remove-invalid-fields)
       ;; Check resentp
       (save-excursion
 	(goto-char (point-min))
@@ -837,6 +872,7 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 	    (setq sendit (y-or-n-p "Really send this message? "))
 	  (setq sendit t)))
       (when sendit
+	;; password should be asked in Summary mode.
 	(if (and (mew-tinfo-get-other-frame) (> (length (frame-list)) 1))
 	    (delete-frame)
 	  (mew-current-get-window-config)
@@ -847,26 +883,29 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 	    (if (mew-smtp-encode pnm case resentp fcc privacy signer)
 		(let ((mdi (concat (buffer-file-name) mew-draft-info-suffix)))
 		  (mew-delete-file mdi)
-		  (cond
-		   ((eq action 'queue)
-		    (mew-smtp-queue case "from Draft mode"))
-		   ((eq action 'send)
-		    (mew-smtp-send-message case))))
+		  (setq msg (mew-smtp-queue case "from Draft mode"))
+		  (if (eq action 'send)
+		      (mew-smtp-send-message case queue (list msg))))
 	      (setq err t))))
-	(if (not err)
-	    (run-hooks 'mew-real-send-hook)
-	  (mew-current-set-window-config)
-	  (switch-to-buffer buf)
-	  (delete-other-windows))))))
+	;; now +queue/1 exists
+	(if err
+	    (progn
+	      (mew-current-set-window-config)
+	      (switch-to-buffer buf)
+	      (delete-other-windows))
+	  (if (and (eq action 'queue) mew-visit-queue-after-sending)
+	      (mew-summary-visit-folder queue))
+	  (run-hooks 'mew-real-send-hook))))))
 
 (defun mew-draft-nntp-process-message (case action &optional privacy signer)
   (run-hooks 'mew-post-hook)
   (let* ((buf (current-buffer))
 	 (pnm (mew-nntp2-info-name case))
-	 fcc sendit err)
+	 (postq (mew-postq-folder case))
+	 fcc sendit msg err)
     (if (get-process pnm)
 	(message "Another message is being posted. Try later")
-      (mew-draft-remove-illegal-fields)
+      (mew-draft-remove-invalid-fields)
       ;; Ask Subject: before the query of "Really post".
       ;; Typing C-g here gets back to the draft.
       (mew-encode-ask-subject)
@@ -877,6 +916,7 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 	    (setq sendit (y-or-n-p "Really post this message? "))
 	  (setq sendit t)))
       (when sendit
+	;; password should be asked in Summary mode.
 	(if (and (mew-tinfo-get-other-frame) (> (length (frame-list)) 1))
 	    (delete-frame)
 	  (mew-current-get-window-config)
@@ -887,19 +927,20 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 	    (if (mew-nntp2-encode pnm case fcc privacy signer)
 		(let ((mdi (concat (buffer-file-name) mew-draft-info-suffix)))
 		  (mew-delete-file mdi)
-		  (cond
-		   ((eq action 'queue)
-		    (mew-nntp2-queue case "from Draft mode"))
-		   ((eq action 'send)
-		    (mew-nntp2-send-message case))))
+		  (setq msg (mew-nntp2-queue case "from Draft mode"))
+		  (if (eq action 'send)
+		      (mew-nntp2-send-message case postq (list msg))))
 	      (setq err t))))
-	(if (not err)
-	    (run-hooks 'mew-real-post-hook)
-	  (mew-current-set-window-config)
-	  (switch-to-buffer buf)
-	  (delete-other-windows))))))
+	(if err
+	    (progn
+	      (mew-current-set-window-config)
+	      (switch-to-buffer buf)
+	      (delete-other-windows))
+	  (if (and (eq action 'queue) mew-visit-queue-after-sending)
+	      (mew-summary-visit-folder postq))
+	  (run-hooks 'mew-real-post-hook))))))
 
-(defun mew-draft-remove-illegal-fields ()
+(defun mew-draft-remove-invalid-fields ()
   (when (mew-header-end)
     (save-excursion
       (save-restriction
