@@ -32,6 +32,19 @@ Execute mew-dinfo-set before calling this."
 ;;;
 
 (defun mew-header-mode ()
+  "\\<mew-header-mode-map> Mew Header mode:: major mode for
+composing a MIME message by editing its header only.
+
+To send the header and its body, type \\[mew-header-make-message] and \\[mew-header-send-message].
+
+Key assignment:
+
+\\[mew-header-make-message]	Compose a MIME message from the header and its body
+	then put it into a queue folder.
+\\[mew-header-send-message]	Compose a MIME message from the header and its body then send it.
+\\[mew-draft-set-case]	Guess case and set the case for output to it.
+\\[mew-draft-kill]	Kill this draft.
+"
   (interactive)
   (mew-draft-set-local-variables)
   (setq major-mode 'mew-header-mode)
@@ -40,7 +53,11 @@ Execute mew-dinfo-set before calling this."
   (mew-header-setup-decoration)
   (mew-draft-mode-name 'header)
   (force-mode-line-update)
-  (run-hooks 'mew-header-mode-hook))
+  (run-hooks 'mew-header-mode-hook)
+  ;; auto-fill-function is set by mew-header-mode-hook
+  (when auto-fill-function
+    (make-local-variable 'auto-fill-function)
+    (setq auto-fill-function 'mew-draft-auto-fill)))
 
 (defmacro mew-summary-header-mode (fields &rest body)
   `(mew-summary-msg-or-part
@@ -54,14 +71,17 @@ Execute mew-dinfo-set before calling this."
        (mew-draft-find-and-switch draft)
        (mew-delete-directory-recursively attachdir)
        ,@body
-       (mew-header-prepared 'header)
+       (mew-header-mode)
+       ;; (mew-header-set "\n") is enough. But highlighting delayed.
+       (mew-header-prepared)
        (goto-char (point-min))
        (end-of-line)
        (mew-tinfo-set-hdr-file (mew-expand-folder fld msg))
-       (mew-tinfo-set-field-del ,fields)
-       (mew-header-mode)))))
+       (mew-tinfo-set-field-del ,fields)))))
 
 (defun mew-summary-send-to-others ()
+  "Prepare Header mode so that this message is to be sent
+other people by a header re-write."
   (interactive)
   (mew-summary-header-mode
    mew-field-delete-for-others
@@ -74,6 +94,8 @@ Execute mew-dinfo-set before calling this."
      (mew-draft-header-insert mew-organization: (mew-organization case)))))
 
 (defun mew-summary-resend ()
+  "Prepare Header mode so that this message is to be forwarded
+by a header re-write."
   (interactive)
   (mew-summary-header-mode
    mew-field-delete-for-resending
@@ -95,9 +117,11 @@ Execute mew-dinfo-set before calling this."
   (let* ((buf (current-buffer))
 	 (case (mew-tinfo-get-case))
 	 (pnm (mew-smtp-info-name case))
-	 sendit resentp fcc err)
+	 (queue (mew-queue-folder case))
+	 sendit resentp fcc msg err)
     (if (get-process pnm)
 	(message "Another message is being sent. Try later")
+      (mew-draft-remove-invalid-fields)
       (goto-char (point-min))
       (setq resentp (mew-draft-resent-p (mew-header-end)))
       (setq fcc (mew-encode-ask-fcc resentp))
@@ -124,24 +148,29 @@ Execute mew-dinfo-set before calling this."
 	      (mew-header-delete-lines mew-field-delete-common)
 	      (mew-header-delete-lines (mew-tinfo-get-field-del)))
 	    (if (mew-smtp-encode pnm case resentp fcc nil nil 'header)
-		(cond
-		 ((eq action 'queue)
-		  (mew-smtp-queue case "from Draft mode"))
-		 ((eq action 'send)
-		  (mew-smtp-send-message case)))
+		(let ((mdi (concat (buffer-file-name) mew-draft-info-suffix)))
+		  (mew-delete-file mdi)
+		  (setq msg (mew-smtp-queue case "from Draft mode"))
+		  (if (eq action 'send)
+		      (mew-smtp-send-message case queue (list msg))))
 	      (setq err t))))
-	(when err
-	  (mew-current-set-window-config)
-	  (switch-to-buffer buf)
-	  (delete-other-windows))))))
+	(if err
+	    (progn
+	      (mew-current-set-window-config)
+	      (switch-to-buffer buf)
+	      (delete-other-windows))
+	  (if (and (eq action 'queue) mew-visit-queue-after-sending)
+	      (mew-summary-visit-folder queue)))))))
 
 (defun mew-header-nntp-process-message (action)
   (let* ((buf (current-buffer))
 	 (case (mew-tinfo-get-case))
 	 (pnm (mew-nntp2-info-name case))
-	 sendit resentp fcc err)
+	 (postq (mew-postq-folder case))
+	 sendit resentp fcc msg err)
     (if (get-process pnm)
 	(message "Another message is being posted. Try later")
+      (mew-draft-remove-invalid-fields)
       (goto-char (point-min))
       (setq resentp (mew-draft-resent-p (mew-header-end)))
       (setq fcc (mew-encode-ask-fcc resentp))
@@ -168,22 +197,29 @@ Execute mew-dinfo-set before calling this."
 	      (mew-header-delete-lines mew-field-delete-common)
 	      (mew-header-delete-lines (mew-tinfo-get-field-del)))
 	    (if (mew-nntp2-encode pnm case fcc nil nil 'header)
-		(cond
-		 ((eq action 'queue)
-		  (mew-nntp2-queue case "from Draft mode"))
-		 ((eq action 'send)
-		  (mew-nntp2-send-message case)))
+		(let ((mdi (concat (buffer-file-name) mew-draft-info-suffix)))
+		  (mew-delete-file mdi)
+		  (setq msg (mew-nntp2-queue case "from Draft mode"))
+		  (if (eq action 'send)
+		      (mew-nntp2-send-message  case postq (list msg))))
 	      (setq err t))))
-	(when err
-	  (mew-current-set-window-config)
-	  (switch-to-buffer buf)
-	  (delete-other-windows))))))
+	(if err
+	    (progn
+	      (mew-current-set-window-config)
+	      (switch-to-buffer buf)
+	      (delete-other-windows))
+	  (if (and (eq action 'queue) mew-visit-queue-after-sending)
+	      (mew-summary-visit-folder postq)))))))
 
 (defun mew-header-make-message ()
+  "Compose a MIME message from the header and its body 
+then put it into a queue folder."
   (interactive)
   (mew-header-process-message 'queue))
 
 (defun mew-header-send-message ()
+  "Compose a MIME message from the header and its body 
+then send it."
   (interactive)
   (mew-header-process-message 'send))
 
@@ -201,6 +237,7 @@ a text object to be saved can be specified."
   (interactive "P")
   (mew-summary-msg-or-part
    ;; need to make a cache or a message buffer.
+   (mew-summary-toggle-disp-msg 'on)
    (mew-summary-display nil)
    (let* ((fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number2))
@@ -260,6 +297,8 @@ a text object to be saved can be specified."
 	   (setq end (mew-syntax-get-end (mew-syntax-get-part syntax)))
 	 (setq end (mew-syntax-get-end syntax)))
        (setq ctl (mew-syntax-get-ct syntax))
+       (setq cdpl (mew-syntax-get-cdp syntax))
+       (setq file (mew-syntax-get-filename cdpl ctl))
        ;;
        (if (and mew-use-samba-encoding
 		file
@@ -301,10 +340,10 @@ a text object to be saved can be specified."
 	       ;; When saving an entire message or its body, 
 	       ;; its charset is unknown since it is not analyzed.
 	       ;; So, we make use of mew-cs-autoconv.
-	       (if (not (mew-coding-system-p fromcs))
-		   (error "Unknown coding system %s" (symbol-name fromcs)))
-	       (if (not (mew-coding-system-p tocs))
-		   (error "Unknown coding system %s" (symbol-name tocs)))
+	       (unless (mew-coding-system-p fromcs)
+		 (error "Unknown coding system %s" (symbol-name fromcs)))
+	       (unless (mew-coding-system-p tocs)
+		 (error "Unknown coding system %s" (symbol-name tocs)))
 	       (mew-cs-decode-region
 		(point-min) (point-max) (or fromcs mew-cs-autoconv))
 	       (mew-cs-encode-region (point-min) (point-max) tocs))
@@ -327,6 +366,7 @@ If executed with '\\[universal-argument]', coding-system is asked."
       (message "This command cannot be used on non-Mule Emacs")
     (mew-summary-msg-or-part
      ;; need to make a cache or a message buffer.
+     (mew-summary-toggle-disp-msg 'on)
      (mew-summary-display nil)
      (let* ((fld (mew-summary-folder-name))
 	    (msg (mew-summary-message-number2))
@@ -438,7 +478,7 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	    (message "Nothing was processed")
 	  (mew-touch-folder folder)
 	  (setq ret (list mstr (int-to-string (1- m)))))))
-      (if errmsg (error errmsg))
+      (if errmsg (error "%s" errmsg))
       ret)))
 
 (defun mew-summary-burst-part (part folder newmsg &optional num)
@@ -498,13 +538,14 @@ If called with '\\[universal-argument]', the current part is de-capsulated."
        (setq ret (mew-summary-burst-body fld msg folder num))
        (when ret
 	 (message "Bursting...done")
-	 (if (y-or-n-p (format "Go to %s? " folder))
-	     (mew-summary-visit-folder folder 'goend))
+	 (when mew-ask-folder-after-burst
+	   (if (y-or-n-p (format "Go to %s? " folder))
+	       (mew-summary-visit-folder folder 'goend)))
 	 (message "Messages from %s to %s were extracted in %s"
 		  (nth 0 ret) (nth 1 ret) folder))))))
 
 (defun mew-summary-burst-multi ()
-  "De-capsulate messages embedded in the messages marked with '@'."
+  "De-capsulate messages embedded in the messages marked with '*'."
   (interactive)
   (mew-summary-multi-msgs
    (let ((folder (mew-input-burst-folder))
@@ -516,8 +557,9 @@ If called with '\\[universal-argument]', the current part is de-capsulated."
 	  (car (car targets)) (cdr (car targets)) folder)
 	 (setq targets (cdr targets)))
        (message "Bursting...done")
-       (if (y-or-n-p (format "Go to %s? " folder))
-	   (mew-summary-visit-folder folder 'goend))))))
+       (when mew-ask-folder-after-burst
+	 (if (y-or-n-p (format "Go to %s? " folder))
+	     (mew-summary-visit-folder folder 'goend)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -525,6 +567,7 @@ If called with '\\[universal-argument]', the current part is de-capsulated."
 ;;;
 
 (defun mew-summary-old-burst ()
+  "Decompose old-fashioned encapsulated messages."
   (interactive)
   (mew-summary-msg
    (let ((fld (mew-summary-folder-name))
@@ -563,8 +606,9 @@ If called with '\\[universal-argument]', the current part is de-capsulated."
 	 (message "Bursting...done")
 	 (mew-touch-folder folder)
 	 (setq done t)))
-     (if (and done (y-or-n-p (format "Go to %s? " folder)))
-	 (mew-summary-visit-folder folder 'goend)))))
+     (when mew-ask-folder-after-burst
+       (if (and done (y-or-n-p (format "Go to %s? " folder)))
+	   (mew-summary-visit-folder folder 'goend))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -629,7 +673,7 @@ See also mew-summary-edit-again."
   ;; X-Mailer: must be the last
   (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
   (mew-header-clear) ;; erase the old header separator
-  (mew-header-prepared))
+  (mew-header-set "\n"))
 
 (defun mew-summary-edit-header-for-draft ()
   (mew-elet
@@ -710,7 +754,7 @@ The message is assumed to be a valid MIME message."
 	((mew-folder-postqp fld)
 	 (setq hdr (mew-nntp2-get-raw-header inf))
 	 (mew-tinfo-set-case (mew-nntp2-get-case inf))))
-       (mew-queue-backup file mew-queue-info-suffix)
+       (mew-queue-backup work mew-queue-info-suffix)
        (and hdr (mew-summary-edit-header-for-queue hdr))))
    (mew-draft-mode)
    (mew-draft-rehighlight-body)
