@@ -109,7 +109,7 @@
 ;;; Header sanity check
 ;;;
 
-(defun mew-encode-remove-illegal-fields ()
+(defun mew-encode-remove-invalid-fields ()
   (when (mew-header-end)
     (save-excursion
       (save-restriction
@@ -137,6 +137,7 @@
 	(setq folder (mew-canonicalize-folder (car folders)))
 	(setq folders (cdr folders))
 	(when (and mew-ask-fcc
+		   (mew-folder-localp folder)
 		   (not (file-directory-p (mew-expand-folder folder))))
 	  (if (y-or-n-p (format "%s does not exist. Create it? " folder))
 	      (mew-local-folder-check folder)
@@ -149,60 +150,73 @@
 ;;; To:, Cc:
 ;;;
 
-(defun mew-encode-canonicalize-address (resentp)
-  (let ((case-fold-search t)
-	(regex (mew-make-field-regex
-		(if resentp mew-resent-dest:-list mew-destination:-list)))
-	key start vals val addrs addr ret insl ins prefix suffix)
-    (save-excursion
-      (save-restriction
-	(narrow-to-region (point-min) (mew-header-end))
-	(goto-char (point-min))
-	(while (re-search-forward regex nil t)
-	  (setq key (match-string 1))
-	  (setq start (match-end 0))
-	  (forward-line)
-	  (while (looking-at mew-lwsp)
-	    (delete-backward-char 1)
-	    (forward-line))
-	  (setq val (mew-buffer-substring start (1- (point))))
-	  (delete-region start (1- (point)))
-	  (backward-char 1)
-	  ;;
-	  (setq vals (mapcar 'mew-chop (mew-split-quoted val ?, ?: ?\;)))
-	  (while vals
-	    (setq val (car vals))
-	    (setq vals (cdr vals))
-	    (setq ins nil addrs nil)
-	    (cond
-	     ((string-match "[^:]+:[^;]+;" val)
-	      (let ((mew-alias-expand-prefix nil))
-		(setq addrs (mew-alias-expand val mew-addrbook-alist 0))
-		(setq addrs (mapcar 'mew-addrstr-append-domain addrs))
-		(setq ins (list (concat mew-alias-expand-prefix ":;")))))
-	     ((and (setq addr (mew-addrstr-parse-address val))
-		   (string-match (concat "\\(.*<\\)" (regexp-quote addr) "\\(>.*\\)")
-				 val))
-	      (setq prefix (match-string 1 val))
-	      (setq suffix (match-string 2 val))
-	      (setq addr (mew-addrstr-append-domain addr))
-	      (setq addrs (list addr))
-	      (setq ins (list (concat prefix addr suffix))))
-	     (t
-	      (let ((mew-alias-expand-prefix nil))
-		(setq addrs (mew-alias-expand val mew-addrbook-alist 0))
-		(setq addrs (mapcar 'mew-addrstr-append-domain addrs))
-		(if mew-alias-expand-prefix
-		    (setq ins (list (concat mew-alias-expand-prefix ":;")))
-		  (setq ins (copy-sequence addrs))))))
-	    (setq insl (nconc insl ins))
-	    (unless (string-match "bcc" key) ;; Removing Bcc:
-	      (setq ret (nconc ret addrs))))
-	  (insert " " (mapconcat 'identity insl ", "))
-	  (setq insl nil)
-	  (forward-line))))
-    ret))
+(defvar mew-draft-append-domain-p t)
 
+(defun mew-draft-append-domain (addr)
+  (if (string-match "@" addr)
+      addr
+    (if mew-draft-append-domain-p
+	(concat addr "@" (mew-mail-domain (mew-tinfo-get-case)))
+      (throw 'jump addr))))
+
+(defun mew-encode-canonicalize-address (resentp)
+  (catch 'jump
+    (let ((case-fold-search t)
+	  (regex (mew-make-field-regex
+		  (if resentp mew-resent-dest:-list mew-destination:-list)))
+	  key start vals val addrs addr ret insl ins prefix suffix)
+      (save-excursion
+	(save-restriction
+	  (narrow-to-region (point-min) (mew-header-end))
+	  (goto-char (point-min))
+	  (while (re-search-forward regex nil t)
+	    (setq key (match-string 1))
+	    (setq start (match-end 0))
+	    (forward-line)
+	    (while (looking-at mew-lwsp)
+	      (delete-backward-char 1)
+	      (forward-line))
+	    (setq val (mew-buffer-substring start (1- (point))))
+	    (delete-region start (1- (point)))
+	    (backward-char 1)
+	    ;;
+	    (setq vals (mapcar 'mew-chop (mew-split-quoted val ?, ?: ?\;)))
+	    (while vals
+	      (setq val (car vals))
+	      (setq vals (cdr vals))
+	      (setq ins nil addrs nil)
+	      (cond
+	       ;; phrase:addr1,addr2;
+	       ((string-match "[^:]+:[^;]+;" val)
+		(let ((mew-alias-expand-prefix nil))
+		  (setq addrs (mew-alias-expand val mew-addrbook-alist 0))
+		  (setq addrs (mapcar 'mew-draft-append-domain addrs))
+		  (setq ins (list (concat mew-alias-expand-prefix ":;")))))
+	       ;; Name <addr>
+	       ((and (setq addr (mew-addrstr-parse-address val))
+		     (string-match (concat "\\(.*<\\)" (regexp-quote addr) "\\(>.*\\)")
+				   val))
+		(setq prefix (match-string 1 val))
+		(setq suffix (match-string 2 val))
+		(setq addr (mew-draft-append-domain addr))
+		(setq addrs (list addr))
+		(setq ins (list (concat prefix addr suffix))))
+	       ;; addr
+	       (t
+		(let ((mew-alias-expand-prefix nil))
+		  (setq addrs (mew-alias-expand val mew-addrbook-alist 0))
+		  (setq addrs (mapcar 'mew-draft-append-domain addrs))
+		  (if mew-alias-expand-prefix
+		      (setq ins (list (concat mew-alias-expand-prefix ":;")))
+		    (setq ins (copy-sequence addrs))))))
+	      (setq insl (nconc insl ins))
+	      (unless (string-match "bcc" key) ;; Removing Bcc:
+		(setq ret (nconc ret addrs)))) ;; end of while
+	    (insert " " (mapconcat 'identity insl ", "))
+	    (setq insl nil)
+	    (forward-line))))
+      ret)))
+	
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Dcc:, Bcc:
@@ -316,8 +330,8 @@
    (addsep ;; reedit
     ;; To:
     ;; Content-*
-    ;; ---
-    (mew-header-clear) ;; mew-header-p returns nil
+    ;; ----
+    (mew-header-clear) ;; mew-in-header-p() returns nil
     ;; To:
     ;; Content-*
     (insert "\n"))
@@ -325,7 +339,7 @@
     ;; To:
     ;; ----
     ;; Content-*
-    (mew-header-clear) ;; mew-header-p returns nil
+    (mew-header-clear) ;; mew-in-header-p returns nil
     ;; To:
     ;; Content-*
     ))
@@ -354,7 +368,7 @@
 	  (mew-smtp-encode-message pnm case resentp fcc privacy signer headerp)
 	(error
 	 (mew-encode-undo)
-	 (message (mew-tinfo-get-encode-err))
+	 (message "%s" (mew-tinfo-get-encode-err))
 	 (setq ret nil))))
     ret))
 
@@ -363,26 +377,31 @@
   (if (buffer-modified-p) (save-buffer)) ;; to make backup
   (widen)
   (let (multip recipients msgid-logtime)
-    (goto-char (point-min))
-    (mew-encode-remove-illegal-fields)
     (mew-smtp-set-raw-header
      pnm (mew-buffer-substring (point-min) (mew-header-end)))
-    ;; Destination check
-    (setq recipients (mew-encode-canonicalize-address resentp))
-    ;; Bcc: is not included.
-    (mew-smtp-set-recipients pnm recipients)
-    (mew-smtp-set-orig-recipients pnm recipients)
     (unless headerp
       ;; Let's backup
       (if (mew-attach-p)
 	  (progn
 	    (setq multip t)
 	    (mew-attach-clear))
-	(setq mew-encode-syntax
-	      (mew-encode-syntax-single "text.txt" (list mew-ct-txt))))
+	(unless mew-encode-syntax
+	  (setq mew-encode-syntax
+		(mew-encode-syntax-single "text.txt" (list mew-ct-txt)))))
       (mew-encode-make-backup))
     ;; Errors can be caused from here.
-    (unless recipients (mew-encode-error "No recipient!"))
+    (goto-char (point-min))
+    (mew-encode-remove-invalid-fields)
+    ;; Destination check
+    (setq recipients (mew-encode-canonicalize-address resentp))
+    ;; Bcc: is not included.
+    (mew-smtp-set-recipients pnm recipients)
+    (mew-smtp-set-orig-recipients pnm recipients)
+    (cond
+     ((null recipients)
+      (mew-encode-error "No recipient!"))
+     ((stringp recipients)
+      (mew-encode-error (format "'%s' is not in the right form!" recipients))))
     ;; Header modifications which are not remained.
     (mew-header-delete-lines (list mew-fcc: mew-resent-fcc:)) ;; anyway
     (mew-smtp-set-dcc pnm (mew-encode-delete-dcc resentp))
@@ -404,7 +423,7 @@
     (unless headerp
       (mew-encode-set-privacy pnm privacy)
       (let ((mew-inherit-encode-signer (or signer (mew-get-my-address))))
-	(goto-char (mew-header-end)) ;; due to illegal null lines in the header
+	(goto-char (mew-header-end)) ;; due to invalid null lines in the header
 	(forward-line)
 	(if multip
 	    (mew-encode-make-multi)
@@ -425,7 +444,7 @@
 	  (mew-nntp2-encode-message pnm case fcc privacy signer headerp)
 	(error
 	 (mew-encode-undo)
-	 (message (mew-tinfo-get-encode-err))
+	 (message "%s" (mew-tinfo-get-encode-err))
 	 (setq ret nil))))
     ret))
 
@@ -434,13 +453,8 @@
   (if (buffer-modified-p) (save-buffer)) ;; to make backup
   (widen)
   (let (multip newsgroups msgid-logtime)
-    (goto-char (point-min))
-    (mew-encode-remove-illegal-fields)
     (mew-nntp2-set-raw-header
      pnm (mew-buffer-substring (point-min) (mew-header-end)))
-    ;;
-    (setq newsgroups (mew-header-get-value mew-newsgroups:))
-    (mew-nntp2-set-newsgroups pnm newsgroups)
     ;; Let's backup
     (unless headerp
       (if (mew-attach-p)
@@ -451,6 +465,11 @@
 	      (mew-encode-syntax-single "text.txt" (list mew-ct-txt))))
       (mew-encode-make-backup))
     ;; Errors can be caused from here.
+    (goto-char (point-min))
+    (mew-encode-remove-invalid-fields)
+    ;; Newsgroups check
+    (setq newsgroups (mew-header-get-value mew-newsgroups:))
+    (mew-nntp2-set-newsgroups pnm newsgroups)
     (unless newsgroups (mew-encode-error "No newsgroups!"))
     ;; Header modifications which are not remained.
     (mew-header-delete-lines (list mew-fcc: mew-resent-fcc:)) ;; anyway
@@ -470,7 +489,7 @@
     (unless headerp
       (mew-encode-set-privacy pnm privacy)
       (let ((mew-inherit-encode-signer (or signer (mew-get-my-address))))
-	(goto-char (mew-header-end)) ;; due to illegal null lines in the header
+	(goto-char (mew-header-end)) ;; due to invalid null lines in the header
 	(forward-line)
 	(if multip
 	    (mew-encode-make-multi)
@@ -639,8 +658,7 @@
 	 charset-cte charset bodybeg cst ask-cst)
     (setq charset-cte (mew-encode-mime-body ctl cte (or buffered file)))
     (goto-char beg)
-    (setq charset (nth 0 charset-cte))
-    (setq cte (nth 1 charset-cte))
+    (mew-set '(charset cte) charset-cte)
     (setq ctl (mew-delete "icharset" ctl))
     (when charset
       (setq ctl (mew-syntax-get-params ctl))
@@ -686,7 +704,7 @@
 
 (defun mew-encode-make-multi ()
   ;; Just after the header
-  (let (beg path multip buffered privacy first ct)
+  (let (beg path multip buffered privacy decrypters first ct)
     (if (mew-encode-syntax-have-one-part)
 	;; A user may want to do any MIME stuff to the body
 	(progn
@@ -720,8 +738,10 @@
 	    (setq path (expand-file-name
 			(mew-syntax-get-file mew-encode-syntax) path))
 	    (setq privacy (mew-syntax-get-privacy mew-encode-syntax))
+	    (setq decrypters (mew-syntax-get-decrypters mew-encode-syntax))
 	    (setq mew-encode-syntax (mew-syntax-get-part mew-encode-syntax))
-	    (mew-syntax-set-privacy mew-encode-syntax privacy)))))
+	    (mew-syntax-set-privacy mew-encode-syntax privacy)
+	    (mew-syntax-set-decrypters mew-encode-syntax decrypters)))))
     (if multip
 	(mew-encode-multipart mew-encode-syntax path 0 buffered)
       (mew-encode-singlepart mew-encode-syntax path nil nil 'cover))))
@@ -733,7 +753,8 @@
   (unless string (setq string "Next_Part"))
   (format mew-default-boundary
 	  string
-	  (mew-replace-character (current-time-string) 32 ?_)
+	  (mew-replace-character
+	   (mew-replace-character (current-time-string) mew-sp ?_) ?: ?_)
 	  (mew-random-string 3 t)))
 
 (defun mew-encode-multipart (syntax path depth &optional buffered)
@@ -793,8 +814,8 @@
     (let (proto ct)
       (while privacy
 	(goto-char (point-min)) 
-	(setq ct (mew-capitalize (nth 0 (car privacy))))
-	(setq proto (nth 1 (car privacy)))
+	(mew-set '(ct proto) (car privacy))
+	(setq ct (mew-capitalize ct))
 	(setq privacy (cdr privacy))
 	(cond 
 	 ((string= mew-ct-mle ct)
@@ -850,8 +871,7 @@
        (mew-encode-error
 	(format "unknown error for %s. Check %s, anyway" 
 		mew-ct-mle mew-temp-dir))))
-    (setq file2 (nth 0 fc) cte2 (nth 1 fc) file3 (nth 2 fc) cte3 (nth 3 fc))
-    (setq errmsg (nth 4 fc))
+    (mew-set '(file2 cte2 file3 cte3 errmsg) fc)
     (if errmsg
 	(progn
 	  (mew-delete-file file1)
@@ -896,9 +916,7 @@
        (mew-encode-error
 	(format "unknown error for %s. Check %s, anyway" 
 		mew-ct-mls mew-temp-dir))))
-    (setq file2 (nth 0 fmc) cte2 (nth 1 fmc) micalg (nth 2 fmc))
-    (setq errmsg (nth 3 fmc))
-    (setq ct2 (nth 4 fmc) cdp2 (nth 5 fmc))
+    (mew-set '(file2 cte2 micalg errmsg ct2 cdp2) fmc)
     (if errmsg
 	(progn
 	  (mew-delete-file file1)
@@ -1152,9 +1170,10 @@
      (if (not (mew-encode-insert-backup))
 	 (message "Cannot undo")
        (mew-header-clear) ;; erase the old header separator
-       (mew-header-prepared (mew-tinfo-get-hdr-file))
+       (mew-header-prepared)
        (if mew-encode-syntax (mew-draft-prepare-attachments))
        (mew-draft-toolbar-update)
+       (setq mew-encode-syntax nil)
        (setq buffer-undo-list nil)))))
 
 (provide 'mew-encode)
