@@ -17,11 +17,7 @@ PIPE is usually recommended for speed but some OSes such as Linux
 requires PTY.")
 
 (defvar mew-connection-type2 t
-  "Connection type for processes that requires a password. ")
-
-(defvar mew-connection-type-for-scan t
-  "Connection type for 'mewls' whose output speed is very fast. 
-To slow down, PTY should be used.")
+  "Connection type for processes that requires a password.")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -201,6 +197,7 @@ To slow down, PTY should be used.")
   (and (overlay-get ovl 'mew) (delete-overlay ovl)))
 
 (defsubst mew-overlay-delete-region (beg end)
+  "Delete overlays in the region."
   (interactive "r")
   (mapcar 'mew-overlay-delete (overlays-in beg end))
   (if (fboundp 'remove-images) (remove-images beg end))) ;; xxx
@@ -317,7 +314,13 @@ on all frames."
  ((fboundp 'base64-decode-string)
   (defun mew-base64-decode-string (str64)
     (condition-case nil
-        (base64-decode-string str64)
+	(let ((r (% (length str64) 4)))
+	  (cond
+	   ((= r 2)
+	    (setq str64 (concat str64 "==")))
+	   ((= r 3)
+	    (setq str64 (concat str64 "="))))
+	  (base64-decode-string str64))
       (error nil))))
  (t
   (defconst mew-base64-char256
@@ -365,9 +368,13 @@ on all frames."
 ;;;
 
 ;; to load the thread separator
-(if (and (boundp 'current-language-environment)
-	 (string= current-language-environment "Japanese"))
-    (require 'mew-lang-jp))
+(cond
+ ((and (boundp 'current-language-environment)
+       (string= current-language-environment "Japanese"))
+  (require 'mew-lang-jp))
+ ((and (boundp 'current-language-environment)
+       (string= current-language-environment "Korean"))
+  (require 'mew-lang-kr)))
 
 (provide 'mew-env)
 
