@@ -13,7 +13,11 @@
 ;;;
 
 (defun mew-mark-kill-refile (src msg)
-  (not (member src (mew-refile-get msg))))
+  (let ((refinfo (mew-refile-get msg))
+	myself)
+    (setq myself (or (null refinfo) (member src refinfo)))
+    (if myself (mew-summary-refile-remove-body))
+    (not myself)))
 
 (defun mew-mark-exec-refile (src msgs)
   "Refile MSGs from the SRC folder."
@@ -36,32 +40,27 @@
 (defun mew-mark-exec-unlink (src dels)
   "Unlink DELS from the SRC folder.
 DELS represents the messages to be deleted."
-  (let ((mew-msg-rm-policy 'always))
-    (mew-mark-exec-delete src dels)))
+  (when dels
+    (mew-summary-unlink-msgs src dels)))
 
 (defun mew-mark-exec-delete (src dels)
-  "Delete messages from the SRC folder according to mew-msg-rm-policy.
+  "Delete messages from the SRC folder according to 'mew-trash-folder'
+and 'mew-trash-folder-list'.
 DELS represents the messages to be deleted."
   (when dels
-    (let ((unlink-it nil))
-      (cond
-       ((eq mew-msg-rm-policy 'always)
-	(setq unlink-it t))
-       ((eq mew-msg-rm-policy 'trashonly)
-	(if (string= src mew-trash-folder)
-	    (setq unlink-it t)))
-       ((eq mew-msg-rm-policy 'uselist)
-	(if (mew-member-match src mew-msg-rm-folder-list)
-	    (setq unlink-it t))))
-      ;; t = 'totrash
-      (if unlink-it
+    ;; see also mew-summary-exec-remote-get-info.
+    (cond
+     ((null mew-trash-folder)
+      (mew-summary-unlink-msgs src dels))
+     (mew-trash-folder-list
+      (if (or (member src mew-trash-folder-list)
+	      (string= src mew-trash-folder))
 	  (mew-summary-unlink-msgs src dels)
-	(if (string= src mew-trash-folder)
-	    (progn
-	      (goto-char (point-min))
-	      (while (re-search-forward mew-regex-msg-delete nil t)
-		(mew-summary-undo-one)))
-	  (mew-summary-totrash-msgs src dels))))))
+	(mew-summary-totrash-msgs src dels)))
+     (t
+      (if (string= src mew-trash-folder)
+	  (mew-summary-unlink-msgs src dels)
+	(mew-summary-totrash-msgs src dels))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -81,7 +80,7 @@ DELS represents the messages to be deleted."
 	(mew-addq uniq-dsts dst)))
     (mew-addq uniq-dsts mew-trash-folder)
     (while uniq-dsts
-      (setq udst (mew-folder-case-folder mew-inherit-exec-case (car uniq-dsts)))
+      (setq udst (mew-case-folder mew-inherit-exec-case (car uniq-dsts)))
       (setq uniq-dsts (cdr uniq-dsts))
       (setq dir (mew-expand-folder udst))
       (if (file-exists-p dir)
@@ -123,7 +122,7 @@ DELS represents the messages to be deleted."
 (defun mew-summary-mv-msgs (src dsts-msgs)
   (let* ((dsts (car dsts-msgs)) ;; (+foo +bar)
 	 (msgs (cdr dsts-msgs)) ;; (1 2 3)
-	 (srcx (mew-folder-folder src))
+	 (srcx (mew-case:folder-folder src))
 	 (myselfp (member srcx dsts))
 	 (cache mew-inherit-offline)
 	 (suffix (mew-folder-suffix srcx))
@@ -139,7 +138,7 @@ DELS represents the messages to be deleted."
 		(setq msgs- (cons msg msgs-))))
 	  (setq msgs- (nreverse msgs-))
 	  (setq msgs msgs-))
-      (setq dst (mew-folder-case-folder mew-inherit-exec-case (car dsts)))
+      (setq dst (mew-case-folder mew-inherit-exec-case (car dsts)))
       (setq dsts (cdr dsts))
       (setq num (string-to-int (mew-folder-new-message dst 'num-only cache)))
       (while msgs
@@ -158,7 +157,7 @@ DELS represents the messages to be deleted."
       (setq msgs- (nreverse msgs-))
       (setq src dst)) ;; myselfp
     (while dsts
-      (setq dst (mew-folder-case-folder mew-inherit-exec-case (car dsts)))
+      (setq dst (mew-case-folder mew-inherit-exec-case (car dsts)))
       (setq dsts (cdr dsts))
       (setq num (string-to-int (mew-folder-new-message dst 'num-only cache)))
       (setq msgs msgs-)
@@ -172,7 +171,7 @@ DELS represents the messages to be deleted."
       (mew-touch-folder dst))))
 
 (defun mew-summary-totrash-msgs (src dels)
-  (let* ((trash (mew-folder-case-folder mew-inherit-exec-case mew-trash-folder))
+  (let* ((trash (mew-case-folder mew-inherit-exec-case mew-trash-folder))
 	 (trashdir (mew-expand-folder trash))
 	 (cache mew-inherit-offline)
 	 (suffix (mew-folder-suffix src))
@@ -180,8 +179,6 @@ DELS represents the messages to be deleted."
     (unless (file-directory-p trashdir) (make-directory trashdir))
     (setq num (string-to-int (mew-folder-new-message trash 'num-only cache)))
     ;; must be here after ensuring that +trash exists.
-
-
     (while dels
       (setq srcfile (mew-expand-folder src (car dels))) ;; xxx
       (setq dels (cdr dels))
@@ -207,6 +204,7 @@ region. To cancel the '*' mark, use '\\[mew-summary-undo]' or
 '\\[mew-summary-undo-all]'."
   (interactive "P")
   (let (beg end region)
+    (if (mew-mark-active-p) (setq arg t))
     (cond
      (arg
       (setq region (mew-summary-get-region))
@@ -249,47 +247,85 @@ all marked messages before the current message."
 	 (mew-mark-spec (list ent)))
     (mew-summary-exec-region (point-min) (point-max))))
 
-(defun mew-summary-exec-refile ()
-  "Process messages marked with 'o'."
-  (interactive)
+(defun mew-summary-exec-refile (&optional arg)
+  "Process messages marked with 'o'.
+If called with '\\[universal-argument]', only messages whose
+destination is the same as that of the current message are processed."
+  (interactive "P")
   (let* ((ent (assoc mew-mark-refile mew-mark-spec))
 	 (mew-mark-spec (list ent)))
-    (mew-summary-exec-region (point-min) (point-max))))
+    (if (not arg)
+	(mew-summary-exec-region (point-min) (point-max))
+      (let* ((msg (mew-summary-message-number))
+	     (dst (cdr (mew-refile-get msg)))
+	     func)
+	(if (not dst)
+	    (message "No message to be refiled here")
+	  (setq func (lambda (num) (equal dst (cdr (mew-refile-get num)))))
+	  (mew-summary-exec-region (point-min) (point-max) func))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Exec region
 ;;;
 
-(defun mew-summary-exec-region (beg end)
+(defun mew-summary-move-and-display (msg)
+  (let ((here (point)))
+    (goto-char (point-min))
+    (if (not (re-search-forward (mew-regex-sumsyn-msg msg) nil t))
+	(goto-char here)
+      (beginning-of-line)
+      (mew-summary-display nil))))
+
+(defun mew-summary-go-back-summary (key)
+  (let ((msg (mew-summary-message-number))
+	fld)
+    (cond
+     ((mew-thread-p)
+      ;; mew-vinfo-get-parent-folder cannot be used here
+      (setq fld (mew-physical-folder (mew-summary-folder-name 'ext))))
+     ((mew-pickable-p)
+      (setq fld (car (mew-vinfo-get-flds)))))
+    (if (not (and fld (get-buffer fld)))
+	(message "No physical folder")
+      (mew-summary-visit-folder fld nil 'no-ls)
+      (if msg
+	  (mew-summary-move-and-display msg)
+	(goto-char (point-max)))
+      (message "Now in %s. Type '%s' again" fld key))))
+
+(defun mew-summary-exec-region (beg end &optional func)
   (let ((folder (mew-sinfo-get-folder)))
     (cond
      ((mew-folder-nntpp folder)
       (message "This command cannot be used in this folder"))
      ((mew-folder-remotep folder)
       (message "Refiling and deleting...")
-      (sit-for 0)
+      (force-mode-line-update)
       (mew-summary-exec-remote beg end))
      ((mew-virtual-p)
-      (if (not (mew-thread-p))
-	  (message "Move to a physical folder first")
-	(let* ((bnm (mew-summary-folder-name 'ext))
-	       (fld (mew-thread-to-folder bnm))
-	       (msg (mew-summary-message-number)))
-	  (if (not (and fld (get-buffer fld)))
-	      (message "No physical folder")
-	    (mew-summary-visit-folder fld nil 'no-ls)
-	    (if msg
-		(mew-summary-jump-message msg)
-	      (goto-char (point-max)))
-	    (message "Now in %s. Type '%s' again"
-		     fld
-		     (substitute-command-keys
-		      "\\<mew-summary-mode-map>\\[mew-summary-exec]"))))))
+      (mew-summary-go-back-summary
+       (mew-substitute-for-summary "\\[mew-summary-exec]")))
      (t
       (message "Refiling and deleting...")
-      (sit-for 0)
-      (mew-summary-exec-local beg end)))))
+      (force-mode-line-update)
+      (mew-summary-exec-local beg end func)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Refile log
+;;;
+
+(defun mew-exec-refile-log (buf)
+  (when (get-buffer buf)
+    (save-excursion
+      (set-buffer buf)
+      (mew-frwlet
+       mew-cs-dummy mew-cs-m17n
+       (write-region (point-min) (point-max)
+		     (expand-file-name mew-refile-log-file mew-conf-path)
+		     'append 'no-msg)))
+    (mew-remove-buffer buf)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -303,16 +339,17 @@ all marked messages before the current message."
     (forward-line)
     (mew-elet (delete-region start (point)))))
 
-(defun mew-summary-exec-local (beg end)
+(defun mew-summary-exec-local (beg end &optional func-decide)
   (when (mew-summary-exclusive-p)
     (save-excursion
       (condition-case nil
-	  (let ((marks (mew-mark-get-all-marks))
-		(src (mew-summary-folder-name 'ext))
-		(m (make-marker))
-		(cnt 0)
-		(case-fold-search nil)
-		msg msgs err-folders mark func-exec func-sanity killp regex)
+	  (let* ((marks (mew-mark-get-all-marks))
+		 (src (mew-summary-folder-name 'ext))
+		 (lbuf (mew-refile-log-buffer src))
+		 (m (make-marker))
+		 (cnt 0)
+		 (case-fold-search nil)
+		 msg msgs err-folders mark func-exec func-sanity killp regex)
 	    (set-marker m end)
 	    (while marks
 	      (setq mark (car marks))
@@ -325,22 +362,21 @@ all marked messages before the current message."
 		(goto-char beg)
 		(while (re-search-forward regex m t)
 		  (setq msg (mew-summary-message-number))
-		  (setq msgs (cons msg msgs))
-		  (cond
-		   ((fboundp killp)
-		    (if (funcall killp src msg)
-			(mew-mark-kill-line)
-		      (mew-mark-delete-here t)))
-		   ((eq killp t)
-		    (mew-mark-kill-line))))
+		  (when (or (not func-decide) (funcall func-decide msg))
+		    (setq msgs (cons msg msgs))
+		    (cond
+		     ((fboundp killp)
+		      (if (funcall killp src msg)
+			  (mew-mark-kill-line)
+			(mew-mark-remove)))
+		     ((eq killp t)
+		      (mew-mark-kill-line)))))
 		(setq msgs (nreverse msgs))
 		(when msgs
 		  (when (= cnt 0)
 		    ;; opening...
 		    (mew-summary-lock t "Executing")
-		    (mew-window-configure 'summary)
-		    (mew-current-set nil nil nil)
-		    (mew-decode-syntax-delete))
+		    (mew-summary-reset))
 		  (setq cnt (1+ cnt))
 		  ;; refiling and deleting...
 		  (setq func-sanity (mew-markdb-func-sanity mark))
@@ -355,7 +391,6 @@ all marked messages before the current message."
 		(message "No marks")
 	      ;; ending...
 	      (mew-cache-clean-up)
-	      (mew-summary-reset-mode-line)
 	      (mew-summary-folder-cache-save)
 	      (set-buffer-modified-p nil)
 	      (mew-summary-unlock)
@@ -364,6 +399,7 @@ all marked messages before the current message."
 		;; This ensures that some messages are displayed in Summary.
 		(recenter))
 	      (run-hooks 'mew-summary-exec-hook)
+	      (mew-exec-refile-log lbuf)
 	      (message "Refiling and deleting...done")))
 	(quit
 	 (set-buffer-modified-p nil)
@@ -382,63 +418,98 @@ all marked messages before the current message."
     (mew-elet (put-text-property start (point) 'invisible t))))
 
 (defun mew-summary-exec-remote-get-info (beg end modify)
+  ;; for POP or IMAP
+  ;; If modify, marked lines get invisible. They are removed
+  ;; IMAP's sentinel.
+  ;; Otherwise, marked lines retain and they are to be processed
+  ;; by mew-summary-exec-local().
   (let* ((case (mew-sinfo-get-case))
 	 (folder (mew-sinfo-get-folder))
-	 (trash (mew-imap-trash-folder case))
 	 (m (make-marker))
-	 siz msg uid ref refs rmvs flds del regex kils
-	 refinfo refile-back)
-    (if (and trash (string= trash folder)) (setq trash nil))
+	 trash trash-list lomov
+	 siz msg uid ref flds del regex trss rmvs kils refs movs
+	 refinfo refile-back mark rttl dttl)
+    (when (mew-folder-imapp folder)
+      ;; see also mew-mark-exec-delete.
+      (setq trash (mew-imap-trash-folder case))
+      (setq trash-list (mew-imap-trash-folder-list case))
+      (setq lomov t)
+      (cond
+       ((null trash)
+	)
+       (trash-list
+	(if (or (member folder trash-list)
+		(string= folder trash))
+	    (setq trash nil)))
+       (t
+	(if (string= folder trash)
+	    (setq trash nil)))))
     (save-excursion
       (set-marker m end)
       (mew-decode-syntax-delete)
       (setq regex (mew-mark-regex mew-mark-refile))
       (goto-char beg)
       (while (re-search-forward regex m t)
-	(setq msg (mew-summary-message-number))
-	(setq uid (mew-summary-message-uid))
-	(setq siz (mew-summary-message-size))
+	(re-search-forward mew-regex-sumsyn-long)
+	(setq msg (mew-sumsyn-message-number))
+	(setq uid (mew-sumsyn-message-uid))
+	(setq siz (mew-sumsyn-message-size))
 	(setq refinfo (mew-refile-get msg))
 	(setq refile-back (cons refinfo refile-back))
 	(setq flds (cdr refinfo))
-	(if modify (mew-refile-reset msg)) ;; reset before delete just in case
+	;;
 	(setq del nil)
 	(if (member folder flds)
 	    (setq flds (delete folder flds)) ;; destructive
 	  (setq del t)
-	  (if modify (setq kils (cons msg kils))))
+	  (when modify
+	    (if lomov
+		(setq movs (cons msg movs))
+	      (setq kils (cons msg kils)))))
 	(when flds
 	  (setq ref (cons uid (cons siz (cons del flds))))
 	  (setq refs (cons ref refs)))
 	(if modify
 	    (if del
 		(mew-mark-invisible-line)
-	      (mew-mark-delete-here t))))
+	      (mew-summary-refile-remove-body)
+	      (mew-mark-remove))))
       (setq refs (nreverse refs))
+      (setq rttl (length refs))
+      (setq movs (nreverse movs))
       (if modify (mew-sinfo-set-refile-back refile-back))
+      (when lomov
+	(let ((mew-inherit-exec-case case))
+	  (mew-mark-sanity-refile movs)))
       ;;
       (setq regex (mew-mark-list-regex (list mew-mark-delete mew-mark-unlink)))
       (goto-char beg)
+      (setq dttl 0)
       (while (re-search-forward regex m t)
-	(setq msg (mew-summary-message-number))
-	(setq uid (mew-summary-message-uid))
-	(setq siz (mew-summary-message-size))
-	(if trash
-	    (setq rmvs (cons (list uid siz t trash) rmvs))
+	(setq mark (match-string 0))
+	(re-search-forward mew-regex-sumsyn-long)
+	(setq msg (mew-sumsyn-message-number))
+	(setq uid (mew-sumsyn-message-uid))
+	(setq siz (mew-sumsyn-message-size))
+	(if (and trash (char-equal (string-to-char mark) mew-mark-delete))
+	    (setq trss (cons (list uid siz t trash) trss))
 	  (setq rmvs (cons uid rmvs)))
 	(when modify
 	  (setq kils (cons msg kils))
-	  (mew-mark-invisible-line)))
+	  (mew-mark-invisible-line))
+	(setq dttl (1+ dttl)))
       (setq rmvs (nreverse rmvs))
-      (when trash
-	(setq refs (nconc rmvs refs))
-	(setq rmvs nil))
+      (setq trss (nreverse trss))
+      (when trss
+	(setq refs (nconc trss refs)))
       (set-buffer-modified-p nil)
-      (if modify
-	  (progn
-	    (sit-for 0)
-	    (list refs rmvs kils))
-	(list refs rmvs)))))
+      (cond
+       ((and (null rmvs) (null refs)) nil)
+       (modify
+	(force-mode-line-update) ;; xxx
+	(list rmvs kils refs movs rttl dttl))
+       (t
+	(list rmvs nil refs nil 0 0))))))
 
 (defun mew-summary-exec-remote (beg end)
   (if (mew-folder-nntpp (mew-sinfo-get-folder))
@@ -447,17 +518,17 @@ all marked messages before the current message."
       (let* ((case (mew-sinfo-get-case))
 	     (folder (mew-sinfo-get-folder))
 	     (bnm (mew-summary-folder-name 'ext))
-	     (info (mew-summary-exec-remote-get-info beg end 'modify))
-	     (refs (nth 0 info))
-	     (rmvs (nth 1 info))
-	     (kils (nth 2 info)))
-	(if (or refs rmvs)
-	    (cond
-	     ((mew-folder-popp folder)
-	      (mew-pop-retrieve case 'exec bnm refs rmvs kils))
-	     ((mew-folder-imapp folder)
-	      (mew-imap-retrieve case 'exec bnm refs rmvs kils)))
-	  (message "No messages to be processed"))))))
+	     (lbuf (mew-refile-log-buffer bnm))
+	     (info (mew-summary-exec-remote-get-info beg end 'modify)))
+	(if (not info)
+	    (message "No messages to be processed")
+	  (mew-summary-reset)
+	  (cond
+	   ((mew-folder-popp folder)
+	    (mew-pop-retrieve case 'exec bnm info))
+	   ((mew-folder-imapp folder)
+	    (mew-imap-retrieve case 'exec bnm info)))
+	  (mew-exec-refile-log lbuf))))))
 
 (defun mew-mark-kill-invisible ()
   (save-excursion
@@ -475,15 +546,20 @@ all marked messages before the current message."
 (defun mew-summary-exec-offline (&optional arg)
   "\\<mew-summary-mode-map>Process marked messages *offline*. Messages to be refiled in 
 a remote folder are moved to the corresponding folder but they are
-marked invalid. A number of an invalid message starts with '0'.
-Invalid messages will be overridden when the remote folder
-is scanned online. 
+marked invalid. Invalid message are marked with '#'.
+Invalid messages will be overridden when the remote folder is scanned online. 
 A job to delete or refile messages, which is created by this command, 
 is queued in a queue folder (%queue for IMAP). To flush jobs in 
 the queue, type '\\[mew-summary-send-message]' in the queue online."
   (interactive "P")
-  (if (not (mew-folder-imapp (mew-sinfo-get-folder)))
-      (message "This command cannot be used in this folder")
+  (cond
+   ((mew-virtual-p)
+    (mew-summary-go-back-summary
+     (mew-substitute-for-summary "\\[mew-summary-exec-offline]")))
+   ((not (mew-folder-imapp (mew-sinfo-get-folder)))
+    (message "This command cannot be used in this folder"))
+   (t
+    (if (mew-mark-active-p) (setq arg t))
     (let (beg end region)
       (cond
        (arg
@@ -494,64 +570,58 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
 	(setq beg (point-min))
 	(setq end (point-max))))
       (message "Refiling and deleting...")
-      (sit-for 0)
-      (mew-summary-exec-offline-region beg end))))
+      (force-mode-line-update)
+      (mew-summary-exec-offline-region beg end)))))
 
 (defun mew-summary-exec-offline-region (beg end)
   "Process the current marked message offline."
   (interactive)
   (let* ((info (mew-summary-exec-remote-get-info beg end nil))
-	 (refs (nth 0 info))
-	 (rmvs (nth 1 info)))
-    (if (or refs rmvs)
-	(let* ((fld (mew-sinfo-get-folder))
-	       (case (mew-sinfo-get-case))
-	       (imapq (mew-folder-case-folder
-		       case (mew-imap-queue-folder case)))
-	       (imapq-dir (mew-expand-folder imapq))
-	       file file-info ref)
-	  (unless (file-directory-p imapq-dir) (mew-make-directory imapq-dir))
-	  (setq file (mew-folder-new-message imapq))
-	  (setq file-info (concat file mew-imapq-info-suffix))
-	  (mew-lisp-save file-info (cons fld info) 'nobackup 'unlimit)
-	  (with-temp-buffer
-	    (mew-header-insert mew-subj: (format "IMAP jobs for %s" fld))
-	    (mew-header-insert mew-date: (mew-time-ctz-to-rfc (current-time)))
-	    (mew-header-insert mew-from: "Mew IMAP manager")
-	    (insert "\n")
-	    (insert "Messages to be refiled:\n")
-	    (while refs
-	      (setq ref (car refs))
-	      (setq refs (cdr refs))
-	      (if (nth 2 ref)
-		  (insert (format "\t%s will be copied to %s\n"
-				  (nth 0 ref)
-				  (mew-join "," (nthcdr 3 ref))))
-		(insert (format "\t%s will be moved to %s\n"
+	 (rmvs (nth 0 info))
+	 (refs (nth 2 info)))
+    (if (not info)
+	(message "No messages to be processed")
+      (let* ((fld (mew-sinfo-get-folder))
+	     (case (mew-sinfo-get-case))
+	     (imapq (mew-case-folder case (mew-imap-queue-folder case)))
+	     (imapq-dir (mew-expand-folder imapq))
+	     file file-info ref)
+	(unless (file-directory-p imapq-dir) (mew-make-directory imapq-dir))
+	(setq file (mew-folder-new-message imapq))
+	(setq file-info (concat file mew-imapq-info-suffix))
+	(mew-lisp-save file-info (cons fld info) 'nobackup 'unlimit)
+	(with-temp-buffer
+	  (mew-header-insert mew-subj: (format "IMAP jobs for %s" fld))
+	  (mew-header-insert mew-date: (mew-time-ctz-to-rfc (current-time)))
+	  (mew-header-insert mew-from: "Mew IMAP manager")
+	  (insert "\n")
+	  (insert "Messages to be refiled:\n")
+	  (while refs
+	    (setq ref (car refs))
+	    (setq refs (cdr refs))
+	    (if (nth 2 ref)
+		(insert (format "\t%s will be copied to %s\n"
 				(nth 0 ref)
-				(mew-join "," (nthcdr 3 ref))))))
-	    (insert "\n")
-	    (when rmvs
-	      (insert "Messages to be deleted:\n")
-	      (insert "\t" (mew-join "," rmvs) "\n"))
-	    (set-buffer-modified-p nil)
-	    (mew-frwlet
-	     mew-cs-dummy nil
-	     (write-region (point-min) (point-max) file nil 'no-msg)))
-	  (mew-touch-folder imapq)
-	  (if (mew-imap-trash-folder case)
-	      (let* ((mew-inherit-exec-case case)
-		     (mew-msg-rm-policy 'trashonly)
-		     (mew-trash-folder (mew-imap-trash-folder case))
-		     (mew-inherit-offline t))
-		(mew-summary-exec-local beg end))
-	    (let ((mew-inherit-exec-case case)
-		  (mew-msg-rm-policy 'always)
-		  (mew-inherit-offline t))
-	      (mew-summary-exec-local beg end))))
-      (message "No messages to be processed"))))
+				(mew-join "," (nthcdr 3 ref))))
+	      (insert (format "\t%s will be moved to %s\n"
+			      (nth 0 ref)
+			      (mew-join "," (nthcdr 3 ref))))))
+	  (insert "\n")
+	  (when rmvs
+	    (insert "Messages to be deleted:\n")
+	    (insert "\t" (mew-join "," rmvs) "\n"))
+	  (mew-frwlet
+	   mew-cs-dummy nil
+	   (write-region (point-min) (point-max) file nil 'no-msg)))
+	(mew-touch-folder imapq)
+	;; refile log is not necessary since mew-summary-exec-local does it.
+	(let ((mew-inherit-exec-case case)
+	      (mew-inherit-offline t)
+	      (mew-trash-folder (mew-imap-trash-folder case)))
+	  (mew-summary-exec-local beg end))))))
 
 (defun mew-summary-exec-offline-one (&optional arg)
+  "Process the message on the cursor offline."
   (interactive "P")
   (if (not (mew-folder-imapp (mew-sinfo-get-folder)))
       (message "This command cannot be used in this folder")
@@ -566,7 +636,7 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
           (setq end (1+ (point))))
         (if (> end (point-max))
             (message "No message")
-          (sit-for 0)
+	  (force-mode-line-update) ;; xxx
           (mew-summary-exec-offline-region beg end))))))
 
 (provide 'mew-exec)
