@@ -24,6 +24,9 @@
 (defsubst mew-draft-p ()
   (eq major-mode 'mew-draft-mode))
 
+(defsubst mew-header-p ()
+  (eq major-mode 'mew-header-mode))
+
 (defsubst mew-draft-or-header-p ()
   (memq major-mode '(mew-draft-mode mew-header-mode)))
 
@@ -33,8 +36,9 @@
 (defsubst mew-summary-or-virtual-p ()
   (or (mew-summary-p) (mew-virtual-p)))
 
-(defsubst mew-summary-or-thread-p ()
-  (or (mew-summary-p) (mew-thread-p)))
+(defsubst mew-pickable-p ()
+  (or (mew-summary-p)
+      (and (mew-virtual-p) (= (length (mew-vinfo-get-flds)) 1))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -310,6 +314,12 @@ in the context of FUNC."
     (setq string (replace-match " " nil t string)))
   string)
 
+(defun mew-replace-white-space2 (string)
+  "Replace white characters to under score."
+  (while (string-match "[\n\t\r ]+" string)
+    (setq string (replace-match "_" nil t string)))
+  string)
+
 (defsubst mew-capitalize (ostr)
   "Syntax table independent version of capitalize.
 Words are separated by '/' and '-'."
@@ -416,21 +426,21 @@ also ignored."
     (catch 'loop
       (while (< i j)
 	(setq c (aref str i))
-	(if (or (char-equal c 32) (char-equal c ?\t))
+	(if (or (char-equal c mew-sp) (char-equal c ?\t))
 	    (setq i (1+ i))
 	  (throw 'loop nil))))
     (setq j (1- j))
     (catch 'loop
       (while (< i j)
 	(setq c (aref str j))
-	(if (or (char-equal c 32) (char-equal c ?\t))
+	(if (or (char-equal c mew-sp) (char-equal c ?\t))
 	    (setq j (1- j))
 	  (throw 'loop nil))))
     (substring str i (1+ j))))
 
 (defun mew-quote-string (str qchar targets)
   (let* ((len (length str))
-	(ret (make-string (* len 2) 32))
+	(ret (make-string (* len 2) mew-sp))
 	(i 0) (j 0) c)
     (while (< i len)
       (setq c (aref str i))
@@ -451,18 +461,22 @@ also ignored."
 
 ;;
 
-(defun mew-folder-case (folder)
+(defun mew-case:folder-case (case:folder)
+  "Extract case from case:folder. If case does not exist, nil is returned."
   ;; "case" must be distinguished from drive
-  (if (string-match mew-regex-case folder)
-      (cons (mew-match-string 1 folder) (mew-match-string 2 folder))))
+  (if (string-match mew-regex-case1 case:folder)
+      (mew-match-string 1 case:folder)))
 
-(defun mew-folder-folder (folder)
-  (let ((case-folder (mew-folder-case folder)))
-    (if case-folder
-	(cdr case-folder)
-      folder)))
+(defun mew-case:folder-folder (case:folder)
+  "Extract folder from case:folder."
+  ;; "case" must be distinguished from drive
+  (if (string-match mew-regex-case1 case:folder)
+      (mew-match-string 3 case:folder)
+    case:folder))
 
-(defun mew-folder-case-folder (case folder)
+(defun mew-case-folder (case folder)
+  "Concat case and folder to produce case:folder.
+If case is \"default\", it is not prepended."
   (if (and case
 	   (not (string= case mew-case-default))
 	   (mew-folder-remotep folder))
@@ -518,8 +532,14 @@ also ignored."
 (defsubst mew-folder-to-thread (folder)
   (concat mew-folder-virtual folder))
 
-(defsubst mew-thread-to-folder (folder)
-  (mew-folder-string folder))
+(defsubst mew-physical-folder (folder)
+  ;; Upper functions may use string-match.
+  ;; So, string-match must not be used here.
+  (if (eq (aref folder 0) ?*)
+      (setq folder (substring folder 1)))
+  (if (eq (aref folder (1- (length folder))) ?*)
+      (setq folder (substring folder 0 -1)))
+  folder)
 
 ;;
 
@@ -551,10 +571,15 @@ also ignored."
 ;;
 
 (defun mew-canonicalize-folder (folder)
-  (if (or (mew-folder-localp folder)
-	  (file-name-absolute-p folder))
-      folder
-    (mew-string-to-local folder)))
+  (cond
+   ((mew-folder-localp folder)
+    folder)
+   ((mew-folder-imapp folder)
+    folder)
+   ((file-name-absolute-p folder)
+    folder)
+   (t
+    (mew-string-to-local folder))))
 
 (defun mew-path-to-folder (path)
   (let ((regex (concat "^" (regexp-quote (file-name-as-directory (expand-file-name mew-mail-path))))))
@@ -563,45 +588,40 @@ also ignored."
       path)))
 
 (defun mew-canonicalize-case-folder (case:folder)
-  (let* ((case-folder (mew-folder-case case:folder))
-	 (case (car case-folder))
-	 (folder (cdr case-folder)))
-    (unless folder (setq folder case:folder))
+  (let ((case (mew-case:folder-case case:folder))
+	(folder (mew-case:folder-folder case:folder))
+	len)
     (cond
-     ((mew-folder-localp folder) folder)
-     ((mew-folder-absolutep folder) folder)
+     ((mew-folder-localp folder)   (directory-file-name folder))
+     ((mew-folder-absolutep folder)(directory-file-name folder))
      ((mew-folder-virtualp folder) folder)
-     (case
-      (if (string-equal case mew-case-default)
-	  folder
-	case:folder))
-     (t case:folder))))
-
-(defun mew-folder-remote-to-local (folder)
-  (let ((case-folder (mew-folder-case folder))
-	case fld)
-    (when case-folder
-      (setq case (car case-folder))
-      (setq folder (cdr case-folder)))
-    (setq fld (mew-folder-string folder))
-    (cond
-     ((mew-folder-localp folder) folder)
-     ((mew-folder-popp folder)
-      (mew-concat-folder (mew-pop-folder case) fld))
-     ((mew-folder-imapp folder)
-      (mew-concat-folder (mew-imap-folder case) fld))
-     ((mew-folder-nntpp folder)
-      (mew-concat-folder (mew-nntp-folder case) fld)))))
+     (t
+      (if (null case)
+	  (setq folder case:folder)
+	(if (string-equal case mew-case-default)
+	    (progn
+	      (setq case nil)
+	      (setq case:folder folder))))
+      (cond
+       ((mew-folder-popp folder) case:folder)
+       ((mew-folder-imapp folder)
+	(mew-imap-directory-file-name
+	 case:folder (or case ;; visit
+			 mew-inherit-case))) ;; refile
+       ((mew-folder-nntpp folder)
+	(setq len (1- (length case:folder)))
+	(if (char-equal (aref case:folder len) ?.)
+	    (substring case:folder 0 len)
+	  case:folder)))))))
 
 ;;
 
-(defun mew-expand-folder2 (folder)
-  (let ((case-folder (mew-folder-case folder))
-	case subdir)
+(defun mew-expand-folder2 (case:folder)
+  "Expanding FOLDER to a relative path to '+'"
+  (let ((case (mew-case:folder-case case:folder))
+	(folder (mew-case:folder-folder case:folder))
+	subdir)
     ;; The length of "case" must be longer than or equal to 2.
-    (when case-folder
-      (setq case (car case-folder))
-      (setq folder (cdr case-folder)))
     (setq subdir (mew-folder-string folder))
     (cond
      ((mew-folder-popp folder)
@@ -609,29 +629,30 @@ also ignored."
      ((mew-folder-nntpp folder)
       (mew-concat-folder (mew-nntp-folder case) subdir))
      ((mew-folder-imapp folder)
-      (mew-concat-folder (mew-imap-folder case) subdir case))
+      (mew-concat-folder (mew-imap-folder case)
+			 (mew-imap-expand-folder case subdir)))
      (t folder))))
 
 (defun mew-expand-folder (folder &optional message)
+  "Expanding FOLDER to its absolute path"
   (when (stringp folder)
     (let (dir)
-    (setq folder (mew-expand-folder2 folder))
-    (if (mew-folder-localp folder)
-	(setq dir (expand-file-name (mew-folder-string folder) mew-mail-path))
-      ;; absolute path
-      ;; "C:/" -> t, "C:" -> nil , "CC:/" -> nil
-      (setq dir (expand-file-name folder)))
-    (if message
-	(expand-file-name message dir)
-      dir))))
+      (setq folder (mew-expand-folder2 folder))
+      (if (mew-folder-localp folder)
+	  (setq dir (expand-file-name (mew-folder-string folder) mew-mail-path))
+	;; absolute path
+	;; "C:/" -> t, "C:" -> nil , "CC:/" -> nil
+	(setq dir (expand-file-name folder)))
+      (if message
+	  (expand-file-name message dir)
+	dir))))
 
-(defun mew-concat-folder (folder subfolder &optional case)
+(defun mew-concat-folder (folder subfolder)
+  (concat (file-name-as-directory folder) subfolder))
+
+(defun mew-concat-folder2 (folder subfolder case)
   (if (mew-folder-imapp folder)
-      (let* ((sep (mew-imap-separator case))
-	     (regex (concat (regexp-quote sep) "$")))
-	(if (string-match regex folder)
-	    (concat folder subfolder)
-	  (concat folder sep subfolder)))
+      (concat (mew-imap-file-name-as-directory folder case) subfolder)
     (concat (file-name-as-directory folder) subfolder)))
 
 (defsubst mew-dir-messages (dir &optional regex)
@@ -654,6 +675,14 @@ also ignored."
     (mew-alet
      (directory-files edir nil regex 'no-sort))))
 
+(defun mew-folder-messages (folder)
+  (let* ((dir (mew-expand-folder folder))
+	 msgs nums)
+    (when (file-directory-p dir)
+      (setq msgs (mew-dir-messages dir))
+      (setq nums (sort (mapcar 'string-to-int msgs) '<))
+      (mapcar 'int-to-string nums))))
+
 (defun mew-folder-new-message (folder &optional num-only cache)
   (let* ((dir (mew-expand-folder folder))
 	 (regex (if cache mew-regex-message-files3 mew-regex-message-files))
@@ -698,13 +727,48 @@ also ignored."
 	maxpath))))
 
 (defun mew-touch-folder (fld)
-  (let (file)
-    (when (and mew-touch-folder-p
-	       (stringp mew-summary-touch-file)
-	       (setq file (mew-expand-folder fld mew-summary-touch-file))
-	       (file-writable-p file))
-      (write-region "touched by Mew." nil file nil 'no-msg)
-      (mew-set-file-modes file))))
+  (when (stringp mew-summary-touch-file)
+    (let ((file (mew-expand-folder fld mew-summary-touch-file)))
+      (when (file-writable-p file)
+	(write-region "touched by Mew." nil file nil 'no-msg)
+	(mew-set-file-modes file)))))
+
+;; this kind of defmacro can't recurse
+;; if +foo exists and we insert +foo/bar,
+;; +foo/ should be inserted, too. But +foo/ does not match to +foo...
+(defmacro mew-folder-insert (folder lst subdir)
+  `(unless (mew-assoc-equal ,folder ,lst 0)
+     (let ((case-fold-search nil)
+	   (max (1- (length ,lst)))
+	   (pair (mew-folder-func ,folder ,subdir))
+	   (min 0)
+	   mid crr prv)
+       (while (> (- max min) 20) ;; 20 is enough?
+	 (setq mid (/ (+ min max) 2))
+	 (if (string< (car (nth mid ,lst)) ,folder)
+	     (setq min mid)
+	   (setq max mid)))
+       (setq crr (nthcdr min ,lst))
+       (while (and crr (string< (car (car crr)) ,folder))
+	 (setq prv crr)
+	 (setq crr (cdr crr)))
+       (if prv
+	   (setcdr prv (cons pair crr))
+	 (setq ,lst (cons pair crr))))))
+
+(defmacro mew-folder-delete (folder lst)
+  `(setq ,lst (delete (assoc ,folder ,lst) ,lst)))
+
+(defun mew-folder-node-p (folder &optional case)
+  (cond
+   ((mew-folder-localp folder)
+    (let* ((dir (file-name-as-directory folder))
+	   (regex (concat "^" (regexp-quote dir))))
+      (mew-assoc-match regex (mew-local-folder-alist) 0)))
+   ((mew-folder-imapp folder)
+    (let* ((dir (mew-imap-file-name-as-directory folder case))
+	   (regex (concat "^" (regexp-quote dir))))
+      (mew-assoc-match regex (mew-imap-folder-alist case) 0)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -723,7 +787,7 @@ also ignored."
   (let ((str-func (mew-folder-spec-func str-type))
 	(lst-func (mew-folder-spec-func lst-type))
 	key keys values ret)
-    (setq folder (mew-folder-folder folder))
+    (setq folder (mew-case:folder-folder folder))
     (catch 'loop
       (while lst
 	(setq keys (car (car lst)))
@@ -879,9 +943,14 @@ Does not examine containing directories for links."
 (defun mew-insert-file-contents (&rest args)
   "A safe version of insert-file-contents.
 This checks -*-coding:ctext;-*- internally when including a file."
-  (let ((after-insert-file-functions nil)
-	(auto-image-file-mode nil))
+  (let ((after-insert-file-functions nil))
     ;; preventing after-insert-file-set-buffer-file-coding-system
+    (apply 'mew-insert-file-contents2 args)))
+
+(defun mew-insert-file-contents2 (&rest args)
+  "Mew version of insert-file-contents.
+This sets  buffer-file-coding-system."
+  (let ((auto-image-file-mode nil))
     (apply 'insert-file-contents args)))
 
 (defun mew-find-file-noselect (&rest args)
@@ -895,11 +964,12 @@ But this does not set buffer-file-coding-system."
 (defun mew-find-file-noselect2 (&rest args)
   "A safe version of find-file-noselect.
 This checks -*-coding:ctext;-*- internally when including a file
-and set buffer-file-coding-system."
+and sets buffer-file-coding-system."
   (let ((auto-image-file-mode nil)
 	(format-alist nil)
 	(auto-mode-alist nil)
         (enable-local-variables nil)
+	(find-file-hook nil)
         (find-file-hooks nil))
     (apply 'find-file-noselect args)))
 
@@ -986,15 +1056,22 @@ and set buffer-file-coding-system."
    (buffer-disable-undo)))
 
 (defun mew-remove-buffer (buf)
-  (if (get-buffer buf) (kill-buffer buf)))
+  (if (and buf (get-buffer buf)) (kill-buffer buf)))
 
 (defmacro mew-elet (&rest body)
   `(let ((buffer-read-only nil)
 	 (inhibit-read-only t)
 	 (after-change-functions nil)
+	 (mark-active nil) ;; for Emacs
 	 (zmacs-regions nil)) ;; for XEmacs
      ,@body))
 
+(defun mew-push-mark ()
+  ;; for C-x C-x
+  (let ((mark-active nil) ;; for Emacs
+	(zmacs-regions nil)) ;; for XEmacs
+    (push-mark (point) t t)))
+
 (defsubst mew-region-bytes (beg end buf)
   ;; string-bytes() acts differently on each Emacs.
   ;; set-buffer-multibyte is also buggy.
@@ -1005,20 +1082,39 @@ and set buffer-file-coding-system."
 	(length (string-as-unibyte (mew-buffer-substring beg end)))
       (- end beg))))
 
-(defun mew-count-lines (beg end &optional skip-regex)
+(defun mew-count-lines (beg end)
   "Return number of lines between BEG and END."
   (save-excursion
+    (goto-char beg)
+    (beginning-of-line)
+    (setq beg (point))
+    (goto-char end)
+    (beginning-of-line)
+    (setq end (point))
+    (if (or (not (mew-decode-syntax-p))
+	    (not (equal (mew-decode-syntax-buffer) (current-buffer))))
+	(mew-count-lines1 beg end)
+      (let ((mbeg (mew-decode-syntax-begin))
+	    (mend (mew-decode-syntax-end)))
+	(if (or (<= end mbeg) (>= beg mend))
+	    (mew-count-lines1 beg end)
+	  (+ (mew-count-lines1 beg mbeg) (mew-count-lines1 mend end)))))))
+
+(defun mew-count-lines1 (beg end)
+  (if (>= beg end) 
+      0
     (save-restriction
       (narrow-to-region beg end)
       (goto-char (point-min))
-      (if skip-regex
-	  (let ((lines 0))
-	    (while (not (eobp))
-	      (forward-line)
-	      (unless (looking-at skip-regex)
-		(setq lines (1+ lines))))
-	    lines)
-	(- (buffer-size) (forward-line (buffer-size)))))))
+      (if (not (and (mew-thread-p) mew-use-thread-separator))
+	  (- (buffer-size) (forward-line (buffer-size)))
+	(let ((regex mew-regex-thread-separator)
+	      (lines 0))
+	  (while (not (eobp))
+	    (unless (looking-at regex)
+	      (setq lines (1+ lines)))
+	    (forward-line))
+	  lines)))))
 
 (defun mew-buffer-list (regex &optional listp mode)
   (let ((bufs (mapcar 'buffer-name (buffer-list)))
@@ -1232,8 +1328,9 @@ by side-effect."
 		(rename-file fullname backname 'override)))
 	  (set-buffer tmp-buf)
 	  (mew-erase-buffer)
-	  (if (and (not unlimit) (> (length lisp) mew-lisp-max-length))
-	      (setcdr (nthcdr (1- mew-lisp-max-length) lisp) nil))
+	  (when (and (not unlimit) (> (length lisp) mew-lisp-max-length))
+	    (setq lisp (copy-sequence lisp)) ;; no side effect
+	    (setcdr (nthcdr (1- mew-lisp-max-length) lisp) nil))
 	  (if (> (length lisp) mew-lisp-max-length)
 	      (print lisp tmp-buf)
 	    (pp lisp tmp-buf))
@@ -1329,7 +1426,7 @@ by side-effect."
   (format "%04d%02d%02d%02d%02d%02d" year mon day hour min sec))
 
 ;; "20000726121835"
-(defsubst mew-time-rfc-to-sortkey (s)
+(defsubst mew-time-rfc-to-sortkey (s &optional tzadj)
   (if (string-match mew-time-rfc-regex s)
       (let ((year (mew-time-rfc-year))
 	    (mon  (mew-time-mon-str-to-int (mew-time-rfc-mon)))
@@ -1347,6 +1444,7 @@ by side-effect."
 	    ;; invalid data
 	    (mew-time-ctz-to-sortkey-invalid sec min hour day mon year)
 	  (setq sec (- sec tmzn))
+	  (if tzadj (setq sec (+ sec (car (current-time-zone)))))
 	  (mew-time-ctz-to-sortkey (encode-time sec min hour day mon year))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1471,6 +1569,15 @@ by side-effect."
 (defsubst mew-timing ()
   (sit-for 0.01))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defun mew-set (vars vals)
+  (while vars
+    (when (car vars)
+      (set (car vars) (car vals)))
+    (setq vars (cdr vars))
+    (setq vals (cdr vals))))
+
 (provide 'mew-func)
 
 ;;; Copyright Notice:
