@@ -51,6 +51,9 @@
 ;;; Toolbar functions
 ;;;
 
+(defvar mew-summary-toolbar nil)
+(defvar mew-draft-toolbar nil)
+
 (defun mew-toolbar-make (alist map)
   (let ((tool-bar-map (make-sparse-keymap)) ;; for tool-bar-add-item-from-menu
 	(data-directory mew-icon-directory)
@@ -92,13 +95,34 @@
 ;;;
 
 (defsubst mew-message-set-end-of-message ()
-  (overlay-put (mew-minfo-get-overlay) 'before-string mew-end-of-message-string))
+  (overlay-put (mew-minfo-get-eom) 'before-string mew-end-of-message-string))
 
 (defsubst mew-message-set-end-of-part ()
-  (overlay-put (mew-minfo-get-overlay) 'before-string mew-end-of-part-string))
+  (overlay-put (mew-minfo-get-eom) 'before-string mew-end-of-part-string))
 
 (defsubst mew-message-set-end-of-nil ()
-  (overlay-put (mew-minfo-get-overlay) 'before-string nil))
+  (overlay-put (mew-minfo-get-eom) 'before-string nil))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Veil
+;;;
+
+(defun mew-header-veil-make ()
+  (let ((ov (mew-overlay-make 1 1)))
+    (overlay-put ov 'invisible t)
+    (overlay-put ov 'before-string mew-header-veil-string)
+    (delete-overlay ov) ;; detach from the buffer
+    ov))
+
+(defun mew-toggle-header-veil (ov)
+  (cond
+   ((overlay-get ov 'invisible)
+    (overlay-put ov 'invisible nil)
+    (overlay-put ov 'before-string nil))
+   (t
+    (overlay-put ov 'invisible t)
+    (overlay-put ov 'before-string mew-header-veil-string))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -119,11 +143,19 @@
 			(set-buffer cache)
 			(string-as-unibyte
 			 (mew-buffer-substring begin end)))
-		      nil t))
+		      format t))
      (error ())))
   (goto-char (point-min))
   (message "Loading image...done"))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Transient
+;;;
+
+(defun mew-mark-active-p ()
+  (and transient-mark-mode mark-active))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; X Face
