@@ -29,7 +29,7 @@
   (mew-header-goto-end)
   (forward-line))
 
-(defun mew-header-get-value (field)
+(defun mew-header-get-value (field &optional as-list)
   "currently, when no match, it returns nil."
   ;; maybe called in narrowed region.
   ;; we cannot widen for citation.
@@ -49,10 +49,16 @@
 	    (setq match (mew-buffer-substring start (1- (point))))
 	    (unless (string= "" match)
 	      (if ret
-		  (setq ret (concat ret "," match))
-		(setq ret match)
+		  (if as-list
+		      (setq ret (cons match ret))
+		    (setq ret (concat ret "," match)))
+		(if as-list
+		    (setq ret (list match))
+		  (setq ret match))
 		(if (string= field mew-from:) (throw 'only-one nil))))))))
-    ret))
+    (if as-list
+	(nreverse ret)
+      ret)))
 
 (defalias 'mew-header-existp 'mew-header-get-value)
 
@@ -199,8 +205,7 @@
 	  (mew-header-encode-text value nil (length key)))
 	(while params
 	  (setq par (car params))
-	  (setq parname (nth 0 par))
-	  (setq parval (nth 1 par))
+	  (mew-set '(parname parval) par)
 	  (setq params (cdr params))
 	  (insert ";")
 	  (cond
@@ -262,7 +267,7 @@
 	 (throw 'max nil))
      (when (and (integerp mew-header-max-depth)
 		(>= sep-cnt mew-header-max-depth))
-       (mew-warn "Too many values. Truncate values over mew-header-max-depth.")
+       (mew-warn "Too many values. Truncate values over mew-header-max-depth")
        (throw 'max nil))))
 
 (defun mew-addrstr-parse-syntax-list (str sep addrp &optional depth)
@@ -295,7 +300,7 @@
 		    (while (< i len)
 		      (setq c (aref str i))
 		      (cond
-		       ;; ((or (char-equal c ?\t) (char-equal c 32))
+		       ;; ((or (char-equal c ?\t) (char-equal c mew-sp))
 		       ;; (setq i (1+ i)))
 		       ((char-equal c ?\")
 			(aset tmp tmp-cnt c)
@@ -331,7 +336,7 @@
 		(while (and (< i len) (not (char-equal (aref str i) ?>)))
 		  (setq i (1+ i)))
 		(when (> i rbeg)
-		  (setq rend i) ;; note: to be used for substring, so not 1-.
+		  (setq rend i)	;; note: to be used for substring, so not 1-.
 		  ;; should not be nested but easy to implement...
 		  (setq ret (cons (car (mew-addrstr-parse-syntax-list
 					(substring str rbeg rend) sep t))
@@ -346,7 +351,7 @@
 		(setq i (1+ i))))))
 	   ((char-equal c ?\n))
 	   ((char-equal c ?\t))
-	   ((char-equal c 32))
+	   ((char-equal c mew-sp))
 	   ((char-equal c sep)
 	    (if (> tmp-cnt 0)
 		(setq ret (cons (substring tmp 0 tmp-cnt) ret)))
@@ -355,7 +360,7 @@
 	   (t 
 	    (aset tmp tmp-cnt c)
 	    (setq tmp-cnt (1+ tmp-cnt)))) ;; end of cond
-	  (setq i (1+ i))) ;; end of while
+	  (setq i (1+ i)))		  ;; end of while
 	(if (> tmp-cnt 0)
 	    (setq ret (cons (substring tmp 0 tmp-cnt) ret)))) ;; end of catch
       (if (featurep 'mew-temacs)
@@ -420,7 +425,7 @@ Remove anonymous addresses."
   "Extracts username from ADDR"
   (if (string-match "@.*:" addr) ;; xxx what's this?
       (setq addr (substring addr (match-end 0) nil))
-    (setq addr (mew-replace-character addr 32 ?_))
+    (setq addr (mew-replace-character addr mew-sp ?_))
     (setq addr (substring addr 0 (string-match "%" addr)))
     (setq addr (substring addr 0 (string-match "@" addr)))
     ;; just for refile:  "To: recipients:;" -> recipients
@@ -437,6 +442,44 @@ Remove anonymous addresses."
 	(concat addr "@" (mew-mail-domain))) ;; xxx
     addr))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Functions for ID
+;;;
+
+(defun mew-idstr-get-first-id (idstr)
+  (when (stringp idstr)
+    (if (string-match mew-regex-id idstr)
+	(mew-replace-white-space2 (match-string 0 idstr)))))
+
+(defun mew-idstr-get-last-id (idstr)
+  (when (stringp idstr)
+    (let (beg end start)
+      (while (string-match mew-regex-id idstr start)
+	(setq beg (match-beginning 0))
+	(setq end (match-end 0))
+	(setq start (match-end 0)))
+      (if (and beg end)
+	  (mew-replace-white-space2 (substring idstr beg end))))))
+
+(defun mew-idstr-to-id-list (idstr &optional rev)
+  (when (stringp idstr)
+    (let (ret start)
+      (while (string-match mew-regex-id idstr start)
+	(setq start (match-end 0))
+	(setq ret (cons (mew-replace-white-space2 (match-string 0 idstr)) ret)))
+      (if rev
+	  ret
+	(nreverse ret)))))
+
+(defun mew-id-list-to-idstr (id-list)
+  (let (skip)
+    (if (integerp mew-references-max-count)
+	(setq skip (- (length id-list) mew-references-max-count)))
+    (if (and (numberp skip) (> skip 0))
+	(setq id-list (nthcdr skip id-list)))
+    (mew-join "\n\t" id-list)))
+
 (provide 'mew-header)
 
 ;;; Copyright Notice:
