@@ -48,11 +48,11 @@
 (defun mew-highlight-mark-line (mark)
   (when mew-use-highlight-mark
     (let ((face (mew-highlight-mark-get-face mark)))
-      (when face
-	(put-text-property
-	 (save-excursion (beginning-of-line) (point))
-	 (save-excursion (end-of-line) (point))
-	 'face face)))))
+      (unless face (setq face 'default))
+      (put-text-property
+       (save-excursion (beginning-of-line) (point))
+       (save-excursion (end-of-line) (point))
+       'face face))))
 
 (defun mew-highlight-unmark-line ()
   (remove-text-properties 
@@ -117,9 +117,11 @@
 
 (defun mew-highlight-body-region (BEG END &optional draft rehighlight)
   (when (and (or mew-use-highlight-body mew-use-highlight-url)
-	     (<= BEG mew-highlight-body-max-size))
-    (if (> END mew-highlight-body-max-size)
-	(setq END mew-highlight-body-max-size))
+	     (or (= 0 mew-highlight-body-max-size)
+		 (<= BEG mew-highlight-body-max-size)))
+    (unless (or (= 0 mew-highlight-body-max-size)
+		(<= END mew-highlight-body-max-size))
+      (setq END mew-highlight-body-max-size))
     (let* ((inhibit-point-motion-hooks t)
 	   (cite-regex mew-highlight-body-regex-cite)
 	   (cmt-regex mew-highlight-body-regex-comment)
@@ -238,7 +240,6 @@
 (defun mew-summary-cook-region (beg end &optional interrupt)
   (when (and (mew-summary-or-virtual-p) mew-summary-buffer-raw)
     (let ((inhibit-point-motion-hooks t)
-	  (regex mew-regex-msg-mark)
 	  ret mark face start med)
       (catch 'loop
 	(save-excursion
@@ -272,10 +273,10 @@
 	   (when mew-use-highlight-mark
 	     (goto-char beg)
 	     (while (and (< (point) end) ;; we cannot trust end
-			 (re-search-forward regex end t))
+			 (re-search-forward mew-regex-mark end t))
 	       (if (and interrupt (input-pending-p))
 		   (throw 'loop (setq ret t)))
-	       (setq mark (string-to-char (mew-match-string 2)))
+	       (setq mark (mew-sumsyn-mark))
 	       (setq face (mew-highlight-mark-get-face mark))
 	       (beginning-of-line)
 	       (setq start (point))
