@@ -46,6 +46,7 @@
   (define-key mew-summary-mode-map ":"
     'mew-summary-analyze-again-alternative)
   (define-key mew-summary-mode-map ","    'mew-summary-display-asis)
+  (define-key mew-summary-mode-map ";"    'mew-summary-trace-path)
   (define-key mew-summary-mode-map "<"    'mew-summary-jump-top)
   (define-key mew-summary-mode-map ">"    'mew-summary-jump-bottom)
   (define-key mew-summary-mode-map "\177" 'mew-summary-prev-page)
@@ -54,7 +55,7 @@
   (define-key mew-summary-mode-map "-"    'mew-summary-scroll-down)
   (define-key mew-summary-mode-map "\e\r" 'mew-summary-scroll-down)
   (define-key mew-summary-mode-map "g"    'mew-summary-goto-folder)
-  (define-key mew-summary-mode-map "j"    'mew-summary-jump-message)
+  (define-key mew-summary-mode-map "j"    'mew-summary-goto-line)
   (define-key mew-summary-mode-map "i"    'mew-summary-retrieve)
   (define-key mew-summary-mode-map "I"    'mew-summary-retrieve-message)
   (define-key mew-summary-mode-map "a"    'mew-summary-reply)
@@ -65,15 +66,20 @@
   (define-key mew-summary-mode-map "f"    'mew-summary-forward)
   (define-key mew-summary-mode-map "F"    'mew-summary-multi-forward)
   (define-key mew-summary-mode-map "r"    'mew-summary-resend)
-  (define-key mew-summary-mode-map "@"    'mew-summary-multi)
+  (define-key mew-summary-mode-map "$"    'mew-summary-escape)
   (define-key mew-summary-mode-map "*"    'mew-summary-review)
+  (define-key mew-summary-mode-map "\eu"  'mew-summary-unread)
   (define-key mew-summary-mode-map "y"    'mew-summary-save)
   (define-key mew-summary-mode-map "Y"    'mew-summary-cite)
   (define-key mew-summary-mode-map "u"    'mew-summary-undo)
   (define-key mew-summary-mode-map "U"    'mew-summary-undo-all)
   (define-key mew-summary-mode-map "n"    'mew-summary-display-down)
   (define-key mew-summary-mode-map "p"    'mew-summary-display-up)
-  (define-key mew-summary-mode-map "M"    'mew-summary-mark-duplicated)
+  (define-key mew-summary-mode-map "\C-n" 'mew-summary-next-line)
+  (define-key mew-summary-mode-map "\C-p" 'mew-summary-previous-line)
+  (define-key mew-summary-mode-map "M"	  (make-sparse-keymap))
+  (define-key mew-summary-mode-map "Md"   'mew-summary-mark-duplicated)
+  (define-key mew-summary-mode-map "M*"   'mew-summary-redo)
   (define-key mew-summary-mode-map "N"    'mew-summary-display-review-down)
   (define-key mew-summary-mode-map "P"    'mew-summary-display-review-up)
   (define-key mew-summary-mode-map "\en"  'mew-summary-find-keyword-down)
@@ -93,9 +99,10 @@
   (define-key mew-summary-mode-map "h"    'mew-summary-goto-msg-mode)
   (define-key mew-summary-mode-map "\C-c\C-a" 'mew-summary-addrbook-add)
   (define-key mew-summary-mode-map "\C-c\C-c" 'mew-summary-send-message)
+  (define-key mew-summary-mode-map "\C-c\C-t" 'mew-summary-execute-internal)
   (define-key mew-summary-mode-map "\C-c\C-e" 'mew-summary-execute-external)
   (define-key mew-summary-mode-map "\C-c\C-y" 'mew-summary-execute-command)
-  (define-key mew-summary-mode-map "\C-c\C-i" 'mew-summary-find-file)
+  (define-key mew-summary-mode-map "\C-c\t"   'mew-summary-find-file)
   (define-key mew-summary-mode-map "\C-c\C-f" 'mew-pgp-fetch-key)
   (define-key mew-summary-mode-map "\C-c\C-v" 'mew-pgp-select)
   (define-key mew-summary-mode-map "\C-c\C-s" 'mew-summary-isearch-forward)
@@ -110,7 +117,7 @@
   (define-key mew-summary-mode-map "\e\\" 'mew-summary-cmd-msgs)
   (define-key mew-summary-mode-map "v"    'mew-summary-toggle-disp-msg)
   (define-key mew-summary-mode-map "\C-c\C-l" 'mew-summary-convert-local-cs)
-  (define-key mew-summary-mode-map "\C-c\C-z" 'mew-summary-decode-pgp)
+  (define-key mew-summary-mode-map "\C-c\C-z" 'mew-summary-decode-old-pgp)
   (define-key mew-summary-mode-map "\C-c\C-q" 'mew-summary-kill)
   (define-key mew-summary-mode-map "\C-c\C-x" 'mew-summary-x-face)
   (define-key mew-summary-mode-map "\C-c\C-k" 'mew-summary-kill-subprocess)
@@ -119,38 +126,56 @@
   (define-key mew-summary-mode-map "("    'mew-summary-thread-sibling-up)
   (define-key mew-summary-mode-map ")"    'mew-summary-thread-sibling-down)
   (define-key mew-summary-mode-map "m"	  (make-sparse-keymap))
-  (define-key mew-summary-mode-map "m@"   'mew-summary-mark-multi)
+  (define-key mew-summary-mode-map "md"   'mew-summary-mark-delete)
+  (define-key mew-summary-mode-map "m\ed" 'mew-summary-mark-unlink)
+  (define-key mew-summary-mode-map "m$"   'mew-summary-mark-escape)
   (define-key mew-summary-mode-map "m*"   'mew-summary-mark-review)
+  (define-key mew-summary-mode-map "m\eu" 'mew-summary-mark-unread)
   (define-key mew-summary-mode-map "ms"   'mew-summary-mark-swap)
+  (define-key mew-summary-mode-map "mo"   'mew-summary-mark-refile)
+  (define-key mew-summary-mode-map "mc"   'mew-summary-mark-copy)
   (define-key mew-summary-mode-map "mr"   'mew-summary-mark-regexp)
   (define-key mew-summary-mode-map "ma"   'mew-summary-mark-all)
   (define-key mew-summary-mode-map "mu"   'mew-summary-mark-undo-all)
   (define-key mew-summary-mode-map "mI"   'mew-summary-mark-retrieve-message)
   (define-key mew-summary-mode-map "ml"	  (make-sparse-keymap))
   (define-key mew-summary-mode-map "mlc"  'mew-summary-mark-local-copy)
-  (define-key mew-summary-mode-map "V"    'mew-summary-virtual)
+  (define-key mew-summary-mode-map "V"    'mew-summary-virtual-old)
   (define-key mew-summary-mode-map "/"    'mew-summary-virtual)
-  (define-key mew-summary-mode-map "?"    'mew-summary-search-mark)
+  (define-key mew-summary-mode-map "?"    'mew-summary-pick-mark)
+  (define-key mew-summary-mode-map "'"    'mew-summary-grep-mark)
+  (define-key mew-summary-mode-map "\""   'mew-summary-cmd-mark)
   (define-key mew-summary-mode-map "mt"   'mew-summary-mark-make-thread)
   (define-key mew-summary-mode-map "t"    (make-sparse-keymap))
   (define-key mew-summary-mode-map "tt"   'mew-summary-make-thread)
-  (define-key mew-summary-mode-map "tn"   'mew-summary-thread-down)
-  (define-key mew-summary-mode-map "tp"   'mew-summary-thread-up)
+  (define-key mew-summary-mode-map "tn"   'mew-thread-down)
+  (define-key mew-summary-mode-map "tp"   'mew-thread-up)
+  (define-key mew-summary-mode-map "t "   'mew-thread-toggle)
+  (define-key mew-summary-mode-map "t("   'mew-thread-all-graft)
+  (define-key mew-summary-mode-map "t)"   'mew-thread-all-prune)
   (define-key mew-summary-mode-map "t*"   'mew-thread-mark-review)
-  (define-key mew-summary-mode-map "t@"   'mew-thread-mark-multi)
+  (define-key mew-summary-mode-map "t$"   'mew-thread-mark-escape)
   (define-key mew-summary-mode-map "td"   'mew-thread-mark-delete)
   (define-key mew-summary-mode-map "to"   'mew-thread-mark-refile)
   (define-key mew-summary-mode-map "tc"   'mew-thread-mark-copy)
   (define-key mew-summary-mode-map "t\ed" 'mew-thread-mark-unlink)
   (define-key mew-summary-mode-map "tu"   'mew-thread-unmark)
+  (define-key mew-summary-mode-map "tg"   'mew-thread-glue)
+  (define-key mew-summary-mode-map "tr"   'mew-summary-regexp-make-thread)
   (define-key mew-summary-mode-map "z"    (make-sparse-keymap))
   (define-key mew-summary-mode-map "z8"   'mew-summary-toggle-8bit)
   (define-key mew-summary-mode-map "zc"   'mew-summary-cache-clean-up)
   (define-key mew-summary-mode-map "zd"   'mew-summary-toggle-debug)
   (define-key mew-summary-mode-map "zp"   'mew-summary-toggle-policy)
+  (define-key mew-summary-mode-map "zo"   'mew-summary-toggle-pgp)
+  (define-key mew-summary-mode-map "zv"   'mew-summary-toggle-header-veil)
   (define-key mew-summary-mode-map "zw"   'mew-summary-toggle-warning)
+  (define-key mew-summary-mode-map "z "   'mew-summary-toggle-mark-regex)
   (define-key mew-summary-mode-map "l"	  (make-sparse-keymap))
   (define-key mew-summary-mode-map "lc"   'mew-summary-local-copy)
+  (define-key mew-summary-mode-map "lh"   'mew-summary-learn-ham)
+  (define-key mew-summary-mode-map "ls"   'mew-summary-learn-spam)
+  (define-key mew-summary-mode-map "="    'mew-summary-info)
   ;;
   ;; not provided in Virtual mode
   ;;
@@ -163,20 +188,18 @@
   (define-key mew-summary-mode-map "X"    'mew-summary-exec-one)
   (define-key mew-summary-mode-map "lx"   'mew-summary-exec-offline)
   (define-key mew-summary-mode-map "lX"   'mew-summary-exec-offline-one)
-  (define-key mew-summary-mode-map "mo"   'mew-summary-mark-refile)
-  (define-key mew-summary-mode-map "mc"   'mew-summary-mark-copy)
-  (define-key mew-summary-mode-map "md"   'mew-summary-mark-delete)
   (define-key mew-summary-mode-map "m/"   'mew-summary-mark-virtual)
-  (define-key mew-summary-mode-map "m\ed" 'mew-summary-mark-unlink)
-  (define-key mew-summary-mode-map "mxd"  'mew-summary-exec-delete)
+  (define-key mew-summary-mode-map "mxd"   'mew-summary-exec-delete)
   (define-key mew-summary-mode-map "mx\ed" 'mew-summary-exec-unlink)
-  (define-key mew-summary-mode-map "mxo"  'mew-summary-exec-refile)
-
-  (define-key mew-summary-mode-map "O"    'mew-summary-pack)
+  (define-key mew-summary-mode-map "mxo"   'mew-summary-exec-refile)
   (define-key mew-summary-mode-map "s"    'mew-summary-ls)
+  (define-key mew-summary-mode-map "O"    'mew-summary-pack)
   (define-key mew-summary-mode-map "S"    'mew-summary-sort)
   (define-key mew-summary-mode-map "\C-c\C-b" 'mew-summary-exchange-point)
   (define-key mew-summary-mode-map "\eo"  'mew-summary-auto-refile)
+  (define-key mew-summary-mode-map "R"	  (make-sparse-keymap))
+  (define-key mew-summary-mode-map "Rd"   'mew-summary-delete-folder)
+  (define-key mew-summary-mode-map "Rr"   'mew-summary-rename-folder)
   ;;
   (if mew-xemacs-p
       (define-key mew-summary-mode-map 'button2 'mew-summary-mouse-show)
@@ -189,13 +212,12 @@
     ["Previous" mew-summary-display-up   t]
     ["Top"      mew-summary-jump-top     t]
     ["Bottom"   mew-summary-jump-bottom  t]
-    ["Jump"     mew-summary-jump-message t]
     "----"
     ["Delete"       mew-summary-delete       (mew-summary-p)]
     ["Refile"       mew-summary-refile       (mew-summary-case1)]
     ["Refile again" mew-summary-refile-again (mew-summary-case1)]
-    ["Mark multi"   mew-summary-mark-multi   t]
     ["Mark review"  mew-summary-mark-review  t]
+    ["Mark escape"  mew-summary-mark-escape  t]
     ["Undo"         mew-summary-undo         t]
     ["Undo all"     mew-summary-undo-all     t]
     ["Execute"      mew-summary-exec         (mew-summary-p)]
@@ -205,7 +227,6 @@
     ("Manipulate folder"
      ["Retrieve"     mew-summary-retrieve    t]
      ["List"         mew-summary-ls          (mew-summary-p)]
-     ["Pack"         mew-summary-pack        (mew-summary-case2)]
      ["Sort"         mew-summary-sort        (mew-summary-case2)]
      ["Burst"        mew-summary-burst       t]
      ["Go to folder" mew-summary-goto-folder t]
@@ -224,8 +245,10 @@
      ["Multi forward" mew-summary-multi-forward (mew-summary-case1)]
      )
     ("Select"
-     ["Search then Mark" mew-summary-search-mark (mew-summary-or-thread-p)]
-     ["Virtual mode"     mew-summary-virtual     (mew-summary-or-virtual-p)]
+     ["Pick then mark"     mew-summary-pick-mark   (mew-pickable-p)]
+     ["Grep then mark"     mew-summary-grep-mark   (mew-pickable-p)]
+     ["Command then mark"  mew-summary-cmd-mark    (mew-pickable-p)]
+     ["Virtual mode"       mew-summary-virtual     (mew-summary-or-virtual-p)]
      )
     ("Misc"
      ["Clean +trash"         mew-summary-clean-trash      t]
@@ -302,11 +325,13 @@
   (define-key mew-draft-header-map "\t"     'mew-draft-header-comp)
   (define-key mew-draft-header-map "\C-c\t" 'mew-draft-circular-comp)
   (define-key mew-draft-header-map "\e\t"   'mew-draft-expand)
+  (define-key mew-draft-header-map "\e\C-e" 'mew-draft-addrbook-expand)
   (mew-draft-share-keymap 'mew-draft-header-map))
 
 (unless mew-draft-body-map
   (setq mew-draft-body-map (make-sparse-keymap))
   (mew-set-keymap-parent mew-draft-body-map text-mode-map)
+  (define-key mew-draft-body-map "\C-x\C-s"     'mew-draft-save-buffer)
   (define-key mew-draft-body-map "\C-c\t"       'mew-draft-insert-signature)
   (define-key mew-draft-body-map "\C-c\C-y"     'mew-draft-cite)
   (define-key mew-draft-body-map "\C-c\C-t"     'mew-draft-yank)
@@ -414,6 +439,7 @@
 
 (unless mew-header-mode-map
   (setq mew-header-mode-map (make-sparse-keymap))
+  (define-key mew-header-mode-map "\C-x\C-s" 'mew-draft-save-buffer)
   (define-key mew-header-mode-map "\t"       'mew-draft-header-comp)
   (define-key mew-header-mode-map "\C-c\t"   'mew-draft-circular-comp)
   (define-key mew-header-mode-map "\e\t"     'mew-draft-expand)
@@ -495,7 +521,7 @@
   (define-key mew-input-map "\t"     'mew-input-complete)
   (define-key mew-input-map [tab]    'mew-input-complete)
   (define-key mew-input-map "?"      'mew-input-complete)
-  (define-key mew-input-map "\C-c\t" 'mew-circular-complete-case))
+  (define-key mew-input-map "\C-c\t" 'mew-circular-complete-switch))
 
 (unless mew-input-folder-map
   (setq mew-input-folder-map (copy-keymap mew-input-map))
