@@ -1,3 +1,4 @@
+;;-*-coding:iso-2022-7bit;-*-
 ;;; mew-lang-jp.el --- Japanese specific stuff for Mew
 
 ;; Author:  Mito <mit@nines.nec.co.jp>
@@ -175,7 +176,7 @@
 
 ;;
 
-(defvar mew-fancy-thread-indent-strings ["$B(2(B" "$B(1(B" "$B(-(B" "$B!!(B"])
+(defvar mew-thread-indent-strings ["$B(2(B" "$B(1(B" "$B(-(B" "$B!!(B"])
 
 (defvar mew-highlight-body-regex-cite
   "^\\(\\([ \t]*\\([>:|$B!S!d!U"d!'!C(B]\\|\\w+\\(['._-]+\\w+\\)*>+\\)\\)+\\).*")
