@@ -14,10 +14,9 @@
 
 (defun mew-mark-unrefile (fld msg)
   "Delete refile state and delete the mark."
-  (when (get-buffer fld)
-    (save-excursion
-      (set-buffer fld)
-      (mew-refile-reset msg))))
+  (mew-summary-refile-remove-body)
+  (unless (mew-virtual-p)
+    (mew-refile-reset msg)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -26,9 +25,9 @@
 
 (defsubst mew-mark-unmark ()
   (save-excursion
-    (beginning-of-line)
-    (if (re-search-forward mew-regex-msg nil t)
-	(mew-mark-delete-here))))
+    (mew-summary-goto-mark)
+    (when (looking-at mew-regex-mark)
+      (mew-mark-remove))))
 
 (defun mew-summary-undo (&optional count)
   "Cancel the mark in COUNT times."
@@ -49,13 +48,10 @@
 	  (or (fboundp func) (setq func nil))
 	  (setq fld (mew-summary-folder-name))
 	  (setq msg (mew-summary-message-number))
-	  (cond
-	   ((mew-virtual-p)
-	    (mew-mark-unmark)
-	    (mew-summary-unmark-in-physical fld msg func))
-	   (t ;; Summary mode
-	    (mew-mark-unmark)
-	    (if func (funcall func fld msg)))))))))
+	  (mew-mark-unmark)
+	  (if func (funcall func fld msg))
+	  (if (mew-virtual-p)
+	      (mew-summary-unmark-in-physical fld msg func)))))))
 
 ;;
 
@@ -66,7 +62,7 @@
     (if (and char (not (char-equal char ?\r)))
 	(mew-mark-undo-mark char))))
 
-(defun mew-mark-undo-mark (mark &optional no-msg)
+(defun mew-mark-undo-mark (mark &optional no-msg virtual-only)
   "Undo MARK on the entire buffer.
 If optional argument NO-MSG is non-nil, no message is displayed."
   (or no-msg (message "Unmarking..."))
@@ -74,20 +70,24 @@ If optional argument NO-MSG is non-nil, no message is displayed."
   (let* ((regex (mew-mark-regex mark))
 	 (func (mew-markdb-func-undo mark))
 	 (case-fold-search nil)
-	 alist fld msg)
+	 (reviewp (char-equal mark mew-mark-review))
+	 alist fld msg reviews)
     (or (fboundp func) (setq func nil))
     (save-excursion
       (goto-char (point-min))
       (while (re-search-forward regex nil t)
 	(setq fld (mew-summary-folder-name))
 	(setq msg (mew-summary-message-number))
-	(mew-mark-delete-here t)
-	(if (mew-virtual-p)
-	    (mew-mark-alist-set alist fld msg)
-	  (if func (funcall func fld msg))) ;; in physical only
-	(forward-line))  ;; make search faster
+	(mew-mark-remove)
+	(when (mew-virtual-p)
+	  (mew-mark-alist-set alist fld msg))
+	(when reviewp (setq reviews (cons msg reviews)))
+	(if func (funcall func fld msg))
+	(forward-line))
       (set-buffer-modified-p nil)
-      (mew-summary-unmark-in-physical-alist alist func)))
+      (mew-sinfo-set-mark-review (nreverse reviews))
+      (unless virtual-only
+	(mew-summary-unmark-in-physical-alist alist func))))
   (or no-msg (message "Unmarking...done")))
 
 ;;
@@ -102,6 +102,24 @@ If optional argument NO-MSG is non-nil, no message is displayed."
       (setq marks (cdr marks)))
     (message "Unmarking...done")))
 
+;;
+
+(defun mew-summary-redo ()
+  (interactive)
+  (let ((mark mew-mark-review)
+	(reviews (mew-sinfo-get-mark-review))
+	msg)
+    (save-excursion
+      (mew-mark-undo-mark mark 'no-msg)
+      (goto-char (point-min))
+      (while reviews
+	(setq msg (car reviews))
+	(setq reviews (cdr reviews))
+	(when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	  (mew-mark-put mark)
+	  (forward-line))))
+    (set-buffer-modified-p nil)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Basic functions and macros for mark
@@ -110,34 +128,49 @@ If optional argument NO-MSG is non-nil, no message is displayed."
 (defun mew-summary-get-mark ()
   "Get a mark on the current message."
   (save-excursion
-    (beginning-of-line)
-    (if (looking-at mew-regex-msg-mark)
-	(string-to-char (mew-match-string 2))
-      nil)))
+    (mew-summary-goto-mark)
+    (when (looking-at mew-regex-mark)
+      (mew-sumsyn-mark)))) ;; return char
 
-(defalias 'mew-summary-marked-p 'mew-summary-get-mark)
+(defun mew-summary-markable ()
+  (let ((mark (mew-summary-get-mark)))
+    (or (null mark) (char-equal mark mew-mark-unread))))
 
-(defsubst mew-summary-mark-as (mark &optional force)
+(defun mew-summary-mark-as (mark)
   "Mark this message with MARK if possible."
-   (when (or force (not (mew-summary-marked-p)))
-     (save-excursion
-       (beginning-of-line)
-       (when (re-search-forward mew-regex-msg nil t)
-	 (mew-mark-put-here mark)))))
+  (when (mew-summary-markable)
+    (mew-mark-put mark)))
 
-(defsubst mew-mark-put-here (mark &optional after-mark)
-  (mew-elet
-   (if after-mark (forward-char -1))
-   (delete-char 1)
-   (insert-and-inherit (char-to-string mark)) ;; inherit highlight
-   (mew-highlight-mark-line mark)))
+(defsubst mew-mark-put (mark)
+  (save-excursion
+    (mew-summary-goto-mark)
+    (let* ((beg (point)) (end (1+ beg)))
+      (mew-elet
+       (mew-front-sticky beg end) ;; for XEmacs
+       ;; This code is awkward but for invisible.
+       (forward-char 1)
+       (insert-and-inherit (char-to-string mark)) ;; inherit highlight
+       (forward-char -2)
+       (delete-char 1)
+       (mew-front-nonsticky beg end)
+       (mew-highlight-mark-line mark))
+      (mew-mark-hist-set (mew-summary-message-number) mark))))
 
-(defsubst mew-mark-delete-here (&optional after-mark)
-  (mew-elet
-   (if after-mark (forward-char -1))
-   (delete-char 1)
-   (insert-and-inherit " ") ;; inherit highlight
-   (mew-highlight-unmark-line)))
+(defsubst mew-mark-remove ()
+  (save-excursion
+    (mew-summary-goto-mark)
+    (let* ((mark mew-mark-read)
+	   (beg (point)) (end (1+ beg)))
+      (mew-elet
+       (mew-front-sticky beg end) ;; for XEmacs
+       ;; This code is awkward but for invisible.
+       (forward-char 1)
+       (insert-and-inherit (char-to-string mark)) ;; inherit highlight
+       (forward-char -2)
+       (delete-char 1)
+       (mew-front-nonsticky beg end)
+       (mew-highlight-unmark-line))
+      (mew-mark-hist-set (mew-summary-message-number) mark))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -182,13 +215,31 @@ cons pairs of folder name and message number."
   (save-excursion
     (let (ret mrk msg)
       (goto-char (point-min))
-      (while (re-search-forward mew-regex-msg-mark nil t)
-	(setq msg (mew-match-string 1))
-	(setq mrk (string-to-char (mew-match-string 2)))
-	(setq ret (cons (cons msg mrk) ret))
+      (while (re-search-forward mew-regex-mark nil t)
+	(setq mrk (mew-sumsyn-mark))
+	(setq msg (mew-summary-message-number))
+	(setq ret (cons (list msg mrk) ret))
 	(forward-line))
       (nreverse ret))))
 
+(defun mew-summary-mark-recover (mdb &optional ref)
+  (let ((opos (point))
+	ent msg mrk new-ref ref-ent)
+    (goto-char (point-min))
+    (while mdb
+      (setq ent (car mdb))
+      (setq mdb (cdr mdb))
+      (mew-set '(msg mrk) ent)
+      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	(if (or (not (char-equal mrk mew-mark-refile)) (null ref))
+	    (mew-mark-put mrk)
+	  (when (setq ref-ent (assoc msg ref))
+	    (mew-mark-put mrk)
+	    (setq new-ref (cons ref-ent new-ref))))
+	(forward-line)))
+    (goto-char opos)
+    (nreverse new-ref)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Base function
@@ -216,27 +267,49 @@ See also mew-mark-afterstep-spec."
      ((eq action 2)
       (mew-summary-goto-message)
       (mew-decode-syntax-delete)
-      ;; for C-x C-x
       (beginning-of-line)
-      (let ((zmacs-regions nil))
-	(push-mark (point) t t))
-      (mew-summary-display-after mew-summary-mark-direction)))))
+      (mew-push-mark)
+      (let ((mew-inherit-after-marking t))
+	(mew-summary-display-after mew-summary-mark-direction))))))
+
+(defun mew-mark-hist-set (msg mark)
+  (let* ((hist (mew-sinfo-get-mark-hist))
+	 (ent (assoc msg hist))
+	 (num (string-to-int msg))
+	 curr prev)
+    (cond
+     (ent
+      (if mark (setcar (nthcdr 1 ent) mark)))
+     (t
+      (setq ent (list msg mark))
+      (setq curr hist)
+      (catch 'loop
+	(while curr
+	  (if (> (string-to-int (nth 0 (car curr))) num)
+	      (progn
+		(if (null prev)
+		    (setq hist (cons ent hist))
+		  (setcdr prev (cons ent (cdr prev))))
+		(throw 'loop nil)))
+	  (setq prev curr)
+	  (setq curr (cdr curr)))
+	(setq hist (nconc hist (list ent))))
+      (mew-sinfo-set-mark-hist hist)))))
 
 (defun mew-mark-put-mark (newmark &optional no-msg valid-only)
   "Put the NEWMARK on the current line if possible.
 If NO-MSG is non-nil, no message is displayed.
 NO-MSG also means that this function is being called in loop."
   (mew-summary-msg-or-part
-   (let (oldmark oldlevel oldname newlevel newname case zmacs-regions
-	 msg fld marked invalidp)
+   (let (oldmark oldlevel oldname newlevel newname case msg fld marked validp)
      (save-excursion
        (mew-summary-goto-message)
-       (setq invalidp (and valid-only (mew-summary-msg-invalidp)))
-       (if invalidp
+       (when (mew-sumsyn-match mew-regex-sumsyn-short)
+	 (setq msg (mew-sumsyn-message-number))
+	 (setq fld (mew-sumsyn-folder-name))
+	 (setq validp (or (not valid-only) (mew-msg-validp msg))))
+       (if (not validp)
 	   (unless no-msg (message "Cannot mark this invalid message with '%c'" newmark))
-	 (when (mew-virtual-p)
-	   (setq msg (mew-summary-message-number))
-	   (setq fld (mew-summary-folder-name)))
 	 (setq oldmark (mew-summary-get-mark))
 	 (setq oldlevel (mew-markdb-level oldmark))
 	 (setq oldname (mew-markdb-name oldmark))
@@ -245,7 +318,7 @@ NO-MSG also means that this function is being called in loop."
 	 (cond
 	  ((null oldmark);; no mark
 	   (setq case 1)
-	   (mew-summary-mark-as newmark)
+	   (mew-mark-put newmark)
 	   (setq marked t))
 	  ((eq oldmark newmark)
 	   (setq case 2)
@@ -255,7 +328,7 @@ NO-MSG also means that this function is being called in loop."
 	  ((< oldlevel newlevel)
 	   (setq case 3)
 	   (mew-summary-undo-one no-msg)
-	   (mew-summary-mark-as newmark)
+	   (mew-mark-put newmark)
 	   (setq marked t))
 	  ((= oldlevel newlevel)
 	   (cond
@@ -266,20 +339,20 @@ NO-MSG also means that this function is being called in loop."
 		 (progn
 		   (setq case 4)
 		   (mew-summary-undo-one no-msg)
-		   (mew-summary-mark-as newmark)
+		   (mew-mark-put newmark)
 		   (setq marked t))
 	       (setq case 5)))
 	    (t
 	     (setq case 6)
 	     (mew-summary-undo-one no-msg)
-	     (mew-summary-mark-as newmark)
+	     (mew-mark-put newmark)
 	     (setq marked t))))
 	  (t ;; > oldlevel newlevel
 	   (setq case 7)
 	   (message "Cannot mark here because '%s' is stronger than '%s'"
 		    oldname newname)))))
-     (unless invalidp
-       (if (and msg fld marked)
+     (when validp
+       (if (and marked (mew-virtual-p))
 	   (mew-summary-mark-in-physical fld msg newmark))
        (or no-msg (mew-mark-afterstep newmark case))
        (set-buffer-modified-p nil)))))
@@ -291,6 +364,15 @@ If COUNT is positive, call FUNC in COUNT times moving the cursor forward.
 If COUNT is negative, call FUNC in COUNT times moving the cursor backward.
 If COUNT is numeric and STAYP is non-nil, the cursor stays in the
 original position."
+  (if (and (not (numberp count)) (mew-mark-active-p))
+      (let ((begend (mew-summary-get-region)))
+	(goto-char (car begend))
+	(setq stayp nil)
+	(setq count (count-lines (point) (cdr begend)))
+	(mew-mark-put-mark-loop1 func count stayp))
+    (mew-mark-put-mark-loop1 func count stayp)))
+
+(defun mew-mark-put-mark-loop1 (func count stayp)
   (when (and func (fboundp func))
     (mew-summary-msg-or-part
      (if (numberp count)
@@ -323,18 +405,20 @@ original position."
   (save-excursion
     (set-buffer fld)
     (save-excursion
-      (goto-char (point-min))
-      (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
-	(mew-mark-put-here new-mark t)
+      (when (or (re-search-forward  (mew-regex-sumsyn-msg msg) nil t)
+		(re-search-backward (mew-regex-sumsyn-msg msg) nil t))
+	(beginning-of-line)
+	(mew-mark-put new-mark)
 	(set-buffer-modified-p nil)))))
 
 (defun mew-summary-unmark-in-physical (fld msg &optional func)
   (save-excursion
     (set-buffer fld)
     (save-excursion
-      (goto-char (point-min))
-      (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
-	(mew-mark-delete-here t)
+      (when (or (re-search-forward  (mew-regex-sumsyn-msg msg) nil t)
+		(re-search-backward (mew-regex-sumsyn-msg msg) nil t))
+	(beginning-of-line)
+	(mew-mark-remove)
 	(if func (funcall func fld msg))
 	(set-buffer-modified-p nil)))))
 
@@ -345,44 +429,47 @@ original position."
 	 (nconc fld-msgs (list imsg))
        (setq ,alist (cons (list ,fld imsg) ,alist)))))
 
-(defun mew-summary-mark-in-physical-alist (alist mark)
-  (let (ent fld msg msgs)
-    (while alist
-      (setq ent (car alist))
-      (setq alist (cdr alist))
-      (setq fld (car ent))
-      (setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
-      (when (get-buffer fld)
-	(set-buffer fld)
-	(save-excursion
-	  (goto-char (point-min))
-	  (while msgs
-	    (setq msg (int-to-string (car msgs)))
-	    (setq msgs (cdr msgs))
-	    (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
-	      (mew-mark-put-here mark t)
-	      (forward-line)))
-	  (set-buffer-modified-p nil))))))
+(defun mew-summary-mark-in-physical-alist (alist mark &optional func)
+  (save-excursion
+    (let (ent fld msg msgs)
+      (while alist
+	(setq ent (car alist))
+	(setq alist (cdr alist))
+	(setq fld (car ent))
+	(setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
+	(when (get-buffer fld)
+	  (set-buffer fld)
+	  (save-excursion
+	    (goto-char (point-min))
+	    (while msgs
+	      (setq msg (int-to-string (car msgs)))
+	      (setq msgs (cdr msgs))
+	      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+		(if func (funcall func))
+		(mew-mark-put mark)
+		(forward-line)))
+	    (set-buffer-modified-p nil)))))))
 
 (defun mew-summary-unmark-in-physical-alist (alist func)
-  (let (ent fld msg msgs)
-    (while alist
-      (setq ent (car alist))
-      (setq alist (cdr alist))
-      (setq fld (car ent))
-      (setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
-      (when (get-buffer fld)
-	(set-buffer fld)
-	(save-excursion
-	  (goto-char (point-min))
-	  (while msgs
-	    (setq msg (int-to-string (car msgs)))
-	    (setq msgs (cdr msgs))
-	    (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
-	      (mew-mark-delete-here t)
-	      (if func (funcall func fld msg))
-	      (forward-line)))
-	  (set-buffer-modified-p nil))))))
+  (save-excursion
+    (let (ent fld msg msgs)
+      (while alist
+	(setq ent (car alist))
+	(setq alist (cdr alist))
+	(setq fld (car ent))
+	(setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
+	(when (get-buffer fld)
+	  (set-buffer fld)
+	  (save-excursion
+	    (goto-char (point-min))
+	    (while msgs
+	      (setq msg (int-to-string (car msgs)))
+	      (setq msgs (cdr msgs))
+	      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+		(mew-mark-remove)
+		(if func (funcall func fld msg))
+		(forward-line)))
+	    (set-buffer-modified-p nil)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -403,19 +490,18 @@ See also '\\[mew-summary-mark-refile]', '\\[mew-summary-mark-delete]', '\\[mew-s
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Multi: "@" in Summary mode
+;;; Escape: "$" in Summary mode
 ;;;
 
-(defun mew-summary-multi (&optional count)
+(defun mew-summary-escape (&optional count)
   "\\<mew-summary-mode-map>
-Put the multi mark (default is '@') in COUNT times
-for '\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', '\\[mew-summary-uudecode]', '\\[mew-summary-burst-multi]'. "
+Put the escape mark (default is '$') in COUNT times."
   (interactive "P")
-  (mew-mark-put-mark-loop 'mew-summary-multi-one count nil))
+  (mew-mark-put-mark-loop 'mew-summary-escape-one count nil))
 
-(defun mew-summary-multi-one (&optional no-msg)
-  "Put the multi mark (default is '@') on this message."
-  (mew-mark-put-mark mew-mark-multi no-msg))
+(defun mew-summary-escape-one (&optional no-msg)
+  "Put the escape mark (default is '$') on this message."
+  (mew-mark-put-mark mew-mark-escape no-msg))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -447,6 +533,21 @@ for '\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', '\\[mew-summary-u
   "Put the unlink mark (default is 'X') on this message."
   (mew-mark-put-mark mew-mark-unlink no-msg 'valid-only))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Unread: "U" in Summary mode
+;;;
+
+(defun mew-summary-unread (&optional count)
+  "\\<mew-summary-mode-map>
+Put the unread mark (default is 'U') in COUNT times."
+  (interactive "P")
+  (mew-mark-put-mark-loop 'mew-summary-unread-one count nil))
+
+(defun mew-summary-unread-one (&optional no-msg)
+  "Put the unread mark (default is 'U') on this message."
+  (mew-mark-put-mark mew-mark-unread no-msg))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; All messages
@@ -456,6 +557,7 @@ for '\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', '\\[mew-summary-u
   "Put the '*' mark onto all messages which are not marked."
   (interactive "P")
   (mew-decode-syntax-delete)
+  (if (mew-mark-active-p) (setq arg t))
   (if arg
       (let ((begend (mew-summary-get-region)))
 	(mew-summary-mark-all-region (car begend) (cdr begend)))
@@ -465,7 +567,7 @@ for '\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', '\\[mew-summary-u
   "Put the '*' mark onto all messages which are not marked between
 BEG and END."
   (interactive "r")
-  (let ((regex (mew-mark-regex ? )) ;; not marked
+  (let ((regex (mew-mark-regex mew-sp)) ;; not marked
 	(mark mew-mark-review) ;; someday ...
 	fld msg alist)
     (save-excursion
@@ -476,7 +578,7 @@ BEG and END."
 	  (setq fld (mew-summary-folder-name))
 	  (setq msg (mew-summary-message-number))
 	  (mew-mark-alist-set alist fld msg))
-	(forward-line)) ;; make search faster
+	(forward-line))
       (set-buffer-modified-p nil))
     (mew-summary-mark-in-physical-alist alist mark)))
 
@@ -485,9 +587,17 @@ BEG and END."
 ;;; Regex
 ;;;
 
-(defun mew-summary-mark-regexp ()
+(defun mew-summary-search-regexp-visible (regex)
+  (catch 'loop
+    (while (re-search-forward regex nil t)
+      (beginning-of-line)
+      (unless (search-forward "\r" (match-end 0) t)
+	(throw 'loop t))
+      (forward-line))))
+    
+(defun mew-summary-mark-regexp (&optional args)
   "Put the '*' mark onto all messages matched to a regular expression."
-  (interactive)
+  (interactive "P")
   (mew-decode-syntax-delete)
   (let ((regex (read-string "Regexp: "))
 	(mark mew-mark-review) ;; someday ...
@@ -497,16 +607,18 @@ BEG and END."
       (setq regex (read-string "Regexp: ")))
     (save-excursion
       (goto-char (point-min))
-      (while (re-search-forward regex nil t)
-	(unless (mew-summary-marked-p)
-	  (when (looking-at "[^\n]*\r") ;; before thread info
-	    (mew-summary-mark-as mark)
-	    (setq n (1+ n))
-	    (when (mew-virtual-p)
-	      (setq fld (mew-summary-folder-name))
-	      (setq msg (mew-summary-message-number))
-	      (mew-mark-alist-set alist fld msg))))
-	(forward-line)) ;; make search faster
+      (while (if args
+		 (re-search-forward regex nil t)
+	       (mew-summary-search-regexp-visible regex))
+	(when (and (mew-summary-markable)
+		   (mew-sumsyn-match mew-regex-sumsyn-short))
+	  (when (mew-virtual-p)
+	    (setq fld (mew-sumsyn-folder-name))
+	    (setq msg (mew-sumsyn-message-number))
+	    (mew-mark-alist-set alist fld msg))
+	  (mew-mark-put mark)
+	  (setq n (1+ n)))
+	(forward-line))
       (set-buffer-modified-p nil))
     (mew-summary-mark-in-physical-alist alist mark)
     (cond
@@ -523,25 +635,25 @@ BEG and END."
 ;;;
 
 (defun mew-summary-exchange-mark (oldmark newmark &optional valid-only)
-  (let ((regex (if valid-only 
-		   (mew-mark-valid-regex oldmark)
-		 (mew-mark-regex oldmark)))
+  (let ((regex (mew-mark-regex oldmark))
 	(case-fold-search nil)
-	fld msg alist)
+	(i 0) fld msg alist)
     (save-excursion
       (goto-char (point-min))
-      (if (not (re-search-forward regex nil t))
-	  (message "No marked messages")
-	(beginning-of-line)
-	(while (re-search-forward regex nil t)
-	  (mew-mark-put-here newmark t)
-	  (when (mew-virtual-p)
-	    (setq fld (mew-summary-folder-name))
-	    (setq msg (mew-summary-message-number))
-	    (mew-mark-alist-set alist fld msg))
-	  (forward-line)) ;; make search faster
-	(set-buffer-modified-p nil)
-	(mew-summary-mark-in-physical-alist alist newmark)))))
+      (while (re-search-forward regex nil t)
+	(when (mew-sumsyn-match mew-regex-sumsyn-short)
+	  (setq fld (mew-sumsyn-folder-name))
+	  (setq msg (mew-sumsyn-message-number))
+	  (when (or (not valid-only) (mew-msg-validp msg))
+	    (mew-mark-put newmark)
+	    (when (mew-virtual-p)
+	      (mew-mark-alist-set alist fld msg))))
+	(forward-line)
+	(setq i (1+ i))))
+    (if (= i 0)
+	(message "No marked messages")
+      (set-buffer-modified-p nil)
+      (mew-summary-mark-in-physical-alist alist newmark))))
    
 (defun mew-summary-mark-delete ()	;; * -> D
   "Put the delete mark onto all messages marked with '*'."
@@ -554,21 +666,26 @@ BEG and END."
   (interactive)
   (mew-summary-exchange-mark mew-mark-review mew-mark-unlink 'valid-only))
 
-(defun mew-summary-mark-multi ()	;; * -> @
-  "Change the '*' mark into the '@' mark."
+(defun mew-summary-mark-escape ()	;; * -> $
+  "Change the '*' mark into the '$' mark."
   (interactive)
-  (mew-summary-exchange-mark mew-mark-review mew-mark-multi))
+  (mew-summary-exchange-mark mew-mark-review mew-mark-escape))
 
-(defun mew-summary-mark-review ()	;; @ -> *
-  "Change the '@' mark into the '*' mark."
+(defun mew-summary-mark-review ()	;; $ -> *
+  "Change the '$' mark into the '*' mark."
   (interactive)
-  (mew-summary-exchange-mark mew-mark-multi mew-mark-review))
+  (mew-summary-exchange-mark mew-mark-escape mew-mark-review))
 
-(defun mew-summary-mark-swap ()		;; @ <-> *
-  "Swap the '@' mark and the '*' mark."
+(defun mew-summary-mark-unread ()	;; * -> U
+  "Change the '*' mark into the 'U' mark."
   (interactive)
-  (mew-summary-exchange-mark mew-mark-multi mew-mark-tmp)
-  (mew-summary-exchange-mark mew-mark-review mew-mark-multi)
+  (mew-summary-exchange-mark mew-mark-review mew-mark-unread))
+
+(defun mew-summary-mark-swap ()		;; $ <-> *
+  "Swap the '$' mark and the '*' mark."
+  (interactive)
+  (mew-summary-exchange-mark mew-mark-escape mew-mark-tmp)
+  (mew-summary-exchange-mark mew-mark-review mew-mark-escape)
   (mew-summary-exchange-mark mew-mark-tmp mew-mark-review))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -586,9 +703,10 @@ region."
     (if (and (or (eq mew-mark-duplicated mew-mark-delete)
 		 (eq mew-mark-duplicated mew-mark-unlink))
 	     (eq mew-summary-mark-duplicated-skip nil))
-	(message "Cannot mark because messages may lost by this setting.")
+	(message "Cannot mark because messages may lost by this setting")
       (let ((reversep (eq mew-summary-mark-duplicated-skip 'last))
 	    (count 0) my-id dup-id size ids beg end region)
+	(if (mew-mark-active-p) (setq arg t))
 	(cond
 	 (arg
 	  (setq region (mew-summary-get-region))
@@ -600,7 +718,7 @@ region."
 	(message "Marking duplications...")
 	(save-excursion
 	  ;; from mew-summary-thread-region in mew-threaed.el
-	  (setq size (mew-count-lines beg end))
+	  (setq size (count-lines beg end))
 	  (cond
 	   ((<= size 211)
 	    (setq size 211))
@@ -617,7 +735,7 @@ region."
 	    (catch 'loop
 	      (while t
 		(when (and (mew-summary-message-number)
-			   (not (mew-summary-marked-p)))
+			   (mew-summary-markable))
 		  (setq my-id (mew-summary-my-id))
 		  (when (> (length my-id) 0)
 		    (setq dup-id (intern-soft my-id ids))
@@ -628,9 +746,8 @@ region."
 			;; second time (first duplication)
 			(unless mew-summary-mark-duplicated-skip
 			  (save-excursion
-			    (when (search-backward
-				   (format "\r <%s> " my-id) nil t)
-			      (beginning-of-line)
+			    (when (re-search-backward
+				   (mew-regex-sumsyn-my-id my-id) nil t)
 			      (mew-mark-put-mark mew-mark-duplicated 'no-msg)
 			      (setq count (1+ count))))
 			  (set (intern my-id ids) nil)))
@@ -662,9 +779,12 @@ region."
 (defun mew-summary-set-walk-mark ()
   (let ((char (mew-input-mark)))
     (cond
+     ((null char)
+      ;; mew-input-mark displays a message.
+      )
      ((char-equal char ?\r)
       (setq mew-mark-walk mew-mark-default-walk)
-      (message "Target mark was set to '%s'" (char-to-string mew-mark-walk)))
+      (message "Target mark was set to '%c'" mew-mark-walk))
      (char (setq mew-mark-walk char)))))
 
 (defun mew-summary-down-mark (mark)
@@ -715,39 +835,22 @@ the specified mark."
 ;;; Clean up marks!
 ;;;
 
-(defun mew-mark-clean (askp &optional sum)
-  "Process marked messages for this folder."
-  (if sum
-      (set-buffer sum)
-    (setq sum (mew-summary-folder-name 'ext)))
-  (if (and (mew-summary-p)
-           (mew-summary-mark-exist-p mew-mark-clean)
-	   (or (not askp)
-	       (y-or-n-p (format "Marks exist in %s. Process them? " sum))))
-      (mew-summary-exec)))
-
 (defun mew-mark-init ()
-  (add-hook 'kill-emacs-hook 'mew-mark-clean-up))
+  (add-hook 'kill-emacs-hook 'mew-sinfo-clean-up))
 
-(defun mew-mark-clean-up ()
-  "Process marked messages for all Summary modes.
-Typically called by kill-emacs-hook."
-  (remove-hook 'kill-emacs-hook 'mew-mark-clean-up)
+(defun mew-sinfo-clean-up ()
+  "Saving marks. Typically called by kill-emacs-hook."
+  (remove-hook 'kill-emacs-hook 'mew-sinfo-clean-up)
   (mew-decode-syntax-delete)
   (let ((bufs mew-buffers) buf)
     (save-excursion
       (while bufs
 	(setq buf (car bufs))
 	(setq bufs (cdr bufs))
-	(when (and (bufferp (get-buffer buf))
-		   (mew-folder-localp buf)) ;; xxx
-	  (mew-mark-clean mew-ask-mark-process buf))))))
-
-;; Saving marks is a really bad idea.
-;; First because there is no way to fill the gap if the folder is newer
-;; than the cache at quitting.
-;; Even if the cache is newer, saving marks faces dilemma if 
-;; multiple Emacses run.
+	(when (bufferp (get-buffer buf))
+	  (set-buffer buf)
+	  (when (mew-summary-p)
+	    (mew-sinfo-save)))))))
 
 (provide 'mew-mark)
 
