@@ -79,7 +79,7 @@
      ((and (listp program) (nth 0 program) (symbolp (nth 0 program)))
       (when (fboundp (nth 0 program))
 	(if (mew-ct-imagep ct)
-	    (funcall (nth 0 program) cache begin end params ct)
+	    (funcall (nth 0 program) cache begin end params fname ct cte)
 	  (if (eq (nth 0 program) 'mew-mime-application/octet-stream)
 	      (funcall (nth 0 program) cache begin end params ct cte fname (nth 1 program))
 	    (funcall (nth 0 program) cache begin end params)))))
@@ -132,44 +132,42 @@
   ;; called in Message buffer
   (when (> end begin)
     (save-excursion
-      (mew-elet
-       ;; We need to keep properties (e.g. citation color)
-       ;; in a message cache.
-       ;; This must be "insert-buffer-substring".
-       (insert-buffer-substring cache begin end)
-       (when mew-insert-final-newline
-	 (save-excursion
-	   (goto-char (point-max))
-	   (if (not (bolp)) (insert "\n")))))
+      ;; We need to keep properties (e.g. citation color)
+      ;; in a message cache.
+      ;; This must be "insert-buffer-substring".
+      (insert-buffer-substring cache begin end)
+      (when mew-insert-final-newline
+	(save-excursion
+	  (goto-char (point-max))
+	  (if (not (bolp)) (insert "\n"))))
       ;; Page breaks
       (when mew-break-pages
 	(goto-char (point-min))
 	(mew-message-narrow-to-page)))))
 
 (defun mew-mime-text/plain-ext (cache begin end &optional params)
-  ;; called in Summary buffer
+  "To call the Text/Plian function for a text attachment whose
+Content-Type: is Application/Octet-Stream."
+  ;; called in Message buffer
   (when (> end begin)
-    (set-buffer (mew-buffer-message))
-    (mew-elet
-     (mew-erase-buffer)
-     ;; We need to keep properties (e.g. citation color)
-     ;; in a message cache.
-     ;; This must be "insert-buffer-substring".
-     (insert-buffer-substring cache begin end)
-     (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
-		  (or (mew-cache-dinfo-get-decode-broken cache)
-		      (mew-charset-to-cs
-		       (mew-syntax-get-param params "charset"))))
-       (mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv))
-     (when mew-insert-final-newline
-       (save-excursion
-	 (goto-char (point-max))
-	 (if (not (bolp)) (insert "\n"))))
-     ;; Page breaks
-     (when mew-break-pages
-       (goto-char (point-min))
-       (mew-message-narrow-to-page))
-     (mew-summary-display-postscript 'nohook))))
+    (mew-erase-buffer)
+    ;; We need to keep properties (e.g. citation color)
+    ;; in a message cache.
+    ;; This must be "insert-buffer-substring".
+    (insert-buffer-substring cache begin end)
+    (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
+		 (or (mew-cache-dinfo-get-decode-broken cache)
+		     (mew-charset-to-cs
+		      (mew-syntax-get-param params "charset"))))
+      (mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv))
+    (when mew-insert-final-newline
+      (save-excursion
+	(goto-char (point-max))
+	(if (not (bolp)) (insert "\n"))))
+    ;; Page breaks
+    (when mew-break-pages
+      (goto-char (point-min))
+      (mew-message-narrow-to-page))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -181,17 +179,16 @@
   (when (> end begin)
     (save-excursion
       (let ((start (point)))
-	(mew-elet
-	 ;; We need to keep composite properties of charset.
-	 ;; This must be "insert-buffer-substring".
-	 (insert-buffer-substring cache begin end)
-	 ;; Highlight
-	 (when mew-use-text/enriched
-	   (condition-case nil
-	       ;; format.el is buggy.
-	       (format-decode-region start (point-max) 'text/enriched)
-	     (args-out-of-range (message nil))) ;; just clear
-	   (enriched-mode 0))))
+	;; We need to keep composite properties of charset.
+	;; This must be "insert-buffer-substring".
+	(insert-buffer-substring cache begin end)
+	;; Highlight
+	(when mew-use-text/enriched
+	  (condition-case nil
+	      ;; format.el is buggy.
+	      (format-decode-region start (point-max) 'text/enriched)
+	    (args-out-of-range (message nil))) ;; just clear
+	  (enriched-mode 0)))
       ;; Page breaks
       (when mew-break-pages
 	(goto-char (point-min))
@@ -206,47 +203,53 @@
   ;; called in Message buffer
   (if (and (symbolp mew-prog-text/html) (fboundp mew-prog-text/html))
       (save-excursion
-	(mew-elet
-	 (let ((start (point)))
-	   ;; We need to keep composite properties of charset.
-	   ;; This must be "insert-buffer-substring".
-	   (insert-buffer-substring cache begin end)
-	   (funcall mew-prog-text/html start (point-max)))))
-    (mew-elet
-     (insert " #     # ####### #     # #\n"
-	     " #     #    #    ##   ## #\n"
-	     " #     #    #    # # # # #\n"
-	     " #######    #    #  #  # #\n"
-	     " #     #    #    #     # #\n"
-	     " #     #    #    #     # #\n"
-	     " #     #    #    #     # #######\n"
-	     "\n\n")
-     (mew-insert "Size:\t\t%d bytes\n" (mew-region-bytes begin end cache))
-     (mew-insert "Browser:\t%s\n" (if (stringp mew-prog-text/html-ext)
-				      mew-prog-text/html-ext
-				    "none"))
-     (insert "\n")
-     (mew-mime-part-messages t))))
+	(let ((start (point))
+	      folder)
+	  ;; We need to keep composite properties of charset.
+	  ;; This must be "insert-buffer-substring".
+	  (insert-buffer-substring cache begin end)
+	  (save-excursion
+	    (set-buffer cache)
+	    (setq folder (mew-cinfo-get-fld)))
+	  (if (or mew-use-text/html
+		  (and mew-use-text/html-list
+		       (mew-folder-spec folder
+					mew-use-text/html-list
+					mew-use-text/html-string-type
+					mew-use-text/html-list-type)))
+	      (funcall mew-prog-text/html start (point-max))
+	    (mew-message-for-summary "To parse HTML, type '\\[mew-summary-analyze-again]'"))))
+    (insert " #     # ####### #     # #\n"
+	    " #     #    #    ##   ## #\n"
+	    " #     #    #    # # # # #\n"
+	    " #######    #    #  #  # #\n"
+	    " #     #    #    #     # #\n"
+	    " #     #    #    #     # #\n"
+	    " #     #    #    #     # #######\n"
+	    "\n\n")
+    (mew-insert "Size:\t\t%d bytes\n" (mew-region-bytes begin end cache))
+    (mew-insert "Browser:\t%s\n" (if (stringp mew-prog-text/html-ext)
+				     mew-prog-text/html-ext
+				   "none"))
+    (insert "\n")
+    (mew-mime-part-messages t)))
 
 (defun mew-mime-text/html-ext (cache begin end &optional params)
-  ;; called in Summary buffer
+  ;; called in Message buffer
   (cond
    ((and (symbolp mew-prog-text/html-ext) (fboundp mew-prog-text/html-ext))
     (message "Displaying HTML...")
-    (set-buffer (mew-buffer-message))
     (save-excursion
-      (mew-elet
-       (mew-erase-buffer)
-       ;; We need to keep composite properties of charset.
-       ;; This must be "insert-buffer-substring".
-       (insert-buffer-substring cache begin end)
-       (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
-		    (or (mew-cache-dinfo-get-decode-broken cache)
-			(mew-charset-to-cs
-			 (mew-syntax-get-param params "charset"))))
-	 (mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv))
-       (funcall mew-prog-text/html-ext (point-min) (point-max))
-       (mew-summary-display-postscript 'nohook)))
+      (mew-erase-buffer)
+      ;; We need to keep composite properties of charset.
+      ;; This must be "insert-buffer-substring".
+      (insert-buffer-substring cache begin end)
+      (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
+		   (or (mew-cache-dinfo-get-decode-broken cache)
+		       (mew-charset-to-cs
+			(mew-syntax-get-param params "charset"))))
+	(mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv))
+      (funcall mew-prog-text/html-ext (point-min) (point-max)))
     (message "Displaying HTML...done"))
    ((stringp mew-prog-text/html-ext)
     (when (> end begin)
@@ -266,6 +269,9 @@
 	  (unless esqp (setq arg (cons file arg)))
 	  (setq arg (nreverse arg))
 	  (setq wcs (mew-text/html-detect-cs begin end params))
+	  (unless (mew-coding-system-p wcs)
+	    (setq wcs (mew-charset-to-cs
+		       (mew-syntax-get-param params "charset"))))
 	  (mew-frwlet
 	   mew-cs-dummy wcs
 	   (write-region begin end file nil 'no-msg)
@@ -306,47 +312,44 @@
   ;; called in Message buffer
   (if (and (symbolp mew-prog-text/xml) (fboundp mew-prog-text/xml))
       (save-excursion
-	(mew-elet
-	 (let ((start (point)))
-	   ;; We need to keep composite properties of charset.
-	   ;; This must be "insert-buffer-substring".
-	   (insert-buffer-substring cache begin end)
-	   (funcall mew-prog-text/xml start (point-max)))))
-    (mew-elet
-     (insert " #     # #     # #\n"
-	     "  #   #  ##   ## #\n"
-	     "   # #   # # # # #\n"
-	     "    #    #  #  # #\n"
-	     "   # #   #     # #\n"
-	     "  #   #  #     # #\n"
-	     " #     # #     # #######\n"
-	     "\n\n")
-     (mew-insert "Size:\t\t%d bytes\n" (mew-region-bytes begin end cache))
-     (mew-insert "Browser:\t%s\n" (if (stringp mew-prog-text/xml-ext)
-				      mew-prog-text/xml-ext
-				    "none"))
-     (insert "\n")
-     (mew-mime-part-messages t))))
+	(let ((start (point)))
+	  ;; We need to keep composite properties of charset.
+	  ;; This must be "insert-buffer-substring".
+	  (insert-buffer-substring cache begin end)
+	  (if mew-use-text/xml
+	      (funcall mew-prog-text/xml start (point-max))
+	    (mew-message-for-summary "To parse XML, type '\\[mew-summary-analyze-again]'"))))
+    (insert " #     # #     # #\n"
+	    "  #   #  ##   ## #\n"
+	    "   # #   # # # # #\n"
+	    "    #    #  #  # #\n"
+	    "   # #   #     # #\n"
+	    "  #   #  #     # #\n"
+	    " #     # #     # #######\n"
+	    "\n\n")
+    (mew-insert "Size:\t\t%d bytes\n" (mew-region-bytes begin end cache))
+    (mew-insert "Browser:\t%s\n" (if (stringp mew-prog-text/xml-ext)
+				     mew-prog-text/xml-ext
+				   "none"))
+    (insert "\n")
+    (mew-mime-part-messages t)))
 
 (defun mew-mime-text/xml-ext (cache begin end &optional params)
-  ;; called in Summary buffer
+  ;; called in Message buffer
   (cond
    ((and (symbolp mew-prog-text/xml-ext) (fboundp mew-prog-text/xml-ext))
     (message "Displaying XML...")
-    (set-buffer (mew-buffer-message))
     (save-excursion
-      (mew-elet
-       (mew-erase-buffer)
-       ;; We need to keep composite properties of charset.
-       ;; This must be "insert-buffer-substring".
-       (insert-buffer-substring cache begin end)
-       (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
-		    (or (mew-cache-dinfo-get-decode-broken cache)
-			(mew-charset-to-cs
-			 (mew-syntax-get-param params "charset"))))
-	 (mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv))
-       (funcall mew-prog-text/xml-ext (point-min) (point-max))
-       (mew-summary-display-postscript 'nohook)))
+      (mew-erase-buffer)
+      ;; We need to keep composite properties of charset.
+      ;; This must be "insert-buffer-substring".
+      (insert-buffer-substring cache begin end)
+      (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
+		   (or (mew-cache-dinfo-get-decode-broken cache)
+		       (mew-charset-to-cs
+			(mew-syntax-get-param params "charset"))))
+	(mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv))
+      (funcall mew-prog-text/xml-ext (point-min) (point-max)))
     (message "Displaying XML...done"))
    ((stringp mew-prog-text/xml-ext)
     (when (> end begin)
@@ -366,6 +369,9 @@
 	  (unless esqp (setq arg (cons file arg)))
 	  (setq arg (nreverse arg))
 	  (setq wcs (mew-text/xml-detect-cs begin end params))
+	  (unless (mew-coding-system-p wcs)
+	    (setq wcs (mew-charset-to-cs
+		       (mew-syntax-get-param params "charset"))))
 	  (mew-frwlet
 	   mew-cs-dummy wcs
 	   (write-region begin end file nil 'no-msg)
@@ -400,42 +406,37 @@
   (if (and (symbolp mew-prog-application/xml)
 	   (fboundp mew-prog-application/xml))
       (save-excursion
-	(mew-elet
-	 (let ((start (point)))
-	   ;; We need to keep composite properties of charset.
-	   ;; This must be "insert-buffer-substring".
-	   (insert-buffer-substring cache begin end)
-	   (funcall mew-prog-application/xml start (point-max)))))
-    (mew-elet
-     (insert " #     # #     # #\n"
-	     "  #   #  ##   ## #\n"
-	     "   # #   # # # # #\n"
-	     "    #    #  #  # #\n"
-	     "   # #   #     # #\n"
-	     "  #   #  #     # #\n"
-	     " #     # #     # #######\n"
-	     "\n\n")
-     (mew-insert "Size:\t\t%d bytes\n" (mew-region-bytes begin end cache))
-     (mew-insert "Browser:\t%s\n" (if (stringp mew-prog-application/xml-ext)
-				      mew-prog-application/xml-ext
-				    "none"))
-     (insert "\n")
-     (mew-mime-part-messages t))))
+	(let ((start (point)))
+	  ;; We need to keep composite properties of charset.
+	  ;; This must be "insert-buffer-substring".
+	  (insert-buffer-substring cache begin end)
+	  (funcall mew-prog-application/xml start (point-max))))
+    (insert " #     # #     # #\n"
+	    "  #   #  ##   ## #\n"
+	    "   # #   # # # # #\n"
+	    "    #    #  #  # #\n"
+	    "   # #   #     # #\n"
+	    "  #   #  #     # #\n"
+	    " #     # #     # #######\n"
+	    "\n\n")
+    (mew-insert "Size:\t\t%d bytes\n" (mew-region-bytes begin end cache))
+    (mew-insert "Browser:\t%s\n" (if (stringp mew-prog-application/xml-ext)
+				     mew-prog-application/xml-ext
+				   "none"))
+    (insert "\n")
+    (mew-mime-part-messages t)))
 
 (defun mew-mime-application/xml-ext (cache begin end &optional params)
-  ;; called in Summary buffer
+  ;; called in Message buffer
   (cond
    ((and (symbolp mew-prog-application/xml-ext) (fboundp mew-prog-application/xml-ext))
     (message "Displaying XML...")
-    (set-buffer (mew-buffer-message))
     (save-excursion
-      (mew-elet
-       (mew-erase-buffer)
-       ;; We need to keep composite properties of charset.
-       ;; This must be "insert-buffer-substring".
-       (insert-buffer-substring cache begin end)
-       (funcall mew-prog-application/xml-ext (point-min) (point-max))
-       (mew-summary-display-postscript 'nohook)))
+      (mew-erase-buffer)
+      ;; We need to keep composite properties of charset.
+      ;; This must be "insert-buffer-substring".
+      (insert-buffer-substring cache begin end)
+      (funcall mew-prog-application/xml-ext (point-min) (point-max)))
     (message "Displaying XML...done"))
    ((stringp mew-prog-application/xml-ext)
     (when (> end begin)
@@ -468,23 +469,25 @@
 ;;; Image/*
 ;;;
 
-(defun mew-mime-image/* (cache begin end &optional params ct)
+(defun mew-mime-image/* (cache begin end &optional params fname ct cte)
   (let* ((format (mew-mime-image-format-name ct)))
     (if (mew-image-inline-p format)
 	(mew-mime-image cache begin end format)
-      (mew-elet
-       (insert "  #####  #     #    #     #####  #######\n"
-	       "    #    ##   ##   # #   #     # #      \n"
-	       "    #    # # # #  #   #  #       #      \n"
-	       "    #    #  #  # #     # #  #### #######\n"
-	       "    #    #     # ####### #     # #      \n"
-	       "    #    #     # #     # #     # #      \n"
-	       "  #####  #     # #     #  #####  #######\n"
-	       "\n\n")
-       (mew-insert "Content-Type:\t%s\n" ct)
-       (mew-insert "Program:\t%s\n" mew-prog-image/*-ext)
-       (insert "\n")
-       (mew-mime-part-messages t)))))
+      (insert "  #####  #     #    #     #####  #######\n"
+	      "    #    ##   ##   # #   #     # #      \n"
+	      "    #    # # # #  #   #  #       #      \n"
+	      "    #    #  #  # #     # #  #### #######\n"
+	      "    #    #     # ####### #     # #      \n"
+	      "    #    #     # #     # #     # #      \n"
+	      "  #####  #     # #     #  #####  #######\n"
+	      "\n\n")
+      (mew-insert "Content-Type:\t%s\n" ct)
+      (mew-insert "Encoding:\t%s\n" cte)
+      (mew-insert "Size:\t\t%d bytes\n" (mew-region-bytes begin end cache))
+      (mew-insert "Filename:\t%s\n" fname)
+      (mew-insert "Program:\t%s\n" mew-prog-image/*-ext)
+      (insert "\n")
+      (mew-mime-part-messages t))))
 
 (defun mew-mime-image/*-ext (cache begin end &optional params fname)
   (let ((file (mew-make-temp-name fname)))
@@ -504,33 +507,31 @@
   ;; called in Message buffer
   (when (> end begin)
     (save-excursion
-      (mew-elet
-       ;; We need to keep properties of a header.
-       ;; This must be "insert-buffer-substring".
-       (insert-buffer-substring cache begin end)
-       (mew-header-arrange (point-min) (point-max))))))
+      ;; We need to keep properties of a header.
+      ;; This must be "insert-buffer-substring".
+      (insert-buffer-substring cache begin end)
+      (mew-header-arrange (point-min) (point-max)))))
 
 (defun mew-mime-message/rfc822 (cache part)
   ;; called in Message buffer
-  (mew-elet
-   (let* ((hbeg (mew-syntax-get-begin part))
-	  (hend (mew-syntax-get-end   part))
-	  (body (mew-syntax-get-part part)))
-     ;; We need to keep properties of a header.
-     ;; This must be "insert-buffer-substring".
-     (insert-buffer-substring cache hbeg hend)
-     (mew-header-arrange (point-min) (point-max))
-     (cond
-      ;; Displaying the text/plain body or the first part of 
-      ;; top level multipart if it is text/plain.
-      ;; see also mew-syntax-singlepart
-      ((mew-syntax-singlepart-p body)
-       (mew-mime-part cache body nil)) ;; nil is single
-      ((mew-syntax-multipart-p body)
-       (let* ((first (mew-syntax-get-part body))
-	      (ct (mew-syntax-get-value (mew-syntax-get-ct first) 'cap)))
-	 (when (and (mew-xinfo-get-text-body) (mew-ct-textp ct))
-	   (mew-mime-part cache first nil)))))))) ;; nil is single
+  (let* ((hbeg (mew-syntax-get-begin part))
+	 (hend (mew-syntax-get-end   part))
+	 (body (mew-syntax-get-part part)))
+    ;; We need to keep properties of a header.
+    ;; This must be "insert-buffer-substring".
+    (insert-buffer-substring cache hbeg hend)
+    (mew-header-arrange (point-min) (point-max))
+    (cond
+     ;; Displaying the text/plain body or the first part of 
+     ;; top level multipart if it is text/plain.
+     ;; see also mew-syntax-singlepart
+     ((mew-syntax-singlepart-p body)
+      (mew-mime-part cache body nil)) ;; nil is single
+     ((mew-syntax-multipart-p body)
+      (let* ((first (mew-syntax-get-part body))
+	     (ct (mew-syntax-get-value (mew-syntax-get-ct first) 'cap)))
+	(when (and (mew-xinfo-get-text-body) (mew-ct-textp ct))
+	  (mew-mime-part cache first nil))))))) ;; nil is single
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -538,26 +539,25 @@
 ;;;
 
 (defun mew-mime-application/octet-stream (cache begin end &optional params ct cte fl func)
-  (mew-elet
-   (insert " ######    ###   #     #    #    ######  #     #\n"
-	   " #     #    #    ##    #   # #   #     #  #   #\n"
-	   " #     #    #    # #   #  #   #  #     #   # #\n"
-	   " ######     #    #  #  # #     # ######     #\n"
-	   " #     #    #    #   # # ####### #   #      #\n"
-	   " #     #    #    #    ## #     # #    #     #\n"
-	   " ######    ###   #     # #     # #     #    #\n"
-	   "\n\n")
-   (mew-insert "Content-Type:\t%s\n" ct)
-   (mew-insert "Encoding:\t%s\n" cte)
-   (when params
-     (mew-insert
-      "Parameters:\t%s\n"
-      (mapconcat (lambda (x) (concat (nth 0 x) "=" (nth 1 x))) params ", ")))
-   (mew-insert "Size:\t\t%d bytes\n" (mew-region-bytes begin end cache))
-   (mew-insert "Filename:\t%s\n" fl)
-   (when func (mew-insert "Function:\t%s\n" func))
-   (insert "\n")
-   (mew-mime-part-messages func)))
+  (insert " ######    ###   #     #    #    ######  #     #\n"
+	  " #     #    #    ##    #   # #   #     #  #   #\n"
+	  " #     #    #    # #   #  #   #  #     #   # #\n"
+	  " ######     #    #  #  # #     # ######     #\n"
+	  " #     #    #    #   # # ####### #   #      #\n"
+	  " #     #    #    #    ## #     # #    #     #\n"
+	  " ######    ###   #     # #     # #     #    #\n"
+	  "\n\n")
+  (mew-insert "Content-Type:\t%s\n" ct)
+  (mew-insert "Encoding:\t%s\n" cte)
+  (when params
+    (mew-insert
+     "Parameters:\t%s\n"
+     (mapconcat (lambda (x) (concat (nth 0 x) "=" (nth 1 x))) params ", ")))
+  (mew-insert "Size:\t\t%d bytes\n" (mew-region-bytes begin end cache))
+  (mew-insert "Filename:\t%s\n" fl)
+  (when func (mew-insert "Function:\t%s\n" func))
+  (insert "\n")
+  (mew-mime-part-messages func))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -578,27 +578,18 @@
 
 (defun mew-mime-application/msword-ext (cache begin end &optional parameter)
   (save-excursion
-    (set-buffer (mew-buffer-message))
-    (mew-elet
-     (mew-erase-buffer)
-     (mew-mime-application/msword cache begin end parameter)
-     (mew-summary-display-postscript 'nohook))))
+    (mew-erase-buffer)
+    (mew-mime-application/msword cache begin end parameter)))
 
 (defun mew-mime-application/msexcel-ext (cache begin end &optional parameter)
   (save-excursion
-    (set-buffer (mew-buffer-message))
-    (mew-elet
-     (mew-erase-buffer)
-     (mew-mime-application/msexcel cache begin end parameter)
-     (mew-summary-display-postscript 'nohook))))
+    (mew-erase-buffer)
+    (mew-mime-application/msexcel cache begin end parameter)))
 
 (defun mew-mime-application/mspowerpoint-ext (cache begin end &optional parameter)
   (save-excursion
-    (set-buffer (mew-buffer-message))
-    (mew-elet
-     (mew-erase-buffer)
-     (mew-mime-application/mspowerpoint cache begin end parameter)
-     (mew-summary-display-postscript 'nohook))))
+    (mew-erase-buffer)
+    (mew-mime-application/mspowerpoint cache begin end parameter)))
 
 (defun mew-mime-application/msoffice (prog cache begin end &optional parameter)
   (let ((doit t) file1 file2)
@@ -607,18 +598,18 @@
 	  (require 'un-define)
 	(file-error
 	 (setq doit nil)
-	 (mew-elet (insert "To display this, install Mule-UCS for UTF-8.\n")))))
+	 (insert "To display this, install Mule-UCS for UTF-8.\n"))))
     (unless (mew-which-exec prog)
       (setq doit nil)
-      (mew-elet (insert "To display this, install \"" prog "\".\n")))
+      (insert "To display this, install \"" prog "\".\n"))
     (condition-case nil
 	(require 'mew-w3m)
       (file-error
        (setq doit nil)
-       (mew-elet (insert "To display this, install \"w3m.el\".\n"))))
+       (insert "To display this, install \"w3m.el\".\n")))
     (if (not doit)
 	(progn
-	  (mew-elet (insert "\n"))
+	  (insert "\n")
 	  (mew-mime-part-messages nil))
       (message "Displaying an MS document...")
       (setq file1 (mew-make-temp-name))
@@ -649,6 +640,41 @@
 		    (w3m-expand-file-name-as-url (file-name-directory file2))))
       (message "Displaying an MS document...done"))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Application/Ms-Tnef
+;;;
+
+(defun mew-mime-application-ms-tnef (cache begin end &optional parameter)
+  (insert " ####### #     # ####### ########\n"
+	  "    #    ##    # #       #\n"
+	  "    #    # #   # #       #\n"
+	  "    #    #  #  # ####### ######\n"
+	  "    #    #   # # #       #\n"
+	  "    #    #    ## #       #\n"
+	  "    #    #     # ####### #\n"
+	  "\n\n")
+  (mew-insert-manual
+   "To extract files, type "
+   "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'.\n"))
+
+(defvar mew-prog-tnef "tnef")
+
+(defun mew-mime-application-ms-tnef-ext (cache begin end &optional parameter)
+  ;; called in Message buffer
+  (let ((file (mew-make-temp-name))
+	(dir (mew-input-directory-name mew-home)))
+    (if (not (mew-which-exec mew-prog-tnef))
+	(message "'%s' not found" mew-prog-tnef)
+      (save-excursion
+	(set-buffer cache)
+	(mew-plet
+	 (write-region begin end file nil 'no-msg)))
+      (mew-erase-buffer)
+      (insert "Extracted: \n\n")
+      (call-process mew-prog-tnef file t nil "--verbose" "-C" dir)
+      (mew-delete-file file))))
+
 (provide 'mew-mime)
 
 ;;; Copyright Notice:
