@@ -70,7 +70,6 @@ at completions."
 
 (defvar mew-input-folder-hist nil)
 
-(defvar mew-input-folder-circular-case nil)
 (defvar mew-input-folder-refile nil)
 
 (defvar mew-input-folder-search-direction nil)
@@ -140,7 +139,7 @@ at completions."
     (if min (goto-char (point-min)))))
 
 (defun mew-input-folder-search-setup1 (&optional min)
-  (let (case:fld case fld alist)
+  (let (case:folder case folder alist)
     (save-excursion
       (goto-char (point-max))
       (if (search-backward "," nil t)
@@ -148,23 +147,23 @@ at completions."
 		(- (match-end 0) (mew-minibuf-point-min)))))
     (setq mew-input-folder-search-original
 	  (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
-    (setq case:fld mew-input-folder-search-original)
+    (setq case:folder mew-input-folder-search-original)
     (if mew-input-folder-search-point
-	(setq case:fld (substring case:fld mew-input-folder-search-point)))
-    (setq case (car (mew-folder-case case:fld)))
-    (if case
-	(setq mew-input-folder-search-point
-	      (+ (or mew-input-folder-search-point 0) (length case) 1))) ;; ":"
-    (setq fld (mew-folder-folder case:fld))
+	(setq case:folder (substring case:folder mew-input-folder-search-point)))
+    (setq case (mew-case:folder-case case:folder))
+    (setq folder (mew-case:folder-folder case:folder))
+    (when case
+      (setq mew-input-folder-search-point
+	    (+ (or mew-input-folder-search-point 0) (length case) 1))) ;; ":"
     (mew-input-folder-display)
-    (if (or (null fld) (string= fld ""))
-	(setq fld (mew-proto case)))
+    (if (or (null folder) (string= folder ""))
+	(setq folder (mew-proto case)))
     (cond
-     ((mew-folder-popp fld)
+     ((mew-folder-popp folder)
       (setq alist (mew-pop-folder-alist)))
-     ((mew-folder-nntpp fld)
+     ((mew-folder-nntpp folder)
       (setq alist (mew-nntp-folder-alist case)))
-     ((mew-folder-imapp fld)
+     ((mew-folder-imapp folder)
       (setq alist (mew-imap-folder-alist case)))
      (t
       (setq alist (mew-local-folder-alist))))
@@ -286,6 +285,9 @@ at completions."
 ;;;
 
 (defun mew-input-folder-exit-minibuffer ()
+  "If called in folder search, this gets back to minibuffer to input
+folders. If called in minibuffer to input folders, this
+gets out of minibuffer."
   (interactive)
   (if mew-input-folder-search-direction
       (mew-elet
@@ -316,12 +318,11 @@ Otherwise, it exits minibuffer."
     (abort-recursive-edit)))
 
 (defun mew-input-folder-comma ()
-  "This function inserts ',' in a single folder input
-and does not insert anything in a multiple folder input."
+  "This function inserts ',' on multiple folder input
+and does not insert anything on single folder input."
   (interactive)
-  (if (and (null mew-input-folder-search-direction)
-	   mew-input-folder-search-multi)
-      (insert ",")))
+  (unless mew-input-folder-search-direction
+    (insert ",")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -495,8 +496,10 @@ it is deleted automatically."
 	(if (and (mew-folder-popp folder) ;; not $inbox
 		 (not (string= mew-pop-inbox-folder folder)))
 	    (throw 'continue nil))
+	(when (string-match " " folder)
+	  (mew-warn "SPC is not allowed!")
+	  (throw 'continue nil))
 	(setq case:folder (mew-canonicalize-case-folder case:folder))
-	(setq case:folder (directory-file-name case:folder))
 	(setq ret (cons case:folder ret))))
     (nreverse ret)))
 
@@ -507,7 +510,7 @@ it is deleted automatically."
   (let* ((default folder)
 	 (init (mew-folder-prefix folder))
 	 (mew-input-complete-function 'mew-complete-local-folder)
-	 (mew-input-folder-circular-case t)
+	 (mew-circular-complete-function 'mew-circular-complete-case:)
 	 (ret (read-from-minibuffer (format "Folder name (%s): " default)
 				    init mew-input-folder-map nil
 				    'mew-input-folder-hist)))
@@ -519,10 +522,10 @@ it is deleted automatically."
   "Input a folder from the minibuffer."
   (mew-input-clear)
   (mew-input-folder-clean-up)
-  (let* ((default (mew-folder-case-folder case folder))
-	 (init (mew-folder-case-folder case (mew-folder-prefix folder)))
+  (let* ((default (mew-case-folder case folder))
+	 (init (mew-case-folder case (mew-folder-prefix folder)))
 	 (mew-input-complete-function 'mew-complete-folder)
-	 (mew-input-folder-circular-case t)
+	 (mew-circular-complete-function 'mew-circular-complete-case:)
 	 ;; mew-inherit-case must be nil
 	 (ret (read-from-minibuffer (format "Folder name (%s): " default)
 				    init mew-input-folder-map nil
@@ -537,7 +540,7 @@ it is deleted automatically."
   (mew-input-folder-clean-up)
   (let* ((init (mew-canonicalize-case-folder case:folder))
 	 (mew-input-complete-function 'mew-complete-folder)
-	 (mew-input-folder-circular-case t)
+	 (mew-circular-complete-function 'mew-circular-complete-case:)
 	 (mew-input-folder-search-multi t)
 	 ;; mew-inherit-case must be nil
 	 (ret (read-from-minibuffer "Folder name: "
@@ -570,8 +573,10 @@ it is deleted automatically."
 	(if (and (= (length folder) 1)
 		 (member folder mew-folder-prefixes)) ;; "prefix" only
 	    (throw 'continue nil))
+	(when (string-match " " folder)
+	  (mew-warn "SPC is not allowed!")
+	  (throw 'continue nil))
 	(setq folder (mew-canonicalize-case-folder folder))
-	(setq folder (directory-file-name folder))
 	(setq ret (cons folder ret))))
     (nreverse ret)))
 
@@ -648,11 +653,12 @@ it is deleted automatically."
 (defun mew-input-pick-pattern ()
   (mew-input-clear)
   (let ((mew-input-complete-function 'mew-complete-pick-pattern)
+	(mew-circular-complete-function 'mew-circular-complete-pick-pattern)
 	(keymap (copy-keymap mew-input-map)))
     (define-key keymap " " nil)
     (mew-pick-macro-expand-string
      (read-from-minibuffer "Pick pattern: "
-			   mew-pick-default-field
+			   (car mew-pick-pattern-list)
 			   keymap
 			   nil
 			   'mew-input-pick-pattern-hist))))
@@ -662,8 +668,12 @@ it is deleted automatically."
 ;;; Grep pattern
 ;;;
 
+(defvar mew-input-grep-pattern-hist nil)
+
 (defun mew-input-grep-pattern ()
-  (read-string "Grep pattern: "))
+  (read-string "Grep pattern: "
+               (car-safe mew-input-grep-pattern-hist)
+               'mew-input-grep-pattern-hist))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -931,10 +941,13 @@ it is deleted automatically."
           nil)
       t)))
 
+(defvar mew-input-case-hist nil)
+
 (defun mew-input-case (default msg &optional edit)
   (mew-input-clear)
   (unless default (setq default mew-case-default))
   (let ((mew-input-complete-function 'mew-complete-case)
+	(mew-circular-complete-function 'mew-circular-complete-case)
 	(mew-input-exit-minibuffer-function 'mew-input-case-check)
 	(mew-input-comma-function 'mew-input-case-check)
 	case lst ret)
@@ -942,11 +955,15 @@ it is deleted automatically."
 	(setq case (read-from-minibuffer
 		    (format "%s case value: " msg)
 		    default
-		    mew-input-map))
+		    mew-input-map
+                    nil
+                    'mew-input-case-hist))
       (setq case (read-from-minibuffer
 		  (format "%s case value (%s): " msg default)
 		  ""
-		  mew-input-map)))
+		  mew-input-map
+                  nil
+                  'mew-input-case-hist)))
     (if (string= case "")
 	default
       (setq lst (mew-split case ?,))
@@ -969,7 +986,7 @@ it is deleted automatically."
 	  (setq cursor-in-echo-area t)
 	  (setq char (read-char))
 	  (unless (char-equal char ?\r)
-	    (message "Input mark: %s" (char-to-string char))))
+	    (message "Input mark: %c" char)))
       (setq cursor-in-echo-area ociea))
     (cond
      ((char-equal char ?\r) char)
