@@ -280,12 +280,14 @@
 	 ((< sw width)
           (if nopad
               str
-            (concat str (make-string (- width sw) 32))))
+            (concat str (make-string (- width sw) mew-sp))))
 	 (t
 	  (if cnt (setq width (- width 2)))
 	  (catch 'loop
 	    (while t
 	      (setq cw (char-width (aref str i)))
+	      (if (get-text-property i 'composition str) ;; for Emacs 21
+		  (setq i (1+ i)))
 	      (if (> (+ w cw) width)
 		  (throw 'loop t))
 	      (setq w (+ w cw))
@@ -299,8 +301,8 @@
 	    ;; workaround for the bug of Emacs 20.7
 	    (setq safe-w (logand (- width w) 255))
 	    (if cnt
-		(concat (substring str 0 i) (make-string safe-w 32) "..")
-	      (concat (substring str 0 i) (make-string safe-w 32)))))))
+		(concat (substring str 0 i) (make-string safe-w mew-sp) "..")
+	      (concat (substring str 0 i) (make-string safe-w mew-sp)))))))
       (error
        (if (> (length mew-error-broken-string) width)
 	   (substring mew-error-broken-string 0 width)
