@@ -65,7 +65,8 @@
 (defun mew-summary-unlock ()
   (setq mew-summary-buffer-process nil)
   (setq mew-summary-buffer-process-status nil)
-  (setq mew-summary-buffer-secure-process nil))
+  (setq mew-summary-buffer-secure-process nil)
+  (force-mode-line-update))
 
 (defun mew-net-status-clear (buf)
   (when mew-use-net-status
@@ -84,11 +85,11 @@
 	(setq mew-summary-buffer-process-status (format " %s" status)))
       (setq mew-summary-buffer-secure-process secure))))
 
-(defun mew-net-status1 (buf dttl dcnt &optional secure)
+(defun mew-net-status1 (buf msg ttl cnt &optional secure)
   (when mew-use-net-status
-    (when (<= dcnt dttl)
-      (let ((substatus (format "%d/%d" dcnt dttl)))
-	(mew-net-status buf "Deleting" substatus secure)))))
+    (when (<= cnt ttl)
+      (let ((substatus (format "%02d%%" (/ (* cnt 100) ttl))))
+	(mew-net-status buf msg substatus secure)))))
 
 (defvar mew-net-status-percent-size 51200)
 
@@ -214,7 +215,7 @@
     (if ent
 	(setcdr ent uidl)
       (setq mew-net-uidl-db (cons (cons tag uidl) mew-net-uidl-db)))
-    (mew-lisp-save mew-net-uidl-file mew-net-uidl-db)))
+    (mew-lisp-save mew-net-uidl-file mew-net-uidl-db nil 'unlimit)))
 
 (defun mew-net-setup ()
   (setq mew-net-uidl-db (mew-lisp-load mew-net-uidl-file))
@@ -222,7 +223,7 @@
 
 (defun mew-net-clean-up ()
   (remove-hook 'kill-emacs-hook 'mew-net-clean-up)
-  (mew-lisp-save mew-net-uidl-file mew-net-uidl-db))
+  (mew-lisp-save mew-net-uidl-file mew-net-uidl-db nil 'unlimit))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -251,17 +252,29 @@
 ;;; Queue
 ;;;
 
+(defun mew-queue-check-new-message (file)
+  (if (not (file-exists-p (concat file mew-queue-work-suffix)))
+      file
+    (let* ((dir (file-name-directory file))
+	   (num (file-name-nondirectory file))
+	   (n (1+ (string-to-int num))))
+      (while (or (file-exists-p (format "%s%d" dir n))
+		 (file-exists-p (format "%s%d%s" dir n mew-queue-work-suffix)))
+	(setq n (1+ n)))
+      (format "%s%d" dir n))))
+
 (defun mew-queue-enqueue (work qfld) ;; 1.wrk or draft/1
   (let (orig info iwrk)
     (if (not (string-match (concat mew-queue-work-suffix "$") work))
 	;; draft/1
-	(setq orig (mew-folder-new-message qfld))
+	(setq orig (mew-queue-check-new-message (mew-folder-new-message qfld)))
       ;; 1.wrk
       (setq orig (file-name-sans-extension work)) ;; 1
       (setq iwrk (concat orig mew-queue-info-suffix mew-queue-work-suffix))
       (mew-delete-file iwrk) ;; 1.mqi.wrk
-      (if (file-exists-p orig) (setq orig (mew-folder-new-message qfld))))
-    (rename-file work orig 'override)
+      (if (file-exists-p orig)
+	  (setq orig (mew-queue-check-new-message (mew-folder-new-message qfld)))))
+    (rename-file work orig) ;; An error is signaled if orig exists.
     (unless mew-use-nfs-hack
       ;; This cause an error when using NFS.
       (mew-set-file-modes orig))
@@ -274,13 +287,14 @@
 	 (info (concat orig mew-imapq-info-suffix)) ;; 1.iqi
 	 (iwrk (concat info mew-queue-work-suffix))) ;; 1.iqi.wrk
     (when (file-exists-p orig)
-      (setq orig (mew-folder-new-message dir))
+      (setq orig (mew-queue-check-new-message (mew-folder-new-message dir)))
       (setq info (concat orig mew-imapq-info-suffix)))
-    (rename-file work orig 'override)
+    (rename-file work orig) ;; An error is signaled if orig exists.
     (rename-file iwrk info 'override)))
 
-(defun mew-queue-insert-file (pnm n file) ;; 1
-  (let* ((work (concat file mew-queue-work-suffix)) ;; 1.wrk
+(defun mew-queue-insert-file (pnm n qfld msg) ;; 1
+  (let* ((file (mew-expand-folder qfld msg))
+	 (work (concat file mew-queue-work-suffix)) ;; 1.wrk
 	 (info (concat file mew-queue-info-suffix)) ;; 1.mqi
 	 (iwrk (concat info mew-queue-work-suffix)) ;; 1.mqi.wrk
 	 (i 0) buf data)
@@ -298,12 +312,12 @@
       (rename-file info iwrk 'override)
       t)))
 
-(defun mew-queue-get-next (pnm msgs n func)
+(defun mew-queue-get-next (pnm qfld msgs n func)
   (let* (msg flushp)
     (while (and msgs (not flushp))
       (setq msg (car msgs))
       (setq msgs (cdr msgs))
-      (if (mew-queue-insert-file pnm n msg) (setq flushp t)))
+      (if (mew-queue-insert-file pnm n qfld msg) (setq flushp t)))
     (if (fboundp func) (funcall func pnm msgs)) ;; set-messages
     flushp))
 
@@ -322,8 +336,15 @@
 ;;; Fcc:
 ;;;
 
-(defun mew-net-fcc-message (flds file)
-  (let (fcc-file link fld)
+(defun mew-net-fcc-message (case fcc file)
+  (let (flds fcc-file link fld imapp)
+    (while fcc
+      (if (mew-folder-imapp (car fcc))
+	  (progn
+	    (setq flds (cons (mew-folder-imap-to-fcc case (car fcc)) flds))
+	    (setq imapp t))
+	(setq flds (cons (car fcc) flds)))
+      (setq fcc (cdr fcc)))
     (catch 'loop
       (while flds
 	(setq fld (car flds))
@@ -342,7 +363,8 @@
       (setq link (mew-folder-new-message fld))
       (when link
 	(mew-link fcc-file link)
-	(mew-touch-folder fld)))))
+	(mew-touch-folder fld)))
+    imapp))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -379,6 +401,7 @@
 	 (msgs (mew-dir-messages dir mew-regex-message-files3)))
     (mew-summary-unlink-msgs bnm msgs)
     (mew-erase-buffer)
+    (setq mew-summary-buffer-raw nil)
     (mew-summary-folder-cache-save)))
 
 (defun mew-net-invalid-cache-clean ()
@@ -388,7 +411,7 @@
 	 beg)
     (save-excursion
       (goto-char (point-min))
-      (when (re-search-forward mew-regex-msg-invalid nil t)
+      (when (re-search-forward mew-regex-sumsyn-invalid nil t)
 	(beginning-of-line)
 	(setq beg (point))
 	(goto-char (point-max))
@@ -417,39 +440,38 @@ If 'mew-auto-flush-queue' is non-nil, +queue is flushed.
 If called with '\\[universal-argument]', +queue is not flushed."
   (interactive "P")
   (let* ((case mew-case-input)
+	 (dont (not (mew-use-retrieve case)))
 	 (inbox (mew-inbox-folder case))
 	 (dir (mew-expand-folder inbox))
 	 (mailbox (mew-mailbox-type case)))
-    (unless (file-directory-p dir)
-      (mew-make-directory dir))
-    ;; for mew-summary-exchange-point.
-    (set-marker mew-summary-inbox-position (point) (current-buffer))
-    (mew-summary-switch-to-folder inbox)
-    ;; in the inbox
-    (when (mew-summary-exclusive-p)
-      (mew-window-configure 'summary)
-      ;;
-      (mew-current-set nil nil nil)
-      (mew-decode-syntax-delete)
-      (mew-unhighlight-cursor-line)
-      ;;
-      (mew-summary-retrieve-gap inbox)
-      ;;
-      (goto-char (point-max))
-      (mew-sinfo-set-start-point (point))
-      (mew-sinfo-set-direction 'down)
-      (cond
-       ((eq mailbox 'pop)
-	(mew-pop-retrieve case 'inc inbox (not no-flush)))
-       ((eq mailbox 'imap)
-	(mew-imap-retrieve case 'inc inbox (not no-flush)))
-       ((eq mailbox 'mbox)
-	(mew-mbox-retrieve case 'inc inbox (not no-flush)))))))
+    (if dont
+	(message "This command is not allowed to use")
+      (unless (file-directory-p dir)
+	(mew-make-directory dir))
+      ;; for mew-summary-exchange-point.
+      (set-marker mew-summary-inbox-position (point) (current-buffer))
+      (mew-summary-switch-to-folder inbox)
+      ;; in the inbox
+      (when (mew-summary-exclusive-p)
+	(mew-summary-reset)
+	;;
+	(mew-summary-retrieve-gap inbox)
+	;;
+	(goto-char (point-max))
+	(mew-sinfo-set-start-point (point))
+	(mew-sinfo-set-direction 'down)
+	(cond
+	 ((eq mailbox 'pop)
+	  (mew-pop-retrieve case 'inc inbox (not no-flush)))
+	 ((eq mailbox 'imap)
+	  (mew-imap-retrieve case 'inc inbox (not no-flush)))
+	 ((eq mailbox 'mbox)
+	  (mew-mbox-retrieve case 'inc inbox (not no-flush))))))))
 
 (defun mew-summary-scan-boot (proto case)
   ;; called by mew()
   (let* ((inbox (mew-proto-inbox-folder proto case))
-	 (case:inbox (mew-folder-case-folder case inbox))
+	 (case:inbox (mew-case-folder case inbox))
 	 (dir (mew-expand-folder case:inbox))
 	 mailbox)
     (unless (file-directory-p dir)
@@ -459,11 +481,7 @@ If called with '\\[universal-argument]', +queue is not flushed."
     ;; for mew-summary-exchange-point.
     (set-marker mew-summary-inbox-position (point) (current-buffer))
     (when (mew-summary-exclusive-p)
-      (mew-window-configure 'summary)
-      ;;
-      (mew-current-set nil nil nil)
-      (mew-decode-syntax-delete)
-      (mew-unhighlight-cursor-line)
+      (mew-summary-reset)
       ;;
       (if (mew-folder-localp inbox) (mew-summary-retrieve-gap case:inbox))
       ;;
@@ -516,14 +534,17 @@ the mailbox. When executed with '\\[universal-argument]',
 In a REMOTE folder: case and protocol are determined by the folder.
 The message in the server side is always retained."
   (interactive "P")
-  (mew-summary-msg
-   (mew-summary-only
-    (unless (mew-summary-msg-invalidp)
-      (when (mew-summary-exclusive-p)
-	(let* ((bnm (mew-summary-folder-name 'ext))
+  (mew-summary-only
+   (mew-summary-msg
+    (when (mew-summary-exclusive-p)
+      (when (mew-sumsyn-match mew-regex-sumsyn-long)
+	(let* ((msg (mew-sumsyn-message-number))
+	       (uid (mew-sumsyn-message-uid))
+	       (siz (mew-sumsyn-message-size))
+	       (bnm (mew-summary-folder-name 'ext))
 	       (folder (mew-sinfo-get-folder))
 	       (case (mew-sinfo-get-case))
-	       msg uid siz del rtr rtrs mailbox)
+	       del rtr rtrs mailbox)
 	  (when (mew-folder-localp folder)
 	    (setq case mew-case-input)
 	    (setq mailbox (mew-mailbox-type case))
@@ -533,27 +554,25 @@ The message in the server side is always retained."
 	     ((eq mailbox 'imap)
 	      (setq del (eq (mew-imap-delete case) t))));; delete may be number
 	    (if rev-del (setq del (not del))))
-	  (setq uid (mew-summary-message-uid))
-	  (when uid
-	    (setq msg (mew-summary-message-number))
-	    (setq siz (mew-summary-message-size))
-	    (setq rtr (list uid siz del (list bnm msg)))
-	    (setq rtrs (list rtr)))
-	  (if (not rtrs)
-	      (message "No message to be retrieved")
+	  (setq rtr (list uid siz del (list bnm msg)))
+	  (setq rtrs (list rtr))
+	  (cond
+	   ((not rtrs)
+	    (message "No message to be retrieved"))
+	   ((not (mew-msg-validp msg))
+	    (message "No valid message to be retrieved"))
+	   ((mew-folder-localp folder)
 	    (cond
-	     ((mew-folder-localp folder)
-	      (cond
-	       ((eq mailbox 'pop)
-		(mew-pop-retrieve case 'get bnm rtrs))
-	       ((eq mailbox 'imap)
-		(mew-imap-retrieve case 'get bnm rtrs))))
-	     ((mew-folder-popp folder)
+	     ((eq mailbox 'pop)
 	      (mew-pop-retrieve case 'get bnm rtrs))
-	     ((mew-folder-imapp folder)
-	      (mew-imap-retrieve case 'get bnm rtrs))
-	     ((mew-folder-nntpp folder)
-	      (mew-nntp-retrieve case 'get bnm rtrs))))))))))
+	     ((eq mailbox 'imap)
+	      (mew-imap-retrieve case 'get bnm rtrs))))
+	   ((mew-folder-popp folder)
+	    (mew-pop-retrieve case 'get bnm rtrs))
+	   ((mew-folder-imapp folder)
+	    (mew-imap-retrieve case 'get bnm rtrs))
+	   ((mew-folder-nntpp folder)
+	    (mew-nntp-retrieve case 'get bnm rtrs)))))))))
 
 (defun mew-summary-mark-retrieve-message (&optional rev-del)
   "Retrieve the rest of truncated('T') messages marked with '*'.
@@ -583,13 +602,14 @@ The messages in the server side is always retained."
        (if rev-del (setq del (not del))))
      (save-excursion
        (goto-char (point-min))
-       (while (re-search-forward mew-regex-msg-valid-review nil t)
-	 (setq uid (mew-summary-message-uid))
-	 (when uid
-	   (setq msg (mew-summary-message-number))
-	   (setq siz (mew-summary-message-size))
-	   (setq rtr (list uid siz del (list bnm msg)))
-	   (setq rtrs (cons rtr rtrs)))
+       (while (re-search-forward mew-regex-msg-review nil t)
+	 (when (mew-sumsyn-match mew-regex-sumsyn-long)
+	   (setq uid (mew-sumsyn-message-uid))
+	   (setq msg (mew-sumsyn-message-number))
+	   (setq siz (mew-sumsyn-message-size))
+	   (when (and (mew-msg-validp msg) (mew-msg-truncatedp siz))
+	     (setq rtr (list uid siz del (list bnm msg)))
+	     (setq rtrs (cons rtr rtrs))))
 	 (forward-line)))
      (if (not rtrs)
 	 (message "No message to be retrieved")
@@ -608,7 +628,6 @@ The messages in the server side is always retained."
 	((mew-folder-nntpp folder)
 	 (mew-nntp-retrieve case 'get bnm rtrs)))))))
 
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Folder clean up
@@ -663,7 +682,9 @@ The messages in the server side is always retained."
     (nreverse ret)))
 
 (defun mew-net-msg-group (lst)
-  (mew-net-msg-cat (mew-net-msg-pack lst)))
+  (if (= (length lst) 1)
+      lst ;; "1:*"
+    (mew-net-msg-cat (mew-net-msg-pack lst))))
 
 (provide 'mew-net)
 
