@@ -14,6 +14,8 @@
 (defvar mew-nntp-folder-alist2-file ".mew-folder-alist2")
 (defvar mew-nntp-folder-alist2 nil)
 
+(defvar mew-nntp-skip-uidl t)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; NNTP info
@@ -21,11 +23,12 @@
 
 (defvar mew-nntp-info-list
   '("server" "port" "process" "ssh-process" "ssl-process" "status"
-    "directive" "bnm"
+    "directive" "bnm" "mdb"
     "rtrs" "refs" "range"
     "rttl" "rcnt" "hlds"
-    "user"
-    "size" "get-body" "no-msg" "case" "msgdb" "done"
+    "user" "account"
+    "size" "get-body" "no-msg" "case" "msgdb" "done" "dispatched" "error"
+    "max"
     "newsgroup" "msgid" "truncated"))
 
 (mew-info-defun "mew-nntp-" mew-nntp-info-list)
@@ -79,11 +82,12 @@
       (mew-nntp-command-group pro pnm))))
 
 (defun mew-nntp-command-authpass (pro pnm)
-  (let ((pass (mew-input-passwd "AUTHINFO: " (mew-nntp-passtag pnm))))
+  (let* ((prompt (format "NNTP password (%s): " (mew-nntp-get-account pnm)))
+         (pass (mew-nntp-input-passwd prompt pnm)))
     (mew-nntp-process-send-string pro "AUTHINFO PASS %s" pass)))
 
 (defun mew-nntp-command-wpwd (pro pnm)
-  (mew-nntp-message pnm "AUTHINFO password is wrong!")
+  (mew-nntp-message pnm "NNTP password is wrong!")
   (mew-passwd-set-passwd (mew-nntp-passtag pnm) nil)
   (mew-nntp-set-status pnm "pre-quit"))
 
@@ -98,30 +102,46 @@
       (mew-nntp-process-send-string pro "GROUP %s" newsgroup)))))
 
 (defun mew-nntp-command-xover (pro pnm)
-  (mew-net-status (mew-nntp-get-bnm pnm)
-		  "Checking"
-		  nil
-		  (mew-nntp-secure-p pnm))
-  (let (beg end)
-    (if (re-search-forward "^[0-9]+ +[0-9]+ +\\([0-9]+\\) +\\([0-9]+\\) +[-.a-zA-Z0-9]+" nil t)
-	(progn
-	  (setq beg (mew-match-string 1))
-	  (setq end (mew-match-string 2))
-	  (mew-nntp-process-send-string pro "XOVER %s-%s" beg end))
-      (mew-nntp-set-status pnm "quit")
-      (mew-nntp-command-quit pro pnm))))
+  (let ((directive (mew-nntp-get-directive pnm))
+	(refs (mew-nntp-get-refs pnm)) ;; (uid siz del (+fld msg))
+	(bnm (mew-nntp-get-bnm pnm))
+	(range (mew-nntp-get-range pnm))
+	max)
+    (if (and mew-nntp-skip-uidl (eq directive 'get))
+	(mew-nntp-command-dispatch pro pnm directive refs nil)
+      (mew-net-status bnm "Checking" nil (mew-nntp-secure-p pnm))
+      (cond
+       ((eq directive 'scan)
+	(if (eq range nil) ;; update
+	    (setq max (mew-lisp-load
+		       (mew-expand-folder bnm mew-nntp-msgid-file))))))
+      (if (re-search-forward "^[0-9]+ +[0-9]+ +\\([0-9]+\\) +[0-9]+ +[-.a-zA-Z0-9]+" nil t)
+	  (progn
+	    (cond
+	     ((stringp max)
+	      (setq max (string-to-int max)))
+	     (max ;; backward compatibility
+	      ;; reversed
+	      (setq max (car max))
+	      (setq max (string-to-int max)))
+	     (t
+	      (setq max (string-to-int (mew-match-string 1)))
+	      (setq max (1- max))))
+	    (mew-nntp-set-max pnm max)
+	    (mew-nntp-process-send-string pro "XOVER %d-" (1+ max)))
+	(mew-nntp-set-status pnm "quit")
+	(mew-nntp-command-quit pro pnm)))))
 
 (defun mew-nntp-command-pre-article (pro pnm)
   (let* ((directive (mew-nntp-get-directive pnm))
+	 (max (mew-nntp-get-max pnm))
 	 (refs (mew-nntp-get-refs pnm))
 	 ;; (uid siz del (+fld msg))
-	 (bnm (mew-nntp-get-bnm pnm))
 	 (range (mew-nntp-get-range pnm))
-	 (old-uidl (mew-lisp-load (mew-expand-folder bnm mew-nntp-msgid-file)))
-	 uid uidl siz rtr rtrs rttl hlds)
+	 uid siz rtr rtrs hlds)
     (goto-char (point-min))
     ;; num subj from date msg-id ref siz lines
-    (while (re-search-forward "^[0-9]+\t[^\t]*\t[^\t]*\t[^\t]*\t<\\([^\t]+\\)>\t[^\t]*\t\\([0-9]+\\)" nil t)
+    (while (re-search-forward "^\\([0-9]+\\)\t[^\t\n]*\t[^\t\n]*\t[^\t\n]*\t<[^>\t\n]+>\t[^\t\n]*\t\\([0-9]*\\)" nil t)
       (setq uid (mew-match-string 1))
       (setq siz (mew-match-string 2))
       (if (string= uid "") (setq uid nil))
@@ -130,26 +150,26 @@
 	(setq rtr (assoc uid refs))
 	(if rtr (setq rtrs (cons rtr rtrs))))
        ((eq directive 'scan)
-	(when uid
-	  (setq uidl (cons uid uidl))
-	  (when (or range ;; all, last:n
-		    (not (member uid old-uidl))) ;; update
-	    (setq rtr (list uid siz))
-	    (setq rtrs (cons rtr rtrs)))))
+	(if (and uid (or range ;; all, last:n
+			 (> (string-to-int uid) max))) ;; update
+	    (setq rtrs (cons (list uid siz) rtrs))))
        ((eq directive 'sync)
-	(when uid
-	  (setq hlds (cons uid hlds))))))
-    (mew-nntp-set-msgid pnm uidl)
+	(if uid (setq hlds (cons uid hlds))))))
+    (mew-nntp-set-msgid pnm (nth 0 (car rtrs))) ;; 'scan
     ;; last:n
     (when (and (eq directive 'scan) (numberp range))
       (if (> (length rtrs) range)
 	  (setcdr (nthcdr (1- range) rtrs) nil)))
     (setq rtrs (nreverse rtrs))
     (setq hlds (nreverse hlds))
+    (mew-nntp-command-dispatch pro pnm directive rtrs hlds)))
+
+(defun mew-nntp-command-dispatch (pro pnm directive rtrs hlds)
+  (let ((rttl (length rtrs)))
     (mew-nntp-set-rtrs pnm rtrs)
+    (mew-nntp-set-rttl pnm rttl)
     (mew-nntp-set-hlds pnm hlds)
-    (setq rttl (length rtrs))
-    (mew-nntp-set-rttl pnm (length rtrs))
+    (mew-nntp-set-dispatched pnm t)
     (cond
      ((eq directive 'sync)
       (mew-nntp-set-status pnm "quit")
@@ -188,19 +208,20 @@
       (mew-nntp-command-quit pro pnm))
      ((eq directive 'get)
       (mew-nntp-set-truncated pnm nil)
-      (mew-nntp-process-send-string pro "ARTICLE <%s>" uid))
+      (mew-nntp-process-send-string pro "ARTICLE %s" uid))
      ((and (eq directive 'scan) (not get-body))
       (mew-nntp-set-truncated pnm t)
-      (mew-nntp-process-send-string pro "HEAD <%s>" uid))
+      (mew-nntp-process-send-string pro "HEAD %s" uid))
      ((or (= lim 0) (<= (string-to-int siz) lim))
       (mew-nntp-set-truncated pnm nil)
-      (mew-nntp-process-send-string pro "ARTICLE <%s>" uid))
+      (mew-nntp-process-send-string pro "ARTICLE %s" uid))
      (t
       (mew-nntp-set-truncated pnm t)
-      (mew-nntp-process-send-string pro "HEAD <%s>" uid)))))
+      (mew-nntp-process-send-string pro "HEAD %s" uid)))))
 
 (defun mew-nntp-command-post-article (pro pnm)
-  (let* ((width (1- (mew-scan-width)))
+  (let* ((directive (mew-nntp-get-directive pnm))
+	 (width (1- (mew-scan-width)))
 	 (rtrs (mew-nntp-get-rtrs pnm))
 	 (rtr (car rtrs))
 	 (uid (nth 0 rtr))
@@ -214,8 +235,7 @@
      ((stringp fld-msg)
       (setq fld fld-msg))
      ((listp fld-msg)
-      (setq fld (nth 0 fld-msg))
-      (setq msg (nth 1 fld-msg))
+      (mew-set '(fld msg) fld-msg)
       (setq lmsg msg)))
     (goto-char (point-min))
     (forward-line)
@@ -223,9 +243,14 @@
     ;; line delimiters
     (mew-eol-fix-for-read)
     (mew-dot-delete)
-    (setq msg-file (mew-net-get-new-message
-		    pnm fld msg 'mew-nntp-get-msgdb 'mew-nntp-set-msgdb))
-    (setq msg (car msg-file) file (cdr msg-file))
+    (cond
+     ((eq directive 'scan)
+      (setq msg uid) 
+      (setq file (mew-expand-folder fld msg)))
+     (t
+      (setq msg-file (mew-net-get-new-message
+		      pnm fld msg 'mew-nntp-get-msgdb 'mew-nntp-set-msgdb))
+      (setq msg (car msg-file) file (cdr msg-file))))
     (goto-char (point-min))
     (if truncated
 	(mew-header-insert-xmu uid siz t)
@@ -240,13 +265,13 @@
 	 (mew-nntp-command-quit pro pnm)
 	 (throw 'write-error nil)))
       (mew-set-file-modes file)
-      ;; xxx links?
       (mew-set-buffer-multibyte t)
-      (setq vec (mew-nntp-scan-header))
+      (setq vec (mew-scan-header))
       (mew-scan-set-folder vec fld)
       (mew-scan-set-message vec msg)
+      (mew-scan-body vec)
       (mew-set-buffer-multibyte nil)
-      (mew-scan-insert-line fld vec width lmsg nil)
+      (mew-scan-insert-line fld vec width lmsg)
       (mew-nntp-command-next-article pro pnm))))
 
 (defun mew-nntp-command-next-article (pro pnm)
@@ -274,7 +299,7 @@
     (mew-eol-fix-for-read)
     (mew-dot-delete)
     (while (not (eobp))
-      (when (looking-at "\\([a-z][^ \t]+\\)")
+      (when (looking-at "\\([a-z][^ \t\n]+\\)")
 	(setq group2 (mew-match-string 1))
 	(setq group (concat mew-folder-nntp group2)) 
 	(setq groups (cons (mew-folder-func group) groups))
@@ -293,6 +318,8 @@
   (mew-nntp-process-send-string pro "QUIT"))
 
 (defun mew-nntp-command-quit2 (pro pnm)
+  (mew-nntp-set-done pnm t)
+  (mew-nntp-set-error pnm t)
   (mew-nntp-set-status pnm "quit")
   (mew-nntp-process-send-string pro "QUIT"))
 
@@ -324,7 +351,9 @@
 (defun mew-nntp-process-send-string (pro &rest args)
   (let ((str (apply 'format args)))
     (mew-nntp-debug "=SEND=" str)
-    (process-send-string pro (concat str mew-cs-eol))))
+    (if (and (processp pro) (eq (process-status pro) 'open))
+	(process-send-string pro (concat str mew-cs-eol))
+      (message "NNTP time out"))))
 
 (defun mew-nntp-passtag (pnm)
   (let ((server (mew-nntp-get-server pnm))
@@ -332,22 +361,19 @@
 	(user (mew-nntp-get-user pnm)))
     (concat user "@" server ":" port)))
 
-(defun mew-nntp-scan-header ()
-  (goto-char (point-min))
-  (unless (re-search-forward mew-eoh nil t)
-    (goto-char (point-max)))
-  (mew-scan-header))
-
 (defsubst mew-nntp-message (pnm &rest args)
   (or (mew-nntp-get-no-msg pnm) (apply 'message args)))
 
 (defsubst mew-bnm-to-newsgroup (bnm)
-  (let ((case-folder (mew-folder-case bnm))
-	folder)
-    (if case-folder
-	(setq folder (cdr case-folder))
-      (setq folder bnm))
-    (mew-folder-string folder)))
+  (mew-folder-string (mew-case:folder-folder bnm)))
+
+(defun mew-nntp-input-passwd (prompt pnm)
+  (let* ((tag (mew-nntp-passtag pnm))
+         (pro (mew-nntp-get-process pnm))
+         (pass (mew-input-passwd prompt tag)))
+    (unless (and (processp pro) (eq (process-status pro) 'open))
+      (mew-passwd-set-passwd tag nil))
+    pass))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -384,6 +410,7 @@
 
 (defun mew-nntp-retrieve (case directive bnm &rest args)
   (let* ((server (mew-nntp-server case))
+         (user (mew-nntp-user case))
 	 (port (mew-nntp-port case))
 	 (sshsrv (mew-nntp-ssh-server case))
 	 (sslp (mew-nntp-ssl case))
@@ -397,7 +424,7 @@
 	(message "Another NNTP process is running. Try later")
       (cond
        (sshsrv
-	(setq sshpro (mew-open-ssh-stream server port sshsrv))
+	(setq sshpro (mew-open-ssh-stream case server port sshsrv))
 	(when sshpro
 	  (setq sshname (process-name sshpro))
 	  (setq lport (mew-ssh-pnm-to-lport sshname))
@@ -414,7 +441,8 @@
 	(setq process (mew-nntp-open pnm server port no-msg))))
       (when process
 	(mew-summary-lock process "NNTPing" (or sshpro sslpro))
-	(mew-sinfo-set-scan-form (mew-summary-scan-form bnm))
+	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
+	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
 	(mew-info-clean-up pnm)
 	(mew-nntp-set-no-msg pnm no-msg) ;; must come here
@@ -424,7 +452,8 @@
 	(mew-nntp-set-ssl-process pnm sslpro)
 	(mew-nntp-set-server pnm server)
 	(mew-nntp-set-port pnm port)
-	(mew-nntp-set-user pnm (mew-nntp-user case))
+	(mew-nntp-set-user pnm user)
+        (mew-nntp-set-account pnm (format "%s@%s" user server))
 	(mew-nntp-set-status pnm "greeting")
 	(mew-nntp-set-directive pnm directive)
 	(mew-nntp-set-bnm pnm bnm)
@@ -436,11 +465,16 @@
 	;;
 	(cond
 	 ((eq directive 'get)
+	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
 	  (mew-nntp-set-refs pnm (nth 0 args)))
 	 ((eq directive 'scan)
 	  (mew-nntp-set-range pnm (nth 0 args))
 	  (mew-nntp-set-get-body pnm (nth 1 args))
-	  (if (mew-nntp-get-range pnm) (mew-net-folder-clean)))
+	  (if (mew-nntp-get-range pnm)
+	      (progn
+		(mew-nntp-set-mdb pnm (mew-summary-mark-collect4))
+		(mew-net-folder-clean))
+	    (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))))
 	 ((eq directive 'sync)
 	  ))
 	(mew-sinfo-set-start-point (point)) ;; after erase-buffer
@@ -486,7 +520,7 @@
        (setq next (mew-nntp-fsm-next status code)))
       ((and (or (and mulrep
 		     (goto-char (point-max))
-		     (forward-line -1)
+		     (= (forward-line -1) 0)
 		     (looking-at "^\\.\r?$"))
 		(and (not mulrep)
 		     (goto-char (1- (point-max)))
@@ -502,63 +536,89 @@
        (setq func (intern-soft (concat "mew-nntp-command-" next)))
        (goto-char (point-min))
        (if (fboundp func)
-	   (and func (funcall func process pnm))
+	   (funcall func process pnm)
 	 (error "No function called %s" (symbol-name func)))
-       (mew-erase-buffer)))))
+       (if (and process (equal (process-buffer process) (current-buffer)))
+	   (mew-erase-buffer))))))
 
 (defun mew-nntp-sentinel (process event)
   (let* ((pnm (process-name process))
 	 (directive (mew-nntp-get-directive pnm))
+	 (mdb (mew-nntp-get-mdb pnm))
 	 (sshpro (mew-nntp-get-ssh-process pnm))
 	 (sslpro (mew-nntp-get-ssl-process pnm))
 	 (rttl (mew-nntp-get-rttl pnm))
-	 (bnm (mew-nntp-get-bnm pnm))
+	 (bnm (or (mew-nntp-get-bnm pnm) (current-buffer)))
 	 (hlds (mew-nntp-get-hlds pnm))
 	 (msgid (mew-nntp-get-msgid pnm))
 	 (done (mew-nntp-get-done pnm))
-	 (file (mew-expand-folder bnm mew-nntp-msgid-file)))
+	 (error (mew-nntp-get-error pnm))
+	 (file (mew-expand-folder bnm mew-nntp-msgid-file))
+	 (buf (process-buffer process)))
     (mew-nntp-debug "NNTP SENTINEL" event)
-    (mew-filter
-     (set-buffer bnm)
-     (when done
-       (cond
-	((eq directive 'list)
-	 (mew-nntp-message pnm "Collecting newsgroup list...done"))
-	((eq directive 'sync)
-	 (mew-nntp-message pnm "Synchronizing messages...")
-	 (mew-net-folder-sync bnm hlds)
-	 (mew-nntp-message pnm "Synchronizing messages...done")
-	 (mew-summary-folder-cache-save))
-	((eq directive 'get)
-	 (cond
-	  ((= rttl 0)
-	   (mew-nntp-message pnm "No new messages"))
-	  ((= rttl 1)
-	   (mew-nntp-message pnm "1 message retrieved"))
-	  (t
-	   (mew-nntp-message pnm "%d messages retrieved" rttl))))
-	((eq directive 'scan)
-	 (cond
-	  ((= rttl 0)
-	   (mew-nntp-message pnm "No messages scanned"))
-	  ((= rttl 1)
-	   (mew-nntp-message pnm "1 message scanned")
-	   (mew-lisp-save file msgid)
-	   (mew-summary-folder-cache-save))
-	  (t
-	   (mew-nntp-message pnm "%d messages scanned" rttl)
-	   (mew-lisp-save file msgid)
-	   (mew-summary-folder-cache-save))))))
-     ;;
-     (mew-net-status-clear (mew-nntp-get-bnm pnm))
-     (mew-info-clean-up pnm)
-     (set-buffer-modified-p nil)
-     (mew-summary-unlock)
-     (if (and (processp sshpro) (not mew-ssh-keep-connection))
-	 (process-send-string sshpro "exit\n"))
-     (if (and (processp sslpro) (not mew-ssl-keep-connection))
-	 (delete-process sslpro))
-     (run-hooks 'mew-nntp-sentinel-hook))))
+    (set-process-buffer process nil)
+    (set-buffer bnm)
+    (mew-summary-mark-recover mdb)
+    (mew-remove-buffer buf)
+    (if (not done)
+	(let* ((rtrs (mew-nntp-get-rtrs pnm))
+	       (lefts (length rtrs))
+	       (msgid (nth 0 (car rtrs)))
+	       recovered)
+	  (mew-nntp-message pnm "NNTP connection is lost")
+	  (when (mew-nntp-get-dispatched pnm)
+	    (cond
+	     ((eq directive 'scan)
+	      (setq msgid (int-to-string (1- (string-to-int msgid))))
+	      (mew-lisp-save file msgid nil 'unlimit)
+	      (setq recovered t)))
+	    (when recovered
+	      (mew-nntp-message
+	       pnm
+	       "%d message retrieved. %d messages are left due to an error."
+	       (- rttl lefts) lefts)
+	      (mew-summary-folder-cache-save))))
+      (cond
+       (error 
+	;; retain the error message
+	)
+       ((eq directive 'list)
+	(mew-nntp-message pnm "Collecting newsgroup list...done"))
+       ((eq directive 'sync)
+	(mew-nntp-message pnm "Synchronizing messages...")
+	(mew-net-folder-sync bnm hlds)
+	(mew-nntp-message pnm "Synchronizing messages...done")
+	(mew-summary-folder-cache-save))
+       ((eq directive 'get)
+	(cond
+	 ((= rttl 0)
+	  (mew-nntp-message pnm "No new messages"))
+	 ((= rttl 1)
+	  (mew-nntp-message pnm "1 message retrieved"))
+	 (t
+	  (mew-nntp-message pnm "%d messages retrieved" rttl))))
+       ((eq directive 'scan)
+	(cond
+	 ((= rttl 0)
+	  (mew-nntp-message pnm "No messages scanned"))
+	 ((= rttl 1)
+	  (mew-nntp-message pnm "1 message scanned")
+	  (mew-lisp-save file msgid nil 'unlimit)
+	  (mew-summary-folder-cache-save))
+	 (t
+	  (mew-nntp-message pnm "%d messages scanned" rttl)
+	  (mew-lisp-save file msgid nil 'unlimit)
+	  (mew-summary-folder-cache-save))))))
+    ;;
+    (mew-net-status-clear bnm)
+    (mew-info-clean-up pnm)
+    (set-buffer-modified-p nil)
+    (mew-summary-unlock)
+    (if (and (processp sshpro) (not mew-ssh-keep-connection))
+	(process-send-string sshpro "exit\n"))
+    (if (and (processp sslpro) (not mew-ssl-keep-connection))
+	(delete-process sslpro))
+    (run-hooks 'mew-nntp-sentinel-hook)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
