@@ -79,10 +79,10 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     ("+batchmode=on" "+language=en")
     ("--decrypt" "--batch"))) ;; --verify does not extract original data
 
-(defconst mew-prog-arg-output '("-o" "-o" "-o" "--output"))
-(defconst mew-prog-arg-input '(nil "-o" nil nil))
-(defconst mew-prog-arg-luserid '("-u" "-u" "-u" "--local-user"))
-(defconst mew-prog-arg-ruserid '(nil "-r" nil "--remote-user"))
+(defconst mew-prog-pgp-arg-output  '("-o" "-o" "-o" "--output"))
+(defconst mew-prog-pgp-arg-input   '(nil  "-o" nil  nil))
+(defconst mew-prog-pgp-arg-luserid '("-u" "-u" "-u" "--local-user"))
+(defconst mew-prog-pgp-arg-ruserid '(nil  "-r" nil  "--remote-user"))
 
 (defconst mew-prog-pgpk-add-arg
   '(("-ka" "+batchmode=on")
@@ -91,7 +91,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     ("--import" "--batch")))
 
 (defconst mew-prog-pgpk-ext-arg
-  '(("-kxfa") ("-xa") ("-kxfa") ("--export" "--armor" "--batch")))
+  '(("-kxfa") ("-xa") ("+force" "-kxfa") ("--export" "--armor" "--batch")))
 
 (defconst mew-pgp-msg-signature
   '("\n\\(.*\\) signature from user \\(.*\\)\\."
@@ -103,7 +103,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
   '("Key ID \\(\\S +\\) not found"
     ": \\(0x[a-zA-Z0-9]+\\)"
     ": \\(0x[a-zA-Z0-9]+\\)"
-    "xxx"))
+    "key ID \\([a-zA-Z0-9]+\\)"))
 
 (defconst mew-pgp-msg-bad-pass
   '("No passphrase"
@@ -144,6 +144,12 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     "^WARNING:[ -9;-~\n]+belongs? to:"
     "There is no indication that this key really belongs to the owner"))
 
+(defconst mew-pgp-msg-pubkey-expired
+  '("xxx"
+    "xxx"
+    "xxx"
+    "encryption failed: unusable public key"))
+
 (defconst mew-pgp-msg-no-vrfkey
   '("Key matching" "unknown keyid" "key does not meet" "public key not found"))
 
@@ -210,19 +216,20 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 (defconst mew-pgp-err-seckey-or-secring 'mew-err-seckey-or-secring)
 (defconst mew-pgp-err-other   'mew-err-other)
 
-(defvar mew-pgp-result-pass     "Pass phrase is wrong.")
-(defvar mew-pgp-result-pubring  "No public keyring.")
-(defvar mew-pgp-result-secring  "No secret keyring.")
-(defvar mew-pgp-result-pubkey   "No his/her public key.")
+(defvar mew-pgp-result-pass     "Pass phrase is wrong")
+(defvar mew-pgp-result-pubring  "No public keyring")
+(defvar mew-pgp-result-secring  "No secret keyring")
+(defvar mew-pgp-result-pubkey   "No his/her public key")
+(defvar mew-pgp-result-expired  "His/her public key is expired")
 (defvar mew-pgp-result-invalid
-  "His/her public key is invalid. Sign the key by yourself, first.")
-(defvar mew-pgp-result-seckey   "No your secret key.")
+  "His/her public key is invalid. Sign the key by yourself, first")
+(defvar mew-pgp-result-seckey   "No your secret key")
 (defvar mew-pgp-result-seckey-or-secring
-  "No secret keyring or no your secret key.")
-(defvar mew-pgp-result-other    "PGP failed for some reasons.")
-(defvar mew-pgp-result-sec-succ "PGP decrypted. ")
-(defvar mew-pgp-result-dec-fail "PGP NOT decrypted for some reasons.")
-(defvar mew-pgp-result-unsup    "PGP unsupported signature.")
+  "No secret keyring or no your secret key")
+(defvar mew-pgp-result-other    "PGP failed for some reasons")
+(defvar mew-pgp-result-sec-succ "PGP decrypted")
+(defvar mew-pgp-result-dec-fail "PGP NOT decrypted for some reasons")
+(defvar mew-pgp-result-unsup    "PGP unsupported signature")
 
 (defvar mew-pgp-prompt-enter-pass   "Enter pass phrase (%s): ")
 (defvar mew-pgp-prompt-reenter-pass "Re-enter pass phrase (%s): ")
@@ -310,9 +317,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 		(goto-char (point-min))
 		(if (not (re-search-forward (mew-pgp-get mew-pgp-msg-key-id) nil t))
 		    (setq keyid "not found")
-		  ;; xxx
-		  (setq keyid (mew-buffer-substring
-			       (match-beginning 1) (match-end 1)))
+		  (setq keyid (mew-match-string 1))
 		  (if (eq mew-pgp-ver mew-pgp-ver2) ;; xxx
 		      (setq keyid (concat "0x" keyid))))
 		(setq ret (concat mew-pgp-result-pubkey " ID = " keyid)))
@@ -359,36 +364,19 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 
 (defun mew-pgp-verify (file1 file2)
   (message "PGP verifying...")
-  (let ((ioption (mew-pgp-get mew-prog-arg-input)) ret)
+  (let ((ioption (mew-pgp-get mew-prog-pgp-arg-input)) ;; detached signature
+	(voptions (mew-pgp-get mew-prog-pgpv-arg))
+	(pgpv (mew-pgp-get mew-prog-pgpv))
+	files ret)
     (with-temp-buffer
       (if ioption
-	  (apply 'mew-call-process-lang
-		 (mew-pgp-get mew-prog-pgpv)
-		 nil t nil 
-		 (append (mew-pgp-get mew-prog-pgpv-arg)
-			 (list ioption file1 file2)))
-	(apply 'mew-call-process-lang
-	       (mew-pgp-get mew-prog-pgpv)
-	       nil t nil 
-	       (append (mew-pgp-get mew-prog-pgpv-arg)
-		       (list file2 file1))))
+	  (setq files (list ioption file1 file2))
+	(setq files (list file2 file1)))
+      (apply 'mew-call-process-lang pgpv nil t nil (append voptions files))
       (setq ret (mew-pgp-verify-check)))
     (message "PGP verifying...done")
     ret))
 
-(defun mew-pgp-verify-old (file)
-  (message "PGP verifying...")
-  (let ((file1 (mew-make-temp-name)) verify)
-    (with-temp-buffer
-      (apply 'mew-call-process-lang
-	     (mew-pgp-get mew-prog-pgpv)
-	     nil t nil 
-	     (append (mew-pgp-get mew-prog-old-pgpv-arg)
-		     (list (mew-pgp-get mew-prog-arg-output) file1 file)))
-      (setq verify (mew-pgp-verify-check)))
-    (message "PGP verifying...done")
-    (list file1 verify)))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; PGP encrypting
@@ -404,37 +392,34 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
           (setq ret mew-pgp-result-pubkey)
         (goto-char (point-min))
         (if (search-forward (mew-pgp-get mew-pgp-msg-no-keyring) nil t)
-            (setq ret mew-pgp-result-pubring))))
+            (setq ret mew-pgp-result-pubring)
+	  (goto-char (point-min))
+	  (if (search-forward (mew-pgp-get mew-pgp-msg-pubkey-expired) nil t)
+	      (setq ret mew-pgp-result-expired)))))
     ret))
 
 (defun mew-pgp-encrypt (file1 decrypters)
   (message "PGP encrypting...")
-  (let ((roption (mew-pgp-get mew-prog-arg-ruserid))
+  (let ((roption (mew-pgp-get mew-prog-pgp-arg-ruserid))
+	(ooption (mew-pgp-get mew-prog-pgp-arg-output))
+	(eoptions (mew-pgp-get mew-prog-pgpe-arg))
+	(pgpe (mew-pgp-get mew-prog-pgpe))
 	(file2 (mew-make-temp-name))
-	check file3) ;; not unique if file3 is made here
+	check file3 decs args) ;; not unique if file3 is made here
     (with-temp-buffer
       (insert "Version: 1\n")
-      (write-region (point-min) (point-max) file2  nil 'no-msg)
-      (setq file3 (concat (mew-make-temp-name) mew-pgp-ascii-suffix))
-      (mew-erase-buffer)
-      (if roption
-	  (let (decs)
-	    (while decrypters
-	      (setq decs (cons (car decrypters) (cons roption decs)))
-	      ;; nreverse later, take care.
-	      (setq decrypters (cdr decrypters)))
-	    (setq decrypters (nreverse decs))
-	    (apply 'mew-call-process-lang
-		   (mew-pgp-get mew-prog-pgpe)
-		   nil t nil
-		   (append (mew-pgp-get mew-prog-pgpe-arg)
-			   decrypters
-			   (list (mew-pgp-get mew-prog-arg-output) file3 file1))))
-	(apply 'mew-call-process-lang
-	       (mew-pgp-get mew-prog-pgpe)
-	       nil t nil
-	       (mew-pgp-get mew-prog-arg-output) file3 file1 
-	       (append (mew-pgp-get mew-prog-pgpe-arg) decrypters)))
+      (write-region (point-min) (point-max) file2 nil 'no-msg))
+    (setq file3 (concat (mew-make-temp-name) mew-pgp-ascii-suffix))
+    (if (not roption)
+	(setq args (append (list ooption file3 file1) eoptions decrypters))
+      (while decrypters
+	(setq decs (cons (car decrypters) (cons roption decs)))
+	;; nreverse later, take care.
+	(setq decrypters (cdr decrypters)))
+      (setq decrypters (nreverse decs))
+      (setq args (append eoptions decrypters (list ooption file3 file1))))
+    (with-temp-buffer
+      (apply 'mew-call-process-lang pgpe nil t nil args)
       (setq check (mew-pgp-encrypt-check)))
     (message "PGP encrypting...done")
     (list file2 "7bit" file3 "7bit" check))) ;; both ctes are 7bit
@@ -453,15 +438,17 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
   (setq mew-pgp-decrypt-msg nil)
   (setq mew-pgp-failure nil)
   (let ((process-connection-type mew-connection-type2)
+	(ooption (mew-pgp-get mew-prog-pgp-arg-output))
+	(doptions (mew-pgp-get mew-prog-pgpd-arg))
+	(pgpd (mew-pgp-get mew-prog-pgpd))
 	file3 process verify)
     (setq file3 (mew-make-temp-name))
     (setq process
 	  (apply 'mew-start-process-lang
 		 "PGP decrypt"
 		 (current-buffer)
-		 (mew-pgp-get mew-prog-pgpd)
-		 (append (mew-pgp-get mew-prog-pgpd-arg)
-			 (list (mew-pgp-get mew-prog-arg-output) file3 file2))))
+		 pgpd
+		 (append doptions (list ooption file3 file2))))
     (mew-set-process-cs process mew-cs-autoconv mew-cs-dummy)
     (set-process-filter process 'mew-pgp-process-filter1)
     (set-process-sentinel process 'mew-pgp-process-sentinel)
@@ -472,9 +459,8 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	  (with-temp-buffer
 	    (insert mew-pgp-string)
 	    (setq verify (mew-pgp-verify-check)))
-	  (if verify 
-	      (setq mew-pgp-decrypt-msg
-		    (concat mew-pgp-decrypt-msg "\n\t" verify))))
+	  (when verify 
+	    (setq mew-pgp-decrypt-msg (concat mew-pgp-decrypt-msg "\n\t" verify))))
       ;; unpredictable error
       (mew-passwd-set-passwd (mew-pgp-passtag) nil)
       (if (string= mew-pgp-decrypt-msg mew-pgp-result-sec-succ)
@@ -504,6 +490,10 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
   (setq mew-pgp-sign-msg nil)
   (setq mew-pgp-failure nil)
   (let ((process-connection-type mew-connection-type2)
+	(loption (mew-pgp-get mew-prog-pgp-arg-luserid))
+	(ooption (mew-pgp-get mew-prog-pgp-arg-output))
+	(soptions (mew-pgp-get mew-prog-pgps-arg))
+	(pgps (mew-pgp-get mew-prog-pgps))
 	file2 process)
     (setq file2 (concat (mew-make-temp-name) mew-pgp-ascii-suffix))
     ;; not perfectly unique but OK
@@ -511,17 +501,15 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	  (apply 'mew-start-process-lang
 		 "PGP sign"
 		 (current-buffer)
-		 (mew-pgp-get mew-prog-pgps)
-		 (append (mew-pgp-get mew-prog-pgps-arg)
-			 (list (mew-pgp-get mew-prog-arg-luserid) mew-inherit-encode-signer
-			       (mew-pgp-get mew-prog-arg-output) file2 file1))))
+		 pgps
+		 (append soptions (list loption mew-inherit-encode-signer ooption file2 file1))))
     (mew-set-process-cs process mew-cs-autoconv mew-cs-dummy)
     (set-process-filter process 'mew-pgp-process-filter1)
     (set-process-sentinel process 'mew-pgp-process-sentinel)
     (mew-rendezvous mew-pgp-running)
     (message "PGP signing...done")
-    (if (not (file-exists-p file2)) ;; for unpredictable error
-	(mew-passwd-set-passwd (mew-pgp-passtag) nil))
+    (unless (file-exists-p file2) ;; for unpredictable error
+      (mew-passwd-set-passwd (mew-pgp-passtag) nil))
     (list file2 nil (mew-pgp-get-micalg) mew-pgp-sign-msg))) ;; return
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -699,16 +687,206 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 (defsubst mew-pgp-encode-message (type &optional ask-singer)
   (if (null mew-pgp-ver)
       (message "%s does not exist" mew-prog-pgp)
-    (if (and ask-singer (string-match "signature" (symbol-name type)))
-	(mew-draft-make-message type (car (mew-input-address "Who's key?: ")))
-      (mew-draft-make-message type))))
+    (let ((func 'mew-draft-make-message))
+      (if (mew-use-old-pgp (mew-tinfo-get-case))
+	  (setq func 'mew-old-pgp-encode))
+      (if (and ask-singer (string-match "signature" (symbol-name type)))
+	  (funcall func type (car (mew-input-address "Who's key?: ")))
+	(funcall func type)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; key distribution
+;;; old PGP encoding
 ;;;
 
-(defun mew-pgp-check-old-pgp ()
+(defconst mew-prog-old-pgps-arg
+  '(("-sat" "+clearsig=on" "+language=en" "+batchmode=off")
+    ("-at" "+clearsig=on" "+language=en" "+batchmode=off")
+    ("-sat" "+clearsig=on" "+language=en" "+batchmode=off")
+    ("--clearsign" "--armor" "--textmode")))
+
+(defconst mew-prog-old-pgpse-arg
+  '(("-seat" "+language=en" "+batchmode=off")
+    ("-eat"  "+language=en" "+batchmode=off")
+    ("-seat" "+language=en" "+batchmode=off")
+    ("--sign" "--encrypt" "--armor" "--textmode")))
+
+(defconst mew-prog-old-pgpe-arg
+  '(("-eat" "+language=en" "+batchmode=on" "+armorlines=0")
+    ("-at" "+language=en" "+batchmode=on" "+armorlines=0")
+    ("-eat" "+language=en" "+batchmode=on" "+armorlines=0")
+    ("--encrypt" "--armor" "--textmode" "--batch")))
+
+(defun mew-old-pgp-encode (type &optional signer)
+  (let (mew-inherit-encode-signer doit syntax ctl ct charset cs)
+    (if (mew-attach-p)
+	(if (mew-encode-syntax-have-one-part)
+	    (progn
+	      (setq syntax (mew-syntax-get-part mew-encode-syntax))
+	      (setq ctl (mew-syntax-get-ct syntax))
+	      (setq ct (mew-syntax-get-value ctl 'cap))
+	      (if (string= ct mew-ct-txt)
+		  (progn
+		    (setq charset (mew-syntax-get-param ctl "charset"))
+		    (setq cs (mew-charset-to-cs charset))
+		    (setq doit t)
+		    (mew-attach-clear))
+		(message "Old PGP cannot be used for non-text.")))
+	  (message "Old PGP cannot be used if multipart exists."))
+      (setq doit t))
+    (when doit
+      (setq mew-inherit-encode-signer (or signer (mew-get-my-address)))
+      (cond
+       ((eq type 'pgp-signature)
+	(mew-old-pgp-sign cs syntax))
+       ((eq type 'pgp-encryption)
+	(mew-old-pgp-encrypt cs))
+       ((eq type 'pgp-signature-encryption)
+	(mew-pgp-old-sign-encrypt cs))
+       (t
+	(message "Not supported."))))))
+
+(defun mew-old-pgp-sign (cs &optional syntax)
+  (let ((file1 (mew-make-temp-name))
+	(mew-prog-pgps-arg mew-prog-old-pgps-arg)
+	file2 fmc errmsg charset)
+    (goto-char (mew-header-end))
+    (forward-line)
+    (unless cs
+      (setq charset (mew-charset-guess-region (point) (point-max)))
+      (setq cs (mew-charset-to-cs charset)))
+    (mew-frwlet
+     mew-cs-dummy cs
+     (write-region (point) (point-max) file1 nil 'no-msg))
+    (condition-case nil
+	(setq fmc (mew-pgp-sign file1))
+      (error
+       (mew-delete-file file1)
+       (mew-encode-error
+	(format "unknown error for %s. Check %s, anyway" 
+		mew-ct-mls mew-temp-dir))))
+    (mew-set '(file2 nil nil errmsg) fmc)
+    (if errmsg
+	(progn
+	  (mew-delete-file file1)
+	  (mew-delete-file file2)
+	  (error errmsg))
+      (delete-region (point) (point-max))
+      (mew-frwlet
+       cs mew-cs-dummy 
+       (insert-file-contents file2))
+      (mew-delete-file file1)
+      (mew-delete-file file2)
+      (setq mew-encode-syntax syntax)
+      (mew-draft-make-message))))
+
+(defun mew-old-pgp-encrypt (cs)
+  (let ((file1 (mew-make-temp-name))
+	(mew-prog-pgps-arg mew-prog-old-pgps-arg)
+	(decrypters (mew-header-parse-address-list mew-destination:-list))
+	file2 file3 fc errmsg charset)
+    (goto-char (mew-header-end))
+    (forward-line)
+    (unless cs
+      (setq charset (mew-charset-guess-region (point) (point-max)))
+      (setq cs (mew-charset-to-cs charset)))
+    (mew-frwlet
+     mew-cs-dummy cs
+     (write-region (point) (point-max) file1 nil 'no-msg))
+    (if mew-encrypt-to-myself
+	(setq decrypters (cons mew-inherit-encode-signer decrypters)))
+    (condition-case nil
+	(setq fc (mew-pgp-encrypt file1 decrypters))
+      (error
+       (mew-delete-file file1)
+       (mew-encode-error
+	(format "unknown error for %s. Check %s, anyway" 
+		mew-ct-mle mew-temp-dir))))
+    (mew-set '(file2 nil file3 nil errmsg) fc)
+    (if errmsg
+	(progn
+	  (mew-delete-file file1)
+	  (mew-delete-file file2)
+	  (mew-delete-file file3)
+	  (error errmsg))
+      ;; Create multipart content-header
+      (delete-region (point) (point-max))
+      (mew-frwlet
+       cs mew-cs-dummy 
+       (insert-file-contents file3))
+      (mew-delete-file file1)
+      (mew-delete-file file2)
+      (mew-delete-file file3)
+      (setq mew-encode-syntax nil)
+      (mew-draft-make-message))))
+
+(defun mew-pgp-old-sign-encrypt (cs)
+  (message "PGP signing then encrypting...")
+  (let ((decrypters (mew-header-parse-address-list mew-destination:-list))
+	(roption (mew-pgp-get mew-prog-pgp-arg-ruserid))
+	(loption (mew-pgp-get mew-prog-pgp-arg-luserid))
+	(ooption (mew-pgp-get mew-prog-pgp-arg-output))
+	(eoptions (mew-pgp-get mew-prog-old-pgpse-arg)) ;; xxx
+	(pgpe (mew-pgp-get mew-prog-pgpe)) ;; xxx
+	(file1 (mew-make-temp-name))
+	file2 check decs args process charset)
+    (goto-char (mew-header-end))
+    (forward-line)
+    (unless cs
+      (setq charset (mew-charset-guess-region (point) (point-max)))
+      (setq cs (mew-charset-to-cs charset)))
+    (mew-frwlet
+     mew-cs-dummy cs
+     (write-region (point) (point-max) file1 nil 'no-msg))
+    (if mew-encrypt-to-myself
+	(setq decrypters (cons mew-inherit-encode-signer decrypters)))
+    (setq file2 (mew-make-temp-name))
+    (setq args (list loption mew-inherit-encode-signer ooption file2 file1))
+    (if (not roption)
+	(setq args (append args eoptions decrypters))
+      (while decrypters
+	(setq decs (cons (car decrypters) (cons roption decs)))
+	;; nreverse later, take care.
+	(setq decrypters (cdr decrypters)))
+      (setq decrypters (nreverse decs))
+      (setq args (append eoptions decrypters args)))
+    (setq mew-pgp-running 'signing)
+    (setq mew-pgp-string nil)
+    (setq mew-pgp-sign-msg nil)
+    (setq mew-pgp-failure nil)
+    (with-temp-buffer
+      (setq process (apply 'mew-start-process-lang
+			   "PGP sign"
+			   (current-buffer)
+			   pgpe args))
+      (mew-set-process-cs process mew-cs-autoconv mew-cs-dummy)
+      (set-process-filter process 'mew-pgp-process-filter1)
+      (set-process-sentinel process 'mew-pgp-process-sentinel)
+      (mew-rendezvous mew-pgp-running)
+      (message "PGP signing then encrypting...done")
+      (setq check (mew-pgp-encrypt-check)))
+    (unless (file-exists-p file2) ;; for unpredictable error
+      (mew-passwd-set-passwd (mew-pgp-passtag) nil))
+    (if (or mew-pgp-sign-msg check)
+	(progn
+	  (mew-delete-file file1)
+	  (mew-delete-file file2)
+	  (error (or mew-pgp-sign-msg check)))
+      (delete-region (point) (point-max))
+      (mew-frwlet
+       cs mew-cs-dummy 
+       (insert-file-contents file2))
+      (mew-delete-file file1)
+      (mew-delete-file file2)
+      (setq mew-encode-syntax nil)
+      (mew-draft-make-message))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; old PGP decoding
+;;;
+
+(defun mew-old-pgp-check ()
   (goto-char (point-min))
   (if (re-search-forward (concat "^" mew-pgp-encryption-begin) nil t)
       ;; this may be 'pgp-signature, but we hope clearsig is used...
@@ -718,58 +896,100 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	'pgp-signature
       nil)))
 
-(defun mew-summary-decode-pgp ()
+(defun mew-old-pgp-verify (file)
+  (message "PGP verifying...")
+  (let ((voptions (mew-pgp-get mew-prog-old-pgpv-arg))
+	(ooption (mew-pgp-get mew-prog-pgp-arg-output))
+	(pgpv (mew-pgp-get mew-prog-pgpv))
+	(file1 (mew-make-temp-name))
+	verify)
+    (with-temp-buffer
+      (apply 'mew-call-process-lang
+	     pgpv nil t nil (append voptions (list ooption file1 file)))
+      (setq verify (mew-pgp-verify-check)))
+    (message "PGP verifying...done")
+    (list file1 verify)))
+
+(defun mew-summary-decode-old-pgp ()
   "Decrypting/verifying old-fashioned PGP messages."
   (interactive)
   (if (null mew-pgp-ver)
       (message "%s does not exist" mew-prog-pgp)
     (mew-summary-msg
-     (let* ((fld (mew-summary-folder-name))
-	    (msg (mew-summary-message-number))
- 	    (file (mew-expand-folder fld msg))
-	    file1 type result win start
-	    mew-inherit-decode-signer)
+     (let ((fld (mew-summary-folder-name))
+	   (msg (mew-summary-message-number2))
+	   type)
        (save-excursion
 	 (set-buffer (mew-buffer-message))
-	 (mew-elet
-	  (widen)
-	  (setq win (get-buffer-window (current-buffer)))
-	  (setq start (window-start win))
-	  (setq type (mew-pgp-check-old-pgp))
-	  (unless type
-	    (with-temp-buffer
-	      (mew-insert-file-contents file)
-	      (setq type (mew-pgp-check-old-pgp))))
-	  (if (null type)
-	      (message "No PGP message was found")
-	    (setq mew-inherit-decode-signer
-		  (mew-addrstr-parse-address
-		   (mew-header-get-value mew-from:)))
-	    (goto-char (mew-header-end))
-	    (forward-line)
-	    (delete-region (point) (point-max))
-	    (cond
-	     ((eq type 'pgp-encryption)
-	      (setq result (mew-pgp-decrypt 'dummy file)))
-	     ((eq type 'pgp-signature)
-	      (setq result (mew-pgp-verify-old file))))
-	    (setq file1 (nth 0 result))
-	    (mew-xinfo-set-pri-result
-	     (concat mew-x-mew: " <body> " (nth 1 result) "\n"))
-	    (save-excursion
-	      (mew-header-delete-lines (list mew-x-mew:))
-	      (goto-char (mew-header-end))
-	      (mew-decode-syntax-insert-privacy))
-	    (when (file-exists-p file1)
-	      (mew-frwlet
-	       mew-cs-text-for-read mew-cs-dummy
-	       (mew-insert-file-contents file1))
-	      (mew-delete-file file1))
-	    (if mew-mule-p
-		(mew-cs-decode-region (point) (point-max) mew-cs-autoconv))
-	    (mew-message-set-end-of)
-	    (set-window-start win start)
-	    (set-buffer-modified-p nil))))))))
+	 ;; We need MIME-decoded buffer to check PGP boundaries.
+	 (setq type (mew-old-pgp-check))
+	 (if type
+	    (mew-old-pgp-decode fld msg type)
+	   (message "No PGP data found")))))))
+
+(defun mew-old-pgp-decode (fld msg type)
+  ;; in Message buffer
+  (let* ((cache (mew-cache-hit fld msg 'must-hit))
+	 (syntax (mew-cache-decode-syntax cache))
+	 (file (mew-expand-folder fld msg))
+	 mew-inherit-decode-signer
+	 file1 file2 result xmew win start cte)
+    (with-temp-buffer
+      (mew-frwlet
+       mew-cs-text-for-read mew-cs-text-for-write
+       (mew-insert-file-contents file)
+       (re-search-forward mew-eoh nil t)
+       (forward-line)
+       (delete-region (point-min) (point))
+       (when (eq type 'pgp-signature)
+	 (setq cte (mew-syntax-get-cte (mew-syntax-get-part syntax)))
+	 (mew-decode-mime-body nil mew-ct-txt cte))
+       (setq file1 (mew-make-temp-name))
+       (write-region (point-min) (point-max) file1 nil 'no-msg)))
+    (mew-elet
+     (widen)
+     (setq win (get-buffer-window (current-buffer)))
+     (setq start (window-start win))
+     (setq mew-inherit-decode-signer
+	   (mew-addrstr-parse-address
+	    (mew-header-get-value mew-from:)))
+     (goto-char (mew-header-end))
+     (forward-line)
+     (delete-region (point) (point-max))
+     (cond
+      ((eq type 'pgp-encryption)
+       (setq result (mew-pgp-decrypt 'dummy file1)))
+      ((eq type 'pgp-signature)
+       (setq result (mew-old-pgp-verify file1))))
+     (mew-set '(file2 xmew) result)
+     (unless xmew (setq xmew "CRC error")) ;; xxx
+     (mew-xinfo-set-pri-result
+      (concat mew-x-mew: " <body> " xmew "\n"))
+     (save-excursion
+       (mew-header-delete-lines (list mew-x-mew:))
+       (goto-char (mew-header-end))
+       (mew-decode-syntax-insert-privacy))
+     (when (and file1 (file-exists-p file1))
+       (mew-delete-file file1))
+     (when (and file2 (file-exists-p file2))
+       (mew-frwlet
+	mew-cs-text-for-read mew-cs-dummy
+	(mew-insert-file-contents file2))
+       (mew-delete-file file2))
+     (if mew-mule-p
+	 (mew-cs-decode-region
+	  (point) (point-max)
+	  ;; This is valid for PGP signature only.
+	  ;; Since cipher text is encoded with Radix 64, charset should
+	  ;; be "US-ASCII". I don't know this code is valid for
+	  ;; cipher text in the case that a wrong charset is specified.
+	  (or (mew-charset-to-cs 
+	       (mew-syntax-get-param
+		(mew-syntax-get-ct (mew-syntax-get-part syntax)) "charset"))
+	      mew-cs-autoconv)))
+     (mew-message-set-end-of)
+     (set-window-start win start)
+     (set-buffer-modified-p nil))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -889,69 +1109,76 @@ public keyring."
 ;;; key fetch
 ;;;
 
-(defvar mew-pgp-fetch-process nil)
+(defun mew-pgp-fetch-key-by-xuri ()
+  (let ((key-list mew-x-pgp-key-list)
+	key val uri)
+    (save-excursion
+      (set-buffer (mew-buffer-message))
+      (catch 'loop
+	(while key-list
+	  (setq key (car key-list))
+	  (setq key-list (cdr key-list))
+	  (setq val (mew-header-get-value key))
+	  (when (and val (string-match "http:[^ \t\n]*" val))
+	    (setq val (mew-match-string 0 val))
+	    (if (y-or-n-p (format "Fetch a PGP public key from %s? " val))
+		(throw 'loop (setq uri val)))))))
+    (if uri
+	(mew-pgp-fetch uri)
+      (message "A PGP key is not fetched"))))
 
 (defun mew-pgp-fetch-key (arg)
-  "Fetch PGP public key."
+  "Fetch a PGP public key. 
+
+A PGP public key is fetch from one of the PGP PKS servers wich a
+key ID which is extracted from the X-Mew: field. The list of the
+PGP PKS servers is specified by 'mew-pgp-pks-servers'.
+
+If called with \\[universal-argument], a PGP public key is fetched 
+according to a URL in a field specified by 'mew-x-pgp-key-list'."
   (interactive "P")
-  (if (processp mew-pgp-fetch-process)
-      (if (y-or-n-p "Fetching process is running. Kill the process? ")
-	  (mew-pgp-fetch-process-kill))
-    (let (key key-list keyid xurl xurl-list xmew uri userid)
+  (if arg
+      (mew-pgp-fetch-key-by-xuri)
+    (let (keyid userid xmew)
       (save-excursion
 	(set-buffer (mew-buffer-message))
 	(setq userid (mew-header-parse-address mew-from:))
-	(cond
-	 (arg
-	  (setq key-list mew-x-pgp-key-list)
-	  (setq key (car key-list))
-	  (while key
-	    (setq xurl (mew-header-get-value key))
-	    (when (and xurl (string-match "http:[^ \t\n]*" xurl))
-	      (setq xurl (substring xurl (match-beginning 0) (match-end 0)))
-	      (setq xurl-list (append xurl-list (list xurl))))
-	    (setq key-list (cdr key-list))
-	    (setq key (car key-list))))
-	 (t
-	  (setq xmew (mew-header-get-value mew-x-mew:)))))
+	(setq xmew (mew-header-get-value mew-x-mew:)))
       (cond
-       ((and arg xurl-list)
-	(setq xurl (car xurl-list))
-	(while (and xurl (not uri))
-	  (if (y-or-n-p (format "fetch from %s? " xurl))
-	      (setq uri xurl))
-	  (setq xurl-list (cdr xurl-list))
-	  (setq xurl (car xurl-list))))
        ((and xmew
 	     (string-match "key. ID = \\(0x[0-9a-fA-F]+\\)" xmew nil)
-	     (setq keyid (substring xmew (match-beginning 1) (match-end 1))))
-	(if (y-or-n-p (format "Fetch Key ID=%s? " keyid))
-	    (setq uri (format mew-pgp-keyserver-url-template keyid))))
-       ((and userid (y-or-n-p (format "fetch User ID=%s? " userid)))
-	(setq uri (format mew-pgp-keyserver-url-template userid))))
-      (if uri
-	  (mew-pgp-fetch uri)
-	(message "Cannot find PGP public key's information")))))
+	     (setq keyid (mew-match-string 1 xmew)))
+	(if (y-or-n-p (format "Fetch the PGP public key for %s? " keyid))
+	    (mew-pgp-fetch-key-from-pks keyid)))
+       (userid
+	(if (y-or-n-p (format "Fetch the PGP public key for %s? " userid))
+	    (mew-pgp-fetch-key-from-pks userid)))
+       (t
+	(message "A PGP key is not fetched"))))))
+
+(defun mew-pgp-fetch-key-from-pks (&optional id)
+  (let* ((alist mew-pgp-pks-servers)
+	 (server (car (car alist)))
+	 uri)
+    (unless id
+      (setq id (read-string "PGP key id or e-mail address: ")))
+    (setq server (completing-read "PGP ker server: " alist nil nil server))
+    (unless (mew-pgp-pks-http-port-p server alist)
+      (setq server (concat server ":" mew-pgp-pks-port)))
+    (setq uri (concat "http://" server mew-pgp-pks-lang id))
+    (mew-pgp-fetch uri)))
 
 (defun mew-pgp-fetch (uri)
   (message "PGP key fetching...")
-  (mew-piolet
-   mew-cs-autoconv mew-cs-autoconv
-   (setq mew-pgp-fetch-process
-	 (apply 'start-process
-		"Mew PGP fetch"
-		(generate-new-buffer mew-buffer-prefix)
-		mew-prog-pgpkey
-		(append mew-prog-pgpkey-arg (list uri)))))
-  (set-process-sentinel mew-pgp-fetch-process 'mew-pgp-fetch-process-sentinel))
-
-(defun mew-pgp-fetch-process-kill ()
-  "Kill the current process fetching PGP keys."
-  (interactive)
-  (unwind-protect
-      (if (processp mew-pgp-fetch-process)
-	  (kill-process mew-pgp-fetch-process))
-    (setq mew-pgp-fetch-process nil)))
+  (let (pro)
+    (mew-piolet
+     mew-cs-autoconv mew-cs-autoconv
+     (setq pro (apply 'start-process
+		      "Mew PGP fetch"
+		      (generate-new-buffer mew-buffer-prefix)
+		      mew-prog-pgpkey
+		      (append mew-prog-pgpkey-args (list uri)))))
+    (set-process-sentinel pro 'mew-pgp-fetch-process-sentinel)))
 
 (defun mew-pgp-fetch-process-sentinel (process event)
   (message "PGP key fetching...done")
@@ -963,8 +1190,7 @@ public keyring."
 	       (search-forward mew-pgp-key-end nil t))
 	  (mew-mime-pgp-keys-ext (current-buffer) (point-min) (point-max))
 	(message "PGP key fetch failed")))
-    (mew-remove-buffer buf)
-    (setq mew-pgp-fetch-process nil)))
+    (mew-remove-buffer buf)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; <Decrypt> or <Decrypt-then-Verify>
