@@ -12,144 +12,259 @@
 ;;; Pick commands
 ;;;
 
-(defun mew-summary-search-mark (&optional arg)
-  "Pick messages according to a pick pattern which you input, 
-then put the '*' mark onto them. If called with '\\[universal-argument]',
-execute 'mew-summary-pick-with-external'. Otherwise, 
-'mew-summary-pick-with-mewls' is called."
+(defun mew-summary-pick-mark (&optional arg)
+  "Pick messages according to a pick pattern which you input,
+then put the '*' mark onto them. 'mewl' is called as a picking
+command. If called with '\\[universal-argument]', the target is
+the region."
   (interactive "P")
-  (mew-summary-or-thread
-   (let ((folder (mew-summary-folder-name))
-	 (preline 0)
-	 pattern msgs msgsback threadmsgs total linenum n)
-     (unless folder
-       ;; probably on a thread separator
-       (save-excursion
-	 (goto-char (point-min))
-	 (setq folder (mew-summary-folder-name))))
-     (if (null folder)
-	 (message "No message to be picked")
-       (setq pattern (if arg (mew-input-grep-pattern)
-		       (mew-input-pick-pattern)))
-       (mew-sinfo-set-find-key nil);; force to ask a user
-       (message "Picking messages in %s..." folder)
-       (if arg
-	   (setq msgs (mew-summary-pick-with-external folder pattern))
-	 (setq msgs (mew-summary-pick-with-mewls folder pattern)))
-       (setq msgsback msgs)
-       (message "Picking messages in %s...done" folder)
-       (if (null msgs)
-	   (message "No message to be marked")
-	 (setq n (length msgs))
-	 (if (= n 1)
-	     (message "Marking 1 message...")
-	   (message "Marking %d messages..." n))
-	 (save-excursion
-	   ;; Thread folder
-	   (when (mew-thread-p)
-	     (setq threadmsgs (mew-summary-thread-get-msglst
-			       (mew-vinfo-get-top) 'separator))
-	     (setq total (length threadmsgs))
-	     (goto-char (point-min))
-	     (while (and msgs (not (eobp)))
-	       (setq linenum (member (car msgs) threadmsgs))
-	       (if (null linenum)
-		   (setq msgsback (delete (car msgs) msgsback))
-		 (setq linenum (- total (length linenum)))
-		 (forward-line (- linenum preline))
-		 (setq preline linenum)
-		 (mew-summary-mark-as mew-mark-review))
-	       (setq msgs (cdr msgs)))
-	     (set-buffer-modified-p nil))
-	   ;; Physical folder
-	   (when (get-buffer folder)
-	     (set-buffer folder)
-	     (save-excursion
-	       (goto-char (point-min))
-	       (setq msgs msgsback)
-	       (while (and msgs (not (eobp)))
-		 (when (re-search-forward (mew-regex-jmp-msg (car msgs)) nil t)
-		   (mew-summary-mark-as mew-mark-review)
-		   (forward-line))
-		 (setq msgs (cdr msgs)))
-	       (set-buffer-modified-p nil))))
-	 (if (= n 1)
-	     (message "Marking 1 message...done")
-	   (message "Marking %d messages...done" n)))))))
+  (mew-pickable
+   (let ((func 'mew-summary-pick-with-mewl))
+     (if (and mew-prog-mewl
+	      (not (string= mew-prog-mewl ""))
+	      (mew-which-exec mew-prog-mewl))
+	 (mew-summary-pick-body func arg t)
+       (message "No command")))))
 
-(defun mew-summary-pick-with-mewls (folder pattern)
-  "A function to pick messages matching PATTERN with 'mewls'"
-  (mew-summary-with-mewls
-   (let ((buf (generate-new-buffer mew-buffer-prefix))
-	 (fld (mew-expand-folder2 folder))
-	 msgs)
-     (save-excursion
-       (set-buffer buf)
-       (mew-erase-buffer)
+(defun mew-summary-grep-mark (&optional arg)
+  "Grep messages according to a grep pattern which you input,
+then put the '*' mark onto them. 'mew-prog-grep' is called as a
+grep command. If called with '\\[universal-argument]', the target
+is the region."
+  (interactive "P")
+  (mew-pickable
+   (let ((func 'mew-summary-pick-with-grep))
+     (if (and mew-prog-grep
+	      (not (string= mew-prog-grep ""))
+	      (mew-which-exec mew-prog-grep))
+	 (mew-summary-pick-body func arg nil)
+       (message "No command")))))
+
+(defvar mew-input-cmd-hist (list mew-prog-grep))
+
+(defun mew-summary-cmd-mark (&optional arg)
+  "Grep messages according to a grep pattern which you input,
+then put the '*' mark onto them. You can asked to specify a grep
+command.  If called with '\\[universal-argument]', the target is
+the region."
+  (interactive "P")
+  (mew-pickable
+   (let ((func 'mew-summary-pick-with-cmd)
+	 (mew-inherit-grep-cmd (read-string "Command opts pattern: "
+                                            (car mew-input-cmd-hist)
+                                            'mew-input-cmd-hist)))
+     (mew-summary-pick-body func arg nil 'nopattern))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Pick body
+;;;
+
+(defun mew-summary-pick-body (func regionp internalp &optional nopattern)
+  (let* ((folder (mew-summary-folder-name))
+	 (dir (mew-expand-folder folder))
+	 (default-directory dir) ;; buffer local
+	 msg msgs beg end region doit pattern)
+    (if (mew-mark-active-p) (setq regionp t))
+    (cond
+     ((and internalp (not regionp))
+      (setq doit t)
+      (setq msgs nil))
+     (regionp
+      (setq region (mew-summary-get-region))
+      (setq beg (car region))
+      (setq end (cdr region))
+      (save-excursion
+	(goto-char beg)
+	(while (re-search-forward mew-regex-msg-or-part end t) ;; for thread
+	  (setq msg (mew-summary-message-number))
+	  (setq msgs (cons msg msgs))
+	  (forward-line)))
+      (setq msgs (delq nil msgs))
+      (setq msgs (nreverse msgs))
+      (if msgs (setq doit t)))
+     (t
+      (setq msgs (mew-dir-messages ".")) ;; all messages
+      (setq msgs (delq nil msgs))
+      (if msgs (setq doit t))))
+    (if (not doit)
+	(message "No message")
+      (if internalp
+	  (progn
+	    (setq pattern (mew-input-pick-pattern))
+	    (mew-sinfo-set-find-key nil)) ;; force to ask a user
+	(unless nopattern
+	  (setq pattern (mew-input-grep-pattern)))
+	(mew-sinfo-set-find-key pattern))
+      (mew-summary-pick-body1 func msgs pattern))))
+
+(defun mew-summary-pick-body1 (func src-msgs pattern)
+  (let ((folder (mew-summary-folder-name))
+	(preline 0)
+	msgs msgsback threadmsgs total linenum n)
+    (message "Picking messages in %s..." folder)
+    (setq msgs (funcall func folder pattern src-msgs))
+    (setq msgsback msgs)
+    (message "Picking messages in %s...done" folder)
+    (if (null msgs)
+	(message "No message to be marked")
+      (setq n (length msgs))
+      (if (= n 1)
+	  (message "Marking 1 message...")
+	(message "Marking %d messages..." n))
+      (save-excursion
+	(cond
+	 ((mew-thread-p)
+	  ;; Thread mode
+	  (setq threadmsgs (mew-summary-thread-get-msglst
+			    (mew-vinfo-get-top) 'separator))
+	  (setq total (length threadmsgs))
+	  (goto-char (point-min))
+	  (while (and msgs (not (eobp)))
+	    (setq linenum (member (car msgs) threadmsgs))
+	    (if (null linenum)
+		(setq msgsback (delete (car msgs) msgsback))
+	      (setq linenum (- total (length linenum)))
+	      (forward-line (- linenum preline))
+	      (setq preline linenum)
+	      (mew-summary-mark-as mew-mark-review))
+	    (setq msgs (cdr msgs)))
+	  (set-buffer-modified-p nil))
+	 ((mew-virtual-p)
+	  ;; Virtual mode
+	  (goto-char (point-min))
+	  (setq msgsback nil)
+	  (while (and msgs (not (eobp)))
+	    (when (re-search-forward (mew-regex-sumsyn-msg (car msgs)) nil t)
+	      (setq msgsback (cons (car msgs) msgsback))
+	      (mew-summary-mark-as mew-mark-review)
+	      (forward-line))
+	    (setq msgs (cdr msgs)))
+	  (setq msgsback (nreverse msgsback))
+	  (set-buffer-modified-p nil)))
+	;; Summary mode (physical)
+	(when (get-buffer folder)
+	  (set-buffer folder)
+	  (save-excursion
+	    (goto-char (point-min))
+	    (setq msgs msgsback)
+	    (while (and msgs (not (eobp)))
+	      (when (re-search-forward (mew-regex-sumsyn-msg (car msgs)) nil t)
+		(mew-summary-mark-as mew-mark-review)
+		(forward-line))
+	      (setq msgs (cdr msgs)))
+	    (set-buffer-modified-p nil))))
+      (if (= n 1)
+	  (message "Marking 1 message...done")
+	(message "Marking %d messages...done" n)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Pick internal
+;;;
+
+(defun mew-summary-pick-with-mewl (folder pattern src-msgs)
+  "A function to pick messages matching PATTERN with 'mewl'"
+  (mew-summary-with-mewl
+   (let ((fld (mew-expand-folder2 folder))
+	 i min max msgs range)
+     (when src-msgs
+       (setq min (string-to-int (car src-msgs)))
+       (setq max min)
+       (setq src-msgs (cdr src-msgs))
+       (while src-msgs
+	 (setq i (string-to-int (car src-msgs)))
+	 (if (< i min) (setq min i))
+	 (if (> i max) (setq max i))
+	 (setq src-msgs (cdr src-msgs)))
+       (setq range (list (format "%d-%d" min max))))
+     (with-temp-buffer
        (mew-set-buffer-multibyte t)
        (mew-piolet
 	mew-cs-text-for-read mew-cs-text-for-write
-	(call-process mew-prog-mewls nil t nil
-		      "-b" mew-mail-path
-		      "-l" (int-to-string mew-scan-max-field-length)
-		      "-p" pattern "-s" fld))
+	(apply 'call-process
+	       mew-prog-mewl nil t nil
+	       "-b" mew-mail-path
+	       "-l" (int-to-string mew-scan-max-field-length)
+	       "-p" pattern fld range))
        (goto-char (point-min))
        (while (not (eobp))
 	 (if (looking-at mew-regex-message-files)
 	     (setq msgs (cons (mew-match-string 0) msgs)))
 	 (forward-line)))
-     (mew-remove-buffer buf)
      (nreverse msgs))))
 
-(defvar mew-summary-pick-with-external-function
-  'mew-summary-pick-with-grep
-  "*A function to be called by '\\[universal-argument] \\<mew-summary-mode-map>\\[mew-summary-search-mark]'.
-This function MUST return text in which each line begins with a file name.")
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Grep
+;;;
 
-(defun mew-summary-pick-with-grep (pattern msgs)
-  "A function to pick messages matching PATTERN with 'grep'."
-  (setq msgs (delq nil msgs))
-  (if (= (length msgs) 1) (setq msgs (cons "/dev/null" msgs)))
-  (setq pattern (mew-cs-encode-arg pattern))
-  (let (nxt)
-    (while msgs
-      (goto-char (point-max))
-      (setq nxt (nthcdr mew-prog-grep-max-msgs msgs))
-      (if nxt (setcdr (nthcdr (1- mew-prog-grep-max-msgs) msgs) nil))
-      (apply 'call-process
-	     mew-prog-grep nil t nil
-	     (append mew-prog-grep-opts (list pattern) msgs))
-      (setq msgs nxt))))
-
-(defun mew-summary-pick-with-external (folder pattern)
-  "A function to pick messages matching PATTERN with 
-'mew-summary-pick-with-external-function'."
+(defun mew-summary-pick-with-grep (folder pattern src-msgs)
+  "A function to pick messages matching PATTERN."
   (let* ((dir (mew-expand-folder folder))
 	 (default-directory dir) ;; buffer local
-	 (msgs (mew-dir-messages "."))
-	 buf)
+	 msgs nxt)
     ;; no sort here
-    (when msgs
-      (save-excursion
-	(setq buf (generate-new-buffer mew-buffer-prefix))
-	(set-buffer buf)
-	(mew-erase-buffer)
-	(cd dir)
-	(mew-piolet
-	 mew-cs-text-for-read mew-cs-text-for-write
-	 (mew-alet
-	  (funcall mew-summary-pick-with-external-function pattern msgs)))
-	(goto-char (point-min))
-	(setq msgs nil)
-	(while (not (eobp))
-	  (if (looking-at mew-regex-message-files2)
-	      (setq msgs (cons (mew-match-string 0) msgs)))
-	  (forward-line)))
-      (mew-remove-buffer buf)
-      (setq msgs (mew-uniq-list msgs))
-      (setq msgs (mapcar 'string-to-int msgs))
-      (setq msgs (sort msgs '<))
-      (mapcar 'int-to-string msgs))))
+    (if (= (length src-msgs) 1) (setq src-msgs (cons "/dev/null" src-msgs)))
+    (setq pattern (mew-cs-encode-arg pattern))
+    (with-temp-buffer
+      (cd dir)
+      (mew-piolet
+       mew-cs-text-for-read mew-cs-text-for-write
+       (while src-msgs
+	 (goto-char (point-max))
+	 (setq nxt (nthcdr mew-prog-grep-max-msgs src-msgs))
+	 (if nxt (setcdr (nthcdr (1- mew-prog-grep-max-msgs) src-msgs) nil))
+	 (apply 'call-process
+		mew-prog-grep nil t nil
+		(append mew-prog-grep-opts (list pattern) src-msgs))
+	 (setq src-msgs nxt)))
+      (goto-char (point-min))
+      (while (not (eobp))
+	(if (looking-at mew-regex-message-files2)
+	    (setq msgs (cons (mew-match-string 0) msgs)))
+	(forward-line)))
+    (setq msgs (mew-uniq-list msgs))
+    (setq msgs (mapcar 'string-to-int msgs))
+    (setq msgs (sort msgs '<))
+    (mapcar 'int-to-string msgs)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Cmd
+;;;
+
+(defun mew-summary-pick-with-cmd (folder pattern src-msgs)
+  "A function to pick messages matching PATTERN."
+  ;; pattern is ignored
+  (let* ((dir (mew-expand-folder folder))
+	 (default-directory dir) ;; buffer local
+	 (cmd mew-inherit-grep-cmd)
+	 msgs nxt arg)
+    ;; no sort here
+    (if (= (length src-msgs) 1) (setq src-msgs (cons "/dev/null" src-msgs)))
+    (with-temp-buffer
+      (cd dir)
+      (mew-piolet
+       mew-cs-text-for-read mew-cs-text-for-write
+       (while src-msgs
+	 (goto-char (point-max))
+	 (setq nxt (nthcdr mew-prog-grep-max-msgs src-msgs))
+	 (if nxt (setcdr (nthcdr (1- mew-prog-grep-max-msgs) src-msgs) nil))
+	 (setq arg (mapconcat 'identity (cons cmd src-msgs) " "))
+	 (setq arg (mew-cs-encode-arg arg))
+	 ;; call shell to handle quote stuff
+	 (call-process shell-file-name nil t nil shell-command-switch arg)
+	 (setq src-msgs nxt)))
+      (goto-char (point-min))
+      (while (not (eobp))
+	(if (looking-at mew-regex-message-files2)
+	    (setq msgs (cons (mew-match-string 0) msgs)))
+	(forward-line)))
+    (setq msgs (mew-uniq-list msgs))
+    (setq msgs (mapcar 'string-to-int msgs))
+    (setq msgs (sort msgs '<))
+    (mapcar 'int-to-string msgs)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -264,6 +379,7 @@ keyword, execute this command with '\\[universal-argument]'."
 (defvar mew-pick-macro-alist nil)
 
 (defun mew-pick-define-macro (str1 str2)
+  "Define pick macro."
   (interactive (list (read-string "Pick pattern: ")
 		     (read-string "Macro body: ")))
   ;; macro-pattern is a string including no #, or
@@ -312,7 +428,7 @@ keyword, execute this command with '\\[universal-argument]'."
       ""
     (let ((first (string-to-char str))
 	  asc key rest eq-flag val args)
-      (if (memq first '(?\( ?\! ?\& ?\| ?= ? ?\)))
+      (if (memq first (list ?\( ?\! ?\& ?\| ?= mew-sp ?\)))
 	  (concat (char-to-string first)
 		  (mew-pick-macro-expand-string (substring str 1)))
 	(if (string-match "=\\| \\|)\\|&\\||" str)
@@ -355,7 +471,7 @@ keyword, execute this command with '\\[universal-argument]'."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
