@@ -12,6 +12,9 @@
 (defun mew-pop-folder-alist ()
   mew-pop-folder-alist)
 
+;; It is impossible to implement mew-pop-skip-uidl since
+;; messages cannot be gaind access by UID.
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; POP info
@@ -19,12 +22,12 @@
 
 (defvar mew-pop-info-list
   '("server" "port" "process" "ssh-process" "ssl-process" "status"
-    "directive" "bnm"
+    "directive" "bnm" "mdb"
     "rtrs" "dels" "refs" "rmvs" "kils" "left" "uidl" "range"
     "rttl" "rcnt" "dttl" "dcnt" "hlds"
-    "user" "auth" "auth-list" "key" "passwd"
+    "user" "auth" "auth-list" "key" "passwd" "account"
     "size" "truncated" "get-body"
-    "flush" "no-msg" "msgdb" "done"
+    "flush" "no-msg" "msgdb" "done" "dispatched" "error"
     "body-lines" "delete"))
 
 (mew-info-defun "mew-pop-" mew-pop-info-list)
@@ -82,7 +85,7 @@
 	  auth func)
       (save-excursion
 	(goto-char (point-min))
-	(if (re-search-forward "SASL \\([^\n\r]+\\)" nil t)
+	(if (re-search-forward "SASL \\([^\r\n]+\\)" nil t)
 	    (setq auth (mew-auth-select (mew-match-string 1) auth-list))))
       (if (and auth
 	       (setq func (mew-pop-auth-get-func auth))
@@ -109,18 +112,20 @@
     (mew-pop-process-send-string pro "USER %s" user)))
 
 (defun mew-pop-command-pass (pro pnm)
-  (let (passwd)
-    (setq passwd (mew-pop-input-passwd "POP password: " pnm))
+  (let* ((prompt (format "POP password (%s): "
+                        (mew-pop-get-account pnm)))
+          (passwd (mew-pop-input-passwd prompt pnm)))
     (mew-pop-message pnm "Sending your POP password to the POP server...")
     (mew-pop-process-send-string pro "PASS %s" passwd)))
 
 (defun mew-pop-command-apop (pro pnm)
   (let ((user (mew-pop-get-user pnm))
+        (prompt (format "APOP password (%s): " (mew-pop-get-account pnm)))
 	(key (mew-pop-get-key pnm))
 	passwd kmd5)
     (cond
      (key
-      (setq passwd (mew-pop-input-passwd "APOP password: " pnm))
+      (setq passwd (mew-pop-input-passwd prompt pnm))
       (setq kmd5 (mew-keyed-md5 key passwd))
       (mew-pop-message pnm "Sending your APOP password to the POP server...")
       (mew-pop-process-send-string pro "APOP %s %s" user kmd5))
@@ -136,7 +141,8 @@
 	(clear-pass t))
     (goto-char (point-min))
     (cond
-     ((re-search-forward " lock" nil t) ;; very ad hoc
+     ((or (looking-at "-ERR \\[IN-USE\\]") ;; RFC2449
+	  (re-search-forward " lock" nil t)) ;; for old servers: very ad hoc
       (mew-pop-message pnm "The mailbox is locked!")
       (setq clear-pass nil))
      ((or (eq auth 'pass) (eq auth 'user))
@@ -147,7 +153,7 @@
       (mew-pop-message pnm "%s password is wrong!" (upcase auth)))
      (t
       ;; pnm may be cleared already
-      (mew-pop-message pnm "Password is wrong!")))
+      (mew-pop-message pnm "POP password is wrong!")))
     (if clear-pass (mew-passwd-set-passwd (mew-pop-passtag pnm) nil))
     (if (eq directive 'exec)
 	(mew-summary-visible-buffer bnm))
@@ -197,7 +203,7 @@
     (if (or (eq directive 'scan) (eq directive 'exec) (eq directive 'sync))
 	(progn
 	  (mew-pop-message pnm "POP server does not support this command")
-	  (mew-pop-command-quit2 pro pnm)) ;; xxx not to set "done"
+	  (mew-pop-command-quit2 pro pnm))
       (mew-pop-set-size pnm 0) ;; To get the entire message
       (mew-pop-set-rttl pnm (mew-pop-get-left pnm))
       (when (mew-pop-get-delete pnm)
@@ -290,6 +296,7 @@
     (mew-pop-set-rttl pnm (length rtrs))
     (mew-pop-set-dttl pnm (length dels))
     (mew-pop-set-left pnm left)
+    (mew-pop-set-dispatched pnm t)
     (cond
      ((or (eq directive 'sync) (eq directive 'biff))
       (mew-pop-set-status pnm "quit")
@@ -321,8 +328,8 @@
 
 (defun mew-pop-command-dels (pro pnm)
   (mew-net-status1
-   (mew-pop-get-bnm pnm) (mew-pop-get-dttl pnm) (mew-pop-get-dcnt pnm)
-   (mew-pop-secure-p pnm))
+   (mew-pop-get-bnm pnm) "Deleting"
+   (mew-pop-get-dttl pnm) (mew-pop-get-dcnt pnm) (mew-pop-secure-p pnm))
   (let ((directive (mew-pop-get-directive pnm))
 	(dels (mew-pop-get-dels pnm))
 	(left (mew-pop-get-left pnm))
@@ -400,15 +407,14 @@
 	 (del  (nth 3 rtr))
 	 (fld-msg (nth 4 rtr))
 	 (truncated (mew-pop-get-truncated pnm))
-	 fld msg vec file msg-file lmsg)
+	 fld msg vec file msg-file lmsg mark)
     (cond
      ((null fld-msg)
       (setq fld (mew-pop-get-bnm pnm)))
      ((stringp fld-msg)
       (setq fld fld-msg))
      ((listp fld-msg)
-      (setq fld (nth 0 fld-msg))
-      (setq msg (nth 1 fld-msg))
+      (mew-set '(fld msg) fld-msg)
       (setq lmsg msg)))
     ;; deleting +OK
     (goto-char (point-min))
@@ -426,7 +432,7 @@
       (mew-header-insert-xmu uid siz t))
      ((eq directive 'scan)
       (mew-header-insert-xmu uid siz nil))
-     ((and (eq directive 'get) (mew-folder-popp (mew-folder-folder fld)))
+     ((and (eq directive 'get) (mew-folder-popp (mew-case:folder-folder fld)))
       (mew-header-insert-xmu uid siz nil)))
     (catch 'write-error
       (condition-case nil
@@ -441,11 +447,14 @@
       (mew-pop-set-rcnt pnm (1+ (mew-pop-get-rcnt pnm)))
       ;;
       (mew-set-buffer-multibyte t)
-      (setq vec (mew-pop-scan-header))
+      (setq vec (mew-scan-header))
       (mew-scan-set-folder vec fld)
       (mew-scan-set-message vec msg)
+      (if (or (eq directive 'inc) (eq directive 'scan))
+	  (setq mark (mew-scan-inbox-action vec)))
+      (mew-scan-body vec)
       (mew-set-buffer-multibyte nil)
-      (mew-scan-insert-line fld vec width lmsg nil)
+      (mew-scan-insert-line fld vec width lmsg mark)
       (mew-pop-set-rtrs pnm (cdr rtrs))
       (if (and del (not truncated))
 	  (progn
@@ -459,8 +468,8 @@
   (mew-pop-process-send-string pro "QUIT"))
 
 (defun mew-pop-command-quit2 (pro pnm)
-  ;; If the server disconnects the connection, this function is not called.
-  ;; If not, the client must send "QUIT".
+  (mew-pop-set-done pnm t)
+  (mew-pop-set-error pnm t)
   (mew-pop-set-status pnm "quit")
   (mew-pop-process-send-string pro "QUIT"))
 
@@ -489,7 +498,7 @@
       (goto-char (point-min))
       (if (re-search-forward" \\([a-zA-Z0-9+/]+=*\\)" nil t)
 	  (setq challenge (mew-match-string 1))))
-    (setq passwd (mew-pop-input-passwd "CRAM-MD5: " pnm))
+    (setq passwd (mew-pop-input-passwd "POP CRAM-MD5: " pnm))
     (setq cram-md5 (mew-cram-md5 user passwd challenge))
     (mew-pop-process-send-string pro cram-md5)))
 
@@ -516,23 +525,12 @@
 (defsubst mew-pop-buffer-name (pnm)
   (concat mew-buffer-prefix pnm))
 
-(defun mew-pop-debug (label string)
-  (when (mew-debug 'net)
-    (save-excursion
-      (set-buffer (get-buffer-create mew-buffer-debug))
-      (goto-char (point-max))
-      (insert (format "\n<%s>\n%s\n" label string)))))
-
 (defun mew-pop-process-send-string (pro &rest args)
   (let ((str (apply 'format args)))
     (mew-pop-debug "=SEND=" str)
-    (process-send-string pro (concat str mew-cs-eol))))
-
-(defun mew-pop-scan-header ()
-  (goto-char (point-min))
-  (unless (re-search-forward mew-eoh nil t)
-    (goto-char (point-max)))
-  (mew-scan-header))
+    (if (and (processp pro) (eq (process-status pro) 'open))
+	(process-send-string pro (concat str mew-cs-eol))
+      (message "POP time out")))) ;; not mew-pop-message
 
 (defsubst mew-pop-message (pnm &rest args)
   (or (mew-pop-get-no-msg pnm) (apply 'message args)))
@@ -563,11 +561,16 @@
 
 (defun mew-pop-input-passwd (prompt pnm)
   (let ((directive (mew-pop-get-directive pnm))
-	(tag (mew-pop-passtag pnm)))
+	(pro (mew-pop-get-process pnm))
+	(tag (mew-pop-passtag pnm))
+	pass)
     (if (eq directive 'biff)
 	(or (mew-pop-get-passwd pnm)       ;; mew-pop-biff
 	    (mew-input-passwd prompt tag)) ;; mew-pop-check
-      (mew-input-passwd prompt tag))))
+      (setq pass (mew-input-passwd prompt tag))
+      (unless (and (processp pro) (eq (process-status pro) 'open))
+	(mew-passwd-set-passwd tag nil))
+      pass)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -605,10 +608,13 @@
 (defun mew-pop-retrieve (case directive bnm &rest args)
   ;; in +inbox
   (let* ((server (mew-pop-server case))
+         (user (mew-pop-user case))
 	 (port (mew-pop-port case))
 	 (sshsrv (mew-pop-ssh-server case))
 	 (sslp (mew-pop-ssl case))
 	 (sslport (mew-pop-ssl-port case))
+         (proxysrv (mew-pop-proxy-server case))
+         (proxyport (mew-pop-proxy-port case))
 	 (pnm (mew-pop-info-name case))
 	 (buf (get-buffer-create (mew-pop-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
@@ -617,7 +623,7 @@
 	(message "Another POP process is running. Try later")
       (cond
        (sshsrv
-	(setq sshpro (mew-open-ssh-stream server port sshsrv))
+	(setq sshpro (mew-open-ssh-stream case server port sshsrv))
 	(when sshpro
 	  (setq sshname (process-name sshpro))
 	  (setq lport (mew-ssh-pnm-to-lport sshname))
@@ -630,13 +636,16 @@
 	  (setq lport (mew-ssl-pnm-to-lport sslname))
 	  (when lport
 	    (setq process (mew-pop-open pnm "localhost" lport no-msg)))))
+       (proxysrv
+	(setq process (mew-pop-open pnm proxysrv proxyport no-msg)))
        (t
 	(setq process (mew-pop-open pnm server port no-msg))))
       (if (null process)
 	  (if (eq directive 'exec)
 	      (mew-summary-visible-buffer bnm))
 	(mew-summary-lock process "POPing" (or sshpro sslpro))
-	(mew-sinfo-set-scan-form (mew-summary-scan-form bnm))
+	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
+	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
 	(mew-info-clean-up pnm)
 	(mew-pop-set-no-msg pnm no-msg) ;; must come here
@@ -646,7 +655,8 @@
 	(mew-pop-set-ssl-process pnm sslpro)
 	(mew-pop-set-server pnm server)
 	(mew-pop-set-port pnm port)
-	(mew-pop-set-user pnm (mew-pop-user case))
+	(mew-pop-set-user pnm user)
+        (mew-pop-set-account pnm (format "%s@%s" user server))
 	(mew-pop-set-auth pnm (mew-pop-auth case))
 	(mew-pop-set-auth-list pnm (mew-pop-auth-list case))
 	(mew-pop-set-status pnm "greeting")
@@ -662,20 +672,25 @@
 	 ((eq directive 'biff)
 	  (mew-pop-set-passwd pnm (nth 0 args))) ;; password
 	 ((eq directive 'inc)
+	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
 	  (mew-pop-set-flush pnm (nth 0 args)) ;; no-flush
 	  (mew-pop-set-delete pnm (mew-pop-delete case)))
 	 ((eq directive 'get)
+	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
 	  (mew-pop-set-refs pnm (nth 0 args)))
 	 ((eq directive 'scan)
 	  (mew-pop-set-range pnm (nth 0 args))
 	  (mew-pop-set-get-body pnm (nth 1 args))
-	  (if (mew-pop-get-range pnm) (mew-net-folder-clean)))
+	  (if (mew-pop-get-range pnm)
+	      (progn
+		(mew-pop-set-mdb pnm (mew-summary-mark-collect4))
+		(mew-net-folder-clean))
+	    (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))))
 	 ((eq directive 'sync)
 	  )
 	 ((eq directive 'exec)
-	  ;; (nth 0 args) is nil
-	  (mew-pop-set-rmvs pnm (nth 1 args))
-	  (mew-pop-set-kils pnm (nth 2 args))))
+	  (mew-pop-set-rmvs pnm (nth 0 (nth 0 args)))
+	  (mew-pop-set-kils pnm (nth 1 (nth 0 args)))))
 	(mew-sinfo-set-start-point (point)) ;; after erase-buffer
 	;;
 	(set-process-sentinel process 'mew-pop-sentinel)
@@ -687,6 +702,13 @@
 ;;; Filter and sentinel
 ;;;
 
+(defun mew-pop-debug (label string)
+  (when (mew-debug 'net)
+    (save-excursion
+      (set-buffer (get-buffer-create mew-buffer-debug))
+      (goto-char (point-max))
+      (insert (format "\n<%s>\n%s\n" label string)))))
+
 (defun mew-pop-filter (process string)
   (let* ((pnm (process-name process))
 	 (status (mew-pop-get-status pnm))
@@ -712,7 +734,7 @@
       ((and (and (goto-char (point-min)) (looking-at "\\+")) ;; +OK
 	    (or (and mulrep
 		     (goto-char (point-max))
-		     (forward-line -1)
+		     (= (forward-line -1) 0)
 		     (looking-at "^\\.\r?$"))
 		(and (not mulrep)
 		     (goto-char (1- (point-max)))
@@ -726,97 +748,130 @@
        (setq func (intern-soft (concat "mew-pop-command-" next)))
        (goto-char (point-min))
        (if (fboundp func)
-	   (and func (funcall func process pnm))
+	   (funcall func process pnm)
 	 (error "No function called %s" (symbol-name func)))
-       (mew-erase-buffer)))))
+       (if (and process (equal (process-buffer process) (current-buffer)))
+	   (mew-erase-buffer))))))
 
 (defun mew-pop-sentinel (process event)
   (let* ((pnm (process-name process))
 	 (directive (mew-pop-get-directive pnm))
+	 (mdb (mew-pop-get-mdb pnm))
 	 (sshpro (mew-pop-get-ssh-process pnm))
 	 (sslpro (mew-pop-get-ssl-process pnm))
 	 (rttl (mew-pop-get-rttl pnm))
 	 (dttl (mew-pop-get-dttl pnm))
 	 (left (mew-pop-get-left pnm))
-	 (bnm (mew-pop-get-bnm pnm))
+	 (bnm (or (mew-pop-get-bnm pnm) (current-buffer)))
 	 (flush (mew-pop-get-flush pnm))
 	 (kils (mew-pop-get-kils pnm))
 	 (hlds (mew-pop-get-hlds pnm))
 	 (uidl (mew-pop-get-uidl pnm))
 	 (done (mew-pop-get-done pnm))
-	 (file (mew-expand-folder bnm mew-pop-msgid-file)))
+ 	 (error (mew-pop-get-error pnm))
+	 (file (mew-expand-folder bnm mew-pop-msgid-file))
+	 (buf (process-buffer process)))
     (mew-pop-debug "POP SENTINEL" event)
-    (mew-filter
-     (set-buffer bnm)
-     (when done
-       (cond
-	((eq directive 'biff)
-	 (funcall mew-biff-function rttl))
-	((eq directive 'sync)
-	 (mew-pop-message pnm "Synchronizing messages...")
-	 (mew-net-folder-sync bnm hlds)
-	 (mew-pop-message pnm "Synchronizing messages...done"))
-	((eq directive 'inc)
-	 (mew-biff-clear)
-	 (mew-net-uidl-db-set (mew-pop-passtag pnm) uidl)
-	 (cond
-	  ((= rttl 0)
-	   (mew-pop-message2 pnm "No new messages" left))
-	  ((= rttl 1)
-	   (mew-pop-message2 pnm "1 message retrieved" left)
-	   (mew-summary-folder-cache-save))
-	  (t
-	   (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
-	   (mew-summary-folder-cache-save))))
-	((eq directive 'get)
-	 (cond
-	  ((= rttl 0)
-	   (mew-pop-message2 pnm "No new messages" left))
-	  ((= rttl 1)
-	   (mew-pop-message2 pnm "1 message retrieved" left)
-	   (mew-summary-folder-cache-save))
-	  (t
-	   (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
-	   (mew-summary-folder-cache-save))))
-	((eq directive 'scan)
-	 (mew-biff-clear)
-	 (cond
-	  ((= rttl 0)
-	   (mew-pop-message pnm "No messages scanned"))
-	  ((= rttl 1)
-	   (mew-pop-message pnm "1 message scanned")
-	   (mew-lisp-save file uidl)
-	   (mew-summary-folder-cache-save))
-	  (t
-	   (mew-pop-message pnm "%d messages scanned" rttl)
-	   (mew-lisp-save file uidl)
-	   (mew-summary-folder-cache-save))))
-	((eq directive 'exec)
-	 (when kils
-	   (mew-mark-exec-unlink bnm kils)
-	   (mew-mark-kill-invisible)
-	   (mew-summary-folder-cache-save))
-	 (cond
-	  ((= dttl 0)
-	   (mew-pop-message pnm "No messages deleted"))
-	  ((= dttl 1)
-	   (mew-pop-message pnm "1 message deleted"))
-	  (t
-	   (mew-pop-message pnm "%d messages deleted" dttl))))))
-     ;;
-     (mew-net-status-clear (mew-pop-get-bnm pnm))
-     (mew-info-clean-up pnm)
-     (set-buffer-modified-p nil)
-     (mew-summary-unlock)
-     (if (and (processp sshpro) (not mew-ssh-keep-connection))
-	 (process-send-string sshpro "exit\n"))
-     (if (and (processp sslpro) (not mew-ssl-keep-connection))
-	 (delete-process sslpro))
-     (unless (eq directive 'biff)
-       (run-hooks 'mew-pop-sentinel-non-biff-hook))
-     (run-hooks 'mew-pop-sentinel-hook)
-     (when (and mew-auto-flush-queue flush)
-       (mew-smtp-flush-queue mew-case-output)))))
+    (set-process-buffer process nil)
+    (set-buffer bnm)
+    (mew-summary-mark-recover mdb)
+    (mew-remove-buffer buf)
+    (if (not done)
+	(let* ((rtrs (mew-pop-get-rtrs pnm))
+	       (lefts (length rtrs))
+	       (uid (nth 1 (car rtrs)))
+	       recovered)
+	  (mew-pop-message pnm "POP connection is lost")
+	  (when (mew-pop-get-dispatched pnm)
+	    (cond
+	     ((eq directive 'scan)
+	      ;; uidl is reversed.
+	      (setq uidl (cdr (member uid uidl)))
+	      (mew-lisp-save file uidl nil 'unlimit)
+	      (setq recovered t))
+	     ((eq directive 'inc)
+	      ;; uidl is reversed.
+	      (setq uid (assoc uid uidl))
+	      (setq uidl (cdr (member uid uidl)))
+	      (mew-net-uidl-db-set (mew-pop-passtag pnm) uidl)
+	      (setq recovered t)))
+	    (when recovered
+	      (mew-pop-message
+	       pnm
+	       "%d message retrieved. %d messages are left due to an error."
+	       (- rttl lefts) lefts)
+	      (mew-summary-folder-cache-save))))
+      (cond
+       (error 
+ 	;; retain the error message
+ 	)
+       ((eq directive 'biff)
+	(funcall mew-biff-function rttl))
+       ((eq directive 'sync)
+	(mew-pop-message pnm "Synchronizing messages...")
+	(mew-net-folder-sync bnm hlds)
+	(mew-pop-message pnm "Synchronizing messages...done"))
+       ((eq directive 'inc)
+	(mew-biff-clear)
+	(mew-net-uidl-db-set (mew-pop-passtag pnm) uidl)
+	(cond
+	 ((= rttl 0)
+	  (mew-pop-message2 pnm "No new messages" left))
+	 ((= rttl 1)
+	  (mew-pop-message2 pnm "1 message retrieved" left)
+	  (mew-summary-folder-cache-save))
+	 (t
+	  (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
+	  (mew-summary-folder-cache-save))))
+       ((eq directive 'get)
+	(cond
+	 ((= rttl 0)
+	  (mew-pop-message2 pnm "No new messages" left))
+	 ((= rttl 1)
+	  (mew-pop-message2 pnm "1 message retrieved" left)
+	  (mew-summary-folder-cache-save))
+	 (t
+	  (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
+	  (mew-summary-folder-cache-save))))
+       ((eq directive 'scan)
+	(mew-biff-clear)
+	(cond
+	 ((= rttl 0)
+	  (mew-pop-message pnm "No messages scanned"))
+	 ((= rttl 1)
+	  (mew-pop-message pnm "1 message scanned")
+	  (mew-lisp-save file uidl nil 'unlimit)
+	  (mew-summary-folder-cache-save))
+	 (t
+	  (mew-pop-message pnm "%d messages scanned" rttl)
+	  (mew-lisp-save file uidl nil 'unlimit)
+	  (mew-summary-folder-cache-save))))
+       ((eq directive 'exec)
+	(when kils
+	  (mew-mark-exec-unlink bnm kils)
+	  (mew-mark-kill-invisible)
+	  (mew-summary-folder-cache-save))
+	(cond
+	 ((= dttl 0)
+	  (mew-pop-message pnm "No messages deleted"))
+	 ((= dttl 1)
+	  (mew-pop-message pnm "1 message deleted"))
+	 (t
+	  (mew-pop-message pnm "%d messages deleted" dttl))))))
+    ;;
+    (mew-net-status-clear bnm)
+    (mew-info-clean-up pnm)
+    (set-buffer-modified-p nil)
+    (mew-summary-unlock)
+    (if (and (processp sshpro) (not mew-ssh-keep-connection))
+	(process-send-string sshpro "exit\n"))
+    (if (and (processp sslpro) (not mew-ssl-keep-connection))
+	(delete-process sslpro))
+    (unless (eq directive 'biff)
+      (run-hooks 'mew-pop-sentinel-non-biff-hook))
+    (run-hooks 'mew-pop-sentinel-hook)
+    (when (and mew-auto-flush-queue flush)
+      (mew-smtp-flush-queue mew-case-output))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -826,23 +881,28 @@
 (defun mew-pop-biff ()
   (let* ((case mew-case-input)
 	 (inbox (mew-proto-inbox-folder nil case))
+	 (case:inbox (mew-case-folder case inbox))
 	 (tag (mew-pop-passtag2 case))
 	 passwd)
-    (save-excursion
-      (set-buffer inbox)
-      (when (and (mew-summary-exclusive-p 'no-msg)
-		 (and mew-use-cached-passwd
-		      (setq passwd (mew-passwd-get-passwd tag))))
-	(mew-pop-retrieve case 'biff inbox passwd)))))
+    (when (get-buffer case:inbox)
+      (save-excursion
+	(set-buffer case:inbox)
+	(when (and (mew-summary-exclusive-p 'no-msg)
+		   (and mew-use-cached-passwd
+			(setq passwd (mew-passwd-get-passwd tag))))
+	  (mew-pop-retrieve case 'biff case:inbox passwd))))))
 
 (defun mew-pop-check ()
+  "See if messages are arrived by POP."
   (interactive)
   (let* ((case mew-case-input)
-	 (inbox (mew-proto-inbox-folder nil case)))
-    (save-excursion
-      (set-buffer inbox)
-      (when (mew-summary-exclusive-p)
-	(mew-pop-retrieve case 'biff inbox)))))
+	 (inbox (mew-proto-inbox-folder nil case))
+	 (case:inbox (mew-case-folder case inbox)))
+    (when (get-buffer case:inbox)
+      (save-excursion
+	(set-buffer case:inbox)
+	(when (mew-summary-exclusive-p)
+	  (mew-pop-retrieve case 'biff case:inbox))))))
 
 (provide 'mew-pop)
 
