@@ -136,54 +136,67 @@ mew-refile-guess-from-me-is-special."
       (setq mew-assoc-folder 'mew-assoc-folder-fast)
     (setq mew-assoc-folder 'mew-assoc-folder-slow)))
 
-(defun mew-assoc-folder-slow (key alist localp)
+(defun mew-assoc-folder-slow (key alist sep)
   ;; Case-sensitive so you can use capital letters for your folders.
   ;; But this is slow.
   (let ((skey (downcase key))
- 	(regex (concat mew-path-separator "$"))
-	ent name ret)
+	(node-regex (concat (regexp-quote sep) "$"))
+	name ret first)
     (catch 'loop
       (while alist
- 	(setq ent (car alist))
- 	(setq name (cdr ent))
+ 	(setq ret (car alist))
+ 	(setq name (cdr ret))
  	(when (and (stringp name) (string= (downcase name) skey))
- 	  (setq ret ent)
- 	  (unless (and localp (string-match regex (car ent)))
+	  (unless first
+	    (setq first ret))
+ 	  (unless (string-match node-regex (car ret))
 	    (throw 'loop nil)))
+	(setq ret nil)
  	(setq alist (cdr alist))))
-    ret))
+    (if mew-use-node-folder
+	(or ret first)
+      ret)))
 
-(defun mew-assoc-folder-fast (key alist localp)
+(defun mew-assoc-folder-fast (key alist sep)
   ;; Case-insensitive so you cannot use capital letters for your folders.
   ;; But this is fast.
-  (let ((skey (downcase key)) ret ret2 regex)
-    (if (not localp)
-	(rassoc skey alist)
-      ;; ("+foo/" . "foo")
-      ;; ("+foo/foo" . "foo")
-      (setq regex (concat mew-path-separator "$"))
-      (setq ret (rassoc skey alist))
-      (setq ret2 ret)
-      (while (and ret (string-match regex (car ret)))
+  ;;
+  ;; ("+foo/" . "foo")
+  ;; ("+foo/foo" . "foo")
+  (let* ((skey (downcase key))
+	 (node-regex (concat (regexp-quote sep) "$"))
+	 ret first)
+    (catch 'loop
+      (while (setq ret (rassoc skey alist))
+	(unless first
+	  (setq first ret))
+	(unless (string-match node-regex (car ret))
+	  (throw 'loop nil))
 	(setq alist (cdr (member ret alist)))
-	(setq ret (rassoc skey alist)))
-      (or ret ret2))))
+	(setq ret nil)))
+    (if mew-use-node-folder
+	(or ret first)
+      ret)))
 
 (defun mew-refile-guess-by-folder ()
   ;; Guess folders by the To:/Cc: field with folder alist.
   ;; Mainly used for mailing-list.
   (let* ((to-cc (mew-header-parse-address-list mew-refile-guess-key-list))
-	 (proto (mew-sinfo-get-proto))
-	 (case (mew-sinfo-get-case))
+	 (proto mew-inherit-refile-proto)
+	 (case mew-inherit-refile-case)
 	 (alist (mew-proto-folder-alist proto case))
-	 (localp (mew-folder-localp proto))
-	 ent ret ml-addr ml-name)
+	 sep ent ret ml-addr ml-name)
+    (cond
+     ((mew-folder-imapp proto)
+      (setq sep (mew-imap-separator case)))
+     (t
+      (setq sep mew-path-separator)))
     (while to-cc
       (setq ml-addr (mew-addrstr-parse-address (or (car to-cc) "")))
       (when ml-addr
 	(setq ml-name (mew-addrstr-extract-user ml-addr))
-	(setq ent (or (funcall mew-assoc-folder ml-addr alist localp)
-		      (funcall mew-assoc-folder ml-name alist localp)))
+	(setq ent (or (funcall mew-assoc-folder ml-addr alist sep)
+		      (funcall mew-assoc-folder ml-name alist sep)))
 	(mew-addq ret (nth 0 ent)))
       (setq to-cc (cdr to-cc)))
     (nreverse ret)))
@@ -195,7 +208,7 @@ mew-refile-guess-from-me-is-special."
 
 (defun mew-refile-guess-by-alist ()
   ;; Guess folders by user-configured alist.
-  (mew-refile-guess-by-alist1 mew-refile-guess-alist))
+  (mew-refile-guess-by-alist1 (mew-refile-guess-alist mew-inherit-refile-case)))
 
 (defun mew-refile-guess-by-alist1 (rule)
   "Return a guessed folder according to the RULE. The syntax of
@@ -208,7 +221,7 @@ RULE is as follows:
 There are two special <key>s: @samp{nil} and @samp{t}. @samp{nil} is
 used to specify <folder> to be returned when nothing is
 guessed. @samp{t} can specify <folder> to be returned in addition to
-guessed values. "
+guessed values."
   (let (key alist val f-or-r header ent ret)
     (while rule
       (setq key (car (car rule)))
@@ -248,13 +261,15 @@ guessed values. "
     (string-match regex string)
     (setq match (cdr (cdr (match-data))))
     (while match
-      (setq str (substring string (car match) (car (cdr match))))
+      (setq str nil)
+      (if (car match)
+	  (setq str (substring string (car match) (car (cdr match)))))
       (setq strs (cons str strs))
       (setq match (cdr (cdr match))))
     (setq strs (cons nil (nreverse strs))) ;; cons nil for 0th
     (while (string-match "\\\\\\([1-9]\\)" template)
       (setq repl (nth (string-to-int (match-string 1 template)) strs))
-      (setq template (replace-match repl nil nil template)))
+      (setq template (replace-match (or repl "") nil nil template)))
     template))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -265,12 +280,11 @@ guessed values. "
 (defun mew-refile-guess-by-thread ()
   ;; Guess folders by thread with alist created by
   ;; mew-refile-guess-by-thread-learn.
-  (let ((msgid (or (mew-header-get-value mew-references:)
-		   (mew-header-get-value mew-in-reply-to:))))
-    ;; search for msgid
-    (if (and msgid 
-	     (string-match "\\(<[^ \t\n]*>\\)[^>]*\0" (concat msgid "\0")))
-	(nth 1 (assoc (match-string 1 msgid) mew-refile-msgid-alist)))))
+  (let ((msgid (or (mew-idstr-get-last-id
+		    (mew-header-get-value mew-references:))
+		   (mew-idstr-get-first-id
+		    (mew-header-get-value mew-in-reply-to:)))))
+    (if msgid (nth 1 (assoc msgid mew-refile-msgid-alist)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -282,8 +296,8 @@ guessed values. "
   (mew-refile-guess-from-dispatch 'mew-refile-guess-by-from-folder-body addr))
 
 (defun mew-refile-guess-by-from-folder-body (&optional addr)
-  (let* ((proto (mew-sinfo-get-proto))
-	 (case (mew-sinfo-get-case))
+  (let* ((proto mew-inherit-refile-proto)
+	 (case mew-inherit-refile-case)
 	 (list (mew-proto-friend-folder-list proto case))
 	 (sep (if (mew-folder-imapp proto)
 		  (mew-imap-separator case)
@@ -320,14 +334,14 @@ guessed values. "
   ;; Guess folders by the Newsgroups field with folder alist.
   (let* ((newsgroups (mew-addrstr-parse-value-list2 
 		     (mew-header-get-value mew-newsgroups:)))
-	 (proto (mew-sinfo-get-proto))
-	 (case (mew-sinfo-get-case))
-	 (alist (mew-proto-folder-alist proto case))
+	 (proto mew-inherit-refile-proto)
+	 (case mew-inherit-refile-case)
+	 (alist (mew-proto-folder-alist proto case)) ;; local folders
 	 newsgroup ent ret)
     (while newsgroups
       (setq newsgroup (car newsgroups))
       (setq newsgroups (cdr newsgroups))
-      (setq ent (funcall mew-assoc-folder newsgroup alist nil))
+      (setq ent (funcall mew-assoc-folder newsgroup alist mew-path-separator))
       (mew-addq ret (nth 0 ent)))
     (nreverse ret)))
 
@@ -341,13 +355,13 @@ guessed values. "
   (mew-refile-guess-from-dispatch 'mew-refile-guess-by-default-body addr))
 
 (defun mew-refile-guess-by-default-body (&optional addr)
-  (let* ((proto (mew-sinfo-get-proto))
-	 (case (mew-sinfo-get-case))
+  (let* ((proto mew-inherit-refile-proto)
+	 (case mew-inherit-refile-case)
 	 (fld (mew-proto-friend-folder proto case))
 	 (from (downcase (or addr (mew-header-parse-address mew-from:) ""))))
     (if mew-refile-guess-strip-domainpart
 	(setq from (mew-addrstr-extract-user from)))
-    (mew-concat-folder fld from case)))
+    (mew-concat-folder2 fld from case)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -378,13 +392,12 @@ guessed values. "
 
 (defun mew-refile-guess-by-thread-learn (chosen info)
   ;; Create mew-refile-msgid-alist for mew-refile-guess-by-thread.
-  (let* ((msgid (mew-header-get-value mew-message-id:)) 
+  (let* ((msgid (mew-idstr-get-first-id
+		 (mew-header-get-value mew-message-id:)))
 	 (folder (car chosen))
 	 ;; ohter people's honest opinion and my honest opinion.
 	 (oho info)
 	 (mho (cdr (assoc 'mew-refile-guess-by-thread info))))
-    (if (and msgid (string-match "<[^ \n>]*>" msgid))
-	(setq msgid (match-string 0 msgid)))
     (when (and msgid chosen)
       ;; if my opninion was right, I learn it.
       ;; or a folder was not in other people's opinion,
@@ -466,15 +479,21 @@ Return t if exists or created. Otherwise, return nil."
      ((mew-folder-nntpp folder) nil)
      ((mew-folder-virtualp folder) nil)
      ((mew-folder-imapp folder)
-      (let* ((case (mew-sinfo-get-case))
+      (let* ((case mew-inherit-refile-case)
 	     (mailboxes (mew-imap-folder-alist case)))
-	(if (assoc folder mailboxes)
-	    t
-	  (if (and (y-or-n-p (format "%s does not exist. Create it? " folder))
-		   (mew-imap-folder-insert case folder))
-	      (progn
-		(message "%s will be created" folder)
-		t)))))
+	(cond
+	 ((or (assoc folder mailboxes)
+	      (assoc (mew-imap-file-name-as-directory folder case) mailboxes))
+	  t)
+	 ;; This is very ad-hoc but no right way...
+	 ((and (string= (mew-imap-separator case) ".")
+	       (string-match mew-path-separator folder))
+	  (message "%s should not contain \"%s\"" folder mew-path-separator)
+	  nil)
+	 ((and (y-or-n-p (format "%s does not exist. Create it? " folder))
+	       (mew-imap-folder-insert case folder))
+	  (message "%s will be created" folder)
+	  t))))
      (t ;; local
       (let ((absdir (mew-expand-folder folder))  ;; /home/Mail/foo
 	    (create-it force-to-create))
@@ -499,14 +518,11 @@ Return t if exists or created. Otherwise, return nil."
   ;;  ((folder1 folder2...)
   ;;   (func1 guess11 guess12...)  ;; for learning
   ;;   (func2 guess12 guess22...))
-  (let ((proto (mew-proto-to-refile (mew-summary-folder-name 'ext)))
-	(case (mew-sinfo-get-case))
+  (let ((proto mew-inherit-refile-proto)
+	(case mew-inherit-refile-case)
 	learn-info folders ret cands singlep lst-lst lst fld)
     (save-excursion
       (set-buffer buf)
-      ;; copy to buffer local variables
-      (mew-sinfo-set-case case)
-      (mew-sinfo-set-proto proto)
       (setq learn-info (mew-refile-guess auto)))
     (if auto
 	;; if auto is set, simply use the guess.
@@ -565,6 +581,9 @@ the current folder as a candidate in addition to guessed folders."
 ;;; Refile (aka move)
 ;;;
 
+(defsubst mew-refile-log-buffer (fld)
+  (concat mew-buffer-prefix "refile " fld))
+
 (defun mew-summary-refile (&optional report)
  "Put the refile mark(default is 'o') on this message. If already
 marked with 'o', it prints where this message will be refiled. This
@@ -577,48 +596,94 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
    (mew-summary-local-or-imap
     (if report (mew-summary-refile-report) (mew-summary-refile-body))))))
 
+(defvar mew-override-body-open "<")
+(defvar mew-override-body-close "> ")
+(defconst mew-regex-override-body
+  (concat (regexp-quote mew-override-body-open)
+	  "[^>\r\n]*"
+	  (regexp-quote mew-override-body-close)))
+
+(defun mew-summary-refile-override-body (folders-str)
+  (save-excursion
+    (when (mew-summary-goto-body 'after)
+      (when (looking-at mew-regex-override-body)
+	(mew-elet
+	 (delete-region (point) (match-end 0))))
+      (mew-elet
+       (insert-and-inherit mew-override-body-open folders-str mew-override-body-close)))))
+
+(defun mew-summary-refile-remove-body ()
+  (save-excursion
+    (when (and (mew-summary-goto-body 'after)
+	       (looking-at mew-regex-override-body))
+      (mew-elet
+       (delete-region (point) (match-end 0))))))
+
+(defun mew-summary-refile-log (fld folders-str)
+  (save-excursion
+    (beginning-of-line)
+    (let ((beg (point))
+	  (cbuf (current-buffer))
+	  (msg-id (mew-summary-my-id))
+	  end)
+      (forward-line)
+      (when (search-backward "\r" nil t)
+	(setq end (point))
+	(set-buffer (get-buffer-create (mew-refile-log-buffer fld)))
+	(goto-char (point-max))
+	(mew-insert-buffer-substring cbuf beg end)
+	(insert "== src=" fld " dst=" folders-str " id=" msg-id)
+	(insert " date=" (mew-time-ctz-to-logtime (current-time)) "\n")))))
+
+(defun mew-refile-decode-subject ()
+  (save-excursion
+    (goto-char (point-min))
+    (if (and (re-search-forward (concat "^$\\|^" mew-subj:) nil t)
+	     (not (looking-at "^$")))
+	(let ((med (point)))
+	  (forward-line)
+	  (mew-header-goto-next)
+	  (mew-header-decode-region mew-subj: med (point))))))
+
 (defun mew-summary-refile-body (&optional exp-flds auto no-msg no-mark exfld)
-  (let (msg folder folders cur-folders mark
-	    buf learn-info fld tmp delbuf invalidp)
-    (save-excursion
-      ;; save the cursor position anyway
-      (mew-summary-goto-message)
-      (setq invalidp (mew-summary-msg-invalidp))
-      ;; on the message
-      (setq fld (mew-summary-folder-name))
-      (setq msg (mew-summary-message-number)) ;; msg is never nil
-      (setq mark (mew-summary-get-mark))) ;; any mark
+  (let ((pos (point))
+	fld msg folder folders cur-folders mark
+	buf learn-info tmp delbuf invalidp folders-str
+	mew-inherit-refile-proto mew-inherit-refile-case)
+    (mew-summary-goto-message)
+    (when (mew-sumsyn-match mew-regex-sumsyn-short)
+      (setq fld (mew-sumsyn-folder-name))
+      (setq msg (mew-sumsyn-message-number)) ;; msg is never nil
+      (setq invalidp (not (mew-msg-validp msg))))
+    (setq mew-inherit-refile-case  (mew-case:folder-case fld))
+    (setq mew-inherit-refile-proto (mew-case:folder-folder fld))
+    (setq mark (mew-summary-get-mark)) ;; any mark
     (cond
      ((and mark (> (mew-markdb-level mark) (mew-markdb-level mew-mark-refile)))
       (or no-msg
 	  (message "Cannot mark here because '%s' is stronger than '%s'"
-		   (mew-markdb-name mark) (mew-markdb-name mew-mark-refile))))
+		   (mew-markdb-name mark) (mew-markdb-name mew-mark-refile)))
+      nil)
      (invalidp
-      (or no-msg (message "Cannot mark this invalid message")))
+      (or no-msg (message "Cannot refile this invalid message"))
+      nil)
      (t
       (if exp-flds
 	  (setq folders exp-flds)
-	;; show message if not displayed
-	(if (or auto (null (mew-sinfo-get-disp-msg)))
-	    (save-excursion
-	      (setq buf (generate-new-buffer mew-buffer-prefix))
-	      (setq delbuf t)
-	      (set-buffer buf)
-	      (mew-erase-buffer)
-	      (mew-insert-message fld msg mew-cs-autoconv
-				  mew-header-reasonable-size)
-	      (goto-char (point-min))
-	      (if (and (re-search-forward (concat "^$\\|^" mew-subj:) nil t)
-		       (not (looking-at "^$")))
-		  (let ((med (point)))
-		    (forward-line)
-		    (mew-header-goto-next)
-		    (mew-header-decode-region mew-subj: med (point)))))
+	(unless auto
 	  ;; need to make a cache or a message buffer.
-	  (mew-summary-display nil)
-	  ;; mew-cache-hit should be first since we want to get 
-	  ;; information form the top level header.
-	  (setq buf (or (mew-cache-hit fld msg) (mew-buffer-message))))
+	  (mew-summary-toggle-disp-msg 'on)
+	  (mew-summary-display nil))
+	(setq buf (mew-cache-hit fld msg))
+	(unless buf
+	  (save-excursion
+	    (setq buf (generate-new-buffer mew-buffer-prefix))
+	    (setq delbuf t)
+	    (set-buffer buf)
+	    (mew-erase-buffer)
+	    (mew-insert-message
+	     fld msg mew-cs-text-for-read mew-header-reasonable-size)
+	    (mew-refile-decode-subject)))
 	(when (and (eq mew-mark-refile mark) (get-buffer fld))
 	  (save-excursion
 	    (set-buffer fld)
@@ -637,40 +702,54 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
       ;; mark refile
       (unless no-mark
 	(when folders
-	  (save-excursion
-	    (mew-summary-goto-message)
-	    (or exp-flds auto (mew-refile-guess-learn buf learn-info))
-	    (when (get-buffer fld)
+	  (or exp-flds auto (mew-refile-guess-learn buf learn-info))
+	  (setq folders-str (mew-join "," folders))
+	  (cond
+	   ((mew-virtual-p)
+	    (save-excursion
+	      (set-buffer fld)
 	      (save-excursion
-		(set-buffer fld)
-		(mew-refile-reset msg)
-		(mew-refile-set msg folders)))
-	    (mew-mark-unmark))
-	  (mew-mark-put-mark mew-mark-refile no-msg) ;; cursor may move
-	  (if (mew-virtual-p)
-	      (mew-summary-mark-in-physical fld msg mew-mark-refile))))
+		(when (or (re-search-forward  (mew-regex-sumsyn-msg msg) nil t)
+			  (re-search-backward (mew-regex-sumsyn-msg msg) nil t))
+		  (mew-refile-reset msg)
+		  (mew-refile-set msg folders)
+		  (mew-summary-refile-log fld folders-str)
+		  (mew-summary-refile-override-body folders-str)
+		  ;; marked in mew-mark-put-mark
+		  (set-buffer-modified-p nil)))))
+	   (t
+	    (mew-refile-reset msg)
+	    (mew-refile-set msg folders)
+	    (mew-summary-refile-log fld folders-str)))
+	  (mew-summary-refile-override-body folders-str)
+	  (mew-mark-put-mark mew-mark-refile no-msg)
+	  (setq pos (point))))
       (if delbuf (mew-remove-buffer buf))
       ;; memorize last-folder
       (setq mew-refile-last-folder folders)
-      (set-buffer-modified-p nil)
-      folders)))) ;; return value
+      (set-buffer-modified-p nil)))
+    (goto-char pos)
+    folders)) ;; return value
 
 (defun mew-summary-refile-report ()
-  (let ((proto (mew-proto-to-refile (mew-summary-folder-name 'ext)))
-	(case (mew-sinfo-get-case))
-	(fld (mew-summary-folder-name))
-	(msg (mew-summary-message-number2))
-	(win (selected-window))
+  (let ((win (selected-window))
 	(customize-var '(mew-refile-ctrl-multi
 			 mew-refile-guess-key-list
 			 mew-refile-guess-strip-domainpart
 			 mew-refile-guess-from-me-is-special))
-	guess)
+	mew-inherit-refile-case
+	mew-inherit-refile-proto
+	fld msg guess)
+    (mew-summary-goto-message)
+    (when (mew-sumsyn-match mew-regex-sumsyn-short)
+      (setq fld (mew-sumsyn-folder-name))
+      (setq msg (mew-sumsyn-message-number)))
+    (setq mew-inherit-refile-case  (mew-case:folder-case fld))
+    (setq mew-inherit-refile-proto (mew-case:folder-folder fld))
     (with-temp-buffer
-      (mew-insert-message fld msg mew-cs-autoconv
-			  mew-header-reasonable-size)
-      (mew-sinfo-set-case case)
-      (mew-sinfo-set-proto proto)
+      (mew-insert-message
+       fld msg mew-cs-text-for-read mew-header-reasonable-size)
+      (mew-refile-decode-subject)
       (setq guess (mew-refile-guess nil t)))
     (mew-window-configure 'message)
     ;; message buffer
@@ -708,15 +787,12 @@ refile folder."
   (interactive)
   (mew-summary-not-in-draft
    (mew-summary-local-or-imap
-    (let (validp)
-      (save-excursion
-	(mew-summary-goto-message)
-	(setq validp (not (mew-summary-msg-invalidp))))
-      (when validp
-	(message "%s to %s"
-		 (mew-summary-message-number)
-		 (mew-join ", " mew-refile-last-folder))
-	(mew-summary-refile-body mew-refile-last-folder))))))
+    (save-excursion
+      (mew-summary-goto-message)
+      (let ((msg (mew-summary-message-number)))
+	(when (mew-msg-validp msg)
+	  (message "%s to %s" msg (mew-join ", " mew-refile-last-folder)))))
+    (mew-summary-refile-body mew-refile-last-folder))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -730,12 +806,12 @@ any previousely marked message will be skipped.
 If '\\[universal-argument]' is specified, only messages marked with
 'mew-mark-review' will be conserned."
   (interactive "P")
-  (mew-summary-or-thread
+  (mew-pickable
    (mew-summary-not-in-draft
     (mew-summary-local-or-imap
      (mew-decode-syntax-delete)
      (let ((mew-use-highlight-x-face nil)
-	   (lines (mew-count-lines (point-min) (point-max)))
+	   (lines (mew-sinfo-get-ttl-line))
 	   (line 1) (mark nil) msg)
        (cond
 	(mew-mark-review-only
@@ -750,23 +826,22 @@ If '\\[universal-argument]' is specified, only messages marked with
 	 (message "Auto refiling...")
 	 (save-window-excursion
 	   (goto-char (point-min))
-	   (while (not (eobp))
-	     (when (looking-at mew-regex-msg-valid)
-	       (setq mark (mew-summary-get-mark))
-	       (if mew-mark-review-only
-		   (and mark
-			(char-equal mark mew-mark-review)
-			(mew-summary-refile-body nil t 'no-msg))
-		 (or (and mark
-			  (or mew-refile-auto-refile-skip-any-mark
-			      (>= (mew-markdb-level mark)
-				  (mew-markdb-level mew-mark-refile))))
-		     (mew-summary-refile-body nil t 'no-msg)))
-	       (if (= (% (/ (* 100 line) lines) 10) 0)
-		   (message "Auto refiling...%s%%" (/ (* 100 line) lines)))
-	       (setq line (1+ line)))
-	     (forward-line))
-	   (message "Auto refiling...done"))))))))
+	   (while (re-search-forward mew-regex-sumsyn-valid nil t)
+	     (setq mark (mew-summary-get-mark))
+	     (if mew-mark-review-only
+		 (and mark
+		      (char-equal mark mew-mark-review)
+		      (mew-summary-refile-body nil t 'no-msg))
+	       (or (and mark
+			(or mew-refile-auto-refile-skip-any-mark
+			    (>= (mew-markdb-level mark)
+				(mew-markdb-level mew-mark-refile))))
+		   (mew-summary-refile-body nil t 'no-msg)))
+	     (if (= (% (/ (* 100 line) lines) 10) 0)
+		 (message "Auto refiling...%s%%" (/ (* 100 line) lines)))
+	     (setq line (1+ line))
+	     (forward-line)))
+	 (message "Auto refiling...done")))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -777,7 +852,7 @@ If '\\[universal-argument]' is specified, only messages marked with
   "\\<mew-summary-mode-map>
 Put the refile mark onto all messages marked with '*', 
 with the current folder as a candidate in addition to guessed folders.
-This is very convenient to refile all messages picked by '\\[mew-summary-search-mark]'."
+This is very convenient to refile all messages  picked by '\\[mew-summary-pick-mark]' and \"\\[mew-summary-grep-mark]\"."
   (interactive)
   (mew-summary-not-in-draft
    (mew-summary-local-or-imap
@@ -788,9 +863,9 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
 	(save-window-excursion
 	  (goto-char (point-min))
 	  (catch 'loop
-	    (while (re-search-forward mew-regex-msg-valid-review nil t)
+	    (while (re-search-forward mew-regex-msg-review nil t)
 	      (setq last (mew-summary-refile-body
-			  last nil 'no-msg nil (mew-summary-folder-name)))
+			  last nil 'no-msg nil (mew-summary-folder-name 'ext)))
 	      (unless last (throw 'loop t))
 	      (forward-line)))
 	  (message "Mark copying...done")))))))
@@ -798,7 +873,7 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
 (defun mew-summary-mark-refile ()
   "\\<mew-summary-mode-map>
 Put the refile mark onto all messages marked with '*'.
-This is very convenient to refile all messages picked by '\\[mew-summary-search-mark]'."
+This is very convenient to refile all messages picked by '\\[mew-summary-pick-mark]' and \"\\[mew-summary-grep-mark]\"."
   (interactive)
   (mew-summary-not-in-draft
    (mew-summary-local-or-imap
@@ -807,9 +882,9 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
       (message "Mark refiling...")
       (save-excursion
 	(save-window-excursion
-	  (goto-char (point-min))
 	  (catch 'loop
-	    (while (re-search-forward mew-regex-msg-valid-review nil t)
+	    (while (or (re-search-forward  mew-regex-msg-review nil t)
+                       (re-search-backward mew-regex-msg-review nil t))
 	      (setq last (mew-summary-refile-body last nil 'no-msg))
 	      (unless last (throw 'loop t))
 	      (forward-line)))
