@@ -22,7 +22,7 @@
 ;;; Variables
 ;;;
 
-(defvar mew-scan-form-size-unit '("" "k" "M" "G" "T"))
+(defvar mew-summary-form-size-unit '("" "k" "M" "G" "T"))
 (defvar mew-vec [0 1 2 3 4 5 6 8 9 10 11 12 13 14 15 16 17 18 19 20]
   "Just for test of (MEW-FOO).")
 
@@ -42,46 +42,49 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Pre-defined functions for mew-scan-form
+;;; Pre-defined functions for mew-summary-form
 ;;;
 
-(defun mew-scan-form-num ()
-  "A function to return a message number."
-  (if VIRTUALP
-      (prog1
-	(int-to-string (mew-vinfo-get-count))
-	(mew-vinfo-set-count (1+ (mew-vinfo-get-count))))
-    (MEW-NUM)))
-
-(defun mew-scan-form-mark ()
+(defun mew-summary-form-mark ()
   "A function to return a mark.
-'mew-scan-form-mark-delete' and 'mew-scan-form-mark-review'
+'mew-summary-form-mark-delete' and 'mew-summary-form-mark-review'
 effect to this function."
-  (let ((id (MEW-ID)) duplicated review)
-    (when mew-scan-form-mark-delete
-      (when (string-match mew-regex-id id)
-	(setq id (match-string 1 id))
-	(if (member id (mew-sinfo-get-scan-id)) ;; in Summary mode
-	    (setq duplicated t)
+  (let ((mark-delete mew-mark-delete)
+	(mark-review mew-mark-review)
+	(unread (and mew-use-unread-mark (mew-sinfo-get-unread-mark)))
+	duplicated review id)
+    (when mew-summary-form-mark-delete
+      (if (mew-characterp mew-summary-form-mark-delete)
+	  (setq mark-delete mew-summary-form-mark-delete))
+      (when (setq id (mew-idstr-get-first-id (MEW-ID)))
+	(if (member id (mew-sinfo-get-scan-id))
+	    (unless (mew-scan-message-invalidp)
+	      (setq duplicated t))
 	  (mew-sinfo-set-scan-id (cons id (mew-sinfo-get-scan-id))))))
-    (when mew-scan-form-mark-review
+    (when mew-summary-form-mark-review
+      (if (mew-characterp mew-summary-form-mark-review)
+	  (setq mark-review mew-summary-form-mark-review))
       (let* ((mew-header-max-depth nil)
-	     (to (mew-addrstr-parse-address-list (MEW-TO))))
+	     (to (mew-addrstr-parse-address-list (MEW-TO)))
+	     (cc (mew-addrstr-parse-address-list (MEW-CC))))
+	(setq to (nconc to cc))
 	(catch 'loop
 	  (while to
 	    (if (mew-is-my-address mew-regex-my-address-list (car to))
 		(throw 'loop (setq review t)))
 	    (setq to (cdr to))))))
     (cond
-     (duplicated (char-to-string mew-mark-delete))
-     (review     (char-to-string mew-mark-review))
-     (t " "))))
+     (duplicated (char-to-string mark-delete))
+     (review     (char-to-string mark-review))
+     (unread     (char-to-string mew-mark-unread))
+     (t          (char-to-string mew-mark-read)))))
 
-(defun mew-scan-form-type ()
+(defun mew-summary-form-type ()
   "A function to return a mark of content type."
   (let ((ct (MEW-CT))
 	(case-fold-search t))
     (cond
+     ((mew-scan-message-invalidp)                  "#")
      ((mew-scan-message-truncatedp)                "T")
      ((string-match "Multipart/Signed"    ct)      "S")
      ((string-match "Multipart/Encrypted" ct)      "E")
@@ -90,7 +93,7 @@ effect to this function."
      ((string-match "Message/Partial"     ct)      "P")
      (t " "))))
 
-(defun mew-scan-form-time ()
+(defun mew-summary-form-time ()
   "A function to return a message time, HH:MM"
   (let ((s (MEW-DATE)))
     (if (or (string= s "")
@@ -103,7 +106,7 @@ effect to this function."
 		(or (mew-time-rfc-min)  0))
       "00:00")))
 
-(defun mew-scan-form-date ()
+(defun mew-summary-form-date ()
   "A function to return a date, MM/DD."
   (let ((s (MEW-DATE)))
     (when (or (string= s "")
@@ -116,11 +119,14 @@ effect to this function."
 		(mew-time-rfc-day))
       "")))
 
-(defun mew-scan-form-year ()
+(defun mew-summary-form-year ()
   "A function to return a message year, YYYY"
   (let ((s (MEW-DATE)) year)
-    (if (or (string= s "")
-	    (not (string-match mew-time-rfc-regex s)))
+    (when (or (string= s "")
+	      (not (string-match mew-time-rfc-regex s)))
+      (setq s (mew-time-ctz-to-rfc
+	       (mew-file-get-time (mew-expand-folder (MEW-FLD) (MEW-NUM))))))
+    (if (not (string-match mew-time-rfc-regex s))
 	"0000"
       (setq year (mew-time-rfc-year))
       (cond
@@ -130,10 +136,10 @@ effect to this function."
 	(setq year (+ year 1900))))
       (int-to-string year))))
 
-(defun mew-scan-form-size ()
+(defun mew-summary-form-size ()
   "A function to return the size of the message. Should be used
-with -4. See also 'mew-scan-form-size-0k' and 'mew-scan-form-size-huge'."
-  (let ((len-1 (1- (length mew-scan-form-size-unit)))
+with -4. See also 'mew-summary-form-size-0k' and 'mew-summary-form-size-huge'."
+  (let ((len-1 (1- (length mew-summary-form-size-unit)))
 	(SIZE (mew-scan-uid-size (MEW-UID)))
 	(i 0) size unit)
     (if (and SIZE (string-match "^[0-9]+$" SIZE))
@@ -142,10 +148,10 @@ with -4. See also 'mew-scan-form-size-0k' and 'mew-scan-form-size-huge'."
     (while (and (< i len-1) (>= size 1000))
       (setq size (/ size 1000))
       (setq i (1+ i)))
-    (if (and mew-scan-form-size-huge (>= size 1000))
+    (if (and mew-summary-form-size-huge (>= size 1000))
 	"HUGE"
-      (setq unit (nth i mew-scan-form-size-unit))
-      (if (and mew-scan-form-size-0k (string= unit ""))
+      (setq unit (nth i mew-summary-form-size-unit))
+      (if (and mew-summary-form-size-0k (string= unit ""))
 	  "0k"
 	(concat
 	 (if (integerp size)
@@ -153,14 +159,13 @@ with -4. See also 'mew-scan-form-size-0k' and 'mew-scan-form-size-huge'."
 	   (format "%.0f" size))
 	 unit)))))
 
-(defun mew-scan-form-extract-addr (addr)
-  "Extract addr according to 'mew-scan-form-extract-rule'."
+(defun mew-summary-form-extract-addr (addr)
+  "Extract addr according to 'mew-summary-form-extract-rule'."
   (let* ((func (if mew-addrbook-for-summary
 		   (mew-addrbook-func mew-addrbook-for-summary)))
 	 (raw (or (mew-addrstr-parse-address addr) ""))
-	 (nickname (if func (funcall func raw)))
-	 (rules mew-scan-form-extract-rule)
-	 rule ret)
+	 (rules mew-summary-form-extract-rule)
+	 rule ret nickname)
     (catch 'matched
       (while rules
 	(setq rule (car rules))
@@ -175,37 +180,40 @@ with -4. See also 'mew-scan-form-size-0k' and 'mew-scan-form-size-huge'."
 	  (throw 'matched (setq ret (mew-chop (match-string 1 addr)))))
 	 ((eq rule 'address)
 	  (throw 'matched (setq ret raw)))
-	 ((and (eq rule 'nickname) nickname)
+	 ((and (eq rule 'nickname)
+	       ;; set nickname here for efficiency
+	       (or nickname
+		   (setq nickname (if func (funcall func raw)))))
 	  (throw 'matched (setq ret nickname)))
 	 ((and (stringp rule)
 	       (string-match rule addr))
 	  (throw 'matched (setq ret (mew-chop (match-string 1 addr))))))))
     (or ret addr)))
 
-(defun mew-scan-form-from ()
+(defun mew-summary-form-from ()
   "A function to return an address.
-If the message is destined to me AND 'mew-scan-form-from-me-prefix'
+If the message is destined to me AND 'mew-summary-form-from-me-prefix'
 is a string, an address on To:, is returned. In this
-case, 'mew-scan-form-from-me-prefix' is prepended to the address. 
+case, 'mew-summary-form-from-me-prefix' is prepended to the address. 
 
 Otherwise, an address on From: is returned.
 
-Address is converted by 'mew-scan-form-extract-addr'. See also
-'mew-scan-form-extract-rule'."
+Address is converted by 'mew-summary-form-extract-addr'. See also
+'mew-summary-form-extract-rule'."
   (let* ((FROM (MEW-FROM)) (TO (MEW-TO))
 	 (from (or (mew-addrstr-parse-address FROM) "")))
     (cond
      ((string= FROM "")
       "")
-     ((and (stringp mew-scan-form-from-me-prefix)
+     ((and (stringp mew-summary-form-from-me-prefix)
 	   (not (string= TO ""))
 	   (mew-is-my-address mew-regex-my-address-list from))
       (mew-replace-white-space
-       (concat mew-scan-form-from-me-prefix (mew-scan-form-extract-addr TO))))
+       (concat mew-summary-form-from-me-prefix (mew-summary-form-extract-addr TO))))
      (t
-      (mew-replace-white-space (mew-scan-form-extract-addr FROM))))))
+      (mew-replace-white-space (mew-summary-form-extract-addr FROM))))))
 
-(defun mew-scan-form-subj ()
+(defun mew-summary-form-subj ()
   "A function to return Subject:. Unnecessary white spaces are removed."
   ;; The beginning white spaces have been removed in mew-scan-header
   ;; (mew-keyval).
@@ -216,35 +224,32 @@ Address is converted by 'mew-scan-form-extract-addr'. See also
       ;; already well-formatted
       (mew-replace-white-space subj))))
 
+(defun mew-summary-form-body ()
+  (mew-header-sanity-check-string (MEW-BODY)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Scan from
 ;;;
 
-(defun mew-summary-scan-form (folder &optional column)
-  "Get scan-form from 'mew-summary-refine-scan-form'.
-See also 'mew-scan-form-list-list'."
+(defun mew-get-summary-form (folder &optional column)
+  "Get summary-form from 'mew-summary-form-list', 
+'mew-summary-form-list-string-type, and 'mew-summary-form-list-list-type'.
+'mew-summary-form-header' is prepended. If COLUMN is non-nil, the number
+of thread indent position is returned."
   (let (ret form col)
-    (setq ret (mew-folder-spec folder mew-scan-form-list
-			       mew-scan-form-list-string-type
-			       mew-scan-form-list-list-type))
+    (setq ret (mew-folder-spec folder
+			       mew-summary-form-list
+			       mew-summary-form-list-string-type
+			       mew-summary-form-list-list-type))
     (if ret
-	(setq form (nth 0 ret) col (nth 1 ret))
-      (setq form mew-scan-form))
-    (setq form (mew-summary-final-scan-form form))
+	(mew-set '(form col) ret)
+      (setq form mew-summary-form))
+    (setq form (append mew-summary-form-header form))
     (if column
 	(if (numberp col) col (mew-thread-column form))
       form)))
 
-(defun mew-summary-final-scan-form (form)
-  "Canonicalize the scan-form specified by FORM.
-If the first element of FORM is an integer N,
-append '((N num) mark) to the FORM.
-Otherwise append 'mew-scan-form-header' to FORM."
-  (if (integerp (car form))
-      (cons (list (car form) 'num) (cons 'mark (cdr form)))
-    (append mew-scan-form-header form)))
-
 (defun mew-thread-column (form)
   (let ((col 0) ret ent)
     (catch 'loop
@@ -262,23 +267,26 @@ Otherwise append 'mew-scan-form-header' to FORM."
 	  (setq col (1+ col))))))
     (or ret mew-thread-column)))
 
-(defun mew-summary-scan-form-num (folder)
-  (abs (car (rassoc '(num) (mew-summary-scan-form folder)))))
+(defun mew-get-unread-mark (folder)
+  (car (mew-folder-spec folder
+			mew-unread-mark-list
+			mew-unread-mark-list-string-type
+			mew-unread-mark-list-list-type)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; The engine function to call mew-scan-form-*
+;;; The engine function to call mew-summary-form-*
 ;;;
 
 (defun mew-scan-get-piece (spec)
   (let (func width str rightp nopad)
     (if (stringp spec)
 	(progn
-	  (setq TOTAL (+ TOTAL (length spec))) ;; xxx length
+	  (setq TOTAL (+ TOTAL (string-width spec)))
 	  spec)
       (if (symbolp spec)
 	  (setq width 1 func spec)
-	(setq width (nth 0 spec) func (nth 1 spec)))
+	(mew-set '(width func) spec))
       (when (and (integerp width) (symbolp func))
 	(when (= width 0)
 	  (setq width (- WIDTH TOTAL 1))
@@ -287,23 +295,21 @@ Otherwise append 'mew-scan-form-header' to FORM."
 	(if (< width 0) (setq width (abs width) rightp t))
 	(setq TOTAL (+ TOTAL width))
 	(setq func (intern-soft
-		    (concat mew-scan-form-func-prefix (symbol-name func))))
+		    (concat mew-summary-form-func-prefix (symbol-name func))))
 	(when (fboundp func)
 	  (setq str (funcall func))
 	  (if rightp
-	      (if (<= (length str) width)
+	      (if (<= (string-width str) width)
 		  (format (format "%%%ds" width) str)
-		(setq TOTAL (+ (- TOTAL width) (length str)))
+		(setq TOTAL (+ (- TOTAL width) (string-width str)))
 		str) ;; width may exceed.
 	    (mew-substring str width nil nopad)))))))
 
-(defun mew-scan-get-line (mew-vec WIDTH &optional VIRTUALP)
-  (let* ((TOTAL 0)
-	 (line (mapconcat 'mew-scan-get-piece (mew-sinfo-get-scan-form) ""))
-	 (my-id "") (par-id "") (first-irt "")
-	 fld msg ld onlyone start uid siz)
-    (if (string-match mew-regex-id (MEW-ID))
-	(setq my-id (match-string 1 (MEW-ID))))
+(defun mew-scan-get-line (mew-vec WIDTH)
+  (let* ((TOTAL 0) (fld "")
+	 (line (mapconcat 'mew-scan-get-piece (mew-sinfo-get-summary-form) ""))
+	 par-id my-id msg ld uid siz irt-list)
+    (setq my-id (or (mew-idstr-get-first-id (MEW-ID)) ""))
     ;; RFC 2822 says: the "In-Reply-To:" field may be used to identify
     ;; the message (or messages) to which the new message is a reply,
     ;; while the "References:" field may be used to identify a
@@ -316,76 +322,75 @@ Otherwise append 'mew-scan-form-header' to FORM."
     ;; (1) The In-Reply-To contains one ID, use it.
     ;; (2) The References contains one or more IDs, use the last one.
     ;; (3) The In-Reply-To contains two or more IDs, use the first one.
-    (when (string-match mew-regex-id (MEW-IRT))
-      (setq start (match-end 0))
-      (setq first-irt (match-string 1 (MEW-IRT)))
-      (unless (string-match mew-regex-id (MEW-IRT) start)
-	(setq onlyone t)))
-    (cond
-     (onlyone
-      (setq par-id first-irt))
-     ((string-match mew-regex-id (MEW-REF))
-      (setq start (match-end 0))
-      (setq par-id (match-string 1 (MEW-REF)))
-      (while (string-match mew-regex-id (MEW-REF) start)
-	(setq start (match-end 0))
-	(setq par-id (match-string 1 (MEW-REF)))))
-     (t
-      (setq par-id first-irt)))
-    (setq uid (mew-scan-uid-uid (MEW-UID)))
-    (cond
-     (VIRTUALP
-      (setq fld (mew-scan-get-folder mew-vec))
-      (setq fld (cdr (assoc fld (mew-vinfo-get-lra))))
-      (setq msg (mew-scan-get-message mew-vec))
-      (setq ld (format "\r <%s> <%s> %s %s \006 %s %s\n"
-		       my-id par-id "" "" fld msg)))
-     (uid
-      (setq siz (mew-scan-uid-size (MEW-UID)))
-      (setq ld (format "\r <%s> <%s> %s %s\n" my-id par-id uid siz)))
-     (t
-      (setq ld (format "\r <%s> <%s>\n" my-id par-id))))
+    (setq par-id (or (mew-idstr-get-first-id (MEW-XREF)) ""))
+    (when (string= par-id "")
+      (setq irt-list (mew-idstr-to-id-list (MEW-IRT)))
+      (if (= (length irt-list) 1)
+	  (setq par-id (car irt-list))
+	(setq par-id (or (mew-idstr-get-last-id (MEW-REF))
+			 (car irt-list)
+			 ""))))
+    (when (mew-virtual-p)
+      (setq fld (cdr (assoc (mew-scan-get-folder mew-vec)
+			    (mew-vinfo-get-lra)))))
+    (setq msg (mew-scan-get-message mew-vec))
+    (setq uid (or (mew-scan-uid-uid (MEW-UID)) ""))
+    (setq siz (or (mew-scan-uid-size (MEW-UID)) ""))
+    (setq ld (format "\r %s %s %s %s %s %s\n" fld msg my-id par-id uid siz))
     (cons line ld)))
 
 ;; See also mew-summary-cook-region
-(defun mew-scan-insert-line (folder vec width lmsg virtualp)
+(defun mew-scan-insert-line (folder vec width lmsg &optional mark-or-dst)
   (when (get-buffer folder)
     (save-excursion
       (set-buffer folder)
-      (let* ((line (mew-scan-get-line vec width virtualp))
+      (let* ((line (mew-scan-get-line vec width))
 	     (opos (point))
 	     (omax (point-max))
-	     beg med face mark olen nlen)
+	     beg med face olen nlen mark msg)
 	(mew-elet
 	 (if (null lmsg)
 	     (goto-char (point-max))
 	   ;; a message marked with 'T'.
 	   (goto-char (point-min))
-	   (if (not (re-search-forward (mew-regex-jmp-msg lmsg) nil t))
+	   (if (not (re-search-forward (mew-regex-sumsyn-msg lmsg) nil t))
 	       (goto-char (point-max)) ;; xxx
+	     (setq mark (mew-summary-get-mark))
 	     (beginning-of-line)
 	     (setq beg (point))
 	     (forward-line)
 	     ;; To avoid inserting a line AFTER the cursor underline,
 	     ;; keep this line and make it invisible.
 	     (put-text-property beg (point) 'invisible t)
+	     (mew-front-nonsticky beg (1+ beg)) ;; for XEmacs
 	     (forward-line -1)))
 	 (setq beg (point))
 	 ;; To "insert" just after mew-marker-decode-syntax-end.
 	 (insert (car line))
 	 (setq med (point))
+	 (insert (cdr line))
 	 (goto-char beg)
 	 (mew-front-nonsticky beg med) ;; for XEmacs
-	 (if (and mew-use-highlight-mark
-		  (looking-at mew-regex-msg-mark)
-		  (setq mark (string-to-char (mew-match-string 2)))
-		  (setq face (mew-highlight-mark-get-face mark)))
-	     (put-text-property beg med 'face face))
+	 (cond
+	  ((stringp mark-or-dst) ;; xxx
+	   (setq msg (mew-scan-get-message vec))
+	   (mew-refile-reset msg)
+	   (mew-refile-set msg (mew-split mark-or-dst ?,))
+	   (mew-summary-refile-log folder mark-or-dst)
+	   (mew-summary-refile-override-body mark-or-dst)
+	   (mew-mark-put mew-mark-refile))
+	  ((mew-characterp mark-or-dst) ;; mew-inbox-action-alist
+	   (mew-mark-put mark-or-dst))
+	  (mark
+	   (mew-summary-mark-as mark))
+	  ((and mew-use-highlight-mark ;; mew-summary-form-mark
+		(setq mark (mew-summary-get-mark)) ;; duplicated, etc
+		(setq face (mew-highlight-mark-get-face mark)))
+	   (put-text-property beg med 'face face)))
 	 (if mew-use-highlight-mouse-line
 	     (put-text-property
 	      beg med 'mouse-face mew-highlight-mouse-line-face))
-	 (goto-char med)
-	 (insert (cdr line))
+	 (forward-line)
 	 (put-text-property med (1- (point)) 'invisible t)
 	 ;; Removing the invisible line.
 	 (when lmsg
@@ -409,8 +414,12 @@ Otherwise append 'mew-scan-form-header' to FORM."
 ;;;
 
 (defun mew-scan-header (&optional draftp)
-  (let (vec key med str n)
-    (setq vec (make-vector (length mew-scan-fields) ""))
+  (let ((vec (make-vector (length mew-scan-fields) ""))
+	(lim (1- mew-scan-max-field-length))
+	i key med str n)
+    (goto-char (point-min))
+    (unless (re-search-forward mew-eoh nil t)
+      (goto-char (point-max)))
     (save-restriction
       (narrow-to-region (point-min) (point))
       (goto-char (point-min))
@@ -419,25 +428,155 @@ Otherwise append 'mew-scan-form-header' to FORM."
 	    (forward-line)
 	  (setq key (mew-capitalize (mew-match-string 1)))
 	  (setq med (match-end 0))
+	  ;; Three lines should be enough for Summary mode.
 	  (forward-line)
-	  (mew-header-goto-next)
-	  (when (setq n (mew-member-case-equal key mew-scan-fields))
-	    ;; xxx this may be able to tune up.
-	    (mew-header-decode-region key med (point) draftp)
+          (setq i 0)
+	  (while (and (< i lim) (looking-at mew-lwsp))
+	    (forward-line)
+	    (setq i (1+ i)))
+	  (when (and (setq n (mew-member-case-equal key mew-scan-fields))
+		     (string= (aref vec n) "")) ;; avoid multiple times
+	    (when (member key mew-scan-decode-fields)
+	      (mew-header-decode-region key med (point) draftp))
 	    ;; We need to keep composite properties of charset.
 	    ;; This must be "buffer-substring".
 	    (setq str (buffer-substring med (1- (point))))
-	    (aset vec n str))))
-      (goto-char (point-max)))
+	    (aset vec n str))
+	  (mew-header-goto-next))))
     vec))
 
+(defun mew-scan-field-to-func (key)
+  (let ((n (mew-member-case-equal key mew-scan-fields))
+	(len (length mew-scan-fields-alias))
+	ali)
+    (if (and n (< n len)) (setq ali (nth n mew-scan-fields-alias)))
+    (if (stringp ali) (symbol-function (intern-soft (concat "MEW-" ali))))))
+
+(defun mew-scan-inbox-action (mew-vec)
+  (let ((alist mew-inbox-action-alist)
+	ent key act val val-func ret mark-or-dst regex-list)
+    (catch 'loop
+      (while alist
+	(setq ent (car alist))
+	(setq alist (cdr alist))
+	(setq key (car ent))
+	(setq val-func (mew-scan-field-to-func key))
+	(if (and val-func (functionp val-func))
+	    (setq val (funcall val-func))
+	  (setq val nil))
+	(when val
+	  (setq ent (cdr ent))
+	  (while ent
+	    (setq act (car ent))
+	    (setq ent (cdr ent))
+	    (if (symbolp act)
+		(when (fboundp act)
+		  (setq ret (funcall act val))
+		  (if ret (throw 'loop nil)))
+	      (when (listp act)
+		(setq mark-or-dst (car act))
+		(setq regex-list (cdr act))
+		(while regex-list
+		  (if (string-match (car regex-list) val)
+		      (throw 'loop (setq ret mark-or-dst)))
+		  (setq regex-list (cdr regex-list)))))))))
+    ret))
+
+(defvar mew-regex-ignore-scan-body-list
+  '("^[ \t]*$"
+    "^[ \t]*[-a-zA-Z0-9]+: "
+    "^[ \t]*[>:|#;/_}]"
+    "^[ \t]*\\w+\\(['._-]+\\w+\\)*>"
+    "^[ \t]*[[</(.]+ *snip"
+    "^   "
+    "^--"
+    "^- --"
+    "^=2D"
+    "^.*\\(:\\|;\\|/\\)[ \t]*$"
+    "^.*\\(wrote\\|writes?\\|said\\|says?\\)[^.!\n]?[ \t]*$"
+    "^[ \t]*\\(On\\|At\\) .*[^.! \t\n][ \t]*$"
+    "^[ \t]*In \\(message\\|article\\|mail\\|news\\|<\\|\"\\|\\[\\|(\\)"))
+
+(defun mew-scan-body (mew-vec &optional draftp)
+  (forward-line)
+  (let* ((i 0) (I mew-scan-max-body-length)
+	 (j 0) (J 3)
+	 (ctr (MEW-CT))
+	 (cte (MEW-CTE))
+	 (body "")
+	 (case-fold-search t)
+	 textp charset cs beg skip boundary found regex)
+    (catch 'break
+      (cond
+       (draftp
+	(setq textp t)
+	(setq cs mew-cs-m17n))
+       ((string= ctr "")
+	(if (mew-case-equal cte mew-b64) (throw 'break nil))
+	(setq textp t)
+	(setq cs mew-cs-autoconv))
+       (t
+	;; The following code is generic but too slow.
+	;; (setq ctl (mew-param-decode ctr))
+	;; (setq ct (mew-syntax-get-value ctl 'cap))
+	;; So, this hard coding is used.
+	(when (and (string-match "^Multipart/" ctr)
+		   (string-match "boundary=\"?\\([^\"\n\t;]+\\)\"?" ctr))
+	  (setq boundary (mew-match-string 1 ctr))
+	  (setq boundary (concat "^--" (regexp-quote boundary)))
+	  (catch 'loop
+	    (while (< i I) 
+	      (if (looking-at boundary) (throw 'loop (setq found t)))
+	      (forward-line)
+	      (setq i (1+ i))))
+	  (if (not found)
+	      (throw 'break nil)
+	    (forward-line)
+	    (save-restriction
+	      (narrow-to-region (point) (point-max))
+	      (setq ctr (mew-header-get-value mew-ct:))
+	      (setq cte (mew-addrstr-parse-value
+			 (mew-header-get-value mew-cte:)))
+	      (mew-header-goto-end)) ;; should be in the narrowed region
+	    (unless ctr ;; not ""
+	      (setq textp t)
+	      (setq cs mew-cs-autoconv)
+	      (throw 'break nil))))
+	(if (and cte (mew-case-equal cte mew-b64)) (throw 'break nil))
+	(when (string-match "^Text/Plain" ctr)
+	  (when (string-match "charset=\"?\\([^\"\n\t;]+\\)\"?" ctr)
+	    (setq charset (mew-match-string 1 ctr))) 
+	  ;; xxx quoted-printable. not enough DB in mew-mule3.el.
+	  (setq textp t)
+	  (setq cs (mew-charset-to-cs charset))
+	  (if (null cs) (setq cs mew-cs-autoconv)))))) ;; end of 'break
+    (when (and textp (mew-coding-system-p cs))
+      (setq i 0)
+      (while (and (not (eobp)) (< i I) (< j J))
+	(setq regex mew-regex-ignore-scan-body-list)
+	(setq skip nil)
+	(catch 'matched
+	  (while regex
+	    (if (looking-at (car regex))
+		(throw 'matched (setq skip t)))
+	    (setq regex (cdr regex))))
+	(if skip
+	    (forward-line)
+	  (when (looking-at "^[ \t]+")
+	    (goto-char (match-end 0)))
+	  (setq beg (point))
+	  (forward-line)
+	  (setq body (concat body (mew-buffer-substring beg (1- (point))) " "))
+	  (setq j (1+ j)))
+	(setq i (1+ i)))
+      (setq body (mew-cs-decode-string (mew-replace-white-space body) cs))
+      (aset mew-vec (1- (length mew-vec)) body))))
+
 (defun mew-scan-width ()
   (if (and (integerp mew-summary-scan-width)
 	   (> mew-summary-scan-width 40)) ;; xxx
       mew-summary-scan-width
-    (if (< (window-width) 80)
-	80
-      (window-width))))
+    (max mew-window-magic (window-width))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -445,10 +584,10 @@ Otherwise append 'mew-scan-form-header' to FORM."
 ;;;
 
 (defsubst mew-scan-uid-uid (uid)
-  (nth 0 (mew-split uid 32)))
+  (nth 0 (mew-split uid mew-sp)))
 
 (defsubst mew-scan-uid-size (uid)
-  (nth 1 (mew-split uid 32)))
+  (nth 1 (mew-split uid mew-sp)))
 
 (defun mew-header-insert-xmu (uid siz truncated)
   (when (and (stringp uid) (stringp siz))
@@ -457,11 +596,15 @@ Otherwise append 'mew-scan-form-header' to FORM."
       (if truncated
 	  (setq fields (concat uid " 0" siz)) ;; e.g. 0500 == truncated
 	(setq fields (concat uid " " siz)))
+      (save-excursion
+	(mew-header-delete-lines (list mew-x-mew-uidl:)))
       (mew-header-insert mew-x-mew-uidl: fields 'no-fold))))
 
 (defun mew-scan-message-truncatedp ()
-  (let ((siz (mew-scan-uid-size (MEW-UID))))
-    (and siz (string-match "^0" siz))))
+  (mew-msg-truncatedp (mew-scan-uid-size (MEW-UID))))
+
+(defun mew-scan-message-invalidp ()
+  (mew-msg-invalidp (MEW-NUM)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -480,71 +623,61 @@ according to the range which you specify. If
 non-nil, only headers of messages are cacched. If executed with
 '\\[universal-argument]', these variables are considered reversed."
   (interactive "P")
-  (when (mew-summary-exclusive-p)
-    (mew-summary-only
+  (mew-summary-only
+   (when (mew-summary-exclusive-p)
      (let* ((bnm (mew-summary-folder-name 'ext))
 	    (case (mew-sinfo-get-case))
 	    (fld (mew-sinfo-get-folder))
 	    (askp mew-ask-range)
 	    (directive 'scan)
 	    (get-body (not header-only))
-	    scanp range oldformatp dir-newp)
+	    scanp range dir-newp)
        (mew-summary-folder-cache-load)
-       (save-excursion
-	 (goto-char (point-min))
-	 (if (and (not (eobp))
-		  (not (looking-at mew-regex-summary))
-		  (mew-folder-localp fld) ;; just in case
-		  (y-or-n-p "The format of this scan cache is old. Type 'y' when ready. "))
-	     (setq oldformatp t)))
-       (if oldformatp
-	   (mew-local-retrieve 'scan bnm mew-range-all 'erase)
-	 (cond
-	  ((interactive-p) ;; "s"
-	   (setq askp t)
-	   (setq scanp t))
-	  ((mew-summary-folder-dir-newp) ;; "g"
-	   (setq askp nil)
-	   (setq scanp t)))
-	 (if (mew-summary-folder-dir-newp) (setq dir-newp t))
-	 (if (or (interactive-p) goend) (goto-char (point-max)))
-	 (set-buffer-modified-p nil)
-	 (if (not scanp)
+       (cond
+	((interactive-p) ;; "s"
+	 (setq askp t)
+	 (setq scanp t))
+	((mew-summary-folder-dir-newp) ;; "g"
+	 (setq askp nil)
+	 (setq scanp t)))
+       (if (mew-summary-folder-dir-newp) (setq dir-newp t))
+       (if (or (interactive-p) goend) (goto-char (point-max)))
+       (set-buffer-modified-p nil)
+       (if (not scanp)
+	   (progn
 	     (run-hooks 'mew-summary-ls-no-scan-hook)
-	   (mew-window-configure 'summary)
-	   ;;
-	   (mew-current-set nil nil nil)
-	   (mew-decode-syntax-delete)
-	   (mew-unhighlight-cursor-line)
-	   ;;
-	   (mew-sinfo-set-direction 'down)
-	   (cond
-	    ((and (mew-folder-remotep fld)
-		  (not (mew-folder-imap-queuep)))
-	     (if (and dir-newp (mew-folder-imapp fld))
-		 (mew-local-retrieve 'scan bnm (mew-range-update bnm))
-	       (setq range (mew-input-range-remote bnm))
-	       (when (eq range 'sync)
-		 (setq range nil)
-		 (setq directive 'sync))
-	       (cond
-		((mew-folder-popp fld)
-		 (if (mew-pop-header-only case)
-		     (setq get-body (not get-body)))
-		 (mew-pop-retrieve  case directive bnm range get-body))
-		((mew-folder-imapp fld)
-		 (if (mew-imap-header-only case)
-		     (setq get-body (not get-body)))
-		 (mew-imap-retrieve case directive bnm range get-body))
-		((mew-folder-nntpp fld)
-		 (if (mew-nntp-header-only case)
-		     (setq get-body (not get-body)))
-		 (mew-nntp-retrieve case directive bnm range get-body)))))
-	    (t ;; local
-	     (setq range (mew-input-range bnm askp))
-	     (if range
-		 (mew-local-retrieve 'scan bnm (nth 0 range) (nth 1 range))
-	       (message "range is wrong"))))))))))
+	     t) ;; return value (not scanned)
+	 (mew-summary-reset)
+	 ;;
+	 (mew-sinfo-set-direction 'down)
+	 (cond
+	  ((and (mew-folder-remotep fld)
+		(not (mew-folder-imap-queuep)))
+	   (if (and dir-newp (mew-folder-imapp fld))
+	       (mew-local-retrieve 'scan bnm (mew-range-update bnm))
+	     (setq range (mew-input-range-remote bnm))
+	     (when (eq range 'sync)
+	       (setq range nil)
+	       (setq directive 'sync))
+	     (cond
+	      ((mew-folder-popp fld)
+	       (if (mew-pop-header-only case)
+		   (setq get-body (not get-body)))
+	       (mew-pop-retrieve case directive bnm range get-body))
+	      ((mew-folder-imapp fld)
+	       (if (mew-imap-header-only case)
+		   (setq get-body (not get-body)))
+	       (mew-imap-retrieve case directive bnm range get-body))
+	      ((mew-folder-nntpp fld)
+	       (if (mew-nntp-header-only case)
+		   (setq get-body (not get-body)))
+	       (mew-nntp-retrieve case directive bnm range get-body)))))
+	  (t ;; local
+	   (setq range (mew-input-range bnm askp))
+	   (if range
+	       (mew-local-retrieve 'scan bnm (nth 0 range) (nth 1 range))
+	     (message "range is wrong"))))
+	 nil))))) ;; return value (scanned)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -564,14 +697,11 @@ non-nil, only headers of messages are cacched. If executed with
 (defsubst mew-summary-folder-dir-newp ()
   (let* ((folder (mew-summary-folder-name 'ext))
 	 (dir (file-chase-links (mew-expand-folder folder)))
-	 (t1 (if mew-touch-folder-p
-		 (mew-file-get-time
-		  (expand-file-name mew-summary-touch-file
-				    (mew-expand-folder dir)))
-	       (mew-file-get-time dir)))
+	 (mfile (expand-file-name mew-summary-touch-file dir))
+	 (t1 (mew-file-get-time mfile))
 	 (cache (expand-file-name mew-summary-cache-file dir))
 	 (t2 (mew-file-get-time cache)))
-    (if (and mew-touch-folder-p (null t1)
+    (if (and (null t1)
 	     (file-directory-p dir)
 	     (mew-dir-messages (mew-expand-folder dir)))
 	t
@@ -584,19 +714,46 @@ non-nil, only headers of messages are cacched. If executed with
 	 (t2 (mew-sinfo-get-cache-time)))
     (mew-compare-times t1 t2)))
 
+(defun mew-summary-set-count-line ()
+  (let* ((ttl-line (mew-count-lines (point-min) (point-max)))
+	 (mid-point (/ (buffer-size) 2))
+	 (mid-marker (mew-sinfo-get-mid-marker))
+	 mid-line)
+    (save-excursion
+      (goto-char mid-point)
+      (beginning-of-line)
+      (if (and (mew-thread-p) mew-use-thread-separator
+	       (looking-at mew-regex-thread-separator))
+	  (forward-line))
+      (setq mid-point (point))
+      (setq mid-line (mew-count-lines (point-min) (point))))
+    (mew-sinfo-set-ttl-line ttl-line)
+    (mew-sinfo-set-mid-line mid-line)
+    (unless (markerp mid-marker)
+      (setq mid-marker (make-marker))
+      (mew-sinfo-set-mid-marker mid-marker))
+    (set-marker mid-marker mid-point)))
+
 (defun mew-summary-folder-cache-load ()
   (let* ((folder (mew-summary-folder-name 'ext))
-	 (cache (mew-expand-folder folder mew-summary-cache-file)))
+	 (cache (mew-expand-folder folder mew-summary-cache-file))
+	 refile)
     (when (and (file-readable-p cache)
 	       (mew-summary-folder-cache-newp))
       (mew-elet
        (mew-erase-buffer)
        (mew-frwlet
 	mew-cs-m17n mew-cs-dummy
-	(insert-file-contents cache))
+	(mew-insert-file-contents cache))
        (mew-sinfo-set-cache-time (mew-file-get-time cache))
-       (setq mew-summary-buffer-raw t)
-       (mew-mark-undo-mark mew-mark-refile 'no-msg)
+       (if (= (point-max) 1)
+	   (setq mew-summary-buffer-raw nil)
+	 (setq mew-summary-buffer-raw t))
+       (mew-sinfo-load)
+       (setq refile (mew-summary-mark-recover
+		     (mew-sinfo-get-mark-hist) (mew-sinfo-get-refile)))
+       (mew-sinfo-set-refile refile)
+       (mew-summary-set-count-line)
        (set-buffer-modified-p nil)))))
 
 (defun mew-summary-folder-cache-save ()
@@ -626,7 +783,10 @@ non-nil, only headers of messages are cacched. If executed with
 	   mew-cs-dummy mew-cs-m17n
 	   (write-region (point-min) (point-max) cache nil 'no-msg))
 	  (mew-set-file-modes cache))
-	(mew-sinfo-set-cache-time (mew-file-get-time cache))))))
+	(mew-summary-set-count-line)
+	(mew-sinfo-set-cache-time (mew-file-get-time cache))
+	(mew-sinfo-save)
+	(mew-sinfo-set-mark-hist nil)))))
 
 ;; See also mew-net-folder-clean.
 (defun mew-summary-folder-cache-clean (folder)
