@@ -78,18 +78,14 @@
 
 (defun mew-sort-index (x) (car x))
 
-(defun mew-sort-adjust-number (form num line)
-  (if (string-match mew-regex-msg line)
-      (let ((n (match-end 0))
-	    (props (text-properties-at 0 line)))
-	(setq num (format form num))
-	(when props
-	  (add-text-properties 0 (length num) props num))
-	(concat num (substring line n)))
-    line)) ;; error
+(defun mew-sort-insert (line msg)
+  (insert line)
+  (save-excursion
+    (forward-line -1)
+    (mew-syntax-change-message-number msg)))
 
 ;; If not found, returns nil.
-(defun mew-summary-sort-rename (src dst pos form &optional lastp)
+(defun mew-summary-sort-rename (src dst pos &optional lastp)
   (if (mew-debug 'sort)
       (mew-elet
        (insert (format "move %s to %s\n" src dst)))
@@ -101,10 +97,11 @@
 	(lastp
 	 (when pos
 	   (goto-char pos)
-	   (insert (mew-sort-adjust-number form dst mew-sort-line))))
+	   (mew-sort-insert mew-sort-line dst)
+	   nil))
 	((null pos) ;; first
 	 (goto-char (point-min))
-	 (when (re-search-forward (mew-regex-jmp-msg src) nil t)
+	 (when (re-search-forward (mew-regex-sumsyn-msg src) nil t)
 	   (beginning-of-line)
 	   (setq beg (point))
 	   (forward-line)
@@ -116,7 +113,7 @@
 	   beg))
 	(t
 	 (goto-char (point-min))
-	 (when (re-search-forward (mew-regex-jmp-msg src) nil t)
+	 (when (re-search-forward (mew-regex-sumsyn-msg src) nil t)
 	   (beginning-of-line)
 	   (setq beg (point))
 	   (forward-line)
@@ -130,7 +127,7 @@
 	     (delete-region beg end)
 	     (save-excursion
 	       (goto-char pos)
-	       (insert (mew-sort-adjust-number form dst line)))
+	       (mew-sort-insert line dst))
 	     (point))
 	    ((= pos beg)
 	     ;; We need to keep properties in Summary mode.
@@ -138,14 +135,14 @@
 	     (setq line (buffer-substring beg end))
 	     (delete-region beg end)
 	     (goto-char pos)                    
-	     (insert (mew-sort-adjust-number form dst line))
+	     (mew-sort-insert line dst)
 	     (point))
 	    (t
 	     ;; We need to keep properties in Summary mode.
 	     ;; This must be "buffer-substring".
 	     (setq line (buffer-substring beg end))
 	     (goto-char pos)
-	     (insert (mew-sort-adjust-number form dst line))
+	     (mew-sort-insert line dst)
 	     (delete-region beg end)
 	     beg)))))))))
 
@@ -156,45 +153,49 @@ After sorting, the cursor moves onto the beginning of the buffer
 or the region. 
 
 If this command is used in a remote folder, local cache messages are
-sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync' would not work well."
+sorted. But if you do so, '\\<mew-summary-mode-map>\\[mew-summary-ls]' + 'sync' would not work well."
   (interactive "P")
   (mew-summary-only
-   (mew-summary-not-in-queue
-    (mew-summary-not-in-draft
-     (mew-summary-with-mewls
-      (when (mew-summary-exclusive-p)
-	(mew-summary-sort-body arg)))))))
+   (mew-summary-local-only
+    (mew-summary-not-in-queue
+     (mew-summary-not-in-draft
+      (mew-summary-with-mewl
+       (when (mew-summary-exclusive-p)
+	 (let ((folder (mew-summary-folder-name)))
+	   (if (null folder)
+	       (message "No message")
+	     (if (mew-mark-active-p) (setq arg t))
+	     (mew-summary-sort-body folder arg))))))))))
 
-(defun mew-summary-sort-body (arg)
-  (let ((folder (mew-summary-folder-name))
-	(win (selected-window))
+(defun mew-summary-sort-body (folder arg)
+  (let ((win (selected-window))
 	key type idx files range beg end)
     ;;
     ;; Summary cache updates
     ;;
-    (mew-current-set nil nil nil)
-    (mew-decode-syntax-delete)
-    (mew-unhighlight-cursor-line)
+    (mew-summary-reset)
     (mew-summary-retrieve-gap folder)
     ;;
     ;; Determining range
     ;;
-    (if (null arg)
-	(setq range mew-range-all)
-      (let ((region (mew-summary-get-region))
-	    rbeg rend)
-	(setq beg (car region))
-	(setq end (cdr region))
-	(if (= beg end) (error "No region"))
-	(goto-char beg)
-	(setq rbeg (mew-summary-message-number))
-	(goto-char end)
-	(forward-line -1)
-	(setq rend (mew-summary-message-number))
-	(forward-line)
-	(if (and rbeg rend)
-	    (setq range (concat rbeg "-" rend))
-	  (error "No region"))))
+    (let (region rbeg rend)
+      (if arg
+	  (progn
+	    (setq region (mew-summary-get-region))
+	    (setq beg (car region))
+	    (setq end (cdr region))
+	    (if (= beg end) (error "No region")))
+	(setq beg (point-min))
+	(setq end (point-max)))
+      (goto-char beg)
+      (setq rbeg (mew-summary-message-number))
+      (goto-char end)
+      (forward-line -1)
+      (setq rend (mew-summary-message-number))
+      (forward-line)
+      (if (and rbeg rend)
+	  (setq range (concat rbeg "-" rend))
+	(error "No region")))
     ;;
     ;; Asking a sort key after range
     ;;
@@ -211,7 +212,7 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
     (condition-case nil
 	(progn
 	  ;;
-	  ;; Calling mewls
+	  ;; Calling mewl
 	  ;;
 	  (let* ((funcs (assoc type mew-sort-switch))
 		 (func1 (nth 1 funcs))
@@ -220,7 +221,7 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 		 (fld (mew-expand-folder2 folder))
 		 num med value ent)
 	    (with-temp-buffer
-	      (call-process mew-prog-mewls nil t nil
+	      (call-process mew-prog-mewl nil t nil
 			    "-b" mew-mail-path "-l" "0"
 			    "-d" key "-s" (concat fld " " range))
 	      (goto-char (point-min))
@@ -240,15 +241,12 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 	    (setq ent (sort ent func2))
 	    (setq idx (vconcat (mapcar 'mew-sort-index ent))))
 	  ;;
-	  (cond
-	   ((mew-debug 'sort)
+	  (when (mew-debug 'sort)
 	    (mew-window-configure 'message)
 	    ;; message buffer
 	    (mew-elet
 	     (mew-erase-buffer)
 	     (insert "Sort as follows:\n")))
-	   (t
-	    (mew-window-configure 'summary)))
 	  ;; for C-xC-x
 	  (when (and (not (mew-debug 'sort)) (not (eobp)))
 	    (beginning-of-line)
@@ -279,7 +277,6 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 	  (let* ((dir (mew-expand-folder folder))
 		 (default-directory dir)
 		 (len (length idx));; not (length files)
-		 (form (format "%%%ds" (mew-summary-scan-form-num folder)))
 		 (tmp (mew-folder-new-message folder 'num-only))
 		 (i 0)
 		 src dst pos)
@@ -288,17 +285,15 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 	      (unless (= i (aref idx i))
 		(setq dst len)
 		(setq src i)
-		(setq pos (mew-summary-sort-rename
-			   (aref files src) tmp nil form))
+		(setq pos (mew-summary-sort-rename (aref files src) tmp nil))
 		(catch 'loop
 		  (while t
 		    (setq dst src)
 		    (setq src (aref idx dst))
 		    (if (= src i) (throw 'loop nil))
-		    (setq pos (mew-summary-sort-rename
-			       (aref files src) (aref files dst) pos form))
+		    (setq pos (mew-summary-sort-rename (aref files src) (aref files dst) pos))
 		    (aset idx dst dst)))
-		(mew-summary-sort-rename tmp (aref files dst) pos form 'last)
+		(mew-summary-sort-rename tmp (aref files dst) pos 'last)
 		(aset idx dst dst))
 	      (setq i (1+ i))))
 	  ;; The cursor moves onto point-min of the region.
@@ -317,7 +312,7 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 			 (point-min) 'mew-sort-orig)))
 	      ;; 'mew-sort-orign may start with bob.
 	      (if (null orig)
-		  (push-mark (point-max) t t)
+		  (mew-push-mark)
 		(save-excursion
 		  (goto-char orig)
 		  (beginning-of-line)
@@ -328,7 +323,7 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 		 ;; it.
 		 (remove-text-properties
 		  orig (+ orig (window-width)) '(mew-sort-orig nil)))
-		(push-mark orig t t)))
+		(mew-push-mark)))
 	    (mew-summary-folder-cache-save)
 	    (set-buffer-modified-p nil)))
 	  ;;
@@ -348,23 +343,13 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 ;;; Packing
 ;;;
 
-(defun mew-summary-pack-rename (src dst form)
-  (if (mew-debug 'pack)
-      (mew-elet
-       (insert (format "move %s to %s\n" src dst)))
-    (mew-elet
-     (let (beg end line)
-       (rename-file src dst)
-       (mew-refile-change src dst)
-       (when (re-search-forward (mew-regex-jmp-msg src) nil t)
-	 (setq beg (match-beginning 0))
-	 (setq end (match-end 0))
-	 ;; We need to keep properties in Summary mode.
-	 ;; This must be "buffer-substring".
-	 (setq line (buffer-substring beg end))
-	 (delete-region beg end)
-	 (insert (mew-sort-adjust-number form dst line))
-	 (forward-line))))))
+(defun mew-summary-pack-rename (src dst)
+  (mew-elet
+   (rename-file src dst)
+   (mew-refile-change src dst)
+   (when (re-search-forward (mew-regex-sumsyn-msg src) nil t)
+     (mew-syntax-change-message-number2 dst)
+     (forward-line))))
 
 (defun mew-summary-pack ()
   "Pack messages and list them up again.
@@ -373,25 +358,24 @@ If this command is used in a remote folder,
 local cache messages are packed."
   (interactive)
   (mew-summary-only
-   (mew-summary-not-in-queue
-    (mew-summary-not-in-draft
-     (when (mew-summary-exclusive-p)
-       (if (or (not mew-ask-pack)
-	       (y-or-n-p (format "Pack %s? " (mew-summary-folder-name))))
-	   (mew-summary-pack-body)))))))
+   (mew-summary-local-only
+    (mew-summary-not-in-queue
+     (mew-summary-not-in-draft
+      (when (mew-summary-exclusive-p)
+	(let ((folder (mew-summary-folder-name)))
+	  (cond
+	   ((null folder)
+	    (message "No message"))
+	   ((or (not mew-ask-pack) (y-or-n-p (format "Pack %s? " folder)))
+	    (mew-summary-pack-body folder))))))))))
 
-(defun mew-summary-pack-body ()
-  (let* ((win (selected-window))
-	 (folder (mew-summary-folder-name))
-	 (dir (mew-expand-folder folder))
+(defun mew-summary-pack-body (folder)
+  (let* ((dir (mew-expand-folder folder))
 	 (default-directory dir)
-	 (form (format "%%%ds" (mew-summary-scan-form-num folder)))
 	 (n 1)
 	 msgs msg src dst)
     ;;
-    (mew-current-set nil nil nil)
-    (mew-decode-syntax-delete)
-    (mew-unhighlight-cursor-line)
+    (mew-summary-reset)
     (mew-summary-retrieve-gap folder)
     ;;
     (message "Packing %s..." folder)
@@ -401,18 +385,9 @@ local cache messages are packed."
 	  (setq msgs (mew-dir-messages "."))
 	  (setq msgs (mapcar 'string-to-int msgs))
 	  (setq msgs (sort msgs '<)) ;; sort is inevitable
-	  (cond
-	   ((mew-debug 'pack)
-	    (mew-window-configure 'message)
-	    ;; message buffer
-	    (mew-elet
-	     (mew-erase-buffer)
-	     (insert "Pack as follows:\n")))
-	   (t
-	    (mew-window-configure 'summary)))
 	  ;; the cursor stays the current position.
 	  (save-excursion
-	    (unless (mew-debug 'pack) (goto-char (point-min)))
+	    (goto-char (point-min))
 	    (while msgs
 	      (setq msg (car msgs))
 	      (setq msgs (cdr msgs))
@@ -427,23 +402,14 @@ local cache messages are packed."
 		(while (file-exists-p dst)
 		  (setq n (1+ n))
 		  (setq dst (int-to-string n)))
-		(mew-summary-pack-rename src dst form)
+		(mew-summary-pack-rename src dst)
 		(setq n (1+ n))))))
-	  (cond
-	   ((mew-debug 'pack)
-	    (mew-message-clear-end-of)
-	    (set-buffer-modified-p nil)
-	    (goto-char (point-min))
-	    (select-window win))
-	   (t
-	    (mew-summary-folder-cache-save)
-	    (set-buffer-modified-p nil)))
+	  (mew-summary-folder-cache-save)
+	  (set-buffer-modified-p nil)
 	  (mew-summary-unlock)
-	  (unless (mew-debug 'pack)
-	    (run-hooks 'mew-pack-hook))
-	  (message "Packing %s...done" folder))
+	  (run-hooks 'mew-pack-hook)
+	  (message "Packing %s...done (the last is %d)" folder (1- n)))
       (quit
-       (select-window win)
        (set-buffer-modified-p nil)
        (mew-summary-unlock)))))
 
