@@ -7,28 +7,32 @@
 
 (require 'mew)
 
-(defvar mew-ssh-ver nil)		;; 0-SSH1, 1-SSH2, 2-OpenSSH
+(defvar mew-ssh-prog-ver nil)		;; 0-SSH1, 1-SSH2, 2-OpenSSH, 3-PuTTY
 
-(defsubst mew-ssh-get (list-or-vec)
-  (elt list-or-vec mew-ssh-ver))
+(defsubst mew-ssh-get (case list-or-vec)
+  (elt list-or-vec (mew-ssh-prog-ver case)))
 
 (defconst mew-ssh-msg-passwd 
   '("password:\\|Enter passphrase"
     "password:\\|Passphrase for\\|Enter passphrase"
-    "password:\\|Enter passphrase"))
+    "password:\\|Enter passphrase"
+    "Password:\\|Passphrase for key"))
 
 (defconst mew-ssh-msg-connected
   '("Entering interactive session"
     "Authentication successful\\|client_authenticated"
-    "Entering interactive session"))
+    "Entering interactive session"
+    "Access granted"))
 
 (defconst mew-ssh-msg-denied
-  '("Permission denied\\." "Permission denied\\." "Permission denied\\."))
+  '("Permission denied\\." "Permission denied\\." "Permission denied\\."
+    "Permission denied\\."))		;XXX
 
 (defconst mew-ssh-msg-refused
   '("Secure connection .* refused\\."
     "FATAL: Connecting .* failed:"
-    "Secure connection .* refused\\."))
+    "Secure connection .* refused\\."
+    "Secure connection .* refused\\."))	;XXX
 
 (defconst mew-ssh-process-exec-cnt 3)
 
@@ -37,7 +41,7 @@
 ;;; SSH info
 ;;;
 
-(defvar mew-ssh-info-list '("status" "try"))
+(defvar mew-ssh-info-list '("case" "status" "try" "account"))
 
 (mew-info-defun "mew-ssh-" mew-ssh-info-list)
 
@@ -58,7 +62,7 @@
 (defsubst mew-ssh-pnm-to-lport (pnm)
   (if (string-match ":\\([0-9]+\\)$" pnm) (match-string 1 pnm)))
 
-;; user@host. port should be used here.
+;; user@host:port should be used here.
 ;; However, SSH does not symbolic port name "ssh" for the "-p" option
 ;; while OpenSSH does. Sigh...
 (defsubst mew-ssh-passtag (name)
@@ -70,24 +74,26 @@
 ;;; Executing SSH
 ;;;
 
-(defun mew-open-ssh-stream (server serv sshserver)
+(defun mew-open-ssh-stream (case server serv sshserver)
   "Open SSH stream for SERVER's SERV via SSHSERVER.
 This function returns a process when an SSH connection is created
 successfully. A local port number can be obtained the process name
-after ':'. "
-  (cond
-   ((not (and (stringp server) (stringp serv) (stringp sshserver)))
-    ())
-   ((not (mew-which-exec mew-prog-ssh))
-    ())
-   (t
-    (let* ((remoteport (mew-serv-to-port serv))
-	   (localport (+ 8000 (% (mew-random) 4000)))
-	   (process-connection-type mew-connection-type2)
-	   (i 0) (N mew-ssh-process-exec-cnt)
-	   (pros (process-list))
-	   (regex (mew-ssh-info-name-regex sshserver server remoteport))
-	   buf name pnm pro status)
+after ':'."
+  (let* ((ssh (mew-ssh-prog case))
+	 (args (mew-ssh-prog-args case))
+	 (remoteport (mew-serv-to-port serv))
+	 (localport (+ 8000 (% (mew-random) 4000)))
+	 (process-connection-type mew-connection-type2)
+	 (i 0) (N mew-ssh-process-exec-cnt)
+	 (pros (process-list))
+	 (regex (mew-ssh-info-name-regex sshserver server remoteport))
+	 buf name pnm pro status)
+    (cond
+     ((not (and (stringp server) (stringp serv) (stringp sshserver)))
+      ())
+     ((not (mew-which-exec ssh))
+      (message "'%s' is not found" ssh))
+     (t
       (catch 'find
 	(while pros
 	  (when (string-match regex (process-name (car pros)))
@@ -104,18 +110,11 @@ after ':'. "
 	(catch 'loop
 	  (while (< i N)
 	    (setq name (mew-ssh-info-name sshserver server remoteport localport))
-	    (setq pro
-		  (apply 'start-process
-			 name buf
-			 mew-prog-ssh "-x" "-v"
-			 "-o" (concat 
-			       (if (= mew-ssh-ver 2)
-				   "NumberOfPasswordPrompts " ;; OpenSSH style
-				 "NumberOfPasswordPrompts=")  ;; SSH style
-			       (int-to-string mew-ssh-number-of-password-prompts))
-			 (format "-L%d:%s:%s" localport server remoteport)
-			 sshserver
-			 mew-prog-ssh-arg))
+	    (setq pro (apply 'start-process
+			     name buf
+			     ssh "-x" "-v" "-L"
+			     (format "%d:%s:%s" localport server remoteport)
+			     (append args (list sshserver))))
 	    ;; An error would occur. So, let's exit in the case.
 	    (mew-timing)
 	    (cond
@@ -131,6 +130,8 @@ after ':'. "
 	    (setq pnm (process-name pro))
 	    (mew-info-clean-up pnm)
 	    (mew-ssh-set-try pnm 0)
+	    (mew-ssh-set-case pnm case)
+            (mew-ssh-set-account pnm (format "%s" sshserver))
 	    (mew-set-process-cs pro mew-cs-text-for-read mew-cs-text-for-write)
 	    (set-process-filter pro 'mew-ssh-filter)
 	    (set-process-sentinel pro 'mew-ssh-sentinel)
@@ -162,13 +163,18 @@ after ':'. "
 
 (defun mew-ssh-filter (process string)
   (mew-ssh-debug "SSH: " string)
-  (let ((pnm (process-name process)) pass)
+  (let* ((pnm (process-name process))
+	 (case (mew-ssh-get-case pnm))
+	 pass)
     (cond
-     ((string-match (mew-ssh-get mew-ssh-msg-passwd) string)
+     ((string-match (mew-ssh-get case mew-ssh-msg-passwd) string)
       (if (= (mew-ssh-get-try pnm) 0)
-	  (setq pass (mew-input-passwd "SSH password: " (mew-ssh-passtag pnm)))
+	  (setq pass (mew-input-passwd (format "SSH password (%s): "
+                                               (mew-ssh-get-account pnm))
+                                       (mew-ssh-passtag pnm)))
 	(mew-passwd-set-passwd (mew-ssh-passtag pnm) nil)
-	(setq pass (mew-input-passwd "SSH password again: "
+	(setq pass (mew-input-passwd (format "SSH password again (%s): "
+                                             (mew-ssh-get-account pnm))
 				     (mew-ssh-passtag pnm))))
       (mew-ssh-set-try pnm (1+ (mew-ssh-get-try pnm)))
       (if (and (processp process)
@@ -178,12 +184,12 @@ after ':'. "
 	    (process-send-string process (concat pass "\n")))
 	(mew-passwd-set-passwd (mew-ssh-passtag pnm) nil)
 	(message "Cannot find SSH process")))
-     ((or (string-match (mew-ssh-get mew-ssh-msg-denied) string)
-	  (string-match (mew-ssh-get mew-ssh-msg-refused) string))
+     ((or (string-match (mew-ssh-get case mew-ssh-msg-denied) string)
+	  (string-match (mew-ssh-get case mew-ssh-msg-refused) string))
       (message "Cannot connect to the SSH server")
       (mew-passwd-set-passwd (mew-ssh-passtag pnm) nil)
       (mew-ssh-set-status pnm 'denied))
-     ((string-match (mew-ssh-get mew-ssh-msg-connected) string)
+     ((string-match (mew-ssh-get case mew-ssh-msg-connected) string)
       (mew-ssh-set-status pnm t)
       (message "Connecting to the SSH server...done"))
      ((string-match "Local: bind: Address already in use" string)
@@ -202,22 +208,25 @@ after ':'. "
 ;;;
 
 (defun mew-ssh-setup ()
-  (if (not (mew-which-exec mew-prog-ssh))
-      (setq mew-ssh-ver nil)
-    (with-temp-buffer
-      (call-process mew-prog-ssh nil t nil "-V")
-      (goto-char (point-min))
-      (cond
-       ((looking-at "SSH Version 1")
-	(setq mew-ssh-ver 0))
-       ((looking-at (concat mew-prog-ssh ": SSH Version 2"))
-	(setq mew-ssh-ver 1))
-       ((looking-at (concat mew-prog-ssh ": SSH Secure Shell [23]"))
-	(setq mew-ssh-ver 1))
-       ((looking-at "\\(SSH Version \\)?\\(OpenSSH\\|NetBSD\\|Sun_SSH\\)")
-	(setq mew-ssh-ver 2))
-       (t
-	(setq mew-ssh-ver nil))))))
+  (let ((ssh (mew-ssh-prog)))
+    (if (not (mew-which-exec ssh))
+	(setq mew-ssh-prog-ver nil)
+      (with-temp-buffer
+	(call-process ssh nil t nil "-V")
+	(goto-char (point-min))
+	(cond
+	 ((looking-at "SSH Version 1")
+	  (setq mew-ssh-prog-ver 0))
+	 ((looking-at (concat ssh ": SSH Version 2"))
+	  (setq mew-ssh-prog-ver 1))
+	 ((looking-at (concat ssh ": SSH Secure Shell [23]"))
+	  (setq mew-ssh-prog-ver 1))
+	 ((looking-at "\\(SSH Version \\)?\\(OpenSSH\\|NetBSD\\|Sun_SSH\\)")
+	  (setq mew-ssh-prog-ver 2))
+	 ((looking-at "plink:")
+	  (setq mew-ssh-prog-ver 3))
+	 (t
+	  (setq mew-ssh-prog-ver nil)))))))
 
 (provide 'mew-ssh)
 
