@@ -22,7 +22,7 @@ A file name of a certificate should be 'cert-hash.0'.
     connection is not created. If not present, an SSL connection is
     not created.
 3 - verify server's certificate which locally installed (not one from
-    the server). ")
+    the server).")
 
 (defvar mew-prog-ssl-arg nil) ;; xxx what about v4?
 
@@ -35,7 +35,7 @@ A file name of a certificate should be 'cert-hash.0'.
 ;;; SSL info
 ;;;
 
-(defvar mew-ssl-info-list '("status" "try"))
+(defvar mew-ssl-info-list '("status" "try" "file"))
 
 (mew-info-defun "mew-ssl-" mew-ssl-info-list)
 
@@ -91,12 +91,13 @@ A file name of a certificate should be 'cert-hash.0'.
   "Open SSL stream for SERVER's SERV.
 This function returns a process when an SSL connection is created
 successfully. A local port number can be obtained the process name
-after ':'. "
+after ':'."
   (cond
    ((not (and (stringp server) (stringp serv)))
-    ())
-   ((not (mew-which-exec mew-prog-ssl))
-    ())
+    nil)
+   ((or (null mew-ssl-ver) (not (mew-which-exec mew-prog-ssl)))
+    (message "'%s' is not found" mew-prog-ssl)
+    nil)
    (t
     (let* ((remoteport (mew-serv-to-port serv))
 	   (localport (+ 8000 (% (mew-random) 4000)))
@@ -104,7 +105,7 @@ after ':'. "
 	   (i 0) (N mew-ssl-process-exec-cnt)
 	   (pros (process-list))
 	   (regex (mew-ssl-info-name-regex server remoteport))
-	   buf name pnm pro status dummy)
+	   buf name pnm pro dummy bound opts)
       (catch 'find
 	(while pros
 	  (when (string-match regex (process-name (car pros)))
@@ -121,8 +122,8 @@ after ':'. "
 	(catch 'loop
 	  (while (< i N)
 	    (setq name (mew-ssl-info-name server remoteport localport))
-	    (setq pro (apply 'start-process name buf mew-prog-ssl
-			     (mew-ssl-options server remoteport localport)))
+	    (setq opts (mew-ssl-options server remoteport localport))
+	    (setq pro (apply 'start-process name buf mew-prog-ssl opts))
 	    ;; An error would occur. So, let's exit in the case.
 	    (mew-timing)
 	    (cond
@@ -138,33 +139,38 @@ after ':'. "
 	    (setq pnm (process-name pro))
 	    (mew-info-clean-up pnm)
 	    (mew-ssl-set-try pnm 0)
+	    (if (= mew-ssl-ver 4) (mew-ssl-set-file pnm (car opts)))
 	    (mew-set-process-cs pro mew-cs-text-for-read mew-cs-text-for-write)
-	    (set-process-filter pro 'mew-ssl-filter)
+	    (set-process-filter pro 'mew-ssl-filter1)
 	    (set-process-sentinel pro 'mew-ssl-sentinel)
 	    (mew-rendezvous (null (mew-ssl-get-status pnm)))
-	    (setq status (mew-ssl-get-status pnm))
-	    (cond
-	     ((eq status t)
-	      ;; "stunnel" does not gain access to the remote port
-	      ;; until a tunneled connection is created.
-	      ;; So, we need to check the SSL tunnel with a dummy
-	      ;; tunneled connection here.
-	      (setq dummy (open-network-stream
-			   " *Mew dummy*" nil "localhost" localport))
-	      (sleep-for 0.1)
-	      (unless (and (processp dummy) (eq (process-status dummy) 'open))
-		(delete-process pro)
-		(setq pro nil))
-	      (delete-process dummy)
-	      (throw 'loop pro)) ;; return value
-	     ((not (eq status 'bound))
-	      (message "Creating an SSL connection...FAILED")
-	      (throw 'loop nil))) ;; return value
-	    ;; 'bound
+	    (if (eq (mew-ssl-get-status pnm) t)
+		(throw 'loop (setq bound t)))
+	    ;; bind-failure
 	    (setq localport (1+ localport))
-	    (setq i (1+ i))
-	    (message "Creating an SSL connection...FAILED")
-	    nil))))))) ;; return value
+	    (setq i (1+ i))))
+	(mew-ssl-set-status pnm nil)
+	(if (not bound)
+	    (progn
+	      (message "Creating an SSL connection...FAILED")
+	      nil)
+	  ;; "stunnel" does not gain access to the remote port
+	  ;; until a tunneled connection is created.
+	  ;; So, we need to check the SSL tunnel with a dummy
+	  ;; tunneled connection here.
+	  (set-process-filter pro 'mew-ssl-filter2)
+	  (setq dummy (open-network-stream " *Mew dummy*" nil "localhost" localport))
+	  (mew-rendezvous (null (mew-ssl-get-status pnm)))
+	  (if (processp dummy) (delete-process dummy))
+	  (if (eq (mew-ssl-get-status pnm) t)
+	      (progn
+		(message "Creating an SSL connection...done")
+		(set-process-filter pro 'mew-ssl-filter3)
+		pro)
+	    ;; verify-failure
+	    (delete-process pro)
+	    (message "Creating an SSL connection...FAILED (cert verify failure)")
+	    nil)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -178,21 +184,37 @@ after ':'. "
       (goto-char (point-max))
       (insert (format "\n<%s>\n%s\n" label string)))))
 
-(defun mew-ssl-filter (process string)
+(defun mew-ssl-filter1 (process string)
   (mew-ssl-debug "SSL: " string)
   (let ((pnm (process-name process)))
     (cond
      ((string-match "bound to" string)
-      (mew-ssl-set-status pnm t)
-      (message "Creating an SSL connection...done"))
+      (mew-ssl-set-status pnm t))
+     ((string-match "gethostbyname: Valid name, no data record of requested type" string)
+      (mew-ssl-set-status pnm 'gethostbyname-failure))
+     ((string-match "gethostbyname: Host not found" string)
+      (mew-ssl-set-status pnm 'gethostbyname-failure))
      ((string-match "Local: bind: Address already in use" string)
-      (mew-ssl-set-status pnm 'bound)
-      (message "Creating an SSL connection...done")))))
+      (mew-ssl-set-status pnm 'bind-failure)))))
+
+(defun mew-ssl-filter2 (process string)
+  (mew-ssl-debug "SSL: " string)
+  (let ((pnm (process-name process)))
+    (cond
+     ((string-match "Negotiated ciphers\\|opened with SSL" string)
+      (mew-ssl-set-status pnm t))
+     ((string-match "Failed to initialize" string)
+      (mew-ssl-set-status pnm t)) ;; xxx
+     ((string-match "verify failed" string)
+      (mew-ssl-set-status pnm 'verify-failure)))))
+
+(defun mew-ssl-filter3 (process string)
+  (mew-ssl-debug "SSL: " string))
 
 (defun mew-ssl-sentinel (process event)
-  (let ((pnm (process-name process)))
-    (unless (mew-ssl-get-status pnm)
-      (mew-ssl-set-status pnm 'exit))
+  (let* ((pnm (process-name process))
+	 (file (mew-ssl-get-file pnm)))
+    (mew-delete-file file)
     (mew-remove-buffer (process-buffer process))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
