@@ -24,7 +24,7 @@
 	   (insert "\n"))
        (mew-message-clear-end-of)
        (when (or mew-end-of-message-string mew-end-of-part-string)
-	 (move-overlay (mew-minfo-get-overlay) (point-max) (point-max))
+	 (move-overlay (mew-minfo-get-eom) (point-max) (point-max))
 	 (if (mew-decode-syntax-p)
 	     (if (mew-summary-end-of-message-p)
 		 (mew-message-set-end-of-message)
@@ -32,8 +32,8 @@
 	   (mew-message-set-end-of-message)))))))
 
 (defsubst mew-message-clear-end-of ()
-  (unless (overlayp (mew-minfo-get-overlay))
-    (mew-minfo-set-overlay (mew-overlay-make (point-max) (point-max))))
+  (unless (overlayp (mew-minfo-get-eom))
+    (mew-minfo-set-eom (mew-overlay-make (point-max) (point-max))))
   (mew-message-set-end-of-nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -49,16 +49,16 @@
       (mew-summary-goto-message)
       (cond
        ((eq (mew-sinfo-get-direction) 'up)
-	(when (re-search-backward mew-regex-msg-or-part nil t)
-	  (setq fld (mew-summary-folder-name))
-	  (setq next (mew-summary-message-number))))
+	(when (re-search-backward mew-regex-sumsyn-short nil t)
+	  (setq fld (mew-sumsyn-folder-name))
+	  (setq next (mew-sumsyn-message-number))))
        ((eq (mew-sinfo-get-direction) 'down)
 	(if (mew-decode-syntax-end)
 	    (goto-char (mew-decode-syntax-end))
 	  (forward-line))
-	(when (re-search-forward mew-regex-msg-or-part nil t)
-	  (setq fld (mew-summary-folder-name))
-	  (setq next (mew-summary-message-number))))))
+	(when (re-search-forward mew-regex-sumsyn-short nil t)
+	  (setq fld (mew-sumsyn-folder-name))
+	  (setq next (mew-sumsyn-message-number))))))
     ;; should get the cursor back for display
     (save-excursion
       (if (and fld next
@@ -123,7 +123,7 @@ If nodisplay is non-nil, displays the cached message."
   ;; message buffer
   (mew-erase-buffer)
   (mew-message-clear-end-of)
-  (mew-overlay-delete-buffer) ;; xxx also delete extents?
+  (mew-overlay-delete-buffer) ;; veil, xxx also delete extents?
   ;; kill mark for cite
   (and (mark-marker) (set-marker (mark-marker) nil)))
 
@@ -139,7 +139,14 @@ If nodisplay is non-nil, displays the cached message."
 (defun mew-summary-cursor-postscript ()
   (mew-summary-mode-line)
   (mew-summary-recenter)
+  (mew-thread-move-cursor)
   (mew-highlight-cursor-line)
+  (when (and mew-use-unread-mark
+	     (mew-summary-message-number)
+	     (equal (mew-summary-get-mark) mew-mark-unread))
+    (unless (and mew-inherit-after-marking
+		 (not mew-delete-unread-mark-by-mark))
+      (mew-mark-put-mark mew-mark-read 'no-msg)))
   (set-buffer-modified-p nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -149,8 +156,7 @@ If nodisplay is non-nil, displays the cached message."
 
 (defsubst mew-summary-message-toobig (fld msg)
   (let ((file (mew-expand-folder fld msg)))
-    (and (mew-folder-localp fld)
-	 (file-readable-p file)
+    (and (file-readable-p file)
 	 (> (mew-file-get-size file) mew-file-max-size))))
 
 (defun mew-summary-display (&optional redisplay)
@@ -219,9 +225,8 @@ function displays it again getting gack to the beginning."
 		   (mew-decode-syntax-clear)
 		   (mew-summary-display-raw fld msg mew-file-max-size)
 		   (setq prefetch t)
-		   (message
-		    (substitute-command-keys
-		     "Too large, truncated. To see the entire message, type '\\<mew-summary-mode-map>\\[mew-summary-analyze-again]'"))))
+		   (mew-message-for-summary
+		    "Too large, truncated. To see the entire message, type '\\[mew-summary-analyze-again]'")))
 		(t
 		 (mew-decode-syntax-clear)
 		 (mew-summary-cache-message fld msg sumbuf)
@@ -251,7 +256,8 @@ function displays it again getting gack to the beginning."
 	 (if (mew-xinfo-get-decode-err)
 	     (message "MIME decoding error: %s" (mew-xinfo-get-decode-err)))
 	 (if (mew-xinfo-get-action)
-	     (message (mew-xinfo-get-action)))
+	     (message "%s" (mew-xinfo-get-action)))
+	 (mew-message-mode-line fld msg)
 	 (select-window win)
 	 ;; summary buffer
 	 (mew-summary-cursor-postscript)
@@ -338,91 +344,81 @@ This function does not create a cache."
 ;;; Analyze again
 ;;;
 
-(defun mew-find-longest-match (nums)
-  (save-excursion
-    (forward-line)
-    (let ((i 1) (max 0) (lim (length nums)) j part lines)
-      (while (and (mew-decode-syntax-end) (< (point) (mew-decode-syntax-end)))
-	(setq part (mew-syntax-nums))
-	(setq j 0)
-	(catch 'loop
-	  (while (< j lim)
-	    (unless (eq (nth j nums) (nth j part))
-	      (throw 'loop nil))
-	    (setq j (1+ j))))
-	(when (> j max)
-	  (setq max j)
-	  (setq lines i))
-	(setq i (1+ i))
-	(forward-line))
-      lines)))
-
 (defun mew-summary-analyze-again-alternative (&optional arg)
   "This command analyzes the message again with 
 'mew-use-alternative' and 'mew-use-text-body' reversed."
   (interactive "P")
+  (mew-summary-goto-message)
   (let ((mew-use-alternative (not mew-use-alternative))
 	(mew-use-text-body   (not mew-use-text-body)))
     (mew-summary-analyze-again arg)))
 
 (defun mew-summary-analyze-again (&optional arg)
-  "This command removes the cache of this message or part and
-analyzes the message, then displays this message or part again.
+  "1) If this command executed on a message, the cache of the
+message is removed and the message is analyzed then displayed.
 
-If the size of the current message exceeds 'mew-file-max-size', MIME
-analysis is skipped then the beginning of the raw message is
-displayed. In this situation, this command analyzes the current
-message without the limitation then displays it.
+1a) If the size of the current message exceeds
+'mew-file-max-size', MIME analysis is skipped then the beginning
+of the raw message is displayed. In this situation, this command
+analyzes the current message without the limitation then displays
+it.
 
-If the length of a header exceeds 'mew-header-max-length', a broken
-message is displayed. In this situation, this command analyzes the
-current message without the limitation then displays it.
+1b) If the length of a header exceeds 'mew-header-max-length', a
+broken message is displayed. In this situation, this command
+analyzes the current message without the limitation then displays
+it.
 
-If called with '\\[universal-argument]', analyze the message
-with 'mew-decode-broken' reversed."
+1c) If the current message is displayed by '\\<mew-summary-mode-map>\\[mew-summary-analyze-again-alternative]', this command gets it back to
+the normal display.
+
+1d) If 'mew-use-text/html' is nil, its HTML body is displayed as
+is. In this situation, this command analyzes the HTML body and
+displays it.
+
+1e) If called with '\\[universal-argument]', analyze the message
+with 'mew-decode-broken' reversed.
+
+2) If this command is called on a part, the part is displayed again.
+
+2a) If 'mew-use-text/html' is nil, its HTML part is displayed as
+is. In this situation, this command analyzes the HTML part and
+displays it."
   (interactive "P")
   (mew-summary-msg-or-part
    (if (mew-folder-draftp (mew-summary-folder-name))
        (mew-summary-display 'redisplay)
      (let ((mew-decode-broken
 	    (if arg (not mew-decode-broken) mew-decode-broken))
+	   (mew-use-text/html t)
+	   (mew-use-text/xml t)
 	   (fld (mew-summary-folder-name))
 	   (vfld (mew-summary-folder-name 'ext))
 	   (msg (mew-summary-message-number2))
 	   (part (mew-syntax-nums))
 	   (win (selected-window))
 	   (sumbuf (current-buffer))
-	   cache lines)
-       (mew-cache-delete2 fld msg)
-       (mew-summary-goto-message)
+	   cache)
        (unwind-protect
 	   (progn
 	     (mew-summary-toggle-disp-msg 'on)
 	     (mew-window-configure 'message)
 	     ;; message buffer
-	     (mew-current-set fld msg part)
-	     (mew-minfo-set-summary vfld)
-	     (mew-decode-syntax-clear)
-	     (mew-decode-syntax-delete)
-	     (if (null part)
-		 (mew-summary-cache-message fld msg sumbuf 'unlimit)
-	       (setq cache (mew-summary-cache-message
-			    fld msg sumbuf 'unlimit 'nodisplay))
-	       ;; Decode syntax is displayed.
-	       ;; Let's move the cursor for end-of-*.
-	       ;; Since its window is not selected, the displayed cursor
-	       ;; does not move. Hence call (forward-line lines) twice.
-	       (save-excursion
-		 (set-buffer sumbuf)
-		 (setq lines (mew-find-longest-match part))
-		 (if lines (forward-line lines))
-		 (setq part (mew-syntax-nums)))
-	       (mew-summary-display-part cache part)))
+	     (if part
+		 (progn
+		   (setq cache (mew-cache-hit fld msg))
+		   (mew-summary-display-part cache part))
+	       (mew-cache-delete2 fld msg)
+	       (mew-summary-goto-message)
+	       (mew-current-set fld msg part)
+	       (mew-minfo-set-summary vfld)
+	       (mew-decode-syntax-clear)
+	       (mew-decode-syntax-delete)
+	       (mew-summary-cache-message fld msg sumbuf 'unlimit)))
 	 (if (mew-xinfo-get-decode-err)
 	     (message "MIME decoding error: %s" (mew-xinfo-get-decode-err)))
+	 (mew-message-mode-line fld msg)
 	 (select-window win)
 	 ;; summary buffer
-	 (if lines (forward-line lines))
 	 (mew-summary-cursor-postscript))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -462,6 +458,9 @@ with 'mew-decode-broken' reversed."
 	   ;; message buffer
 	   (mew-current-set fld msg nums)
 	   (mew-minfo-set-summary vfld)
+	   ;; The following codes are not necessary. 
+	   ;; (mew-decode-syntax-clear)
+	   ;; (mew-decode-syntax-delete)
 	   (mew-elet
 	    (mew-summary-display-preamble)
 	    (mew-decode-syntax-copy buf)
@@ -484,7 +483,10 @@ with 'mew-decode-broken' reversed."
 	      (goto-char (point-min)))))
 	   (mew-summary-display-postscript 'no-hook))
        (mew-remove-buffer buf)
-       (select-window win)))))
+       (mew-message-mode-line fld msg)
+       (select-window win)
+       ;; summary buffer
+       (mew-summary-cursor-postscript)))))
 
 (defun mew-summary-find-file (&optional arg)
   "Open this message and makes it read-only.
@@ -499,8 +501,88 @@ If called with '\\[universal-argument]', it stays writable."
      (mew-frwlet
       (if (mew-folder-draftp fld) mew-cs-m17n mew-cs-autoconv) mew-cs-dummy
       (if arg
-	  (find-file file)
-	(view-file file))))))
+	  (switch-to-buffer (mew-find-file-noselect2 file))
+	(view-buffer (mew-find-file-noselect2 file)))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Trace Path
+;;;
+
+(defun mew-summary-trace-path ()
+  "Parse the Received: fields and display them in Message mode."
+  (interactive)
+  (mew-summary-msg-or-part
+   (let* ((win (selected-window))
+	  fld msg cache lst ent svr-date snd rcv svr date tmp)
+     (save-excursion
+       (mew-summary-goto-message)
+       (when (mew-sumsyn-match mew-regex-sumsyn-short)
+	 (setq fld (mew-sumsyn-folder-name))
+	 (setq msg (mew-sumsyn-message-number))) ;; msg is never nil
+       (setq cache (mew-cache-hit fld msg))
+       (cond
+	(cache
+	 (set-buffer cache)
+	 (setq lst (mew-header-get-value mew-received: 'as-list)))
+	(t
+	 (with-temp-buffer
+	   (mew-insert-message fld msg mew-cs-autoconv
+			       mew-header-reasonable-size)
+	   (setq lst (mew-header-get-value mew-received: 'as-list))))))
+     (mew-summary-toggle-disp-msg 'on)
+     (mew-window-configure 'message)
+     ;; messge buffer
+     (mew-summary-display-preamble)
+     (setq lst (nreverse lst))
+     (while lst
+       (setq ent (car lst))
+       (setq lst (cdr lst))
+       (setq svr-date (mew-split ent ?\;))
+       (mew-set '(svr date) svr-date)
+       (unless date
+	 ;; for broken MTAs
+	 (if (string-match "Mon\\|Tue\\|Wed\\|Thu\\|Fri\\|Sat\\|Sun" ent)
+	     (setq date (substring ent (match-beginning 0)))))
+       (if date (setq date (mew-time-rfc-to-sortkey date 'tzadj)))
+       (unless date (setq date "19700101000000"))
+       (setq tmp (cons (list date svr) tmp))
+       (cond
+	((string-match "(\\(qmail [0-9]+\\)" svr)
+	 (setq rcv (match-string 1 svr)))
+	((string-match "by[ \t]*\\([^() \t\n]+\\)" svr)
+	 (setq rcv (match-string 1 svr))
+	 (if (string-match "from[ \t]*\\([^() \t\n]+\\)" svr)
+	     (setq snd (match-string 1 svr)))
+	 (when (and (string= snd "unknown")
+		    (string-match "from unknown (HELO \\([^() \t\n]+\\))" svr))
+	   (setq snd (concat (match-string 1 svr) "(unknown)"))))
+	((string-match "[ \t]*\\([^() \t\n]+\\)" svr)
+	 (setq rcv (match-string 1 svr))
+	 (if (string-match "from[ \t]*\\([^() \t\n]+\\)" svr)
+	     (setq snd (match-string 1 svr)))))
+       (mew-elet
+	(insert date)
+	(beginning-of-line)
+	(forward-char 4)
+	(insert "/")
+	(forward-char 2)
+	(insert "/")
+	(forward-char 2)
+	(insert " ")
+	(forward-char 2)
+	(insert ":")
+	(forward-char 2)
+	(insert ":")
+	(forward-char 2)
+	(insert " (" (mew-time-tmzn-int) ") ")
+	(if snd (insert snd))
+	(insert " => ")
+	(if rcv (insert rcv))
+	(insert "\n")
+	(setq snd nil rcv nil)))
+     (mew-summary-display-postscript 'no-hook)
+     (select-window win))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -516,37 +598,49 @@ If called with '\\[universal-argument]', it stays writable."
   "This function is called by mew-summary-down(). This is a very
 ad-hoc solution for putting the review mark on a thread.")
 
+(defun mew-re-search-forward-visibile (regex)
+  (catch 'loop
+    (while (re-search-forward regex nil t)
+      (unless (eq (get-text-property (point) 'invisible) t)
+	(throw 'loop t)))))
+
+(defun mew-re-search-backward-visible (regex)
+  (catch 'loop
+    (while (re-search-backward regex nil t)
+      (unless (eq (get-text-property (point) 'invisible) t)
+	(throw 'loop t)))))
+
 (defun mew-summary-down ()
   (funcall mew-summary-down-function)
   (cond 
-   ((re-search-forward mew-regex-msg-or-part nil t)
-    (mew-summary-thread-move-cursor)
+   ((mew-re-search-forward-visibile mew-regex-msg-or-part)
+    (mew-thread-move-cursor)
     (mew-sinfo-set-direction 'down)
     t)
    (t
     (mew-decode-syntax-delete)
     (forward-line -1)
+    (while (and (eq (get-text-property (point) 'invisible) t)
+		(= (forward-line -1) 0)))
     (unless (pos-visible-in-window-p)
       ;; This ensures that some messages are displayed in Summary.
       (recenter))
-    (mew-summary-thread-move-cursor)
-    (mew-window-configure 'summary)
-    (mew-current-set nil nil nil)
+    (mew-thread-move-cursor)
+    (mew-summary-reset)
     (message "No more messages")
     nil)))
 
 (defun mew-summary-up ()
   (beginning-of-line)
   (cond 
-   ((re-search-backward mew-regex-msg-or-part nil t)
-    (mew-summary-thread-move-cursor)
+   ((mew-re-search-backward-visible mew-regex-msg-or-part)
+    (mew-thread-move-cursor)
     (mew-sinfo-set-direction 'up)
     t)
    (t 
-    (mew-summary-thread-move-cursor)
+    (mew-thread-move-cursor)
     (mew-decode-syntax-delete)
-    (mew-window-configure 'summary)
-    (mew-current-set nil nil nil)
+    (mew-summary-reset)
     (message "No more messages")
     nil)))
 
@@ -585,8 +679,7 @@ parts are skipped."
 ;;;
 
 (defun mew-summary-prev-page ()
-  "\\<mew-summary-mode-map>
-Back-scroll this message. Unnecessary header fields are hidden
+  "\\<mew-summary-mode-map>Back-scroll this message. Unnecessary header fields are hidden
 over the window. Type '\\[mew-summary-prev-page]' to see them when a message is displayed."
   (interactive)
   (mew-summary-scroll-down 'fullpage))
@@ -610,6 +703,7 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 	     (and part (equal part opart))) ;; MUST be equal
 	 (unwind-protect
 	     (progn
+	       (mew-summary-toggle-disp-msg 'on)
 	       (mew-window-configure 'message)
 	       ;; message buffer
 	       (mew-message-next-page 1))
@@ -630,12 +724,48 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 	     (and part (equal part opart))) ;; MUST be equal
 	 (unwind-protect
 	     (progn
+	       (mew-summary-toggle-disp-msg 'on)
 	       (mew-window-configure 'message)
 	       ;; message buffer
 	       (mew-message-prev-page (if fullpage nil 1)))
 	   (select-window win))
        (call-interactively 'mew-summary-display)))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Cursor motion
+;;;
+
+(defun mew-summary-next-line (&optional arg)
+  "Go to the next line."
+  (interactive "p")
+  (or arg (setq arg 1))
+  (if (< arg 0)
+      (mew-summary-previous-line (- arg))
+    (let ((col (current-column)))
+      (while (> arg 0)
+	(forward-line)
+	(while (or (looking-at mew-regex-thread-separator)
+		   (eq (get-text-property (point) 'invisible) t))
+	  (forward-line))
+	(setq arg (1- arg)))
+      (move-to-column col))))
+
+(defun mew-summary-previous-line (&optional arg)
+  "Go to the previous line."
+  (interactive "p")
+  (or arg (setq arg 1))
+  (if (< arg 0)
+      (mew-summary-next-line (- arg))
+    (let ((col (current-column)))
+      (while (> arg 0)
+	(forward-line -1)
+	(while (or (looking-at mew-regex-thread-separator)
+		   (eq (get-text-property (point) 'invisible) t))
+	  (forward-line -1))
+	(setq arg (1- arg)))
+      (move-to-column col))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Mouse
