@@ -25,7 +25,7 @@
 		      (specifier-instance default-toolbar-visible-p))
 	     (set-specifier default-toolbar-visible-p nil)
 	     (set-specifier default-toolbar-visible-p t)))
-       (save-buffer)) ;; to make sure not to use this draft again
+       (mew-draft-save-buffer)) ;; to make sure not to use this draft again
      (mew-set-file-modes (buffer-file-name))
      (mew-touch-folder mew-draft-folder)
      (message "Draft is prepared")))
@@ -147,36 +147,28 @@
     (list to cc newsgroups fromme)))
 
 (defun mew-in-reply-to-references ()
-  (let ((old-message-id  (mew-header-get-value mew-message-id:))
-	(old-in-reply-to (mew-header-get-value mew-in-reply-to:))
-	(old-references  (mew-header-get-value mew-references:))
-	(regex "<[^>]+>")
-	(start 0) tmp-ref skip in-reply-to references)
-    (if (and old-message-id (string-match regex old-message-id))
-	(setq old-message-id (match-string 0 old-message-id))
-      (setq old-message-id nil))
+  (let ((old-message-id  (mew-idstr-get-first-id
+			  (mew-header-get-value mew-message-id:)))
+	(old-in-reply-to (mew-idstr-get-first-id
+			  (mew-header-get-value mew-in-reply-to:)))
+	(old-references  (mew-idstr-to-id-list
+			  (mew-header-get-value mew-references:) 'rev))
+	id-list in-reply-to references)
     ;; Assuming that In-Reply-To: contains one ID.
-    (if (and old-in-reply-to (string-match regex old-in-reply-to))
-	(setq old-in-reply-to (match-string 0 old-in-reply-to))
-      (setq old-in-reply-to nil))
-    (if (null old-message-id)
-	() ;; we do not care even if old-references exist.
+    (when old-message-id
+      ;; when old-message-id is nil, 
+      ;; we do not care even if old-references exist.
       (setq in-reply-to old-message-id)
-      (if (null old-references)
-	  (setq tmp-ref (if old-in-reply-to 
-			    (list old-in-reply-to old-message-id)
-			  (list old-message-id)))
-	(while (string-match "<[^>]+>" old-references start)
-	  (setq start (match-end 0))
-	  (setq tmp-ref (cons (match-string 0 old-references) tmp-ref)))
-	;; described in old drums but not in RFC2822
-	(mew-addq tmp-ref old-in-reply-to)
-	(setq tmp-ref (nreverse (cons old-message-id tmp-ref))))
-      (if (integerp mew-references-max-count)
-	  (setq skip (- (length tmp-ref) mew-references-max-count)))
-      (if (and (numberp skip) (> skip 0))
-	  (setq tmp-ref (nthcdr skip tmp-ref)))
-      (setq references (mew-join "\n\t" tmp-ref)))
+      (cond
+       (old-references
+	(setq id-list old-references)
+	(mew-addq id-list old-in-reply-to)
+	(mew-addq id-list old-message-id)
+	(setq id-list (nreverse id-list)))
+       (t
+	(setq id-list (list old-message-id))
+	(mew-addq id-list old-in-reply-to)))
+      (setq references (mew-id-list-to-idstr id-list)))
     (list in-reply-to references)))
 
 (defun mew-summary-reply (&optional replysender)
@@ -184,13 +176,13 @@
 Values of To:, Cc:, and Newsgroups: are prepared accoring to
 three alists.
 
-(1) If called with '\\[universal-argument]', replying to the
+\(1) If called with '\\[universal-argument]', replying to the
     sender/poster only. In this case, 'mew-reply-sender-alist' is used.
 
-(2) If this message is sent by ME, you probably intend to reply with
+\(2) If this message is sent by ME, you probably intend to reply with
     the original header. In this case, 'mew-reply-fromme-alist' is used.
 
-(3) Otherwise, replying to all people listed. In this case, 
+\(3) Otherwise, replying to all people listed. In this case, 
     'mew-reply-all-alist' is used.
 
 The default value of 'mew-reply-sender-alist' is as follows:
@@ -299,7 +291,7 @@ You may want to set 'mew-reply-all-alist' to:
 	 ;; see also mew-draft-cite
 	 (set-buffer (or (save-excursion
 			   (set-buffer (mew-buffer-message))
-			   (if (mew-header-p) (current-buffer)))
+			   (if (mew-msghder-p) (current-buffer)))
 			 ;; header exists only in cache if multipart
 			 (mew-cache-hit fld msg)))
 	 (when mew-case-guess-when-replied
@@ -320,15 +312,11 @@ You may want to set 'mew-reply-all-alist' to:
 	   ;;
 	   ;; To:, Cc:, Newsgroups:
 	   (setq ret (mew-to-cc-newsgroups replysender))
-	   (setq to (nth 0 ret))
-	   (setq cc (nth 1 ret))
-	   (setq newsgroups (nth 2 ret))
-	   (setq fromme (nth 3 ret))
+	   (mew-set '(to cc newsgroups fromme) ret)
 	   ;;
 	   ;; In-Reply-To:, References:
 	   (setq ret (mew-in-reply-to-references))
-	   (setq in-reply-to (nth 0 ret))
-	   (setq references (nth 1 ret)))
+	   (mew-set '(in-reply-to references) ret))
 	 ;;
 	 (if (window-live-p cwin)
 	     (select-window cwin) ;; draft
@@ -405,7 +393,7 @@ Draft mode and this message is automatically attached."
        ;;
        (set-buffer (or (save-excursion
 			 (set-buffer (mew-buffer-message))
-			 (if (mew-header-p) (current-buffer)))
+			 (if (mew-msghder-p) (current-buffer)))
 		       ;; header exists only in cache if multipart
 		       (mew-cache-hit fld msg)))
        (setq subject (mew-header-get-value mew-subj:))
@@ -427,7 +415,7 @@ Draft mode and this message is automatically attached."
 ;;;
 
 (defun mew-summary-multi-forward ()
-  "Forward messages marked with '@' to a third person. A new draft 
+  "Forward messages marked with '*' to a third person. A new draft 
 is prepared in Draft mode and this message is automatically 
 attached."
   (interactive)
