@@ -20,30 +20,34 @@
 ;;; Switching to folder
 ;;;
 
-(defun mew-summary-switch-to-folder (folder &optional thread)
+(defun mew-summary-switch-to-folder (case:folder &optional p-folder func)
   (let ((ofolder (mew-summary-folder-name 'ext))
-	case-folder new-folder)
+	new-folder)
     (cond
-     ((get-buffer folder)
-      (switch-to-buffer folder)
+     ((get-buffer case:folder)
+      (switch-to-buffer case:folder)
       (mew-set-buffer-cs mew-cs-m17n)
-      (unless (mew-folder-virtualp folder) (mew-summary-folder-cache-load))
-      (unless (string= ofolder folder) (mew-window-configure 'summary)))
+      (when (and p-folder (mew-folder-virtualp case:folder))
+	(mew-vinfo-set-thread-p t)
+	(mew-vinfo-set-parent-folder p-folder))
+      (unless (mew-folder-virtualp case:folder) (mew-summary-folder-cache-load))
+      (unless (string= ofolder case:folder) (mew-window-configure 'summary)))
      (t
       (setq new-folder t)
-      (switch-to-buffer (get-buffer-create folder))
+      (unless func (setq func 'switch-to-buffer))
+      (funcall func (get-buffer-create case:folder))
       (mew-set-buffer-cs mew-cs-m17n)
-      (mew-buffers-setup folder)
-      (if (null (setq case-folder (mew-folder-case folder)))
-	  (mew-sinfo-set-folder folder)
-	(mew-sinfo-set-case (car case-folder))
-	(mew-sinfo-set-folder (cdr case-folder)))
-      (if (mew-folder-virtualp folder)
+      (mew-buffers-setup case:folder)
+      (mew-sinfo-set-case (mew-case:folder-case case:folder))
+      (mew-sinfo-set-folder (mew-case:folder-folder case:folder))
+      (if (mew-folder-virtualp case:folder)
 	  (progn
-	    (if thread (mew-vinfo-set-thread-p t))
+	    (when p-folder
+	      (mew-vinfo-set-thread-p t)
+	      (mew-vinfo-set-parent-folder p-folder))
 	    (mew-virtual-mode))
 	(mew-summary-mode)
-	(if mew-summary-trace-directory (cd (mew-expand-folder folder)))
+	(if mew-summary-trace-directory (cd (mew-expand-folder case:folder)))
 	(mew-summary-folder-cache-load))
       (mew-window-configure 'summary)))
     new-folder))
@@ -59,15 +63,25 @@
 Summary mode."
   (interactive "P")
   (let* ((proto (mew-proto-to-go (mew-summary-folder-name 'ext)))
-	 (case (if (mew-folder-remotep proto)
+	 (case1 (if (mew-folder-remotep proto)
 		   (mew-sinfo-get-case) ;; a remote folder
 		 mew-case-input)) ;; a local/virtual folder
-	 (inbox (mew-proto-inbox-folder	proto case))
-	 (folder (mew-input-folder case inbox))
-	 buf win frame goend)
-    (when folder
+	 (inbox (mew-proto-inbox-folder	proto case1))
+
+	 (case:folder (mew-input-folder case1 inbox))
+	 (case (mew-case:folder-case case:folder))
+	 (folder (mew-case:folder-folder case:folder))
+	 buf win frame goend folder-alist sep)
+    (when (mew-folder-imapp folder)
+      (setq folder-alist (mew-imap-folder-alist case))
+      (setq sep (mew-imap-separator case))
+      (unless (or (assoc folder folder-alist)
+		  (assoc (concat folder sep) folder-alist))
+	(message "%s does not exist" case:folder)
+	(setq case:folder nil)))
+    (when case:folder
       (when mew-use-other-frame-for-summary
-	(if (setq buf (get-buffer folder))
+	(if (setq buf (get-buffer case:folder))
 	    (if (setq win (get-buffer-window buf 0))
 		(progn
 		  (setq frame (window-frame win))
@@ -81,7 +95,7 @@ Summary mode."
 		       (selected-frame) (1- (frame-width)) 0))))
 	      (select-frame (make-frame)))
 	  (select-frame (make-frame))))
-      (mew-summary-visit-folder folder goend))))
+      (mew-summary-visit-folder case:folder goend))))
 
 (defun mew-summary-visit-folder (folder &optional goend no-ls)
   (let ((dir (mew-expand-folder folder))
@@ -92,14 +106,14 @@ Summary mode."
 	  (if (mew-virtual-thread-p folder)
 	      (if (mew-thread-cache-valid-p folder)
 		  (mew-summary-switch-to-folder folder)
-		(message "%s is old. " folder))
+		(message "%s is old" folder))
 	    (mew-summary-switch-to-folder folder))
         (message "No such virtual folder: %s" folder)))
      (t ;; local/remote folders
       (if (null dir)
 	  (message "Folder is wrong")
 	(cond
-	 ((mew-folder-remotep (mew-folder-folder folder))
+	 ((mew-folder-remotep (mew-case:folder-folder folder))
 	  (unless (file-directory-p dir)
 	    (mew-make-directory dir))
 	  (setq scanp t))
@@ -110,36 +124,30 @@ Summary mode."
 	(when scanp
 	  (if no-ls
 	      (progn
-		(mew-summary-switch-to-folder folder) ;; Virtual mode
-		(if goend (goto-char (point-max))))
+		(mew-summary-switch-to-folder folder)
+		(if goend (goto-char (point-max)))
+		(if (pos-visible-in-window-p (point-min))
+		    (mew-summary-cook-window)))
 	    (setq new-folder (mew-summary-switch-to-folder folder))
-	    (mew-summary-ls nil (or goend new-folder)))))))))
+	    (if (and (mew-summary-ls nil (or goend new-folder))
+		     (pos-visible-in-window-p (point-min)))
+		(mew-summary-cook-window)))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Moving to message
+;;; Going to a line
 ;;;
 
-(defun mew-summary-jump-message (&optional msg)
-  "Jump to a message according to the number which you input.
-If 'mew-summary-jump-message-then-display' is non-nil,
+(defun mew-summary-goto-line ()
+  "Jump to a line according to the number which you input.
+If 'mew-summary-goto-line-then-display' is non-nil,
 the message is then displayed."
-  (interactive "P")
-  (if (integerp msg) (setq msg (int-to-string msg))
-    (if (not (stringp msg)) (setq msg nil)))
-  (let ((here (point)))
-    (unless msg (setq msg (read-string "Message No.: " "")))
-    (cond 
-     ((string= msg "") ())
-     ((eq msg t) ;; xxx
-      (goto-char (point-max))) ;; (forward-line -1)
-     (t 
-      (goto-char (point-min))
-      (if (not (re-search-forward (mew-regex-jmp-msg msg) nil t))
-	  (goto-char here)
-	(beginning-of-line)
-	(if mew-summary-jump-message-then-display
-	    (mew-summary-display nil)))))))
+  (interactive)
+  (let ((line (string-to-int (read-string "Line No.: " ""))))
+    (when (/= line 0)
+      (goto-line line)
+      (if mew-summary-goto-line-then-display
+	  (mew-summary-display nil)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -147,6 +155,7 @@ the message is then displayed."
 ;;;
 
 (defun mew-summary-goto-msg-mode ()
+  "Go to Header mode."
   (interactive)
   (let (msgp)
     (save-excursion
@@ -193,6 +202,7 @@ the message is then displayed."
 If 'mew-summary-jump-top-then-display' is non-nil, 
 the top message is then displayed."
   (interactive)
+  (if mew-use-push-mark (mew-push-mark))
   (goto-char (point-min))
   (if mew-summary-jump-top-then-display
       (mew-summary-display nil)))
@@ -202,6 +212,7 @@ the top message is then displayed."
 If 'mew-summary-jump-bottom-then-display' is non-nil, 
 the top message is then displayed."
   (interactive)
+  (if mew-use-push-mark (mew-push-mark))
   (goto-char (point-max))
   (unless (bobp) (forward-line -1))
   (if mew-summary-jump-bottom-then-display
@@ -235,7 +246,7 @@ the top message is then displayed."
 	 (if (mew-virtual-thread-p folder)
 	     (if (mew-thread-cache-valid-p folder)
 		 (setq getback t)
-	       (message "%s is old. " folder))
+	       (message "%s is old" folder))
 	   (setq getback t)))
 	(t
 	 (setq getback t)))
@@ -286,12 +297,13 @@ the top message is then displayed."
 (defun mew-summary-pipe-message (prefix command)
   "Send the content of Message buffer to a command via pipe.
 If called with '\\[universal-argument]', the body of the message
-(excluding its header) is sent."
+\(excluding its header) is sent."
   (interactive
    (list current-prefix-arg 
 	 (read-string "Shell command on message: " mew-last-shell-command)))
   (mew-summary-display 'redisplay)
-  (when (y-or-n-p "Send this message to pipe? ")
+  (when (or (not mew-ask-pipe)
+            (y-or-n-p "Send this message to pipe? "))
     (save-excursion
       (set-buffer (mew-buffer-message))
       (save-restriction
@@ -322,7 +334,7 @@ If called with '\\[universal-argument]', you can specify a printer name."
 	(set-buffer (mew-buffer-message))
 	(save-restriction
 	  (widen)
-	  (setq have-header (mew-header-p))
+	  (setq have-header (mew-msghder-p))
 	  (setq str (mew-buffer-substring (point-min) (point-max)))))
       (with-temp-buffer
 	(insert str)
@@ -369,21 +381,21 @@ argument."
 	  (command (read-string "Command: ")))
      (while (not (mew-which-exec command))
        (setq command (read-string "Command: ")))
-     (message (format "Executing %s for %s..." command msg))
+     (message "Executing %s for %s..." command msg)
      (call-process command nil nil nil file)
-     (message (format "Executing %s for %s...done" command msg)))))
+     (message "Executing %s for %s...done" command msg))))
 
 (defun mew-summary-cmd-msgs ()
   "Executing an external command specifying messages
-marked with '@' as arguments."
+marked with '*' as arguments."
   (interactive)
   (mew-summary-multi-msgs
    (let ((command (read-string "Command: ")))
      (while (not (mew-which-exec command))
        (setq command (read-string "Command: ")))
-     (message (format "Executing %s ..." command))
+     (message "Executing %s ..." command)
      (apply 'call-process command nil nil nil FILES)
-     (message (format "Executing %s ...done" command)))))
+     (message "Executing %s ...done" command))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -403,6 +415,76 @@ marked with '@' as arguments."
 	    (mew-draft-cite arg))
 	(message "No such draft buffer!")))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Executing internal commands
+;;;
+
+(defun mew-summary-ask-ct (ct fname)
+  (let (pt fl)
+    (cond
+     (fname
+      (setq ct (or (mew-ctdb-ct (mew-ctdb-by-file fname))
+		   (mew-content-type (mew-sinfo-get-case))))
+      (setq pt "Type for %s (%s): ")
+      (setq fl fname))
+     (t
+      (setq pt "Type %s(%s): ")
+      (setq fl "")))
+    (mew-input-type pt fl ct mew-mime-content-type-list)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Executing internal commands
+;;;
+
+(defun mew-summary-execute-internal ()
+  "Visualize MIME parts in Message mode.
+This is useful, for example, an image file attached but
+whose Content-Type: is Application/Octet-Stream. With this
+command, you can specify its Content-Type: correctly and
+visualize it in Message mode."
+  (interactive)
+  (let* ((fld (mew-summary-folder-name))
+	 (msg (mew-summary-message-number2))
+	 (nums (mew-syntax-nums))
+	 (cache (mew-cache-hit fld msg 'must-hit))
+	 (syntax (mew-cache-decode-syntax cache))
+	 (stx (mew-syntax-get-entry syntax nums))
+	 (ctl (mew-syntax-get-ct stx))
+	 (ct (mew-syntax-get-value ctl 'cap))
+	 (mew-inherit-ct ct)
+	 (cte (mew-syntax-get-cte stx))
+	 (begin (mew-syntax-get-begin stx))
+	 (end (mew-syntax-get-end stx))
+	 (params (mew-syntax-get-params ctl))
+	 (cdpl (mew-syntax-get-cdp stx))
+	 (fname (mew-syntax-get-filename cdpl ctl))
+	 (win (selected-window))
+	 program)
+    (setq ct (mew-summary-ask-ct ct fname))
+    (setq program (mew-ctdb-prog (mew-ctdb-by-ct ct)))
+    (mew-summary-toggle-disp-msg 'on)
+    (mew-window-configure 'message)
+    ;; message buffer
+    (unwind-protect
+	(mew-elet
+	  (mew-erase-buffer)
+	  (cond
+	   ((and (listp program) (nth 0 program)
+		 (symbolp (nth 0 program)) (fboundp (nth 0 program)))
+	    (if (or (mew-ct-imagep ct) (mew-ct-modelp ct))
+		(funcall (nth 0 program) cache begin end params fname ct cte)
+	      (funcall (nth 0 program) cache begin end params)))
+	   ;; This is not possible since the part may be analyzed as
+	   ;; a single part, not a message.
+	   ;; ((and (symbolp program) (eq program 'mew-mime-message/rfc822))
+	   ;; (funcall program cache stx))
+	   (t
+	    (message "Not supported yet.")))
+	  (mew-summary-display-postscript 'no-hook))
+      (select-window win))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Executing external commands
@@ -424,7 +506,7 @@ If called with '\\[universal-argument]', options are asked."
     nil
     (read-string "Command: ")
     (if arg (delete "" (mew-split-quoted
-			(read-string "Options: ") ?\  ?\" ?\"))))))
+			(read-string "Options: ") mew-sp ?\" ?\"))))))
 
 (defun mew-summary-execute-base (ask-type com args)
   (let* ((fld (mew-summary-folder-name))
@@ -435,92 +517,87 @@ If called with '\\[universal-argument]', options are asked."
 	 (stx (mew-syntax-get-entry syntax nums))
 	 (ctl (mew-syntax-get-ct stx))
 	 (ct (mew-syntax-get-value ctl 'cap))
-	 begin end params program fname cdpl options async doit pt fl
-	 mew-inherit-ct)
-    (if (not (string= ct mew-ct-msg))
-	(setq doit t)
-      ;; Message/Rfc822
-      (setq stx (mew-syntax-get-part stx))
-      (if (mew-syntax-multipart-p stx)
-	  (progn
-	    (setq stx (mew-syntax-get-entry stx '(1)))
-	    (setq ctl (mew-syntax-get-ct stx))
-	    (setq ct (mew-syntax-get-value ctl 'cap))
-	    (if (mew-ct-textp ct)
-		(setq doit t)))
-	;; singlepart
-	(setq ctl (mew-syntax-get-ct stx))
-	(setq ct (mew-syntax-get-value ctl 'cap))
-	(setq doit t)))
-    (if (not doit)
-	(message "No body")
-      (setq begin   (mew-syntax-get-begin stx))
-      (setq end     (mew-syntax-get-end stx))
-      (setq params  (mew-syntax-get-params ctl))
-      (setq cdpl    (mew-syntax-get-cdp stx))
-      (setq fname   (mew-syntax-get-filename cdpl ctl))
-      (when ask-type
-	(setq mew-inherit-ct ct)
+	 (mew-inherit-ct ct)
+	 (begin (mew-syntax-get-begin stx))
+	 (end (mew-syntax-get-end stx))
+	 (params (mew-syntax-get-params ctl))
+	 (cdpl (mew-syntax-get-cdp stx))
+	 (fname (mew-syntax-get-filename cdpl ctl))
+	 (win (selected-window))
+	 program)
+    (when ask-type
+      (setq ct (mew-summary-ask-ct ct fname)))
+    (setq program (or com (mew-ctdb-prog (mew-ctdb-by-ct ct))))
+    (mew-summary-toggle-disp-msg 'on)
+    (mew-window-configure 'message)
+    ;; message buffer
+    (unwind-protect
+	(mew-elet
+	  (mew-summary-execute-base1 program ct ctl cache begin end params fname args)
+	  (mew-summary-display-postscript 'no-hook))
+      (select-window win))))
+
+(defun mew-summary-execute-base1 (program ct ctl cache begin end params fname args)
+  (let (options async)
+    (if (and (listp program) (nth 1 program) (symbolp (nth 1 program)))
 	(cond
-	 (fname
-	  (setq ct (or (mew-ctdb-ct (mew-ctdb-by-file fname))
-		       (mew-content-type (mew-sinfo-get-case))))
-	  (setq pt "Type for %s (%s): ")
-	  (setq fl fname))
+	 ((not (fboundp (nth 1 program)))
+	  (mew-message-for-summary "Use '\\[mew-summary-execute-command]' or \\[universal-argument] \\[mew-summary-execute-external]'"))
+	 ((or (mew-ct-imagep ct) (mew-ct-modelp ct))
+	  (funcall (nth 1 program) cache begin end params fname))
 	 (t
-	  (setq pt "Type %s(%s): ")
-	  (setq fl "")))
-	(setq ct (mew-input-type pt fl ct mew-mime-content-type-list)))
-      (setq program (or com (mew-ctdb-prog (mew-ctdb-by-ct ct))))
+	  (funcall (nth 1 program) cache begin end params)))
+      (if (listp program)
+	  (mew-set '(program options async) program)
+	(setq options args)
+	(setq async t))
       (cond
-       ((and (listp program) (nth 1 program) (symbolp (nth 1 program)))
-	(if (fboundp (nth 1 program))
-	    (save-excursion
-	      (if (or (mew-ct-imagep ct) (mew-ct-modelp ct))
-		  (funcall (nth 1 program) cache begin end params fname)
-		(funcall (nth 1 program) cache begin end params)))
-	  (message (substitute-command-keys "Use '\\<mew-summary-mode-map>\\[mew-summary-execute-command]' or \\[universal-argument] \\<mew-summary-mode-map>\\[mew-summary-execute-external]'"))))
+       ((symbolp program)
+	(message "Function %s is not executable here" program))
+       ((not (mew-which-exec program))
+	(message "Program %s is not found" program))
+       (t
+	(mew-summary-execute-base2 program ct ctl cache begin end params fname options async))))))
+
+(defun mew-summary-execute-base2 (program ct ctl cache begin end params fname options async)
+  (let ((file (mew-make-temp-name fname))
+	wcs)
+    (save-excursion
+      (set-buffer cache)
+      ;; NEVER use call-process-region for privacy reasons
+      (cond
+       ((not (mew-ct-linebasep ct))
+	(setq wcs mew-cs-binary))
+       ((not (mew-ct-textp ct))
+	(setq wcs mew-cs-text-for-write))
        (t
 	(cond
-	 ((listp program)
-	  (setq options (nth 1 program))
-	  (setq async (nth 2 program))
-	  (setq program (nth 0 program)))
+	 ((string= mew-ct-htm ct)
+	  (setq wcs (mew-text/html-detect-cs begin end ctl))
+	  (unless (mew-coding-system-p wcs)
+	    (setq wcs (mew-charset-to-cs
+		       (mew-syntax-get-param ctl "charset")))))
+	 ((string= mew-ct-xml ct)
+	  (setq wcs (mew-text/xml-detect-cs begin end ctl))
+	  (unless (mew-coding-system-p wcs)
+	    (setq wcs (mew-charset-to-cs
+		       (mew-syntax-get-param ctl "charset")))))
+	 
 	 (t
-	  (setq options args)
-	  (setq async t)))
-	(if (symbolp program)
-	    (message "Function %s is not executable here" program)
-	  (if (not (mew-which-exec program))
-	      (message "Program %s is not found" program)
-	    (let ((file (mew-make-temp-name fname))
-		  rcs)
-	      (save-excursion
-		(set-buffer cache)
-		;; NEVER use call-process-region for privacy reasons
-		(if (mew-ct-linebasep ct)
-		    (if (mew-ct-textp ct)
-			(cond
-			 ((string= mew-ct-htm ct)
-			  (setq rcs (mew-text/html-detect-cs begin end ctl)))
-			 ((string= mew-ct-xml ct)
-			  (setq rcs (mew-text/xml-detect-cs begin end ctl)))
-			 (t
-			  (setq rcs (or (mew-charset-to-cs
-					 (mew-syntax-get-param ctl "charset"))
-					(if mew-decode-broken
-					    (mew-charset-to-cs
-					     (mew-charset-guess-region
-					      begin end))
-					  mew-cs-text-for-write)))))
-		      (setq rcs mew-cs-text-for-write)) ;; xxx
-		  (setq rcs mew-cs-binary))
-		(mew-frwlet
-		 mew-cs-dummy rcs
-		 (write-region begin end file nil 'no-msg))
-		(if async
-		    (mew-mime-start-process program options file)
-		  (mew-mime-call-process program options file)))))))))))
+	  (setq wcs (mew-charset-to-cs
+		     (mew-syntax-get-param ctl "charset")))))
+	(unless (mew-coding-system-p wcs)
+	  (setq wcs (if mew-decode-broken
+			(mew-charset-to-cs
+			 (mew-charset-guess-region
+			  begin end))
+		      mew-cs-text-for-write)))))
+      (mew-frwlet
+       mew-cs-dummy wcs
+       (write-region begin end file nil 'no-msg))
+      (if async
+	  (mew-mime-start-process program options file)
+	(mew-mime-call-process program options file)))))
   
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -563,46 +640,45 @@ If executed with '\\[universal-argument]', you can set the sending case."
 ;;; Unlocking
 ;;;
 
-(defun mew-summary-kill-subprocess (&optional kill-ssh)
+(defun mew-summary-kill-subprocess (&optional kill-ssx)
   "\\<mew-summary-mode-map>
 Kill a process in Summary mode.
 Sometime a process accidentally remains in Summary mode. 
 In this situation, you cannot execute '\\[mew-summary-retrieve]', '\\[mew-summary-ls]', nor '\\[mew-summary-exec]'.
 Use this command to solve this problem.
 
-If called with '\\[universal-argument]', corresponding SSH process, if
-any, is also killed. Please note that the SSH process is shared by 
-other network processes. In this situation, if you kill the SSH process,
-the other network processes are also hung up."
+If called with '\\[universal-argument]', all SSH processes, if
+any, are killed."
   (interactive "P")
-  (unwind-protect
-      (let ((process mew-summary-buffer-process)
-	    (disp-msg (interactive-p))
-	    pnm ssh-process)
-	(if (not (processp process))
-	    (if disp-msg (message "No process to kill. This buffer is unlocked anyway"))
-	  (cond
-	   ((eq process t)
-	    ;; Emacs is locking
-	    )
-	   ((memq (process-status process) '(open closed))
-	    ;; Network process
-	    (setq pnm (process-name process))
-	    (setq ssh-process (mew-net-get-ssh-process pnm))
-	    (mew-info-clean-up pnm)
-	    (delete-process process)
-	    (mew-summary-visible-buffer (current-buffer))
-	    (when (mew-sinfo-get-refile-back)
-	      (mew-sinfo-set-refile
-	       (nconc (mew-sinfo-get-refile) (mew-sinfo-get-refile-back)))
-	      (mew-sinfo-set-refile-back nil))
-	    (if (and kill-ssh ssh-process) (kill-process ssh-process))
-	    (mew-summary-unlock))
-	   (t
-	    ;; Local process or SSL
-	    (kill-process process)))
-	  (if disp-msg (message "The process was killed"))))
-    (mew-summary-unlock)))
+  (let ((process mew-summary-buffer-process)
+	(disp-msg (interactive-p))
+	pros sentinel)
+    (if kill-ssx
+	(progn
+	  (setq pros (process-list))
+	  (while pros
+	    (if (string-match "mew" (process-name (car pros)))
+		(delete-process (car pros)))
+	    (setq pros (cdr pros)))
+	  (if disp-msg (message "All SSH/SSL processes were killed")))
+      (cond
+       ((not (processp process))
+	(if disp-msg (message "No process to kill. This buffer is unlocked anyway")))
+       ((eq process t)
+	;; Emacs is locking
+	(if disp-msg (message "This buffer was unlocked")))
+       ((memq (process-status process) '(open closed))
+	;; Network process
+	(setq sentinel (process-sentinel process))
+	(if (fboundp sentinel)
+	    (funcall sentinel process "killed by the user"))
+	(delete-process process)
+	(if disp-msg (message "The process was killed")))
+       (t
+	;; Local process or SSH/SSL
+	(delete-process process)
+	(if disp-msg (message "The process was killed"))))
+      (mew-summary-unlock))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -610,7 +686,7 @@ the other network processes are also hung up."
 ;;;
 
 (defun mew-summary-unshar ()
-  "Apply 'unshar' on messages marked with '@'."
+  "Apply 'unshar' on messages marked with '*'."
   (interactive)
   (mew-summary-multi-msgs
    (if (y-or-n-p (format "Execute %s for these messages? " mew-prog-unshar))
@@ -620,7 +696,7 @@ the other network processes are also hung up."
 	 (message "Executing %s...done" mew-prog-unshar)))))
 
 (defun mew-summary-uudecode ()
-  "Apply 'uudecode' on messages marked with '@'."
+  "Apply 'uudecode' on messages marked with '*'."
   (interactive)
   (mew-summary-multi-msgs
    (cond
@@ -639,7 +715,7 @@ the other network processes are also hung up."
 	  mew-cs-text-for-read mew-cs-text-for-write
 	  (while files
 	    (mew-erase-buffer)
-	    (insert-file-contents (car files))
+	    (mew-insert-file-contents (car files))
 	    (goto-char (point-min))
 	    (if (re-search-forward mew-eoh nil t)
 		(forward-line))
@@ -684,10 +760,11 @@ the other network processes are also hung up."
 ;;;
 
 (defun mew-summary-join ()
-  "Concat Message/Partial fragments marked with '@' to an original
-message."   (interactive)
+  "Concat Message/Partial fragments marked with '*' to an original
+message."
+  (interactive)
   (mew-summary-only
-   (mew-summary-with-mewls
+   (mew-summary-with-mewl
     (mew-summary-multi-msgs
      (let ((cfld (mew-summary-folder-name))
 	   (folder (mew-input-burst-folder))
@@ -707,7 +784,7 @@ message."   (interactive)
 	  mew-cs-text-for-read mew-cs-text-for-write
 	  (write-region (point-min) (point-max) tfile nil 'no-msg))
 	 (mew-erase-buffer)
-	 (call-process mew-prog-mewls nil t nil
+	 (call-process mew-prog-mewl nil t nil
 		       "-b" mew-mail-path "-l" "0"
 		       "-i" tfile "-d" "content-type")
 	 (mew-delete-file tfile)
@@ -743,7 +820,7 @@ message."   (interactive)
 	   (setq idx (1- (string-to-int idx)))
 	   (if (and (>= idx 0) (< idx len))
 	       (aset vec idx num)
-	     (error "illegal number")))
+	     (error "invalid number")))
 	 (unless TTL (error "no total"))
 	 (setq i 0)
 	 (while (< i len)
@@ -801,32 +878,48 @@ message."   (interactive)
 			nil 'no-msg)))
        (mew-touch-folder folder)
        (message "Joining...done")
-       (if (y-or-n-p (format "Go to %s? " folder))
-	   (mew-summary-visit-folder folder 'goend)))))))
+       (when mew-ask-folder-after-join
+	 (if (y-or-n-p (format "Go to %s? " folder))
+	     (mew-summary-visit-folder folder 'goend))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Cleaning +trash
 ;;;
 
-(defun mew-summary-clean-trash (&optional arg)
-  "Remove all messages in +trash.
-When called with '\\[universal-argument]', you can specify a target folder
-to clean up."
-  (interactive "P")
-  (let* ((folder mew-trash-folder)
-	 trashdir msgs)
-    (if arg (setq folder (mew-input-folder nil mew-trash-folder)))
-    (when folder
-      (setq trashdir (mew-expand-folder folder))
+(defun mew-summary-clean-trash ()
+  "Remove all messages in +trash."
+  (interactive)
+  (let* ((case (mew-sinfo-get-case))
+	 (folder (mew-sinfo-get-folder))
+	 case:trash trash trashdir msgs)
+    (cond
+     ((mew-folder-imapp folder)
+      (setq trash (mew-imap-trash-folder case)))
+     (t
+      (setq trash mew-trash-folder)))
+    (unless trash
+      (setq trash folder))
+    (setq case:trash (mew-input-folder case trash))
+    (if (null case:trash)
+	(message "Nothing was processed")
+      (setq case (mew-case:folder-case case:trash))
+      (setq trash (mew-case:folder-folder case:trash))
+      (setq trashdir (mew-expand-folder trash))
       (setq msgs (mew-dir-messages trashdir))
-      (if (null msgs)
-	  (message "No messages to be removed in %s" folder)
-	(when (y-or-n-p (format "Remove all messages in %s? " folder))
-	  (message "Removing all messages in %s..." folder)
-	  (mew-summary-unlink-msgs folder msgs)
-	  (mew-summary-folder-cache-clean folder)
-	  (message "Removing all messages in %s...done" folder))))))
+      (if (and (mew-folder-localp trash) (null msgs))
+	  (message "No messages to be removed in %s" trash)
+	(if (not (y-or-n-p (format "Remove all messages in %s? " case:trash)))
+	    (message "No messages removed in %s" trash)
+	  (message "Removing all messages in %s..." case:trash)
+	  (mew-summary-unlink-msgs case:trash msgs)
+	  (mew-summary-folder-cache-clean case:trash)
+	  (message "Removing all messages in %s...done" case:trash)
+	  (when (mew-folder-imapp trash)
+	    (if (get-buffer case:trash)
+		(set-buffer case:trash)
+	      (mew-summary-switch-to-folder case:trash nil 'set-buffer))
+	    (mew-imap-retrieve case 'exec case:trash '(("1:*") nil nil nil 0 -1))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -870,7 +963,8 @@ folder."
 	 (fld (mew-summary-folder-name))
 	 (exclusivep t)
 	 msgs file dstmsg dstfld dstfile
-	 beg end region)
+	 beg end region total i)
+     (if (mew-mark-active-p) (setq arg t))
      (cond
       (arg
        (setq region (mew-summary-get-region))
@@ -894,12 +988,16 @@ folder."
 	 (message "Try again later"))
 	(t
 	 (message "Copying...")
+	 (setq total (length msgs))
+	 (setq i 1)
 	 (while msgs
 	   (setq file (mew-expand-folder fld (car msgs)))
 	   (setq dstfile (mew-expand-folder dstfld dstmsg))
+	   (message "Copying (%d/%d)..." i total)
 	   (mew-summary-local-copy-one file dstfile)
 	   (setq dstmsg (number-to-string (1+ (string-to-number dstmsg))))
-	   (setq msgs (cdr msgs)))
+	   (setq msgs (cdr msgs))
+	   (setq i (1+ i)))
 	 (mew-touch-folder dstfld)
 	 (message "Copying...done")))))))
 
@@ -912,6 +1010,100 @@ folder."
      (mew-header-delete-lines (list mew-x-mew-uidl:))
      (write-region (point-min) (point-max) dstfile nil 'no-msg))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Deleting and moving a folder
+;;;
+
+(defun mew-summary-delete-folder ()
+  "Delete this folder."
+  (interactive)
+  (mew-summary-only
+   (let* ((folder (or (mew-sinfo-get-folder)
+		      (mew-summary-folder-name 'ext)))
+	  (case (mew-sinfo-get-case))
+	  (case:folder (mew-summary-folder-name 'ext))
+	  (dir (mew-expand-folder case:folder))
+	  (buf (current-buffer)))
+     (cond
+      ((mew-folder-localp folder)
+       (if (mew-folder-node-p folder)
+	   (message "%s cannot be deleted" folder)
+	 (when (y-or-n-p "Delete this folder? ")
+	   (mew-delete-directory-recursively dir)
+	   (mew-local-folder-delete folder)
+	   (mew-summary-visit-folder mew-inbox-folder)
+	   (mew-kill-buffer buf))))
+      ((mew-folder-imapp folder)
+       (if (mew-folder-node-p folder case)
+	   (message "%s cannot be deleted" folder)
+	 (when (y-or-n-p "Delete this folder? ")
+	   (setq mew-summary-buffer-process-error nil)
+	   (mew-imap-retrieve case 'delete case:folder)
+	   (mew-timing)
+	   (mew-rendezvous mew-summary-buffer-process)
+	   (if mew-summary-buffer-process-error
+	       (progn
+		 (message "%s cannot be deleted" folder)
+		 (setq mew-summary-buffer-process-error nil))
+	     (mew-imap-folder-delete case folder)
+	     (mew-delete-directory-recursively dir) ;; cache
+	     (mew-summary-visit-folder mew-imap-inbox-folder)
+	     (mew-kill-buffer buf)))))))))
+
+(defun mew-summary-rename-folder ()
+  "Rename this folder."
+  (interactive)
+  (mew-summary-only
+   (let* ((folder (or (mew-sinfo-get-folder)
+		      (mew-summary-folder-name 'ext)))
+	  (case (mew-sinfo-get-case))
+	  (case:folder (mew-summary-folder-name 'ext))
+	  (dir (mew-expand-folder case:folder))
+	  (buf (current-buffer))
+	  new-folder new-dir case:new-folder)
+     (cond
+      ((mew-folder-localp folder)
+       (setq new-folder (mew-input-local-folder folder))
+       (cond
+	((string= folder new-folder)
+	 (message "%s was not moved" folder))
+	(t
+	 (setq new-dir (mew-expand-folder new-folder))
+	 ;; xxx if new-dir exists?
+	 (rename-file dir new-dir)
+	 (mew-summary-switch-to-folder new-folder)
+	 (mew-kill-buffer buf)
+	 (if (mew-folder-node-p folder)
+	     ;; xxx remove all related buffer?
+	     (mew-local-update 'force)
+	   (mew-local-folder-delete folder)
+	   (mew-local-folder-insert new-folder)))))
+      ((mew-folder-imapp folder)
+       (setq new-folder (car (mew-input-refile-folders (list folder) t case mew-folder-imap)))
+       (setq case:new-folder (mew-case-folder case new-folder))
+       (cond
+	((string= folder new-folder)
+	 (message "%s was not moved" folder))
+	(t
+	 (setq new-dir (mew-expand-folder new-folder))
+	 (mew-imap-retrieve case 'rename case:folder case:new-folder)
+	 (mew-timing)
+	 (mew-rendezvous mew-summary-buffer-process)
+	 (if mew-summary-buffer-process-error
+	     (progn
+	       (message "%s cannot be renamed" folder)
+	       (setq mew-summary-buffer-process-error nil))
+	   (rename-file dir new-dir t) ;; cache
+	   (mew-summary-switch-to-folder case:new-folder)
+	   (mew-kill-buffer buf)
+	   (if (mew-folder-node-p folder case)
+	       (if (y-or-n-p "Collect a folder list now? ")
+		   (mew-imap-update case)
+		 (mew-message-for-summary "Type '3\\[mew-status-update]' to collect a folder list."))
+	     (mew-imap-folder-delete case folder)
+	     (mew-imap-folder-insert case new-folder))))))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Toggle
@@ -938,11 +1130,8 @@ you choose 'Summary mode only', you can quickly put the delete
     (if (mew-sinfo-get-disp-msg)
 	(mew-summary-display 'redisplay)
       (mew-summary-goto-message)
-      (mew-decode-syntax-delete)
-      (mew-window-configure 'summary)
-      (mew-current-set nil nil nil)
-      (mew-summary-cook-window)
-      (mew-summary-reset-mode-line))))
+      (mew-summary-reset)
+      (mew-summary-cook-window))))
   (run-hooks 'mew-summary-toggle-disp-msg-hook))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1016,6 +1205,72 @@ If 'mew-warning-field-level' is 1, set it to 2."
       (message "'mew-debug' is now 't'")
     (message "'mew-debug' is now 'nil'")))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Toggle PGP/MIME and old PGP
+;;;
+
+(defun mew-summary-toggle-pgp ()
+  "Toggle PGP/MIME and old PGP (i.e. 'mew-use-old-pgp')."
+  (interactive)
+  (setq mew-use-old-pgp (not mew-use-old-pgp))
+  (if mew-use-old-pgp
+      (message "mew-use-old-pgp has been set to 't'")
+    (message "mew-use-old-pgp has been set to 'nil'")))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Learning spam and ham
+;;;
+
+(defun mew-summary-learn-spam ()
+  "Learn that this message is a spam.
+See also 'mew-spam-prog' and 'mew-spam-prog-args."
+  (interactive)
+  (mew-summary-msg
+   (let* ((fld (mew-summary-folder-name))
+	  (msg (mew-summary-message-number))
+	  (file (mew-expand-folder fld msg))
+	  (case (mew-sinfo-get-case))
+	  (prog (mew-spam-prog case))
+	  (args (mew-spam-prog-args case))
+	  beg end)
+     (message "Learning as spam...")
+     (when (mew-which-exec prog)
+       (with-temp-buffer
+	 (apply 'call-process prog file (current-buffer) nil args)
+	 (setq beg (point-min))
+	 (goto-char beg)
+	 (forward-line)
+	 (if (= (point) beg)
+	     (message "Learning as spam...done")
+	   (setq end (1- (point)))
+	   (message (concat "Learned as spam: " (mew-buffer-substring beg end)))))))))
+
+(defun mew-summary-learn-ham ()
+  "Learn that this message is a ham.
+See also 'mew-ham-prog' and 'mew-ham-prog-args."
+  (interactive)
+  (mew-summary-msg
+   (let* ((fld (mew-summary-folder-name))
+	  (msg (mew-summary-message-number))
+	  (file (mew-expand-folder fld msg))
+	  (case (mew-sinfo-get-case))
+	  (prog (mew-ham-prog case))
+	  (args (mew-ham-prog-args case))
+	  beg end)
+     (message "Learning as ham...")
+     (when (mew-which-exec prog)
+       (with-temp-buffer
+	 (apply 'call-process prog file (current-buffer) nil args)
+	 (setq beg (point-min))
+	 (goto-char beg)
+	 (forward-line)
+	 (if (= (point) beg)
+	     (message "Learning as ham...done")
+	   (setq end (1- (point)))
+	   (message (concat "Learned as ham: " (mew-buffer-substring beg end)))))))))
+
 (provide 'mew-summary4)
 
 ;;; Copyright Notice:
