@@ -42,6 +42,8 @@
 (defvar mew-prog-mspowerpoint    '(mew-mime-application/mspowerpoint
 				   mew-mime-application/mspowerpoint-ext))
 (defvar mew-prog-visio           'mew-mime-application/octet-stream)
+(defvar mew-prog-mstnef          '(mew-mime-application-ms-tnef
+				   mew-mime-application-ms-tnef-ext))
 
 ;;;
 ;;; Text/Html, Application/Xml, Image
