@@ -18,25 +18,23 @@
 ;;; Marks
 ;;;
 
-(defvar mew-mark-multi  ?@)
 (defvar mew-mark-review ?*)
+(defvar mew-mark-escape ?$)
 (defvar mew-mark-delete ?D)
 (defvar mew-mark-unlink ?X)
 (defvar mew-mark-refile ?o)
+(defvar mew-mark-read   mew-sp)
+(defvar mew-mark-unread ?U)
 (defvar mew-mark-tmp    ?\0) ;; temporary use only.
-(defvar mew-mark-clean
-  `(,mew-mark-delete ,mew-mark-unlink ,mew-mark-refile))
-(defvar mew-mark-all
-  `(,mew-mark-multi ,mew-mark-review ,mew-mark-delete ,mew-mark-unlink ,mew-mark-refile))
 (defvar mew-mark-default-walk mew-mark-review)
 (defvar mew-mark-walk         mew-mark-default-walk)
 (defvar mew-mark-duplicated   mew-mark-delete)
-(defvar mew-mark-show-list (list mew-mark-review))
+(defvar mew-mark-show-list (list mew-mark-read mew-mark-unread mew-mark-review))
+(defvar mew-mark-show-list2 (list mew-mark-unread mew-mark-review))
 
 (defvar mew-summary-mark-undo-marks
-  `(,mew-mark-delete ,mew-mark-unlink ,mew-mark-refile)
-  "A list of marks to be canceled by \\<mew-summary-mode-map>\\[mew-summary-mark
--undo-all].")
+  `(,mew-mark-delete ,mew-mark-unlink ,mew-mark-refile ,mew-mark-read)
+  "*A list of marks to be canceled by '\\<mew-summary-mode-map>\\[mew-summary-mark-undo-all]'.")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -78,7 +76,7 @@
    (cons mew-was-regex nil)) ;; regexp
   "*Replacement alist to simplify Subject: field body
 Each element is cons cell whos car is REGEXP to replace,
-cdr is new text. ")
+cdr is new text.")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -87,8 +85,8 @@ cdr is new text. ")
 
 (defvar mew-error-unknown-charset    "**UNKNOWN CHARSET**")
 (defvar mew-error-broken-string      "**BROKEN STRING**")
-(defvar mew-error-illegal-b-encoding " **B ENCODING ERROR** ")
-(defvar mew-error-illegal-q-encoding " **Q ENCODING ERROR** ")
+(defvar mew-error-invalid-b-encoding " **B ENCODING ERROR** ")
+(defvar mew-error-invalid-q-encoding " **Q ENCODING ERROR** ")
 (defvar mew-error-no-subject         "** no subject **")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -98,10 +96,10 @@ cdr is new text. ")
 
 ;; In each folder
 
-(defvar mew-summary-cache-file ".mew-cache"
+(defvar mew-summary-cache-file ".mew-summary"
   "Cache file for Summary mode contents.")
 
-(defvar mew-summary-touch-file ".mew-touch"
+(defvar mew-summary-touch-file ".mew-mtime"
   "Time-stamp file for message folders.")
 
 ;; Under mew-conf-path
@@ -119,6 +117,7 @@ cdr is new text. ")
 
 (defvar mew-smtp-log-file "Smtplog")
 (defvar mew-nntp-log-file "Nntplog")
+(defvar mew-refile-log-file "Refilelog")
 
 ;; xxx Home
 
@@ -288,16 +287,16 @@ in draft buffer.")
     ("audio/x-midi" "\\.midi?$" mew-b64 mew-prog-audio mew-icon-audio)
     ("audio/x-mpeg" "\\.mpga$\\|\\.mp[23]$" mew-b64 mew-prog-audio mew-icon-audio)
     ;;
-    ("image/gif"   "\\.gif$"   mew-b64 mew-prog-image mew-icon-image)
-    ("image/tiff"  "\\.tif?f$" mew-b64 mew-prog-image mew-icon-image)
-    ("image/jpeg"  "\\.jpe?g$" mew-b64 mew-prog-image mew-icon-image)
-    ("image/png"   "\\.png$"   mew-b64 mew-prog-image mew-icon-image)
-    ("image/x-xwd" "\\.xwd$"   mew-b64 mew-prog-image mew-icon-image)
-    ("image/x-xbm" "\\.xbm$"   mew-b64 mew-prog-image mew-icon-image)
-    ("image/x-xpm" "\\.xpm$"   mew-b64 mew-prog-image mew-icon-image)
-    ("image/x-bmp" "\\.bmp$"   mew-b64 mew-prog-image mew-icon-image)
-    ("image/x-pcx" "\\.pcx$"   mew-b64 mew-prog-image mew-icon-image)
-    ("image/x-tga" "\\.tga$"   mew-b64 mew-prog-image mew-icon-image)
+    ("image/gif"   "\\.gif$"   mew-b64 mew-prog-image mew-icon-image gif)
+    ("image/tiff"  "\\.tif?f$" mew-b64 mew-prog-image mew-icon-image tiff)
+    ("image/jpeg"  "\\.jpe?g$" mew-b64 mew-prog-image mew-icon-image jpeg)
+    ("image/png"   "\\.png$"   mew-b64 mew-prog-image mew-icon-image png)
+    ("image/x-xwd" "\\.xwd$"   mew-b64 mew-prog-image mew-icon-image xwd)
+    ("image/x-xbm" "\\.xbm$"   mew-b64 mew-prog-image mew-icon-image xbm)
+    ("image/x-xpm" "\\.xpm$"   mew-b64 mew-prog-image mew-icon-image xpm)
+    ("image/x-bmp" "\\.bmp$"   mew-b64 mew-prog-image mew-icon-image bmp)
+    ("image/x-pcx" "\\.pcx$"   mew-b64 mew-prog-image mew-icon-image PCX)
+    ("image/x-tga" "\\.tga$"   mew-b64 mew-prog-image mew-icon-image TGA)
     ("image"       "^$"        mew-b64 mew-prog-image mew-icon-image)
     ;;
     ("model/iges" "\\.ige?s$" mew-b64 mew-prog-iges  mew-icon-image) ;; xxx
@@ -324,10 +323,14 @@ in draft buffer.")
      mew-prog-xml2                   mew-icon-text)
     ("application/msword"            "\\.doc$"  mew-b64
      mew-prog-msword                 mew-icon-text)
-    ("application/vnd.ms-excel"      "\\.xls$"  mew-b64
+    ("application/vnd.ms-excel"      "\\.xl[st]$" mew-b64
      mew-prog-msexcel                mew-icon-text)
     ("application/vnd.ms-powerpoint" "\\.ppt$"  mew-b64
      mew-prog-mspowerpoint           mew-icon-text)
+    ("application/vnd.visio"         "\\.vsd$"  mew-b64
+     mew-prog-visio                  mew-icon-unknown)
+    ("application/ms-tnef"           "\\.dat$" mew-b64
+     mew-prog-mstnef                 mew-icon-unknown)
     ("application/pgp-keys"          "\\.pka$"  nil
      mew-prog-pgp-keys               mew-icon-unknown)
     ("application/x-pkcs7-signature" "\\.p7s$"  mew-b64
@@ -357,7 +360,44 @@ in draft buffer.")
     ;; Unknown suffix matches here and return the entry specified 
     ;; by mew-content-type.
     (nil             ".*"))
-  "(content-type filename <encoding> <prog> <icon>)")
+  "Database for MIME content type.
+This is a list of entries which consist of <content-type>, 
+<regular expression of file suffix>, <content-transfer-encoding>, 
+<symbol for visualizer>, <symbol for icon>, <auxiliary data>.
+
+Here is an example:
+
+	(\"image/png\" \"\\\\.png$\" mew-b64 mew-prog-image mew-icon-image png)
+
+This database is used both when visualizing and composing. For
+example, when visualizing, a visualizer is chosen according to
+its <content-type>. When composing, content-type is selected
+according to the suffix of the file to be attached with <regular
+expression of file suffix>.
+
+For <content-transfer-encoding>, 'nil', 'mew-b64', or 'mew-qp'
+should be specified. 'mew-b64' and 'mew-qp' means Base64 and
+Quoted-Printable, respectively. This is used as an appropriate
+content-transfer-encoding when composing.
+
+Each symbol specified at <symbol for visualizer> should have a value
+whose format is one of followings:
+
+- symbol
+	symbol is a name of Elisp function for inline.
+- (symbol1 symbol2)
+	symbol1 is a name of Elisp function for inline.
+	symbol2 is a name of Elisp function for external.
+- (string list nil-or-t)
+	string is a name of an external program.
+	list is a list of arguments for the external program.
+	nil-or-t: 
+		If nil, the external program is called synchronously.
+		If t, the external program is called asynchronously.
+
+If you want to add an entry for new content-type, please let the author
+know. Such an entry should be shared by all users and be included in
+the next version of Mew.")
 
 (defsubst mew-ctdb-by-ct (ct)
   (mew-assoc-match2 ct mew-mime-content-type 0))
@@ -383,19 +423,8 @@ in draft buffer.")
 (defsubst mew-ctdb-icon (attr)
   (symbol-value (nth 4 attr)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; MIME image control
-;;;
-
-(defvar mew-mime-image-alist
-  '(("Image/Tiff" . tiff) ("Image/Gif" . gif) 
-    ("Image/Jpeg" . jpeg) ("Image/Png" . png)
-    ("Image/X-Xwd" . xwd) ("Image/X-Xbm" . xbm)
-    ("Image/X-Xpm" . xpm) ("Image/X-Bmp" . bmp)))
-
 (defsubst mew-mime-image-format-name (ct)
-  (cdr (assoc ct mew-mime-image-alist)))
+  (nth 5 (mew-ctdb-by-ct ct)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -588,8 +617,8 @@ in draft buffer.")
   "*Face to highlight the review mark"
   :group 'mew-highlight)
 
-(defface mew-face-mark-multi nil
-  "*Face to highlight the multi mark"
+(defface mew-face-mark-escape nil
+  "*Face to highlight the escape mark"
   :group 'mew-highlight)
 
 (defface mew-face-mark-delete nil
@@ -604,6 +633,10 @@ in draft buffer.")
   "*Face to highlight the refile mark"
   :group 'mew-highlight)
 
+(defface mew-face-mark-unread nil
+  "*Face to highlight the unread mark"
+  :group 'mew-highlight)
+
 (defface mew-face-eof-message nil
   "*Face to highlight the 'end of message' string"
   :group 'mew-highlight)
@@ -657,7 +690,7 @@ in draft buffer.")
     ("^Face:$" nil
      mew-face-header-private
      mew-face-header-marginal)
-    ("^X-" nil
+    ("^\\(X\\|Original\\)-" nil
      mew-face-header-private
      mew-face-header-marginal))
   "*An alist of field spec for Message mode. Each spec
@@ -720,10 +753,11 @@ present, mew-face-header-marginal is used."
 
 (defvar mew-highlight-mark-keywords
   `((,mew-mark-review . mew-face-mark-review)
-    (,mew-mark-multi  . mew-face-mark-multi)
+    (,mew-mark-escape . mew-face-mark-escape)
     (,mew-mark-delete . mew-face-mark-delete)
     (,mew-mark-unlink . mew-face-mark-unlink)
-    (,mew-mark-refile . mew-face-mark-refile))
+    (,mew-mark-refile . mew-face-mark-refile)
+    (,mew-mark-unread . mew-face-mark-unread))
   "A list of mark-face pair to used in Summary/Virtual mode.")
 
 (defsubst mew-highlight-mark-get-face (mark)
@@ -736,7 +770,8 @@ present, mew-face-header-marginal is used."
 
 (defvar mew-mark-afterstep-spec
   `((,mew-mark-review 0 0 0 0 0 0 0)
-    (,mew-mark-multi  0 0 0 0 0 0 0)
+    (,mew-mark-escape 0 0 0 0 0 0 0)
+    (,mew-mark-read   0 0 0 0 0 0 0)
     (,mew-mark-delete 2 0 2 0 0 0 0)
     (,mew-mark-unlink 2 0 2 0 0 0 0)
     (,mew-mark-refile 2 0 2 0 0 0 0))
@@ -775,11 +810,13 @@ For more detail, see mew-mark-put-mark and mew-mark-afterstep.")
   (nth case (assoc mark mew-mark-afterstep-spec)))
 
 (defvar mew-mark-spec
-  `((,mew-mark-review "review" 0 nil nil nil nil nil )
-    (,mew-mark-multi  "multi"  0 nil nil nil nil nil)
-    (,mew-mark-delete "delete" 1 nil t   nil mew-mark-exec-delete nil)
-    (,mew-mark-unlink "unlink" 1 nil t   nil mew-mark-exec-unlink nil)
-    (,mew-mark-refile "refile" 1 t   mew-mark-kill-refile mew-mark-unrefile
+  `((,mew-mark-read   "read"   0 nil nil nil nil nil)
+    (,mew-mark-unread "unread" 0 nil nil nil nil nil)
+    (,mew-mark-review "review" 1 nil nil nil nil nil)
+    (,mew-mark-escape "escape" 1 nil nil nil nil nil)
+    (,mew-mark-delete "delete" 2 nil t   nil mew-mark-exec-delete nil)
+    (,mew-mark-unlink "unlink" 2 nil t   nil mew-mark-exec-unlink nil)
+    (,mew-mark-refile "refile" 2 t   mew-mark-kill-refile mew-mark-unrefile
 		                     mew-mark-exec-refile mew-mark-sanity-refile))
   "*A list of lists which consists of
 mark, name, level, statefullp, kill-line-p, 
@@ -815,6 +852,69 @@ undo-func, exec-func, and sanity-fucn.")
 (defsubst mew-markdb-func-sanity (mark)
   (nth 7 (mew-markdb-by-mark mark)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Inbox action
+;;;
+
+(defvar mew-inbox-action-alist nil
+  "*This variable controls actions when retrieving messages.
+Currently this works only when '\\<mew-summary-mode-map>\\[mew-summary-retrieve]' with POP or 'mewl/incm'.
+You can put any marks, typically 'o' and 'D' according to message
+headers.
+
+This value must be a list of entries. An entry is a list whose first
+element is a field defined in 'mew-scan-fields'. 
+
+If the second element is a list, the first element of the list must be
+a mark (e.g. ?D) or a string which is a comma separated folders (e.g
+\"+foo,+bar\" for ?o). Other elements of the list are regular
+expressions to be matched to the value of the field.
+
+With the following configuration, Mew marks messages whose Subject:
+matches \"adult\" or \"money\" with 'D'. And Mew marks messages whose
+Subject is \"daily log\" with 'o' to be refiled \"+log\". Also Mew
+marks messages whose Content-Type: contains \"gb2312\" with 'D'.
+
+(setq mew-inbox-action-alist
+      '((\"Subject:\"
+	 (?D \"adult\" \"money\")
+	 (\"+log\" \"daily log\"))
+	(\"Content-Type:\"
+	 (?D \"gb2312\"))))
+
+If the second element of the entry is a symbol, it will be called as a
+function which returns nil or a mark or a string.
+
+If you are using 'spamassassin' which adds the \"X-Spam-Flag:\" field
+to messages and puts them in your mailbox, configure as follows:
+
+(setq mew-spam: \"X-Spam-Flag:\") ;; must be in \".emacs\"
+
+(defun mew-spam-assassin (val)
+  (let ((case-fold-search t))
+    (if (string-match \"yes\" val) ?D)))
+
+(setq mew-inbox-action-alist
+      '((\"X-Spam-Flag:\" mew-spam-assassin)))
+
+With this configuration, messages with the \"X-Spam-Flag:\" whose
+value is \"Yes\" are automatically marked with 'D' since the
+'mew-spam-assassin' function returns 'D' marks when the value is
+\"Yes\".
+
+If you are using 'bogofilter' which adds the \"X-Bogosity:\" field
+to messages and puts them to your mailbox, configure as follows:
+
+(setq mew-spam: \"X-Bogosity:\") ;; must be in \".emacs\" but default
+
+(defun mew-spam-bogofilter (val)
+  (let ((case-fold-search t))
+    (if (string-match \"yes\" val) ?D)))
+
+(setq mew-inbox-action-alist
+      '((\"X-Bogosity:\" mew-spam-bogofilter)))
+")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -835,12 +935,10 @@ If PROGRAM is a symbol, the lisp function whose name is PROGRAM is called.
 'mew-ext-url-show-by-w3' and 'mew-ext-url-fetch-by-w3' are pre-defined
 as lisp function.
 
-If you want to use \"w3.el\" instead of \"mozilla\", put the 
-following in .emacs.
+If you want to use \"w3.el\" instead of \"mozilla\", configure like this:
 	(setq mew-ext-url-alist
               '((t (\"Browse by w3\" mew-ext-url-show-by-w3 nil))))
-If you want to use \"lynx\" instead of \"mozilla\", put the 
-following in .emacs.
+If you want to use \"lynx\" instead of \"mozilla\", configure like this:
         (setq mew-ext-url-alist
               '((t (\"Browse by lynx\" \"kterm\" (\"-e\" \"lynx\" \"-color\")))))
 ")
@@ -874,8 +972,8 @@ following in .emacs.
 
 (defun mew-proto-to-body (fld n)
   (if (mew-folder-virtualp fld)
-      (setq fld (mew-thread-to-folder fld)))
-  (setq fld (mew-folder-folder fld))
+      (setq fld (mew-physical-folder fld)))
+  (setq fld (mew-case:folder-folder fld))
   (let ((proto (mew-folder-prefix fld)))
     (unless (member proto mew-folder-prefixes)
       (setq proto (mew-proto))) ;; default proto
@@ -939,27 +1037,26 @@ following in .emacs.
 
 Currently, the following keys are supported: 
 \"name\", \"user\", \"mail-domain\", 
-\"cc\", \"fcc\", \"dcc\", \"reply-to\",
-\"organization\", \"header-alist\", 
-\"smtp-server\", \"smtp-port\", \"smtp-ssh-server\", 
-\"smtp-ssl\", \"smtp-ssl-port\", 
-\"smtp-msgid-user\", \"smtp-msgid-domain\", 
-\"smtp-helo-domain\", \"smtp-mail-from\", 
+\"cc\", \"fcc\", \"dcc\", \"reply-to\", \"organization\", \"header-alist\", 
+\"proto\", 
+\"smtp-server\", \"smtp-port\", \"smtp-ssh-server\", \"smtp-ssl\", \"smtp-ssl-port\", 
 \"smtp-user\", \"smtp-auth\", \"smtp-auth-list\", 
-\"pop-server\", \"pop-port\", \"pop-ssh-server\", 
-\"pop-ssl\", \"pop-ssl-port\", 
-\"pop-user\", \"pop-auth\", 
-\"pop-size\", \"pop-body-lines\", \"pop-delete\", \"pop-folder\", 
-\"imap-server\", \"imap-port\", \"imap-ssh-server\", 
-\"imap-ssl\", \"imap-ssl-port\", 
-\"imap-user\", \"imap-auth\", \"imap-size\", \"imap-folder\", 
-\"nntp-server\", \"nntp-port\", \"nntp-ssh-server\", 
-\"nntp-ssl\", \"nntp-ssl-port\", 
-\"nntp-user\", \"nntp-size\", \"nntp-folder\",
+\"smtp-msgid-user\", \"smtp-msgid-domain\", \"smtp-helo-domain\", \"smtp-mail-from\", 
+\"pop-server\", \"pop-port\", \"pop-ssh-server\", \"pop-ssl\", \"pop-ssl-port\", 
+\"pop-user\", \"pop-auth\", \"pop-auth-list\",
+\"pop-size\", \"pop-body-lines\", \"pop-delete\", \"pop-body-lines\",
+\"pop-proxy-server\", \"pop-proxy-port\",
+\"imap-server\", \"imap-port\", \"imap-ssh-server\", \"imap-ssl\", \"imap-ssl-port\", 
+\"imap-user\", \"imap-auth\", \"imap-auth-list\", 
+\"imap-size\", \"imap-header-only\", \"imap-delete\", 
+\"imap-trash-folder\", \"imap-queue-folder\", \"imap-spam-field\", \"imap-spam-word\",
+\"nntp-server\", \"nntp-port\", \"nntp-ssh-server\", \"nntp-ssl\", \"nntp-ssl-port\", 
+\"nntp-user\", \"nntp-size\", \"nntp-header-only\", 
 \"nntp-msgid-user\", \"nntp-msgid-domain\", 
-\"inbox-folder\", \"queue-folder\",
+\"inbox-folder\", \"queue-folder\", \"postq-folder\", 
 \"mailbox-type\", \"mbox-command\", \"mbox-command-arg\", 
-\"signature-file\", \"content-type\" .
+\"use-retrieve\", \"signature-file\", \"content-type\", \"refile-guess-alist\",
+\"spam-prog\", \"spam-prog-args\", \"ham-prog\", \"ham-prog-args\".
 
 from = name <user@mail-domain>
 message-id = *random*.smtp-msgid-user@smtp-msgid-domain
