@@ -13,7 +13,7 @@
 ;;;
 
 (defvar mew-vinfo-list
-  '("count" "func" "lra" "top" "db" "thread-p" "column" "flds"))
+  '("func" "lra" "top" "db" "thread-p" "parent-folder" "column" "flds"))
 
 (mew-blinfo-defun 'mew-vinfo mew-vinfo-list)
 
@@ -76,14 +76,14 @@ execute 'mew-summary-virtual-with-external'. Otherwise,
 (defun mew-virtual-folder-name ()
   (if (mew-virtual-p)
       (car (mew-vinfo-get-flds))
-    (mew-summary-folder-name)))
+    (mew-summary-folder-name 'ext)))
 
 (defun mew-summary-virtual-with-internal ()
-  "Create Virtual mode with mewls."
+  "Create Virtual mode with mewl."
   (let* ((flds (mew-input-folders (mew-virtual-folder-name)))
 	 folders vfolder pattern opts lra)
     (when flds
-      (setq folders (mapcar 'mew-folder-remote-to-local flds))
+      (setq folders (mapcar 'mew-expand-folder2 flds))
       (setq pattern (mew-input-pick-pattern))
       (setq vfolder (mew-summary-get-vfolder flds))
       (mew-summary-switch-to-folder vfolder)
@@ -95,7 +95,7 @@ execute 'mew-summary-virtual-with-external'. Otherwise,
 	  (setq flds (cdr flds))
 	  (setq folders (cdr folders)))
 	(setq opts (cons "-a" (cons "-p" (cons pattern (nreverse opts)))))
-	(mew-local-retrieve 'vir opts 1 nil lra)))))
+	(mew-local-retrieve 'vir opts nil lra)))))
 
 (defvar mew-summary-virtual-with-external-function
   'mew-summary-virtual-with-grep
@@ -108,7 +108,7 @@ This function MUST returns a file name which contains message numbers.")
   (let* ((flds (mew-input-folders (mew-virtual-folder-name)))
 	 folders vfolder pattern file func opts lra)
     (when flds
-      (setq folders (mapcar 'mew-folder-remote-to-local flds))
+      (setq folders (mapcar 'mew-expand-folder2 flds))
       (setq pattern (mew-input-grep-pattern))
       (mew-sinfo-set-find-key pattern) ;; xxx necessary?
       (setq vfolder (mew-summary-get-vfolder flds))
@@ -127,7 +127,7 @@ This function MUST returns a file name which contains message numbers.")
 	  (setq lra (cons (cons (car folders) (car flds)) lra))
 	  (setq flds (cdr flds))
 	  (setq folders (cdr folders)))
-	(mew-local-retrieve 'vir opts 1 func lra)))))
+	(mew-local-retrieve 'vir opts func lra)))))
 
 (defun mew-summary-virtual-with-grep (flds pattern)
   (setq pattern (mew-cs-encode-arg pattern))
@@ -160,9 +160,7 @@ This function MUST returns a file name which contains message numbers.")
       (setq msgs (mapcar 'string-to-int msgs))
       (setq msgs (sort msgs '<))
       (setq msgs (mapcar 'int-to-string msgs))
-      (if (null msgs)
-	  ()
-	(setq picks (cons (cons dir msgs) picks)))
+      (if msgs (setq picks (cons (cons dir msgs) picks)))
       (setq flds (cdr flds)))
     (mew-erase-buffer)
     (setq picks (nreverse picks))
@@ -184,42 +182,57 @@ This function MUST returns a file name which contains message numbers.")
 ;;; mark virtual
 ;;;
 
-(defun mew-summary-mark-virtual ()
-  "Making Virtaul mode for messages marked with '*'."
-  (interactive)
+(defun mew-summary-mark-virtual (&optional ask-mark)
+  "Making Virtual mode for messages marked with '*'.
+If called with '\\[universal-argument]', you can specify a target mark."
+  (interactive "P")
   (if (not (mew-summary-p))
       (message "This command cannot be used in this mode")
     (let* ((folder (mew-summary-folder-name 'ext))
 	   (vfolder (mew-summary-get-vfolder (list folder)))
-	   (form (format "%%%ds" (mew-summary-scan-form-num folder)))
+	   (mark mew-mark-review)
 	   (start (point))
-	   (i 1)
-	   beg line msg)
+	   beg line med regex)
+      (if ask-mark (setq mark (mew-input-mark)))
+      (setq regex (mew-mark-regex mark))
       (mew-summary-switch-to-folder vfolder)
       (mew-erase-buffer)
       (mew-vinfo-set-flds (list folder))
       (set-buffer folder)
       (goto-char (point-min))
-      (while (re-search-forward mew-regex-msg-review nil t)
+      (while (re-search-forward regex nil t)
 	(beginning-of-line)
 	(setq beg (point))
-	(setq msg (mew-summary-message-number))
 	(forward-line)
 	;; This must be buffer-substring
 	(setq line (buffer-substring beg (point)))
 	(save-excursion
 	  (set-buffer vfolder)
 	  (mew-elet
-	   (insert (mew-sort-adjust-number form (int-to-string i) line))
-	   (forward-line -1)
-	   (end-of-line)
-	   (insert " \006 " folder " " msg)
-	   (forward-line)))
-	(setq i (1+ i)))
+	   (insert line)
+	   (save-excursion
+	     (when (and (search-backward "\r")
+			(setq med (point))
+			(looking-at mew-regex-sumsyn-short))
+	       (goto-char (match-beginning 1))
+	       (insert folder)
+	       (put-text-property med (point) 'invisible t))))))
       (goto-char start)
       (set-buffer vfolder)
+      (mew-summary-set-count-line)
       ;; Unmarking in both Summary and Thread
-      (mew-mark-undo-mark mew-mark-review 'no-msg))))
+      (if (char-equal mark mew-mark-review)
+	  (mew-mark-undo-mark mark 'no-msg 'virtual-only)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Old
+;;;
+
+(defun mew-summary-virtual-old ()
+  "Obsoleted command."
+  (interactive)
+  (mew-message-for-summary "This command was obsoleted. Use '\\[mew-summary-virtual]' to make Virtual mode."))
 
 (provide 'mew-virtual)
 
