@@ -2,7 +2,7 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1994
-;; Revised: Jun  1, 2004
+;; Revised: Oct 18, 2004
 
 ;;; Commentary:
 
@@ -19,7 +19,7 @@
 ;;; Mew version
 ;;;
 
-(defconst mew-version-number "4.0.65"
+(defconst mew-version-number "4.1"
   "Version number for this version of Mew.")
 (defconst mew-version (format "Mew version %s" mew-version-number)
   "Version string for this version of Mew.")
@@ -499,6 +499,7 @@ It is typically called by kill-emacs-hook."
 ;;; Regular expressions
 ;;;
 
+(defvar mew-regex-msg-show-list nil)
 (defvar mew-regex-msg-show nil)
 (defvar mew-regex-msg-or-part nil)
 (defvar mew-regex-msg-review nil)
@@ -515,10 +516,14 @@ It is typically called by kill-emacs-hook."
 (defsubst mew-mark-list-regex (mark-list)
   (concat "^" "[" (mapconcat 'char-to-string mark-list "") "]"))
 
+(defun mew-regex-setup-msg (lst)
+  (setq mew-regex-msg-show-list lst) 
+  (setq mew-regex-msg-show (mew-mark-list-regex lst))
+  (setq mew-regex-msg-or-part (concat mew-regex-msg-show "\\|" mew-regex-part)))
+
 (defun mew-regex-setup ()
   (setq mew-eoh (format "^\\(%s\\|\\)$" (regexp-quote mew-header-separator)))
-  (setq mew-regex-msg-show (mew-mark-list-regex (cons 32 mew-mark-show-list)))
-  (setq mew-regex-msg-or-part (concat mew-regex-msg-show "\\|" mew-regex-part))
+  (mew-regex-setup-msg mew-mark-show-list)
   (setq mew-regex-msg-review (mew-mark-regex mew-mark-review))
   (setq mew-regex-msg-delete (mew-mark-regex mew-mark-delete))
   (setq mew-regex-thread-separator
@@ -548,6 +553,15 @@ It is typically called by kill-emacs-hook."
 	    ((,mew-draft-folder) ,mew-range-str-all)
 	    (t ,mew-range-str-update)))))
 
+(defun mew-summary-toggle-mark-regex ()
+  (interactive)
+  (let (lst)
+    (if (equal mew-regex-msg-show-list mew-mark-show-list)
+	(setq lst mew-mark-show-list2)
+      (setq lst mew-mark-show-list))
+    (mew-regex-setup-msg lst)
+    (message "Target marks are: %s" (mapconcat (lambda (x) (message "'%c'" x)) lst ", "))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Theme
@@ -583,7 +597,7 @@ It is typically called by kill-emacs-hook."
 	(save-excursion
 	  (set-buffer buf)
 	  (when (mew-summary-or-virtual-p)
-	    (mew-summary-kill-subprocess 'no-msg)
+	    (mew-summary-kill-subprocess)
 	    (setq obuf (mew-local-buffer-name folder))
 	    ;; xxx should kill pop's buffer...
 	    (mew-remove-buffer obuf))
@@ -624,7 +638,7 @@ Mew remain, so you can resume with buffer operations."
     (mew-buffer-clean-up (mew-folder-regex mew-draft-folder)) ;; +draft/*
     (mew-buffer-clean-up mew-buffer-regex) ;; other buffers
     ;;
-    (mew-mark-clean-up)
+    (mew-sinfo-clean-up)
     (mew-buffers-clean-up) ;; Summary mode and Virtual mode
     ;;
     (mew-temp-dir-clean-up)
